/*Las secuencias deben empezar en el cdigo de tienda e incrementarse de 1000 en 1000*/
/*Los atributos que aparecen con (*) son parte de la clave. */


/*
 * Familia: Es la tabla que representa al tipo entidad FAMILIA.
 * Es la abstracin de las familias a las que pertenecen todos los productos.
 * Consta de:
 *   [(*) codigoFamilia] (cdigo dado por C.E.C.A.)
 *   [descripcion] (descripcin dada por C.E.C.A.)
 *
 */

DROP TABLE Familia;

CREATE TABLE Familia (
        codigoFamilia VARCHAR(10),
        descripcion   VARCHAR(20) NOT NULL,
	CONSTRAINT codigoFamilia_PK PRIMARY KEY (codigoFamilia)
);


/*
 * Producto: Es la tabla que representa al tipo entidad PRODUCTO.
 * Representa todas los elementos que factura la aplicacin.
 * Consta de:
 *   [(*) codigoProducto] Clave sustituta que hay que generar debido a que
 *                         es una categora con superclases de distinta clave.
 *   [codigoFamilia] Clave fornea de la tabla familia.
 *   [descripcion] Descripcin del producto utilizado.
 *   [tipoIVA] Porcentaje de IVA a aplicar por defecto al producto.
 *   [precio] Precio que lle pon Tere  producto.
 *   [dtoMax] Descuento mximo que se le debera realizar a las piezas.
 *   
 *
 */

DROP SEQUENCE seqProducto;

CREATE SEQUENCE seqProducto INCREMENT 1000 START 1;

DROP TABLE Producto;

CREATE TABLE Producto (
        codigoProducto BIGINT,
        codigoFamilia  VARCHAR(10)   NOT NULL,
        descripcion    CHARACTER(100) NOT NULL,
        tipoIVA        DECIMAL(5,2)      NOT NULL,
        precio         DECIMAL(10,2),
	dtoMax	       DECIMAL(5,2),
	CONSTRAINT codigoProducto_PK PRIMARY KEY (codigoProducto),
        CONSTRAINT codigoProducto_codigoFamilia_FK
                  FOREIGN KEY (codigoFamilia)
        	REFERENCES Familia (codigoFamilia) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX producto_familia
ON producto(codigoFamilia);

CREATE INDEX producto_descripcion
ON producto(descripcion);


/*
 * ManoDeObra: Es la tabla que representa al tipo entidad MANO DE OBRA.
 * Agrupa los distintos tipos de manos de obra que se facturan.
 * Consta de:
 *   [(*) codigoManoDeObra]
 *   [codigoProducto] Clave de enlace con el producto.
 *
 */

DROP SEQUENCE seqManoDeObra;

CREATE SEQUENCE seqManoDeObra INCREMENT 1000 START 1;

DROP TABLE ManoDeObra;

CREATE TABLE ManoDeObra (
        codigoManoDeObra BIGINT,
        codigoProducto   BIGINT  NOT NULL,
	CONSTRAINT codigoManoDeObra_PK PRIMARY KEY (codigoManoDeObra),
        CONSTRAINT manoDeObra_codigoProducto_FK
                  FOREIGN KEY (codigoProducto)
        	REFERENCES Producto (codigoProducto) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX manodeobra_producto
ON ManoDeObra(codigoProducto);

DROP SEQUENCE seqMedida;

CREATE SEQUENCE seqMedida  START 1 INCREMENT 1000;

DROP TABLE MEDIDA;

/*
 * Medida. Tabla que representa el tipo de entidad MEDIDA. Las medidas son utilizadas
 *         para identificar en qu se mide una unidad de una pieza que se vende por fragmentos. Por
 *         ejemplo: metro, kilogramo, ...
 *
 * Consta de:
 * [(*) codigoMedida] Cdigo identificador de la medida.
 * [nombre] Nombre de la medida. Por ejemplo: "kilogramos"
 * [abreviatura] Abreviatura de la medida. Por ejemplo: "kg"
 *
 */

CREATE TABLE MEDIDA (
        codigoMedida INTEGER,
        nombre VARCHAR(50),
        abreviatura VARCHAR(4),
	CONSTRAINT codigoMedida_PK
                PRIMARY KEY (codigoMedida)
);

/*
 * Pieza: Es la tabla que representa al tipo entidad PIEZA.
 * Agrupa los distintos tipos de piezas que se factura.
 * Consta de:
 *   [(*) EuroCode] Cdigo universal para su identificacin.
 *   [codigoProducto] Clave de enlace con el producto.
 *   [largo] Medida del largo del cristal (mm).
 *   [ancho] Medida del ancho del cristal (mm).
 *   [precioCoste] PRecio de coste de una unidad de la pieza.
 *   [unidadesPorPaquete] Unidades que trae un paquete de una pieza. Las piezas que se venden
 *                        individualmente contienen 1 en este campo. En cambio un rollo de
 *                        3 metros de cristal "x" contiene 3 en este campo.
 *   [codigoMedida] Cdigo que identifica la medida de la pieza. Con medida nos referimos a:
 *                  metros, kilogramos, etc. Es una clave fornea de la tabla medida que contiene
 *                  los nombres.
 *   [codigoProveedor] Proveedor asignado al producto.
 */

DROP TABLE Pieza;

CREATE TABLE Pieza (
        EuroCode       CHARACTER(16),
        codigoProducto BIGINT   UNIQUE  NOT NULL,
        largo          SMALLINT ,
        ancho          SMALLINT ,
        precioCoste    DECIMAL(10,2),
        unidadesPorPaquete  DECIMAL(10,2),
        codigoMedida         BIGINT,
        codigoProveedor      BIGINT,
	CONSTRAINT EuroCode_PK PRIMARY KEY (EuroCode),
        CONSTRAINT pieza_codigoProducto_FK
                  FOREIGN KEY (codigoProducto)
        	REFERENCES Producto (codigoProducto) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT pieza_codigoMedida_FK
                  FOREIGN KEY (codigoMedida)
        	REFERENCES Medida (codigoMedida) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX pieza_producto
ON Pieza(codigoProducto);

/*
 * Participante. Tabla que representa al tipo entidad PARTY.
 * Es la "puesta en prctica" del patrn Party.
 * Consta de:
 *   [(*) codigoParty] Cdigo que identifica al participante, es una clave generada
 *   [nombre] Nombre simple del participante. En el caso de particulares almacena el nombre y
 *            en caso de empresas la razn social.
 *   [CIF] NIF/CIF del participante.
 *   [nick] Nombre completo del participante. En el caso de particulares almacena el nombre y los
 *          apellidos. En caso de empresas almacena la razn social.
 *   [E-mail] Direccin de correo del participante.
 *   [Telfono] Primer telfono de contacto.
 *   [Telfono2] Segundo telfono de contacto.
 *   [Fax] Nmero de fax.
 *   [Calle] Calle en la que reside el participante.
 *   [Nmero] Nmero de edificio.
 *   [Piso] Piso dentro del edificio.
 *   [CP] Cdigo postal
 *   [Municipio] Municipio del participante.
 *   [Provincia] Provincia del municipio
 *   Campos tipo: Booleanos para saber que tipo de rol realiza el participante.
 *   -----------
 *   [proveedor]  Booleano que identifica a los proveedores
 *   [taller]     Booleano que identifica a los talleres
 *   [particular] Booleano que identifica a los particulares.
 *   [consorcio]  Booleano que identifica a los consorcios
 *   [compana]   Booleano que identifica a las compaas de seguros
 *   [empresa]     Booleano que identifica a las empresas.
 *   [tienda]     Booleano que identifica a las tiendas.
 */

DROP SEQUENCE seqParticipante;

CREATE SEQUENCE seqParticipante INCREMENT 1000 START 1;

DROP TABLE Participante;

CREATE TABLE Participante (
        codigoParty BIGINT,
        nombre      VARCHAR(256),
        CIF         VARCHAR(12),
        nick        VARCHAR(256),
        telefono    VARCHAR(25),
        telefono2    VARCHAR(25),
        fax         VARCHAR(25),
        email       VARCHAR(40),
        calle       VARCHAR(40),
        numero      VARCHAR(25),
        piso        VARCHAR(25),
        CP          BIGINT,
        municipio   VARCHAR(40),
        provincia   VARCHAR(25),
        proveedor   BOOLEAN     NOT NULL,
        taller      BOOLEAN     NOT NULL,
        particular  BOOLEAN     NOT NULL,
        consorcio   BOOLEAN     NOT NULL,
        compania    BOOLEAN     NOT NULL,
        empresa     BOOLEAN     NOT NULL,
        tienda      BOOLEAN     NOT NULL,
        nombreCorto      VARCHAR(256),
	CONSTRAINT codigoParticipante_PK PRIMARY KEY (codigoParty)
);

CREATE INDEX participante_nombre
ON Participante(nombre);

CREATE INDEX participante_codigo
ON Participante(codigoPArty);


/*
 * Direcciones de envo. Tabla para el almacenamiento de direcciones.
 * alternativas de los participantes.
 * Consta de:
 *   [(*) codigo] Cdigo para la identificacin de la direccin.
 *   [codigoParty] Cdigo que identifica al participante.
 *   [calle] Calle de la direccin de envo.
 *   [numero] Nmero del edificio
 *   [piso] Piso en el edificio.
 *   [CP] Cdigo postal.
 *   [municipio] Municipio de la direccin.
 *   [provincia] Provincia del municipio.
 */

DROP SEQUENCE seqDireccionEnvio;

CREATE SEQUENCE seqDireccionEnvio INCREMENT 1000 START 1;

DROP TABLE DireccionEnvio;

CREATE TABLE DireccionEnvio (
	codigo	    BIGINT,
        codigoParty BIGINT,
        calle       VARCHAR(40),
        numero      VARCHAR(25),
        piso        VARCHAR(25),
        CP          BIGINT,
        municipio   VARCHAR(40),
        provincia   VARCHAR(25),
	CONSTRAINT codigoDirecciones_PK PRIMARY KEY (codigo),
	CONSTRAINT Direcciones_codigoParty_FK FOREIGN KEY (codigoParty)
		REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

/*
 * Cliente: Tabla que representa al tipo entidad Cliente. Cualquier participante, excepto las
 * tiendas pueden desempear el rol de cliente del grupo.
 * Consta de:
 *  [(*) codigoParty] Cdigo que identifica al cliente, es la clave de la fornea que referencia al participante.
 *  [dtoCliente] Descuento al cliente.
 *  [agente] Indica si el cliente ha hecho alguna vez de agente
 */

DROP TABLE Cliente;

CREATE TABLE Cliente (
        codigoParty   BIGINT,
        dtoCliente    DECIMAL(5,2),
	agente	      BOOLEAN     NOT NULL,
	CONSTRAINT codigoCliente_PK PRIMARY KEY (codigoParty),
        CONSTRAINT participante_codigoParty_FK FOREIGN KEY (codigoParty)
		REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX cliente_codCliente
ON Cliente(codigoParty);


/*
 * Empresa: Tabla que representa al tipo entidad EMPRESA. una empresa puede representar cualquier
 *          empresa o bien representar a una empresa del grupo.
 * Consta de:
 *  [(*) codigoParty] Cdigo que identifica al cliente, es la clave de la fornea que referencia al participante.
 *  [perteneceGrupo] true si pertenece al grupo.
 */

DROP TABLE Empresa;

CREATE TABLE Empresa (
        codigoParty       BIGINT,
        perteneceGrupo BOOLEAN NOT NULL,
	CONSTRAINT codigoEmpresa_PK PRIMARY KEY (codigoParty),
        CONSTRAINT empresa_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES  Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX empresa_codEmpresa
ON Empresa(codigoParty);

/*
 * Tienda: Tabla que representa al tipo entidad TIENDA. Las tiendas no se pueden insertar desde
 *         la aplicacin ya que provocan cambios importantes en la aplicacin. Tiene dependencia
 *         de identificacin con el tipo entidad EMPRESA.
 * Consta de:
 *  [(*) codigoTienda] Cdigo que identifica a una tienda concreta
 *   de una empresa.
 *  [(*) codigoEmpresa] Cdigo que identifica a la empresa a la que
 *   pertenece la tienda.
 *  [codigoPertenenciaGrupo] Cdigo de cada tienda. Es una letra que distingue las facturas de cada tienda.
 *  [codigoReparador] Cdigo de cada tienda como centro reparador de ACR. Es un nmero asignado por
 *                    el consorcio.
 *
 */

DROP TABLE Tienda;

CREATE TABLE Tienda (
        codigoTienda   BIGINT,
        codigoEmpresa  BIGINT,
        codigoPertenenciaGrupo VARCHAR(5),
	codigoreparador VARCHAR(4),
	CONSTRAINT codigoTienda_PK PRIMARY KEY (codigoEmpresa,codigoTienda),
        CONSTRAINT tienda_codigoEmpresa_FK FOREIGN KEY (codigoEmpresa)
		REFERENCES Empresa (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT participante_codigoParty_FK FOREIGN KEY (codigoTienda)
		REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);


CREATE INDEX tienda_empresa
ON Tienda(codigoEmpresa);

CREATE INDEX tienda_tienda
ON Tienda(codigoTienda);

CREATE INDEX tienda_tienda_empresa
ON Tienda(codigoTienda,codigoEmpresa);

/*
 * Banco. Tabla que representa al tipo entidad BANCO.
 * Consta de:
 *   [(*) codigoBanco] Cdigo que identifica al banco.
 *   [nombre] Nombre del banco.
 */


DROP TABLE Banco;

CREATE TABLE Banco (
        codigoBanco SMALLINT,
        nombre      VARCHAR(96),
	CONSTRAINT codigoBanco_PK PRIMARY KEY (codigoBanco)
);

CREATE INDEX banco_nombre
ON Banco(nombre);

/*
 * Oficina. Tabla que representa al tipo entidad OFICINA.
 * Tiene dependencia de identificacin con el tipo entidad BANCO.
 * Consta de:
 *   [(*) codigoBanco] Cdigo que identifica al banco al que pertenece
 *     la oficina.
 *   [(*) codigoOficina] Cdigo que identifica a la oficina de un banco.
 *   [nombre] Nombre de la oficina bancaria.
 *   [direccion] Deireccin de la sucursal.
 */

DROP TABLE Oficina;

CREATE TABLE Oficina (
        codigoBanco   SMALLINT,
        codigoOficina SMALLINT,
        nombre        VARCHAR(50),
        direccion     VARCHAR(50),
	CONSTRAINT codigoOficina_PK PRIMARY KEY (codigoBanco, codigoOficina),
        CONSTRAINT oficina_codigoBanco_FK FOREIGN KEY (codigoBanco)
		REFERENCES Banco (codigoBanco) ON DELETE CASCADE ON UPDATE CASCADE

);

CREATE INDEX oficina_banco
ON oficina(codigoBanco);

/*
 * CuentaCorriente. Tabla que representa al tipo entidad CUENTA CORRIENTE. Esta tabla solo se
 *                  utiliza para las cuentas corrientes de las empresas del grupo. En esta tabla
 *                  solo se insertan tuplas ya insertadas en la tabla CCParty.
 * Tiene dependencia de identificacin con el tipo entidad OFICINA.
 * Consta de:
 *   [(*) codigoBanco] Cdigo que identifica al banco al que
 *      pertenece la oficina.
 *   [(*) codigoOficina] Cdigo que identifica a la oficina de un banco.
 *   [(*) numeroCuenta] Cdigo que identifica una cuenta corriente de una oficina
 *   [DC] Dgito de control. NO se necesita, es un nmero calculado.
 *   [SaldoInicial] Indica el saldo inicial de las cuentas corrientes del grupo al iniciar las
 *                  operaciones.
 */

DROP TABLE CuentaCorriente;

CREATE TABLE CuentaCorriente (
        codigoBanco   SMALLINT,
        codigoOficina SMALLINT,
        numeroCuenta  BIGINT,
        DC            SMALLINT NOT NULL,
        SaldoInicial  DECIMAL(10,2),
	CONSTRAINT codigoCuentaCorriente_PK
                PRIMARY KEY (codigoBanco, codigoOficina, numeroCuenta),
        CONSTRAINT cuentaCorriente_codigoOficina_FK
                  FOREIGN KEY (codigoBanco, codigoOficina)
        	REFERENCES Oficina (codigoBanco, codigoOficina) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX numeroCuenta_cuentaCorriente
ON CuentaCorriente(codigoBanco, codigoOficina, DC, numeroCuenta);

CREATE INDEX numeroCuenta_oficina
ON CuentaCorriente(codigoBanco, codigoOficina);


/*
 * Movimiento. Tabla que representa al tipo entidad MOVIMIENTO de una cuenta corriente..
 * Tiene dependencia de identificacin con el tipo entidad CUENTA CORRIENTE.
 * Consta de:
 *     [(*) codigomovimiento] Cdigo que identifica el movimiento
 *    en concreto.
 *     [codigoBanco] Cdigo del banco en el que se realiza el movimiento.
 *     [codigoOficina] Cdigo de la oficina bancaria en la que se realiza el movimiento.
 *     [codigoPago] Cdigo del pago con el que est relacionado en movimiento bancario.
 *     [numeroCuenta] Nmero de la cuenta corriente en la que se produce el movimiento.
 *     [Descripcin] Descripcin del movimiento.
 *     [Fecha] Fecha en el que se hizo el movimiento.
 *     [Cantidad] Cantidad que se mueve.
 *     [referenciaRecibo] Si es domiciliacin la referencia del recibo bancario.
 *     [fechaNotificacion] Fecha en la que se notifica el ingreso.
 */

DROP SEQUENCE seqMovimiento;

CREATE SEQUENCE seqMovimiento INCREMENT 1000 START 1;

DROP TABLE Movimiento;

CREATE TABLE Movimiento (
        codigoMovimiento BIGINT,
        codigoBanco      SMALLINT,
        codigoOficina    SMALLINT,
        codigoPago       BIGINT,
        numeroCuenta     BIGINT,
        descripcion      VARCHAR(200) NOT NULL,
        fecha            TIMESTAMP,
        cantidad         DECIMAL(10,2),
        referenciaRecibo VARCHAR(40),
        fechaNotificacion  TIMESTAMP,
	CONSTRAINT movimiento_PK
                PRIMARY KEY (codigoMovimiento),
        CONSTRAINT movimiento_codigoCuentaCorriente_FK
                  FOREIGN KEY (codigoBanco, codigoOficina, numeroCuenta)
        	REFERENCES CuentaCorriente (codigoBanco, codigoOficina, numeroCuenta) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX movimiento_codigoCuenta
ON Movimiento(codigoBanco, codigoOficina, numeroCuenta);

/*
 * CCParty. Es la tabla que representa al tipo relacin "es de" que existe
 * entre una CUENTA CORRIENTE y un PARTY (participante). Esta tabla almacena todas las cuentas corrientes
 * del sistema.
 *
 * Consta de:
 *   [(*) codigoBanco] Cdigo que identifica al banco al que
 *      pertenece la oficina.
 *   [(*) codigoOficina] Cdigo que identifica a la oficina de un banco.
 *   [(*) numeroCuenta] Cdigo que identifica una cuenta corriente de una oficina.
 *   [(*) codigoParty] Cdigo que identifica al participante.
 */

DROP TABLE CCParty;

CREATE TABLE CCParty (
        codigoBanco   SMALLINT,
        codigoOficina SMALLINT,
        numeroCuenta  BIGINT,
        codigoParty   BIGINT,
	CONSTRAINT codigoCCParty_PK
                PRIMARY KEY (codigoBanco, codigoOficina, numeroCuenta, codigoParty),
        CONSTRAINT ccParty_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES  Cliente (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX CCParty_codigoCuenta
ON CCParty(codigoBanco, codigoOficina, numeroCuenta);

CREATE INDEX CCParty_codigoParty
ON CCParty(codigoParty);

/*
 * Proveedor. Tabla que representa al tipo entidad PROVEEDOR.
 * Especializacin de "participante". Los proveedores del grupo de empresas apareceran en esta tabla.
 * Consta de:
 *    [(*) codigoParty] Cdigo que identifica al participante.
 */

DROP TABLE Proveedor;

CREATE TABLE Proveedor (
        codigoParty BIGINT,
	CONSTRAINT codigoProveedor_PK
                PRIMARY KEY (codigoParty),
        CONSTRAINT proveedor_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);


/*
 * Taller. Tabla que representa al tipo entidad TALLER.
 * Especializacin de "participante". Los talleres externos al grupo y que enven vehculos
 * a reparar a tiendas del grupo actuarn como talleres.
 * Consta de:
 *    [(*) codigoParty] Cdigo que identifica al participante.
 *    [comision] Comisin que se le aplica a un taller.
 */

DROP TABLE Taller;

CREATE TABLE Taller (
        codigoParty BIGINT,
        comision    DECIMAL(5,2),
	CONSTRAINT codigoTaller_PK
                PRIMARY KEY (codigoParty),
        CONSTRAINT taller_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);


/*
 * Particular. Tabla que representa al tipo entidad PARTICULAR. Las personas
 * que van a reparar vehculos a una tienda del grupo son particulares. En el momento en que una persona
 * acta como empresa debera aparecer como empresa.
 * Especializacin de "participante".
 * Consta de:
 *    [(*) codigoParty] Cdigo que identifica al participante.
 *    [primerApellido] Primer apellido del particular.
 *    [segundoApellido] Segundo apellido del particular.
 */

DROP TABLE Particular;

CREATE TABLE Particular (
        codigoParty     BIGINT,
        primerApellido  VARCHAR(48),
        segundoApellido VARCHAR(48),
	CONSTRAINT codigoParticular_PK
                PRIMARY KEY (codigoParty),
        CONSTRAINT particular_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX particular_primerApellido
ON Particular(primerApellido);

CREATE INDEX particular_segundoApellido
ON Particular(segundoApellido);

CREATE INDEX particular_apellido
ON Particular(primerApellido, segundoApellido);

/*
 * Consorcio. Tabla que representa al tipo entidad CONSORCIO. Los consorcios de seguros desempearn
 *            este rol. Especializacin de "participante".
 * Consta de:
 *    [(*) codigoParty] Cdigo que identifica al participante.
 *    [dtoConsorcio] Descuento general que se le aplicar a las compaas de seguros que pertenezcan
 *                   a este consorcio. Existe alguna excepcin como Winthertur cuyo descuento se
 *                   impone al del consorcio.
 */

DROP TABLE Consorcio;

CREATE TABLE Consorcio (
        codigoParty BIGINT,
	dtoConsorcio DECIMAL(5,2),
	CONSTRAINT codigoConsorcio_PK
                PRIMARY KEY (codigoParty),
        CONSTRAINT consorcio_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);


/*
 * Compaa. Tabla que representa al tipo entidad COMPAA (solo referente a las compaas de seguros).
 *           La compaas de seguros van todas
 *           en esta tabla. Especializacin de "participante".
 * Consta de:
 *    [(*) codigoParty] Cdigo que identifica al participante.
 *    [codigoConsorcio] Consorcio al que pertenece la compaa de seguros. Clave fornea de
 *    la tabla consorcio.
 *    [dtoCompania] Descuento de la compaa de seguros, este descuento se utiliza cuando
 *                  la compaa de seguros no pertenece a ningn consorcio o en el caso de
 *                  de que la compa sea Winthertur.
 *    [plantilla] La plantilla es la plantilla con la que se imprimirn los albaranes y facturas
 *                que vayan a esta compaa de seguros.
 */

DROP TABLE Compania;

CREATE TABLE Compania (
        codigoParty     BIGINT,
        codigoConsorcio BIGINT,
        dtoCompania     DECIMAL(5,2),
        plantilla       INT4,
	CONSTRAINT codigoCompania_PK
                PRIMARY KEY (codigoParty),
        CONSTRAINT compania_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT compania_codigoConsorcio_FK
                  FOREIGN KEY (codigoConsorcio)
        	REFERENCES Consorcio (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);


CREATE INDEX compania_codigoConsorcio
ON Compania(codigoConsorcio);


/*
 * Pliza. Esta es la tabla que representa al tipo entidad PLIZA. Tiene
 * dependencia de identificacin con el tipo entidad COMPAA. Es la tabla que se
 * utiliza para almacenar las plizas de seguro de los clientes.
 * Sus atributos son:
 *    [(*) codigoCompania] Cdigo de la compaa de seguros con la que se contrat la
 *                         pliza de seguros. Clave fornea de la tabla cliente y participante.
 *    [(*) numeroPoliza] Nmero de la pliza de seguros.
 *    [(*) codigoTitular] Cdigo que identifica el titular de
 *    			la pliza.
 */

DROP TABLE Poliza;

CREATE TABLE Poliza (
        codigoCompania  BIGINT,
        numeroPoliza    VARCHAR(20),
        codigoTitular   BIGINT,
	CONSTRAINT codigoParty_PK
                PRIMARY KEY (codigoCompania, numeroPoliza, codigoTitular),
        CONSTRAINT poliza_codigoCompania_FK
                  FOREIGN KEY (codigoCompania)
        	REFERENCES Compania (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT poliza_codigoTitular_FK
                  FOREIGN KEY (codigoTitular)
        	REFERENCES Cliente (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX poliza_codigoCompania
ON Poliza(codigoCompania);

CREATE INDEX poliza_codigoTitular
ON Poliza(codigoTitular);
 
/*
 * Recibo. Esta es la tabla que representa al tipo entidad RECIBO, que se utiliza para almacenar
 * los recibos de las plizas de seguros. Tiene
 * dependencia de identificacin con el tipo entidad PLIZA.
 * Sus atributos son:
 *    [(*) codigoCompania] Cdigo de la compaa de seguros con la que se contrat la
 *                         pliza de seguros. Clave fornea de la tabla cliente y participante.
 *    [(*) numeroPoliza] Nmero de la pliza de seguros.
 *    [(*) codigoTitular] Cdigo que identifica el titular de
 *    			la pliza.
 *    [(*) numeroRecibo] Nmero del ltimo recibo pagado de la pliza de seguros.
 *    [sucursalMediador] Sucursal mediadora en la contratacin de la pliza.
 *    [inicioValidez] Inicio de la validez del recibo.
 *    [finValidez] Fin de la validez del recibo.
 * 
 */

DROP TABLE Recibo;

CREATE TABLE Recibo (
        codigoCompania   BIGINT,
        numeroPoliza     VARCHAR(20),
        codigoTitular    BIGINT,
        numeroRecibo     VARCHAR(20),
        sucursalMediador VARCHAR(64),
	inicioValidez	 TIMESTAMP,
        finValidez       TIMESTAMP ,
	CONSTRAINT codigoRecibo_PK
                PRIMARY KEY (codigoCompania, numeroPoliza, codigoTitular, numeroRecibo),
	CONSTRAINT recibo_fechasMenores CHECK (inicioValidez <= finValidez),
        CONSTRAINT recibo_codigoCompaniayPoliza_FK
                  FOREIGN KEY (codigoCompania, numeroPoliza, codigoTitular)
        	REFERENCES Poliza (codigoCompania, numeroPoliza, codigoTitular) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX recibo_poliza
ON Recibo(codigoCompania, numeroPoliza, codigoTitular);

/*
 * Tasacin. Es la tabla que representa al tipo relacin tasa. Una tasacin el el valor que una compaa
 *           de seguros considera que cuenta una mano de obra, de modo que se cobrar un precio
 *           u otro por la mano de obra segn la compaa de seguros.
 *    [(*) codigoAseguradora] codigo que identifica el tasador del producto.
 *    [(*) codigoManoDeObra] Cdigo de la mano de obra tasada.
 *    [precio] Precio que le ha puesto el tasador al producto sin IVA.
 */

DROP TABLE Tasacion;

CREATE TABLE Tasacion (
        codigoAseguradora  BIGINT,
        codigoManoDeObra BIGINT,
        precio         DECIMAL(10,2) NOT NULL,
	CONSTRAINT tasacion_PK
                PRIMARY KEY (codigoAseguradora, codigoManoDeObra),
        CONSTRAINT tasacion_codigoParty_FK
                  FOREIGN KEY (codigoAseguradora)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT tasacion_codigoProducto_FK
                  FOREIGN KEY (codigoManoDeObra)
        	REFERENCES ManoDeObra (codigoManoDeObra) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX tasacion_codigoAseguradora
ON Tasacion(codigoAseguradora);

CREATE INDEX tasacion_codigoProducto
ON Tasacion(codigoManoDeObra);

/*
 * Vehculo. Tabla que representa al tipo entidad VEHCULO. Los vehculos que se reparen en
 *           los talleres del grupo quedarn almacenados en esta tabla. Los datos color, matricula y
 *           kilmetros quedarn almacenados en la reparacin por si un vehculo cambia estos datos de la
 *           primera vez que es reparado a la siguiente.
 * Consta de:
 *     [(*) codigoVehiculo] Cdigo generado para la identificacin unvoca de un
 *      vehculo.
 *     [Matrcula] Matrculo actual del vehculo.
 *     [numeroBastidor] Nmero de bastidor del vehculo.
 *     [kilometros] kilmetros recorridos hata el momento por el vehculo.
 *     [modelo] Modelo del vehculo.
 *     [marca] Marca.
 *     [color] Color del vehculo.
 */

DROP SEQUENCE seqVehiculo;

CREATE SEQUENCE seqVehiculo INCREMENT 1000 START 1;

DROP TABLE Vehiculo;

CREATE TABLE Vehiculo (
        codigoVehiculo BIGINT,
        matricula      VARCHAR(12),
        numeroBastidor VARCHAR(48),
        kilometros     BIGINT,
        modelo         VARCHAR(16),
        marca          VARCHAR(16),
        color          VARCHAR(16),
	CONSTRAINT vehiculo_PK
                PRIMARY KEY (codigoVehiculo)
);

CREATE INDEX vehiculo_codigo
ON Vehiculo(codigoVehiculo);

CREATE INDEX vehiculo_matricula
ON Vehiculo(matricula);

CREATE INDEX vehiculo_numeroBastidor
ON Vehiculo(numeroBastidor);


/*
 * Reparacin. Tabla que representa al tipo entidad REPARACIN. La reparacin es la entidad central de
 *             la aplicacin. Las reparacines se almacenan en el momento en el que el coche
 *             ha sido reparado en uno de los talleres del grupo.
 * Tiene dependencia de identificacin con el tipo entidad TIENDA.
 * Consta de:
 *   [(*) codigoReparacion] Cdigo que identifica unvocamente una reparacin de una tienda.
 *   [(*) codigoTienda] Cdigo que identifica la tienda donde se ha
 *     			realizado la reparacin.
 *   [(*) codigoEmpresa] Cdigo que identifica la empresa donde se ha
 *     			 realizado la reparacin.
 *   [codigoTtaller] Cdigo del taller que envi el vehculo para la reparacin (a null/-1 si el vehculo
 *     no fue enviado por ningn taller).
 *   [fechaEntrada] Hora y fecha en la que el vehculo entr en el taller para reparar.
 *   [fechaSalida] Hora y fecha en la que el vehculo sali del taller de reparar.
 *   [codigoCliente] Cdigo de participante que indica el cliente
 *     que al que se le realiza la reparacin.
 *   [codigoCompania] Cdigo de la compaa de seguros con la que se contrata la pliza.
 *   [numeroPoliza] Cdigo que identifica la pliza que cubre la
 *     reparacin.
 *   [numeroRecibo] Nmero del ltimo recibo pagado.
 *   [codigoVehiculo] Identificador del vehculo al que se le realiza
 *     la reparacin.
 *   [observaciones] Observaciones que desee introducir el reparador.
 *   [distanciaUnidadMovil] Distancia a la que se encontraba la unidad mvil.
 *   [lugarUnidadMovil] Lugar de la unidad mvil.
 *   [importeTotal] Importe total de la reparacin
 *   [codigoAgente] Cdigo del participante que acta como agente en la reparacin.
 *   [matricula] Matrcula del vehculo reparado.
 *   [kilometros] Kilmetros del vehculo reparado.
 *   [color] Color del vehculo reparado.
 */

DROP SEQUENCE seqReparacion;

CREATE SEQUENCE seqReparacion INCREMENT 1000 START 1;

DROP TABLE Reparacion;

CREATE TABLE Reparacion (
        codigoReparacion BIGINT,
        codigoEmpresa    BIGINT,
        codigoTienda     BIGINT,
        codigoTaller     BIGINT,
        fechaEntrada     TIMESTAMP,
        fechaSalida      TIMESTAMP,
        codigoCliente    BIGINT NOT NULL,
        codigoCompania   BIGINT,
	numeroPoliza     VARCHAR(20),
	numeroRecibo     VARCHAR(20),				
        codigoVehiculo   VARCHAR(48) NOT NULL,
        observaciones    VARCHAR(500),
        distanciaUnidadMovil BIGINT,
        lugarUnidadMovil VARCHAR(32),
        importeTotal     DECIMAL(10,2),
        codigoAgente     BIGINT,
        matricula        VARCHAR(12),
        kilometros       BIGINT,
        color            VARCHAR(16),
        CONSTRAINT codigoReparacion_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda, codigoReparacion),
	CONSTRAINT reparacion_fechasMenores CHECK (fechaEntrada<=fechaSalida),
        CONSTRAINT reparacion_codigoTienda_FK
                   FOREIGN KEY    (codigoEmpresa, codigoTienda)
		REFERENCES Tienda (codigoEmpresa, codigoTienda) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT reparacion_codigoAgente_FK
                  FOREIGN KEY (codigoAgente)
        	REFERENCES Cliente (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT reparacion_codigoTaller_FK
                   FOREIGN KEY    (codigoTaller)
		REFERENCES Taller (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT reparacion_codigoCliente_FK
                   FOREIGN KEY    (codigoCliente)
		REFERENCES Cliente (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT reparacion_codigoRecibo_FK
                   FOREIGN KEY    (codigoCompania, numeroPoliza, codigoCliente, numeroRecibo)
		REFERENCES Recibo (codigoCompania, numeroPoliza, codigoTitular, numeroRecibo) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT reparacion_codigoVehiculo_FK
                   FOREIGN KEY      (codigoVehiculo)
		REFERENCES Vehiculo (codigoVehiculo) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX r_codigoTienda
ON Reparacion(codigoEmpresa, codigoTienda);

CREATE INDEX r_codTaller
ON Reparacion(codigoTaller);

CREATE INDEX r_codTallerYCliente
ON Reparacion(codigoTaller, codigoCliente);

CREATE INDEX r_codCliente
ON Reparacion(codigoCliente);

CREATE INDEX r_codPoliza
ON Reparacion(codigoCompania, numeroPoliza, codigoCliente);

CREATE INDEX r_codRecibo
ON Reparacion(codigoCompania, numeroPoliza, codigoCliente, numeroRecibo);

CREATE INDEX r_codVehiculo
ON Reparacion(codigoVehiculo);

CREATE INDEX r_codVehiculoYCliente
ON Reparacion(codigoVehiculo, codigoCliente);

CREATE INDEX r_codVehiculoYTaller
ON Reparacion(codigoVehiculo, codigoTaller);

CREATE INDEX r_codVehiculoYTallerYCliente
ON Reparacion(codigoVehiculo, codigoTaller, codigoCliente);

/*
 * Siniestro. Tabla que representa al tipo entidad SINIESTRO. En esta tabla se almacenarn todos los
 *            accidentes que se produjeran con cobertura del seguro.
 * Tiene dependencia de identificacin con el tipo entidad REPARACIN
 * Consta de:
 * [(*) codigoReparacion] Cdigo que identifica la reparacin que
 *      viene provocada por este siniestro.
 * [(*) codigoempresa] Cdigo de la empresa a la que pertenece la tienda
 *                 en la que se repar el vehculo siniestrado.
 * [(*) codigotienda] Cdigo de la tienda a la que pertenece la tienda
 *                 en la que se repar el vehculo siniestrado.
 * [referenciasiniestro] Cdigo que proporciona la compaa de
 *      seguros/consorcio para la identificacin del siniestro
 *            [CECA] Nmero de expediente.
 *            [ROYAL] Nmero de siniestro.
 *            [ACR] Ref. siniestro
 * [fecha] Fecha en la que se profujo el siniestro.
 * [lugar] Lugar donde se produjo el siniestro.
 * [ramo] Ramo del siniestro.
 * [danos] Daos que se aprecian aparentemente.               
 * [modo]  Modo en el que se produjo el siniestro.                
 * [codprov] Cdigo de provincia en la que se produjo el siniestro.
 * [codcausa] Cdigo de la causa de siniestro.
 * [descripcion] Descripcin del siniestro.
 * [foto1] Foto del vehculo antes de la reparacin
 * [foto2] Segunda foto del vehculo antes de la reparacin por se fuese necesario reparar ms de una luna.
 * [fechaapertura] Fecha de apertura del expediente, se solicita a la compaa de seguros.
 * 
 */

DROP TABLE Siniestro;

CREATE TABLE Siniestro (
        codigoReparacion     BIGINT,
        codigoEmpresa        BIGINT,
        codigoTienda         BIGINT,
        referenciaSiniestro  VARCHAR(48),
        fecha                TIMESTAMP,
        lugar                VARCHAR(48),
        ramo                 VARCHAR(20),
        danos                VARCHAR(400),
        modo                 VARCHAR(48),
	codProv		     VARCHAR(2),
	codCausa	     VARCHAR(8),
	descripcion	     VARCHAR(500),
	foto1		     VARCHAR(128),
	foto2		     VARCHAR(128),
        fechaApertura        TIMESTAMP,
	CONSTRAINT codigoSiniestro_PK
                   PRIMARY KEY (codigoReparacion, codigoEmpresa, codigoTienda),
        CONSTRAINT siniestro_codigohTienda_FK
                   FOREIGN KEY    (codigoReparacion, codigoEmpresa, codigoTienda)
		REFERENCES Reparacion (codigoReparacion, codigoEmpresa, codigoTienda) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX siniestro_codigoTienda
ON Siniestro(codigoReparacion, codigoEmpresa, codigoTienda);

/*
 * Lnea de reparacin. Tabla que representa al tipo entidad
 * LNEA DE REPARACIN.
 * Tiene dependencia de identificacin con el tipo entidad REPARACIN.
 * Consta de:
 * [(*) codigoreparacion] Cdigo que identifica unvocamente una
 *      reparacin.
 * [(*) codigoempresa] Cdigo de empresa a la que pertenece la tienda en la que se gener la reparacin.
 * [(*) codigotienda] Cdigo de  la tienda en la que se gener la reparacin.
 * [codigoalmacen] Cdigo del almacn en el que se extray la pieza.
 * [(*) numerolinea] Nmero de la lnea de la reparacin.
 * [codigoproducto] Producto que se referencia en la lnea de reparacin.
 * [unidades] Unidades que se utilizaron del producto.
 * [precio] Precio de la lnea de reparacin. Se utiliza como importe de la lnea. Para calcular precios
 *          no tiene en cuenta el descuento.
 * [dto] Descuento que se le aplica sobre el precio del producto.
 * [preciocoste] Precio de coste de la pieza.
 * [preciohistorico] Precio anterior de la pieza cuando se modific el precio de la pieza.
 */

DROP SEQUENCE seqLineaReparacion;

CREATE SEQUENCE seqLineaReparacion INCREMENT 1000 START 1;

DROP TABLE LineaReparacion;

CREATE TABLE LineaReparacion (
        codigoReparacion     BIGINT,
        codigoEmpresa        BIGINT,
        codigoTienda         BIGINT,
	codigoAlmacen	     BIGINT,
        numeroLinea          BIGINT,
        codigoProducto       BIGINT,
        unidades             DECIMAL(10,3),
        precio               DECIMAL(10,2),
        dto                  DECIMAL(5,2),
        precioCoste          DECIMAL(10,2),
        precioHistorico       DECIMAL(10,2),
        tipoIvaHistorico       DECIMAL(10,2),
	CONSTRAINT codigoLineaReparacion_PK PRIMARY KEY
               (codigoReparacion, codigoEmpresa, codigoTienda, numeroLinea),
        CONSTRAINT lineaReparacion_codigoReparacion_FK
                  FOREIGN KEY (codigoReparacion, codigoEmpresa, codigoTienda)
        	REFERENCES Reparacion (codigoReparacion, codigoEmpresa, codigoTienda) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT lineaReparacion_codigoProducto_FK
                  FOREIGN KEY (codigoProducto)
        	REFERENCES Producto (codigoProducto) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX lineaReparacion_codigoReparacion
ON LineaReparacion(codigoReparacion, codigoEmpresa, codigoTienda);

CREATE INDEX lineaReparacion_codigoProducto
ON LineaReparacion(codigoProducto);

/*
 * Pago. Tabla que representa el tipo entidad PAGO. Los pagos son generados automticamante al almacenar
 *        una factura en la base de datos. Las facturas que generan pagos son: facturas emitidas, facturas a pagar,
 *        notas emitidas y notas a pagar. Tiene dependencia de identificacin con el tipo entidad TIENDA y EMPRESA.
 * Sus atributos son:
 * [codigoempresa] Cdigo de la empresa a la que pertenece la tienda a la que pertenece un pago
 *      concreto.
 * [codigotienda] Cdigo de la tienda a la que pertenece un pago
 *      concreto.
 * [codigopago] Cdigo que identifica unvocamente un
 *      pago de una tienda concreta.
 * [pagosrealizados] Indica los pagos que ya se han realizado en el caso de pagos a plazos.
 * [fecharealizado] Indica la fecha en la que se realiza el pago.
 * [esaplazos] Booleano para detectar si es un pago a plazos.
 * [fechainicio] Fecha en la que se realizar el primer pago.
 * [porcentajeentrada] Porcentaje de entrada  (sobre el importe total)
 *      que se aplica en el primer vencimiento. Los importes segundo y siguientes
 *      son siempre del mismo importe.
 * [numerovencimientos] Nmero de plazos en que se realiza el pago.
 * [diasintervalo]  Nmero de das que existe entre los pagos.
 * [esdomiciliacion] Indica si el pago se realizar por domiciliacin.
 * [codigobanco] Cdigo del banco en el que domiciliar el pago.
 * [codigooficina] Cdigo de oficina en el que domiciliar el pago.
 * [numerocuenta] Nmero de cuenta en el que se domiciliar el pago.
 * [estransferencia] Booleano para detectar si es un pago a travs de una transferencia.
 * [estalon] Indica si el pago se realiza con taln.
 * [numerotalon] Nmero del taln en caso de pago por taln.
 * [estarjeta] Indica si el pago se realiza con tarjeta.
 * [numerotarjeta] Nmero del tarjeta en caso de pago por tarjeta.
 * [validahasta] Validez de la tarjeta.
 * [espagocontado] Indica si se paga en el momento de emisin de la factura.
 */

DROP SEQUENCE seqPago;

CREATE SEQUENCE seqPago INCREMENT 1000 START 1;

DROP TABLE Pago;

CREATE TABLE Pago (
        codigoEmpresa      BIGINT,
        codigoTienda       BIGINT,
        codigoPago         BIGINT,
        pagosRealizados    BIGINT,
        fechaRealizado     TIMESTAMP,
        esAPlazos          BOOLEAN,
        fechaInicio        TIMESTAMP,
        porcentajeEntrada  DECIMAL(5,2),
        numeroVencimientos SMALLINT,
        diasIntervalo      SMALLINT,
        esDomiciliacion    BOOLEAN,
        codigoBanco        SMALLINT,
        codigoOficina      SMALLINT,
        numeroCuenta       BIGINT,
        esTransferencia    BOOLEAN,
        esTalon            BOOLEAN,
        numeroTalon        VARCHAR(12),
        esTarjeta          BOOLEAN,
        numeroTarjeta      VARCHAR(20),
        validaHasta        TIMESTAMP,
        esPagoContado      BOOLEAN,
	CONSTRAINT codigoPago_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda, codigoPago),
        CONSTRAINT pago_codigoTienda_FK
                   FOREIGN KEY    (codigoEmpresa, codigoTienda)
		REFERENCES Tienda (codigoEmpresa, codigoTienda) ON DELETE CASCADE ON UPDATE CASCADE
);

/*
 * Nota. Tabla que representa el tipo entidad NOTA.
 * Tiene dependencia de identificacin con el tipo entidad REPARACIN.
 * Sus atributos son:
 * [(*) codigonota] Cdigo que identifica unvocamente una nota.
 * [codigoreparacion] Cdigo que identifica unvocamente una
 *     reparacin.
 * [(*) codigoempresa] Cdigo de la empresa a la que pertenece la tienda en la que se emiti la nota.
 * [(*) codigotienda] Cdigo de la tienda en la que se emiti la nota.
 * [codigopago] Cdigo del pago generado al guardar la nota.
 * [total] Total de la nota.
 * [fechaemision] Fecha de emisin de la nota.
 */


DROP SEQUENCE seqNota;

CREATE SEQUENCE seqNota INCREMENT 1000 START 1;

DROP TABLE Nota;

CREATE TABLE Nota (
        codigoNota              BIGINT,
        codigoReparacion        BIGINT,
        codigoEmpresa           BIGINT,
        codigoTienda            BIGINT,
        codigoPago              BIGINT,
        total              	DECIMAL(10,2),
        fechaEmision            TIMESTAMP,
	CONSTRAINT codigoNota_PK PRIMARY KEY
           (codigoNota, codigoReparacion, codigoEmpresa, codigoTienda),
        CONSTRAINT nota_codigoReparacion_FK
                   FOREIGN KEY        (codigoReparacion, codigoEmpresa, codigoTienda)
		REFERENCES Reparacion (codigoReparacion, codigoEmpresa, codigoTienda) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT nota_codigoPago_FK
                   FOREIGN KEY  (codigoEmpresa, codigoTienda, codigoPago)
		REFERENCES Pago (codigoEmpresa, codigoTienda, codigoPago) ON DELETE CASCADE ON UPDATE CASCADE
);

/*
 * Factura de abono. Tabla que representa al tipo entidad FACTURA ABONO. Las facturas de abono son las
 *                   facturas que emiten las tiendas del grupo cuando realizan reparaciones o albaranes.
 * Consta de:
 * [(*) codigoempresa] Cdigo de la empresa a la que pertenece la tienda en la que se emiti la factura.
 * [(*) codigotienda]  Cdigo de la tienda en la que se emiti la factura.
 * [numerofactura] Nmero de la factura. Es el nmero que se muestra en la factura.
 * [codigopagador] Cdigo del participante que paga la factura.
 * [codigopago] Cdigo que identifica el pago que se ha generado automticamente al generar la factura.
 * [fechaemision] Fecha de emisin de la factura.
 * [dtofactura] Descuento general de la factura. Tambin utilizado como descuento por pronto pago.
 * [baseimponiblenueva] Total de la factura OJO: CON IVA INCLUDO. En futuras versiones se debera modificar
 *                       el nombre de este atributo.
 * [baseimponiblemodificada] Indica si la factura se realiza por mdulo pactado, es decir, es impone una
 *                           base imponible a la factura.
 * [estado] Estado de la factura: 0 es pendiente de emisin, 1 es pendiente de pago, 2 es pagada y 3 en gestin.
 * [codigoreparacion] Cdigo de la reparacin a la que hace referencia la factura. Si este campo es NULL
 *                    la factura se gener a partir de uno o varios albaranes.
 * [refactura] Indica si la factura es una refactura.
 * [tipoiva] Este campo estabe inutilizado y actualmente se utiliza para las facturas sin IVA. Si este campo
 *           tiene algn valor la factura es sin Iva. En prinicpio, solo se utiliza el 0.0
 * [(*) codigofactura] Cdigo que identifica la tienda que realiza
 *       la factura.

 */

/**
 * NOTA: TIENE SENTIDO QUE SE GUARDE EL CLIENTE, LA EMPRESA Y LA TIENDA DE LA FACTURA,
 *       YA QUE EL CLIENTE SER DISTINTO DEL RECEPTOR DE LA REPARACION EN LA PRIMERA
 *       FACTURA DE LA REFACTURACIN, Y LA EMPRESA Y TINDA SERN DISTINTOS EN LA SEGUNDA
 *       FACTURA DE LA REFACTURACIN.
*/

DROP SEQUENCE seqFacturaAbono;

CREATE SEQUENCE seqFacturaAbono INCREMENT 1000 START 1;

DROP TABLE FacturaAbono;

CREATE TABLE FacturaAbono (
        codigoEmpresa     BIGINT,
        codigoTienda      BIGINT,
        numeroFactura     BIGINT,
        codigoPagador     BIGINT,
        codigoPago        BIGINT,
        fechaEmision      TIMESTAMP,
        dtoFactura        DECIMAL(5,2),
        baseImponibleNueva DECIMAL(10,2),
        baseImponibleModificada BOOLEAN,
        estado            SMALLINT,
        codigoReparacion     BIGINT,
        refactura         BOOLEAN,
        tipoIVA        	  DECIMAL(5,2),
        codigoFactura     BIGINT,
	CONSTRAINT codigoFacturaAbono_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda, codigoFactura),
        CONSTRAINT facturaAbono_codigoPagador_FK
                  FOREIGN KEY (codigoPagador)
        	REFERENCES Cliente (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT facturaAbono_codigoReparacion_FK
                  FOREIGN KEY (codigoEmpresa, codigoTienda, codigoReparacion)
        	REFERENCES Reparacion (codigoEmpresa, codigoTienda, codigoReparacion) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT facturaAbono_codigoPago_FK
                   FOREIGN KEY  (codigoEmpresa, codigoTienda, codigoPago)
		REFERENCES Pago (codigoEmpresa, codigoTienda, codigoPago) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX facturaabono_codigoPago_index
ON FacturaAbono(codigoPago);

CREATE INDEX facturaabono_codigoFactura_index
ON FacturaAbono(codigoFactura,codigoEmpresa,codigoTienda);

CREATE INDEX facturaabono_codigoRep_index
ON FacturaAbono(codigoReparacion);

CREATE INDEX facturaabono_codigoRepTotal_index
ON FacturaAbono(codigoReparacion,codigoEmpresa,codigoTienda);

CREATE INDEX facturaabono_codigoPagador_index
ON FacturaAbono(codigoPagador);




/*
 * Albarn de abono. Tabla que representa el tipo entidad ALBARN ABONO. El albarn de abono se emite
 *                   para las reparaciones con seguro para que el cliente firme la orden de trabajo.
 * Tiene dependencia de identificacin con el tipo entidad REPARACIN.
 * Sus atributos son:
 * [(*) codigoempresa] Cdigo de la empresa a la que pertenece la tienda en la que se emiti el albarn.
 * [(*) codigotienda]  Cdigo de la tienda en la que se emiti el albarn.
 * [(*) numeroalbaran] Nmero de albarn. Identifica el albarn univocamente con el cdigo de empresa y tienda.
 * [numerofactura] Factura en la que se refleja este albarn.
 * [codigoreparacion] Cdigo que identifica la reparacin a la que se refiere el albarn.
 * [fechaemision] Fecha de emisin del albarn.
 * [totaltaller] Total que se reintegrar a un taller por enviar una reparacin a un taller del grupo.
 */
DROP SEQUENCE seqAlbaranAbono;

CREATE SEQUENCE seqAlbaranAbono INCREMENT 1000 START 1;

DROP TABLE AlbaranAbono;

CREATE TABLE AlbaranAbono (
        codigoEmpresa        BIGINT,
        codigoTienda         BIGINT,
        numeroAlbaran        BIGINT,
        numeroFactura        BIGINT,
        codigoReparacion     BIGINT,
        fechaEmision         TIMESTAMP,
        totalTaller          DECIMAL(10,2),
	CONSTRAINT codigoAlbaranAbono_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda, numeroAlbaran),
        CONSTRAINT reparacion_codigoReparacion_FK
                   FOREIGN KEY  (codigoEmpresa, codigoTienda, codigoReparacion)
		REFERENCES Reparacion (codigoEmpresa, codigoTienda, codigoReparacion) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT albaranAbono_codigoFactura_FK
                   FOREIGN KEY  (codigoEmpresa, codigoTienda, numeroFactura)
		REFERENCES FacturaAbono (codigoEmpresa, codigoTienda, codigoFactura) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX albaranabono_codigoRep_index
ON AlbaranAbono(codigoReparacion);

CREATE INDEX albaranabono_codigoRepTotal_index
ON AlbaranAbono(codigoReparacion,codigoTienda,codigoEmpresa);

CREATE INDEX albaranabono_codigoFact_index
ON AlbaranAbono(numeroFactura);

CREATE INDEX albaranabono_codigoFactTot_index
ON AlbaranAbono(numeroFactura,codigoTienda,codigoEmpresa);

CREATE INDEX albaranabono_codigoAlb_index
ON AlbaranAbono(numeroalbaran);

CREATE INDEX albaranabono_codigoAlbTot_index
ON AlbaranAbono(numeroAlbaran,codigoTienda,codigoEmpresa);




/*
 * Factura de cargo. Tabla que representa al tipo entidad FACTURA CARGO. Las facturas de cargo
 *                   son las facturas emitidas por los proveedores que tienen que ser pagadas por cualquier
 * 		     empresa del grupo.
 * Tiene dependencia de identificacin con el tipo entidad PROVEEDOR.
 * Consta de:
 * [(*) codigoproveedor] Cdigo que identifica el participante proveedor
 *     que genera la factura. Parte de la clave de la factura de cargo.
 * [codigoempresa] Cdigo de la empresa a la que pertenece la tienda en la que se recibi la factura.
 * [codigotienda] Cdigo de la tienda en la que se recibi la factura.
 * [numerofactura] Nmero de la factura que le asign el proveedor a la misma.
 * [codigopago] Cdigo del pago asociado a la factura generado automticamente cuando se genera la factura.
 * [fecha] Fecha de emisin de la factura de cargo. En caso de la la fecha sea anterior a la ltimo perodo
 *         de contabilidad cerrado se almacenar la siguiente fecha al perodo de contabilidad.
 * [concepto] Concepto de la factura.
 * [total] Total de la factura IVA incluido.
 * [estado] Estado de la factura.
 * [numerorecepcion] Nmero que se le asigna a la factura cuando se cierra el perodo de contabilidad.
 * [(*) codigofactura] Cdigo que identifica la factura de pago univocamente.
 */
DROP SEQUENCE seqFacturaCargo;

CREATE SEQUENCE seqFacturaCargo INCREMENT 1000 START 1;


DROP TABLE FacturaCargo;

CREATE TABLE FacturaCargo (
        codigoProveedor BIGINT,
        codigoEmpresa	BIGINT,
        codigoTienda    BIGINT,
        numeroFactura   VARCHAR(40),
        codigoPago      BIGINT,
        fecha           TIMESTAMP,
        concepto        VARCHAR(200),
        total           DECIMAL(10,2),
        estado          INT4,
	numeroRecepcion INT4,
        codigoFactura   BIGINT,
	CONSTRAINT codigoFacturaCargo_PK
                   PRIMARY KEY (codigoFactura, codigoProveedor),
        CONSTRAINT facturaCargo_codigoProveedor_FK
                   FOREIGN KEY (codigoProveedor)
               REFERENCES Proveedor (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT facturaCargo_codigoPago_FK
                   FOREIGN KEY (codigoEmpresa,codigoTienda,codigoPago)
               REFERENCES Pago (codigoEmpresa,codigoTienda,codigoPago) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX facturacargo_codigoTFact_index
ON FacturaCargo(codigoFactura,codigoProveedor);
CREATE INDEX facturacargo_numeroFact_index
ON FacturaCargo(numeroFactura);
CREATE INDEX facturacargo_codigoFact_index
ON FacturaCargo(codigoFactura);
CREATE INDEX facturacargo_codigoEmpresa_index
ON FacturaCargo(codigoFactura,codigoEmpresa,codigoTienda);
CREATE INDEX facturacargo_codigoPago_index
ON FacturaCargo(codigoEmpresa,codigoTienda,codigoPago);



/*
 * Nota de cargo. Tabla que representa al tipo entidad NOTA de CARGO. Las notas de cargo
 *                   son las notas emitidas por los proveedores que tienen que ser pagadas por cualquier
 *                empresa del grupo.
 *                   Las notas son sin IVA.
 * Tiene dependencia de identificacin con el tipo entidad PROVEEDOR.
 * Consta de:
 * [(*) codigoproveedor] Cdigo que identifica el participante proveedor
 *     que genera la nota. Parte de la clave de la nota de cargo.
 * [codigoempresa] Cdigo de la empresa a la que pertenece la tienda en la que se recibi la nota.
 * [codigotienda] Cdigo de la tienda en la que se recibi la nota.
 * [numeroNotaCargo] Nmero de la nota que le asign el proveedor a la misma.
 * [codigopago] Cdigo del pago asociado a la nota generado automticamente cuando se genera la nota.
 * [fecha] Fecha de emisin de la nota de cargo. En caso de la la fecha sea anterior a la ltimo perodo
 *         de contabilidad cerrado se almacenar la siguiente fecha al perodo de contabilidad.
 * [concepto] Concepto de la nota.
 * [total] Total de la nota.
 * [estado] Estado de la nota.
 * [numerorecepcion] Nmero que se le asigna a la nota cuando se cierra el perodo de contabilidad.
 * [(*) codigoNota] Cdigo que identifica la nota de pago univocamente.
 */

DROP SEQUENCE seqNotaCargo;

CREATE SEQUENCE seqNotaCargo INCREMENT 1000 START 1;


DROP TABLE NotaCargo;

CREATE TABLE NotaCargo (
        codigoProveedor         BIGINT,
        codigoEmpresa	        BIGINT,
        codigoTienda            BIGINT,
        codigoNotaCargo         VARCHAR(40),
        codigoPago              BIGINT,
        fecha                   TIMESTAMP,
        concepto                VARCHAR(200),
        total              	DECIMAL(10,2),
        codigoNota              BIGINT,
	CONSTRAINT codigoNotaCargo_PK PRIMARY KEY
           (codigoNota, codigoProveedor),
        CONSTRAINT nota_codigoProveedor_FK
                   FOREIGN KEY        (codigoProveedor)
		REFERENCES Proveedor (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT nota_codigoPago_FK
                   FOREIGN KEY  (codigoPago,codigoTienda,codigoEmpresa)
		REFERENCES Pago (codigoPago,codigoTienda,codigoEmpresa) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX notacargo_codigoTNota_index
ON NotaCargo(codigoNota,codigoProveedor);
CREATE INDEX notacargo_notacargo_index
ON NotaCargo(codigoNotaCargo);
CREATE INDEX notacargo_codigoNota_index
ON NotaCargo(codigoNota);
CREATE INDEX notacargo_codigoEmp_index
ON NotaCargo(codigoEmpresa,codigoTienda,codigoNota);
CREATE INDEX notacargo_codigoPago_index
ON NotaCargo(codigoEmpresa,codigoTienda,codigoPago);


/*
 * Almacn: Es la tabla que representa al tipo entidad ALMACN. Los almacenes son los del grupo y
 *          no se pueden aadir almacenes desde la aplicacin.
 * Almacena toda la informacin necesaria de cada almacn. Tiene dependencia
 * de identificacin con el tipo entidad TIENDA.
 * Consta de:
* [(*) codigoempresa] Cdigo que identifica la empresa a la que pertenecea el almacn.
* [(*) codigoalmacen] Cdigo que identifica el almacn.
* [nick] Nombre completo del almacn.
* [telefono] Telfono del almacn.
* [fax] Nmero de fax del almacn.
* [email] Direccin de correo del almacn.
* [calle] Calle de la direccin del almacn.
* [numero] Nmero del almacn en la calle.
* [piso] Piso del edificio.
* [cp] Cdigo postal
* [municipio] Municipio de ubicacin del almacn.
* [provincia] Provincia del municipio.
 */


DROP SEQUENCE seqAlmacen;

CREATE SEQUENCE seqAlmacen INCREMENT 1000 START 1;

DROP TABLE Almacen;

CREATE TABLE Almacen (
        codigoEmpresa BIGINT,
        codigoAlmacen BIGINT,
        nick          VARCHAR(256),
        telefono      VARCHAR(20) NOT NULL,
        fax           VARCHAR(20),
        email         VARCHAR(30),
        calle         VARCHAR(30) NOT NULL,
        numero        VARCHAR(5)  NOT NULL,
        piso          VARCHAR(5)  NOT NULL,
        CP            BIGINT        NOT NULL,
        municipio     VARCHAR(20) NOT NULL,
        provincia    VARCHAR(20) NOT NULL,
	CONSTRAINT codigoAlmacen_PK
                   PRIMARY KEY (codigoEmpresa, codigoAlmacen),
        CONSTRAINT almacen_codigoEmpresa_FK
                   FOREIGN KEY     (codigoEmpresa)
		REFERENCES Empresa (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX a_codigoEmpresa_index
ON Almacen(codigoEmpresa);

/*
 * Stock: Es la tabla que representa al tipo relacin ``tiene en stock''.
 * Almacena el stock que tiene cada almacn de cada tipo de pieza.
 * Sus atributos son:
 * [(*) codigoproducto] Cdigo del producto al que se refiere el stock.
 * [(*) codigoempresa] Cdigo de la empresa a la que pertenece el almacn.
 * [(*) codigoalmacen] Cdigo del almacn en el que se encuentra este stock.
 * [unidades] El nmero de piezas que tiene de ese tipo de piezas.
 * [stockminimo] Stock mnimo que se permite tener en un almacn.
 * [ubicacion] Ubicacin fsica de las piezas en el almacn.
 */

DROP TABLE Stock;

CREATE TABLE Stock (
        codigoProducto BIGINT,
        codigoEmpresa  BIGINT,
        codigoAlmacen  BIGINT,
        unidades       DECIMAL(10,3),
        stockMinimo    DECIMAL(10,3),
        ubicacion      VARCHAR(64),
	CONSTRAINT codigoStock_PK
                   PRIMARY KEY (codigoProducto, codigoEmpresa, codigoAlmacen),
        CONSTRAINT stock_codigoproducto_FK
                  FOREIGN KEY      (codigoProducto)
		  REFERENCES Producto (codigoProducto) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT stock_codigoAlmacen_FK
                   FOREIGN KEY        (codigoEmpresa, codigoAlmacen)
		   REFERENCES Almacen (codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX s_codigoProducto_index
ON Stock(codigoProducto);

CREATE INDEX s_codigoAlmacen_index
ON Stock(codigoEmpresa, codigoAlmacen);

CREATE INDEX s_codigoAlmacenSingle_index
ON Stock(codigoAlmacen);

CREATE INDEX s_codigoEmpresa_index
ON Stock(codigoEmpresa);

/*
 * Movimiento. Secuencia que permite identificar un movimiento de un
 * determinado almacn.
 */

DROP SEQUENCE seqMovimientoAlmacen;

CREATE SEQUENCE seqMovimientoAlmacen INCREMENT 1000 START 1;

/*
 * Intercambio. Es la tabla que representa al tipo entidad INTERCAMBIO,
 * subclase del tipo entidad MOVIMIENTO. Esta tabla almacena los itercambios de piezas entre
 * almacenes del grupo, guardando el almacn del que salen las piezas y el almacn destino de las
 * mismas.
 * Tiene dependencia de identificacin con el tipo entidad ALMACN.
 * Sus atributos son:
 *   [(*) codigoEmpresa]  Cdigo de la empresa a la que pertenece el almacn origen del intercambio.
 *   [(*) codigoAlmacen] Cdigo que identifica el almacn en el que se
 *   produce el movimiento.
 *   [(*) codigoMovimiento] Cdigo que identifica un movimiento de
 *   intercambio concreto de un almacn.
 *   [fecha] Fecha en la que se produce el intercambio.
 *   [unidades] Nmero de piezas que se intercambian. Un nmero positivo es
 *   un movimiento de entrada; uno negativo, de salida.
 *   [codigoEmpresaDestino] Cdigo de la empresa a la que pertenece el almacn destino del intercambio.
 *   [codigoAlmacenDestino] Almacn del que vienen (movimiento positivo) o
 *   al que van (movimiento negativo) las piezas de este momento.
 *   [codigoProducto] Producto para el que se produjo el movimiento.
 */

DROP TABLE Intercambio;

CREATE TABLE Intercambio (
        codigoEmpresa        BIGINT,
        codigoAlmacen        BIGINT,
        codigoMovimiento     BIGINT,
        fecha                TIMESTAMP,
        unidades             DECIMAL(10,3),
        codigoEmpresaDestino BIGINT,
        codigoAlmacenDestino BIGINT,
        codigoProducto       BIGINT,
	CONSTRAINT codigoIntercambio_PK
                   PRIMARY KEY (codigoEmpresa, codigoAlmacen, codigoMovimiento),
        CONSTRAINT intercambio_codigoAlmacen_FK
                   FOREIGN KEY     (codigoEmpresa, codigoAlmacen)
		REFERENCES Almacen (codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT intercambio_codigoAlmacenDestino_FK
                   FOREIGN KEY     (codigoEmpresaDestino, codigoAlmacenDestino)
		REFERENCES Almacen (codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE
);

/*
 * Regularizacin. Es la tabla que representa al tipo entidad REGULARIZACIN
 * subclase del tipo entidad MOVIMIENTO. Almacena todos los datos relativos
 * a un movimiento de regularizacin de piezas en un almacn. La regularizacin es
 * simplemente la insercin de stock para una pieza sin mediar un pedido o un intercambio.
 * Tiene dependencia de identificacin con el tipo entidad ALMACN.
 * Sus atributos son:
 *   [(*) codigoEmpresa] Cdigo de la empresa a la que pertenece el almacn.
 *   [(*) codigoAlmacn] Cdigo que identifica el almacn en el que se
 *     produce el movimiento.
 *   [(*) codigoMovimiento] codigo que identifica un movimiento de
 *     regularizacin concreto de un almacn.
 *   [fecha] Fecha en la que se produce el intercambio.
 *   [unidades] Nmero de piezas que se intercambian. Un nmero positivo es
 *     un movimiento de entrada; uno negativo, de salida.
 *   [codigoProducto] Producto para el que se produjo el movimiento.
 */

DROP TABLE Regularizacion;

CREATE TABLE Regularizacion (
        codigoEmpresa        BIGINT,
        codigoAlmacen        BIGINT,
        codigoMovimiento     BIGINT,
        fecha                TIMESTAMP,
        unidades             DECIMAL(10,3),
        codigoProducto       BIGINT,
	CONSTRAINT codigoRegularizacion_PK
                   PRIMARY KEY (codigoEmpresa, codigoAlmacen, codigoMovimiento),
        CONSTRAINT regularizacion_codigoAlmacen_FK
                   FOREIGN KEY     (codigoEmpresa, codigoAlmacen)
		REFERENCES Almacen (codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE
);



/*
 * MovimientoPorReparacion. esta tabla almacena los movimientos de piezas 
 * en un almacn usadas para realizar reparaciones. Se identifican por el
 * nmero de almacn del que salen las piezas, la empresa y un cdigo
 * de movimiento. Adems se relaciona con una lnea de repacin de alguna
 * de las reparaciones de una tienda. Admite hacer reparaciones en una tienda
 * diferente a la que las almacena.
 *
 * Sus atributos son:
 *   [(*) codigoEmpresa] Cdigo que identifica la empresa a la que pertenece el almacn en el que se
 *      produce el movimiento.
 *   [(*) codigoAlmacen] Cdigo que identifica el almacn en el que se
 *      produce el movimiento.
 *   [(*) codigoMovimiento] Cdigo que identifica el movimiento.
 *   [fecha] Fecha en la que se produce el intercambio.
 *   [unidades] Nmero de piezas que se mueven.
 *   [codigoReparacion] Cdigo de la reparacin que produce el movimiento por reparacin.
 *   [codigoEmpresaReparacion] Cdigo de la empresa a la que pertenceze la tienda en la que se
 *                             produce el movimiento.
 *   [codigoTienda] Cdigo de la tienda en la que se realiza la reparacin.
 *   [numeroLinea] Lnea de la reparacin a partir de la que se realiza el movimiento.
 *   [codigoProducto] Producto para el que se produjo el movimiento.
 *
 */

DROP TABLE MovimientoPorReparacion;

CREATE TABLE MovimientoPorReparacion (
        codigoEmpresa        BIGINT,
        codigoAlmacen        BIGINT,
        codigoMovimiento     BIGINT,
        fecha                TIMESTAMP,
        unidades             DECIMAL(10,3),
        codigoReparacion     BIGINT,
        codigoEmpresaReparacion BIGINT,
        codigoTienda         BIGINT,
        numeroLinea          BIGINT,
        codigoProducto          BIGINT,
	CONSTRAINT codigoMovReparacion_PK
                   PRIMARY KEY (codigoEmpresa, codigoAlmacen, codigoMovimiento),
        CONSTRAINT movimientoRep_codigoAlmacen_FK
                   FOREIGN KEY (codigoEmpresa, codigoAlmacen)
		REFERENCES Almacen (codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT movimientoRep_LReparacion_FK
                   FOREIGN KEY (codigoReparacion, codigoEmpresaReparacion, codigoTienda, numeroLinea)
		REFERENCES LineaReparacion (codigoReparacion, codigoEmpresa, codigoTienda, numeroLinea) ON DELETE CASCADE ON UPDATE CASCADE
);


/*
 * Caja. Tabla que representa al tipo entidad CAJA. La caja representa las
 *       cajas de cada tienda. En las cajas se pueden producir movimientos de
 *       entrada y salida de dinero, segn la cantidad sea positiva o negativa.
 * Tiene dependencia de identificacin con el tipo entidad TIENDA.
 * Consta de:
 *    [(*) codigoEmpresa] Cdigo que identifica la empresa a la que pertenece la tienda.
 *    [(*) codigoTienda] Cdigo que identifica la tienda a la
 *       que pertenece la caja.
 *    [(*) codigoCaja] Cdigo que identifica la caja de la tienda.
 *    [saldoInicial] Importe inicial de la caja.
 *    [saldoActual] Importe actual de la caja.
 */
 

DROP SEQUENCE seqCaja;

CREATE SEQUENCE seqCaja INCREMENT 1000 START 1;

DROP TABLE Caja;

CREATE TABLE Caja (
        codigoEmpresa BIGINT,
        codigoTienda  BIGINT,
        codigoCaja    BIGINT,
        saldoInicial  DECIMAL(10,2),
	saldoActual   DECIMAL(10,2),
	CONSTRAINT codigoCaja_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda, codigoCaja),
        CONSTRAINT caja_codigoTienda_FK
                   FOREIGN KEY    (codigoEmpresa, codigoTienda)
		REFERENCES Tienda (codigoEmpresa, codigoTienda) ON DELETE CASCADE ON UPDATE CASCADE
);

 
/*
 * Movimiento de Caja: Tabla que representa al tipo entidad MOVIMIENTO DE CAJA. Los movimientos
 *                     en caja sern positivos si entran en caja o negativos si salen.
 * Tiene dependencia de identificacin con el tipo entidad CAJA.
 * Consta de:
 *    [(*) codigoEmpresa] Cdigo de la empresa
 *    [(*) codigoTienda] Cdigo de la tienda a la que pertenece la caja.
 *    [(*) codigoCaja] Cdigo que identifica la caja en concreto a la
 *      que pertenece el movimiento.
 *    [(*) codigoMovimiento] Cdigo que identifica el movimiento que se realiza.
 *    [codigoPago] Cdigo que identifica el pago que provoc el
 *      movimiento en caja.
 *    [cantidad] Cantidad que entra/sale en el movimiento de caja.
 *    [fecha] Fecha en la que se produce el movimiento.
 *    [concepto] Descripcin del movimiento.
 */
 
DROP SEQUENCE seqMovimientoCaja;

CREATE SEQUENCE seqMovimientoCaja INCREMENT 1000 START 1;

DROP TABLE MovimientoCaja;

CREATE TABLE MovimientoCaja (
        codigoEmpresa     BIGINT,
        codigoTienda      BIGINT,
        codigoCaja        BIGINT,
        codigoMovimiento  BIGINT,
        codigoPago        BIGINT,
        cantidad          DECIMAL(10,2),
        fecha	          TIMESTAMP,
        concepto	  VARCHAR(200),
	CONSTRAINT codigoMovimientoCaja_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda, codigoCaja, codigoMovimiento),
        CONSTRAINT caja_codigoTienda_FK
                   FOREIGN KEY    (codigoEmpresa, codigoTienda, codigoCaja)
		REFERENCES Caja (codigoEmpresa, codigoTienda, codigoCaja) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT caja_codigoPago_FK
                   FOREIGN KEY    (codigoEmpresa, codigoTienda, codigoPago)
		REFERENCES Pago (codigoEmpresa, codigoTienda, codigoPago) ON DELETE CASCADE ON UPDATE CASCADE
);

/*
 * Refacturacin. Tabla que representa al tipo relacin "refactura".
 * Es una tabla que est desnormalizada, ya no estar en 3 forma normal;
 * pero ser ms eficiente. Esta tabla contiene la relacin entre facturas que son refacturas de
 * otras, para poder acceder a la reparacin a la que referencia la refactura.

 * Consta de:
 * [(*) codigoempresaabono] Cdigo de la empresa a la que pertenece la tienda en la que se gener la factura
 *                          de la que se refactur.
 * [(*) codigotiendaabono] Cdigo de la tienda en la que se gener la factura
 *                          de la que se refactur.
 * [(*) numerofacturaabono] Cdigo de la factura a la que se refactur.
 * [(*) codigoempresarefactura] Cdigo de la empresa a la que pertenece la tienda que realiza la refactura.
 * [(*) codigotiendarefactura] Cdigo de la tienda que realiza la refactura.
 * [(*) numerorefactura] Cdigo de la refactura.
 * [codigoreparacion] Cdigo de la reparacin a la que referencian las dos facturas.
 */
DROP TABLE Refacturacion;

CREATE TABLE Refacturacion (
        codigoEmpresaAbono     BIGINT,
        codigoTiendaAbono      BIGINT,
        numeroFacturaAbono     BIGINT,
        codigoEmpresaRefactura BIGINT,
        codigoTiendaRefactura  BIGINT,
        numeroRefactura        BIGINT,
        codigoReparacion       BIGINT,
        CONSTRAINT codigoRefacturacion_PK
              PRIMARY KEY (codigoEmpresaAbono, codigoTiendaAbono,numeroFacturaAbono,codigoEmpresaRefactura,codigoTiendaRefactura,numeroRefactura),
        CONSTRAINT refacturacion_codigoFacturaAbono_FK
                   FOREIGN KEY    (codigoEmpresaAbono, codigoTiendaAbono, numeroFacturaAbono)
		REFERENCES FacturaAbono (codigoEmpresa, codigoTienda, codigoFactura) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT refacturacion_codigoRefactura_FK
                   FOREIGN KEY    (codigoEmpresaRefactura, codigoTiendaRefactura, numeroRefactura)
		REFERENCES FacturaAbono (codigoEmpresa, codigoTienda, codigoFactura) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT refacturacion_codigoReparacion_FK
                   FOREIGN KEY    (codigoEmpresaAbono, codigoTiendaAbono, codigoReparacion)
		REFERENCES Reparacion (codigoEmpresa, codigoTienda, codigoReparacion) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX refacturacion_codigoFact_index
ON Refacturacion(codigoEmpresaAbono,codigoTiendaAbono,numeroFacturaAbono);

CREATE INDEX refacturacion_codigoRe_index
ON Refacturacion(codigoEmpresaRefactura,codigoTiendaRefactura,numeroRefactura);





/*
 * FacturaTaller: Es utilizada para representar la factura que se le pasa a un taller.
 *      Explicacin: A un taller se le pueden facturar las facturas de abono que tengan al taller como
 *      cliente, y todos los albaranes se facturan mensualmente. Pero adems, existen reparaciones por el
 *      seguro que implican una factura negativa al taller (es decir se le paga x% al taller de comisin).
 *      Estas ltimas no habra forma de conocer en que factura se han reflejado si no existiese
 *      esta tabla. As: los albaranes que tengan como cliente al taller ya tendrn el nmero de
 *      factura asociado y los albaranes que indiquen un pago al taller se reflejarn en esta tabla.
 * Consta de:
 * [(*) codigoempresafactura] Cdigo de la empresa a la que pertenece la tienda en la que emite la factura
 *                            para el taller.
 * [(*) codigotiendafactura] Cdigo de la tienda en la que emite la factura
 *                            para el taller.
 * [(*) numerofacturataller] Cdigo de la factura que se le emite al taller.
 * [(*) codigoempresaalbaran] Cdigo de la empresa a la que pertenece la tienda en la que emiti el albarn
 *                           a partir del cul se emitir la factura.
 * [(*) codigotiendaalbaran] Cdigo de la tienda en la que emiti el albarn
 *                           a partir del cul se emitir la factura.
 * [(*) numeroalbaran] Nmero del albarn a partir del que se emite la factura del taller.
 */
DROP TABLE FacturaTaller;

CREATE TABLE FacturaTaller (
        codigoEmpresaFactura    BIGINT,
        codigoTiendaFactura     BIGINT,
        numeroFacturaTaller    BIGINT,
        codigoEmpresaAlbaran   BIGINT,
        codigoTiendaAlbaran    BIGINT,
        numeroAlbaran          BIGINT,
        CONSTRAINT codigoFacturaTaller_PK
              PRIMARY KEY (codigoEmpresaFactura, codigoTiendaFactura,numeroFacturaTaller,codigoEmpresaAlbaran,codigoTiendaAlbaran,numeroAlbaran),
        CONSTRAINT facturaTaller_codigoFactura_FK
                   FOREIGN KEY    (codigoEmpresaFactura, codigoTiendaFactura, numeroFacturaTaller)
		REFERENCES FacturaAbono (codigoEmpresa, codigoTienda, codigoFactura) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT facturaTaller_codigoAlbaran_FK
                   FOREIGN KEY    (codigoEmpresaAlbaran, codigoTiendaAlbaran, numeroAlbaran)
		REFERENCES AlbaranAbono (codigoEmpresa, codigoTienda, numeroAlbaran) ON DELETE CASCADE ON UPDATE CASCADE
);


CREATE INDEX facturataller_codigoFAC_index
ON FacturaTaller(codigoEmpresaFactura,codigoTiendaFactura,numeroFacturaTaller);
CREATE INDEX facturataller_codigoAlba_index
ON FacturaTaller(codigoEmpresaAlbaran,codigoTiendaAlbaran,numeroAlbaran);
CREATE INDEX facturataller_codigoFT_index
ON FacturaTaller(codigoEmpresaFactura,codigoTiendaFactura,numeroFacturaTaller,codigoEmpresaAlbaran,codigoTiendaAlbaran,numeroAlbaran);



/*
 * Documento. Tabla que indica los tipos de documentos existentes. Estos
 * documentos son lo que necesitar cada aseguradora.
 * Consta de:
 *    [(*) codigoDocumento] Cdigo que identifica el documento.
 *    [(*) nombreDocumento] Nombre del documento en cuestin
 *    [(*) descripcion] Descripcin del documento.
 *    [deFacturacion] Indica si el documento es para la pantalla de factura.
 *    [deReparacion] Indica si el documento es para la pantalla de reparacin.
 */

DROP SEQUENCE seqDocumento;

CREATE SEQUENCE seqDocumento INCREMENT 1000 START 1;

DROP TABLE Documento;

CREATE TABLE Documento (
        codigoDocumento   BIGINT,
        nombreDocumento   VARCHAR(40) ,
        descripcionDocumento   VARCHAR(500),
        deFacturacion     BOOLEAN     NOT NULL,
        deReparacion      BOOLEAN     NOT NULL,			
	CONSTRAINT codigoDocumento_PK
              PRIMARY KEY (codigoDocumento)
);

/*
 * NecesitaDocumento. Tabla que asocia los documentos con las compaas de seguros que lo solicitan.
 *
 * Consta de:
 *    [(*) codigoAseguradora] Cdigo de la aseguradora que solicita el documento.
 *    [(*) codigoDocumento] Cdigo del documento asociado a la compaa.
 */

DROP TABLE NecesitaDocumento;

CREATE TABLE NecesitaDocumento (
        codigoAseguradora BIGINT,
        codigoDocumento BIGINT,
        CONSTRAINT codigoNecesitaDocumento_PK
                PRIMARY KEY (codigoAseguradora,codigoDocumento),
        CONSTRAINT aseguradoraNecesitaDocumento_PK
                FOREIGN KEY (codigoAseguradora)
       		REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT documentoNecesitado_PK
                FOREIGN KEY (codigoDocumento)
       		REFERENCES Documento (codigoDocumento) ON DELETE CASCADE ON UPDATE CASCADE
);

DROP TABLE NumerosCorrelativosFactura;
DROP TABLE NumerosCorrelativosAlbaran;

/*
 * NumerosCorrelativosFactura. Esta tabla se utiliza para garantizar que los nmeros de la factura
 *                             sern siempre correlativos, ya que contienen el nmero de factura que
 *                             se ha utilizado en la anterior factura y no se actualiza hasta que se
 *                             genera una nueva factura en una transaccin de la base de datos.
 *
 * Consta de:
 *    [numeroFactura] Nmero de la ltima factura emitida por la empresa identificada por codigoEmpresa y
 *                        tienda identificada por codigoTienda.
 *    [(*) codigoEmpresa] Cdigo de la empresa a la que pertenece la tienda para la que se almacena el
 *                        ltimo nmero de factura. Si el cdigo es cero es otra numeracin...
 *    [(*) codigoTienda] Cdigo de la tienda para la que se almacena el ltimo nmero de factura.
 */


CREATE TABLE NumerosCorrelativosFactura (
        numeroFactura BIGINT NOT NULL,
        codigoEmpresa BIGINT NOT NULL,
        codigoTienda BIGINT NOT NULL,
	CONSTRAINT codigoNumerosFactura_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda)
);


/*
 * NumerosCorrelativosAlbaran. Esta tabla se utiliza para garantizar que los nmeros de los albaranes
 *                             sern siempre correlativos, ya que contienen el nmero de albarn que
 *                             se ha utilizado en la anterior factura y no se actualiza hasta que se
 *                             genera un nuevo albarn en una transaccin de la base de datos.
 *
 * Consta de:
 *    [numeroAlbaran] Nmero del ltimo albarn emitido por la empresa identificada por codigoEmpresa y
 *                        tienda identificada por codigoTienda.
 *    [(*) codigoEmpresa] Cdigo de la empresa a la que pertenece la tienda para la que se almacena el
 *                        ltimo nmero de albaran.
 *    [(*) codigoTienda] Cdigo de la tienda para la que se almacena el ltimo nmero de albarn.
 */
CREATE TABLE NumerosCorrelativosAlbaran (
        numeroAlbaran BIGINT NOT NULL,
        codigoEmpresa BIGINT NOT NULL,
        codigoTienda  BIGINT NOT NULL,
	CONSTRAINT codigoNumerosAlbaran_PK
                   PRIMARY KEY (codigoEmpresa, codigoTienda)
);

/*
 * Pedido. Tabla que representa al tipo entidad PEDIDO.
 * Tiene dependencia de identificacin con el tipo entidad ALMACN.
 * Consta de:
 *   [(*) codigoPedido] Cdigo que identifica unvocamente un pedido de un almacn.
 *   [(*) codigoEmpresa] Cdigo de la empresa a la que pertenece el almacn.
 *   [(*) codigoAlmacen] Cdigo que identifica el almacn donde se ha
 *     realizado el pedido.
 *   [codigoProveedor] Cdigo de participante que identifica el
 *     proveedor al que se le realiz el pedido.
 *   [fecha] Fecha en la que se gener el pedido.
 *   [solicitado] Indica si el pedido ha sido solicitado.
 */


DROP SEQUENCE seqPedido;

CREATE SEQUENCE seqPedido INCREMENT 1000 START 1;

DROP TABLE Pedido;

CREATE TABLE Pedido (
        codigoPedido    BIGINT,
        codigoEmpresa   BIGINT,
        codigoAlmacen   BIGINT,
        codigoProveedor BIGINT,
        fecha           TIMESTAMP,
        solicitado      BOOLEAN,
        CONSTRAINT codigoPedido_PK
                   PRIMARY KEY (codigoEmpresa, codigoAlmacen, codigoPedido),
        CONSTRAINT reparacion_codigoAlmacen_FK
                   FOREIGN KEY    (codigoEmpresa, codigoAlmacen)
		REFERENCES Almacen (codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT reparacion_codigoProveedor_FK
                  FOREIGN KEY (codigoProveedor)
        	REFERENCES Proveedor (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX p_codigoAlmacen
ON Pedido(codigoEmpresa, codigoAlmacen);

CREATE INDEX p_codProveedor
ON Pedido(codigoAlmacen);

CREATE INDEX p_fecha
ON Pedido(fecha);

CREATE INDEX p_codigoAlmacenProv
ON Pedido(codigoEmpresa, codigoAlmacen, codigoProveedor);

CREATE INDEX p_codigoAlmacenProvFecha
ON Pedido(codigoEmpresa, codigoAlmacen, codigoProveedor, fecha);

/*
 * Lnea de factura de cargo. Tabla que representa al tipo entidad
 * LNEA DE FACTURA DE CARGO genrica, son las lneas de las facturas a pagar por
 * cualquier empresa del grupo.
 * Consta de:
 * [(*) numerofactura] Cdigo de la factura a la que pertenece la lnea.
 * [(*) codigoproveedor] Cdigo del proveedor al que pertenece la lnea de la factura.
 * [(*) numerolineafactura] Nmero de lnea.
 * [descripcion] Descripcin de la lnea de la factura, si la lnea se proviene de una lnea de factura
 *               la descripcin es la de la pieza.
 * [tipoiva] Tipo de iva de la lnea de la factura.
 * [total] Total de la lnea de factura.
 */
DROP SEQUENCE seqLineaFacturaCargo;

CREATE SEQUENCE seqLineaFacturaCargo INCREMENT 1000 START 1;

DROP TABLE LineaFacturaCargo;

CREATE TABLE LineaFacturaCargo (
        numeroFactura      BIGINT,
        codigoProveedor    BIGINT,
        numeroLineaFactura BIGINT,
        descripcion        VARCHAR(200),
        tipoIva            DECIMAL(5,2),
        total              DECIMAL(10,2),
	CONSTRAINT codigoLineaFactura_PK PRIMARY KEY
               (numeroFactura, codigoProveedor, numeroLineaFactura),
        CONSTRAINT lineaFactura_codigoFactura_FK
                  FOREIGN KEY (numeroFactura,codigoProveedor)
        	REFERENCES FacturaCargo (codigoFactura, codigoProveedor) ON DELETE CASCADE
);

CREATE INDEX lp_codigoLineaFacturaCargo
ON LineaFacturaCargo(numeroFactura,codigoProveedor, numeroLineaFactura);


/*

 * Lnea de pedido. Tabla que representa al tipo entidad
 * LNEA DE PEDIDO. Representa las lneas de un pedido.
 * Tiene dependencia de identificacin con el tipo entidad PEDIDO.
 * Consta de:
 * [(*) codigopedido] Cdigo que identifica unvocamente un
 *      pedido.
 * [(*) codigoempresa] Cdigo de la empresa a la que pertenece el el almacn para
 *                     el cul se realiza el pedido.
 * [(*) codigoalmacen] Cdigo del almacn para el que se solicita un pedido.
 * [(*) numerolinea] Nmero de lnea.
 * [codigoproducto] Pieza que se pide en cada lnea de pedido.
 *      (No se pone el eurocode (character) y s el cdigoproducto (int4)
 *      para mejorar la eficiencia)
 * [unidades] Unidades que se solicitan en de la pieza en cuestin.
 * [numeroalbaran] Nmero del albann al que se asocia esta lnea de pedido. En el momento en el
 *                 que se mete un valor en este campo, es decir, se asocia un albarn se producir un
 *                 movimiento de entrada de piezas.
 * [numerolineacargo] Nmero de lnea de factura a la que referencia est lnea de pedido.
 * [numerofacturacargo] Nmero de factura a la que pertenece la lnea de factura a la que se referencia.
 * [codigoproveedorfacturacargo] Cdigo de proveedor parte de la clave de factura a la que se referencia.
 */

DROP SEQUENCE seqLineaPedido;

CREATE SEQUENCE seqLineaPedido INCREMENT 1000 START 1;

DROP TABLE LineaPedido;

CREATE TABLE LineaPedido (
        codigoPedido   BIGINT,
        codigoEmpresa  BIGINT,
        codigoALmacen  BIGINT,
        numeroLinea    BIGINT,
        codigoProducto BIGINT,
        unidades       DECIMAL(10,3),
        numeroAlbaran   VARCHAR(40),
	numeroLineaCargo BIGINT,
        numeroFacturaCargo   BIGINT,
        codigoProveedorFacturaCargo BIGINT,
	CONSTRAINT codigoLineaPedido_PK PRIMARY KEY
               (codigoPedido, codigoEmpresa, codigoAlmacen, numeroLinea),
        CONSTRAINT lineaPedido_codigoPedido_FK
                  FOREIGN KEY (codigoPedido, codigoEmpresa, codigoAlmacen)
        	REFERENCES Pedido (codigoPedido, codigoEmpresa, codigoAlmacen) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT lineaPedido_numeroLinFac_FK
                  FOREIGN KEY (numeroLineaCargo,numeroFacturaCargo,codigoProveedorFacturaCargo)
        	REFERENCES LineaFacturaCargo (numeroLineaFactura, numeroFactura, codigoProveedor) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT lineaPedido_codigoProducto_FK
                  FOREIGN KEY (codigoProducto)
        	REFERENCES Producto (codigoProducto) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX lp_codigoPedido
ON LineaPedido(codigoPedido, codigoEmpresa, codigoAlmacen);

CREATE INDEX lp_codigoProducto
ON LineaPedido(codigoProducto);



/*
 * Periodo de contabilidad. Tabla que representa al tipo entidad PERODO DE CONTABILIDAD. El perodo de
 *                          contabilidad se utiliza para poder tener una referencia de a que perodo pertenecen
 *                          las facturas emitidas y recibidas por las empresa del grupo. Por ejemplo,
 *                          no se pueden recibir facturas de perodos de contabilidad cerrados.
 *
 * Consta de:
 * [codigocontabilidad] Cdigo que identifica el perodo de contabilidad.
 * [fechainicio] Fecha de inicio del perodo de contabilidad en cuestin.
 * [fechafin] Fecha de fin del perodo de contabilidad en cuestin.
 * [cerrado] Indica si un perodo de contabilidad de una empresa est cerrado.
 * [codigoempresa] Cdigo de la empresa para la cual se introduce el perodo de contabilidad actual, ya que
 *                 diferentes empresas pueden tener contabilidades trimestrales, semestrales...
 */

DROP SEQUENCE seqContabilidad;

CREATE SEQUENCE seqContabilidad INCREMENT 1000 START 1;

DROP TABLE PeriodoContabilidad;

CREATE TABLE PeriodoContabilidad (
        codigoContabilidad        BIGINT NOT NULL,
        fechaInicio               TIMESTAMP,
        fechaFin                  TIMESTAMP,
        cerrado                   BOOLEAN,
        codigoEmpresa             BIGINT,
        CONSTRAINT codigoContabilidad_PK PRIMARY KEY
                (codigoContabilidad,codigoEmpresa),
        CONSTRAINT codigoEmpresa_PK FOREIGN KEY (codigoEmpresa)
        REFERENCES Empresa (codigoParty)
);




/*
 * Provincia. Tabla que representa el tipo de entidad PROVINCIA. Las provincias se
 *            almacenan con su cdigo y nombre.
 *
 * Consta de:
 * [(*) codigoProvincia] Cdigo de la provincia.
 * [nombreProvincia] Nombre de la provincia.
 */

DROP TABLE Provincia;

CREATE TABLE Provincia (
        codigoProvincia INT4,
        nombreProvincia VARCHAR(32),
	CONSTRAINT codigoProvincia_PK
                PRIMARY KEY (codigoProvincia)
);

CREATE INDEX provincia_codigoProvincia
ON Provincia(codigoProvincia);

CREATE INDEX provincia_nombreProvincia
ON Provincia(nombreProvincia);

DROP SEQUENCE seqDatosCK;

CREATE SEQUENCE seqDatosCK START 50;

/*
 * DatosCK. Tabla que representa el tipo de entidad DATOSCK.
 *
 * Consta de:
 * [(*) codigo] Cdigo identificador del los datos.
 * [codigoparty] Cdigo de la compaa de seguros para la que se almacenan los datos CK.
 * [codigock] Cdigo que asigna el Consorcio de cristaleros (CK) a una compaa.
 * [descripcionck] Nombre que asigna el Consorcio de cristaleros (CK) a una compaa.
 *
 */

DROP TABLE DatosCK;

CREATE TABLE DatosCK (
        codigo         BIGINT,
        codigoParty    BIGINT,
        codigoCK       VARCHAR(16),
        descripcionCK  VARCHAR(256),
	CONSTRAINT codigoDatosCK_PK
                PRIMARY KEY (codigo),
        CONSTRAINT datosCK_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX datosCK_codigo
ON DatosCK(codigo);

CREATE INDEX datosCK_codigoParty
ON DatosCK(codigoParty);

CREATE INDEX datosCK_codigoCK
ON DatosCK(codigoCK);

/***************************************************************************/
/* Nmero de envo */
/***************************************************************************/

DROP TABLE NumeroEnvioCK;

/*
 * NumeroEnvioCK. Tabla que representa el tipo de entidad NUMEROENVIOCK. Se utiliza para 
 * simular una secuencia de nmeros y garantizar que los nmeros sern correlativos.
 *
 * Consta de:
 * [(*) codigoParty] Cdigo de la empresa que realiza los envos a CK. Actualmente solo dos
 *                   empresas lo realizan Luis Manuel Garca Rodellino y Coruesa de Parabrisas.
 * [numerosEnvio] Contiene el valor del nmero de envo. Es necesario para poder garantizar que los
 *                nmeros de envo son correlativos.
 *
 */

CREATE TABLE NumeroEnvioCK (
        codigoParty  BIGINT,
        numeroEnvio  BIGINT,
	CONSTRAINT numeroEnvioCK_PK
                PRIMARY KEY (codigoParty),
        CONSTRAINT numeroEnvioCK_codigoParty_FK
                  FOREIGN KEY (codigoParty)
        	REFERENCES Participante (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX numeroEnvioCK_codigoParty
ON NumeroEnvioCK(codigoParty);


/*
 * CodigoIneOficina. Tabla que representa el tipo de entidad CODIGOINEOFICINA.
 *                   Se utiliza para mantener los cdigos Ine de las oficinas bancarias
 *                   con las que se realicen los intercambios de datos como las domiciliaciones.
 *
 * Consta de:
 * [codigoBanco] Cdigo del banco al que pertenece la oficina bancaria.
 * [codigoOficina] Oficina para la que se almacenar el cdigo Ine.
 * [codigoIne] Cdigo de identificacin de una oficina bancaria dado por el Instituto Nacional de Estadstica.
 *
 */

DROP TABLE CODIGOINEOFICINA;

CREATE TABLE CODIGOINEOFICINA (
        codigoBanco INTEGER,
        codigoOficina INTEGER,
        codigoIne INTEGER,
	CONSTRAINT codigoIneOficina_PK
                PRIMARY KEY (codigoBanco,codigoOficina,codigoIne),
        CONSTRAINT codigoIneOficina_FK
                  FOREIGN KEY (codigoBanco,codigoOficina)
        	REFERENCES Oficina (codigoBanco,codigoOficina) ON DELETE CASCADE ON UPDATE CASCADE
);


/*
 * Domiciliacion. Tabla que representa el tipo de entidad DOMICILIACION. Esta tabla contiene la relacin entre
 *                un cdigo de domiciliacin y las facturas a las que se realizan esa domiciliacin.
 *
 * Consta de:
 * [codigodomiciliacion] Cdigo que identifica la domiciliacin. Una domiciliacin se corresponde con
 *                       un registro individual del cuaderno 58.
 * [codigofactura] Cdigo de factura a la que se asociar una domiciliacin.
 * [codigoempresa] Cdigo de empresa a la que pertenece la tienda en la que se gener la factura.
 * [codigotienda] Cdigo de la tienda en la que se gener la factura.
 * [estado] Estado de la domiciliacin. Puede estar gestionada (9), o en uno de los errores de devolucin (1-8).
 *          o bien, pagada (10).
 *
 */
DROP SEQUENCE seqDomiciliacion;

CREATE SEQUENCE seqDomiciliacion INCREMENT 1000 START 1;

DROP TABLE DOMICILIACION;

CREATE TABLE DOMICILIACION (
 codigodomiciliacion  bigint,
 codigofactura        bigint,
 codigoempresa        bigint,
 codigotienda         bigint,
 estado               integer,
 esaplazos            boolean,
 plazoactual          integer,
	CONSTRAINT codigodomiciliacion_pk
                PRIMARY KEY ( codigodomiciliacion,codigofactura,codigoempresa,codigotienda)
);

CREATE INDEX domiciliacion_codigodomi
ON Domiciliacion(codigoDomiciliacion);

CREATE INDEX domiciliacion_codigofactura
ON Domiciliacion(codigoFactura,codigoEmpresa,codigoTienda);


DROP TABLE PRECIOSCOSTE;
/*
 * PreciosCoste. Tabla que representa el tipo de relacin PRECIOSCOSTE. Esta tabla contiene la relacin entre
 *                un cdigo de producto (una pieza) y un proveedor.
 *
 * Consta de:
 * [codigoProducto] Cdigo del producto para el que se almacenar el precios de coste
 * [codigoProveedor] Cdigo del proveedor que tiene el precio para el producto indicado.
 * [precioCoste] Precio del producto referenciado por cdigo de producto para el proveedor
 *       con cdigo de proveedor codigoProveedor.
 *
 */
CREATE TABLE PRECIOSCOSTE (
 codigoProducto  	bigint,
 codigoProveedor        bigint,
 precioCoste        	DECIMAL(10,2),
	CONSTRAINT codigoprecioscoste_pk
                PRIMARY KEY (codigoProducto,codigoProveedor),
        CONSTRAINT codigoProducto_FK
                  FOREIGN KEY (codigoProducto)
        	REFERENCES Producto (codigoProducto) ON DELETE CASCADE ON UPDATE CASCADE,
        CONSTRAINT codigoProveedor_FK
                  FOREIGN KEY (codigoProveedor)
        	REFERENCES Proveedor (codigoParty) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX precioscoste_codigo
ON PRECIOSCOSTE(codigoProducto,codigoProveedor);

CREATE INDEX precioscoste_codigoProducto
ON PRECIOSCOSTE(codigoProducto);
                                                      
CREATE INDEX precioscoste_codigoProveedor
ON PRECIOSCOSTE(codigoProveedor);

/* PARA METER O ATRIBUTO CODIGOPROVEEDOR en PIEZA:
 alter table pieza add column codigoProveedor BIGINT constraint
 codigoProveedorProducto_fk references PROVEEDOR (codigoParty)
 on delete set null on update cascade;
*/





