 /* accounting_period_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "accounting_period_model.h"

/**
 * m_accounting_period_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Accounting_period con los campos rellenados con los datos de la base de datos.
 */
AccountingPeriod * m_accounting_period_create (GdaRecordset *rs) {

    GdaField * field;
    AccountingPeriod * accounting_period;

    accounting_period = d_accounting_period_create_empty();



    field = gda_recordset_field_name(rs,"codigoContabilidad" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
            accounting_period->accounting_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs,"codigoEmpresaContabilidad" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
            accounting_period->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs,"fechainicioContabilidad");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp;
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    accounting_period->initial_date = (*my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechafinContabilidad");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp;
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    accounting_period->ending_date = (*my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"cerradoContabilidad" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
            accounting_period->closed = gda_field_get_boolean_value(field);
	}
	gda_field_free(field);
    }

    return accounting_period;
}



/**
 * m_accounting_period_insert: 
 * @accounting_period: Objeto que representa un perodo de contabilidad.
 *
 * Inserta el objeto accounting_period en la BD.
 * Permite insertar en la base de datos perodos de contabilidad para diferentes empresas,
 * ya que las empresas pueden necesitar perodos trimestrales o semestrales. Por esta razn
 * se puede elegir cualquier fecha para introducir el perodo de contabilidad.
 *
 * Returns: %ULONG_MAX si NO se pudo completar la insercin, el cdigo del perodo en otro caso.
 */
gulong m_accounting_period_insert (AccountingPeriod * accounting_period) {
    gulong result=0;
    gchar * query=NULL;
    gchar * enterprise=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    time_t initial_date=d_accounting_get_initial_date (accounting_period);
    time_t ending_date=d_accounting_get_ending_date (accounting_period);

    gulong enterprise_code;
    gulong logged_shop_code;
    gulong logged_warehouse_code;
    gchar * temp_name;
    gchar * next_val_accounting=NULL;
    gchar * initial_date_char=g_strdup(ctime(&initial_date));
    gchar * ending_date_char=g_strdup(ctime(&ending_date));
    gboolean closed=d_accounting_get_closed (accounting_period);

    gchar * closed_char=NULL;



    if (m_accounting_period_exists (accounting_period)==FALSE) {
	temp_name = u_parameter_manager_get_parameter ("tienda");
	logged_shop_code = m_shop_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("almacen");
	logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

	g_free(temp_name);

	enterprise_code=d_accounting_get_enterprise_code (accounting_period);

	gda_command_set_connection(command,connection);
	m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,logged_shop_code,logged_warehouse_code);

	if (closed == FALSE) closed_char = g_strdup("FALSE");
	else closed_char = g_strdup("TRUE");
	next_val_accounting = m_model_nextval_sequence ("seqContabilidad",connection);
	enterprise = g_strdup_printf("%lu",enterprise_code);

	query = g_strconcat("insert into PeriodoContabilidad values(",
			    next_val_accounting,",'",
			    initial_date_char,"','",
			    ending_date_char,"',",
			    closed_char,",",
			    enterprise,")",
			    NULL);

	if (m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE)==FALSE) {
	    result = ULONG_MAX;
	} else {
	    gchar ** stop=(gchar **) g_malloc0(sizeof(gchar*));
	    result = strtoul(next_val_accounting,stop,10);
	    g_free(stop);
	}

	if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	    result = ULONG_MAX;
	}

	if (query!=NULL) g_free(query);

	g_free(tuples);
	g_free(next_val_accounting);
	g_free(initial_date_char);
	g_free(ending_date_char);
	g_free(closed_char);
	g_free(enterprise);

	if (record_set!=NULL)
	    gda_recordset_free(record_set);

    } else {
        result = (ULONG_MAX-1);
    }

    igalia_command_free(command);

    return result;
}

/**
 * m_accounting_period_close:
 * @accounting_code: Cdigo del perodo de contabilidad que se quiere cerrar.
 * @enterprise_code: Cdigo de la empresa para la cual se desea cerrar el perodo de contabilidad.
 * @connection: Conexin con la BD en la que vamos a ejecutar el comando
 * @command: Comando del tipo GdaCommand con el que ejecutaremos la query.
 * 
 * Cierra el perodo de contabilidad deseado.
 * Las facturas recibidas tienen un orden de recepcin dentro del perodo de contabilidad, que
 * se puede modificar antes de estar cerrado dicho perodo. Con lo cual, se utiliza este mtodo
 * para impedir que se puedan modificar los nmeros de recepcin de las facturas, una vez se
 * ha cerrado el perodo de contabilidad.
 *
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_accounting_period_close (gulong accounting_code,gulong enterprise_code,GdaConnection * connection,GdaCommand * command) {
    gboolean result=FALSE;
    gchar * query=NULL;
    gchar * enterprise=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    gchar * accounting_code_gchar;

    gulong logged_shop_code;
    gulong logged_warehouse_code;

    gchar * temp_name;




    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    accounting_code_gchar=g_strdup_printf("%lu",accounting_code);

    gda_command_set_connection(command,connection);

    enterprise = g_strdup_printf("%lu",enterprise_code);

    query = g_strconcat("update PeriodoContabilidad set cerrado=TRUE where (codigoContabilidad=",
				 accounting_code_gchar," and codigoEmpresa=",
				 enterprise,")",
				 NULL);

    g_print(_("\nQuery: %s\n"),query);
    if (m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE)==FALSE) {
	result = FALSE;
    } else {
        result = TRUE;
    }


    if (query!=NULL) g_free(query);

    g_free(tuples);
    g_free(enterprise);
    g_free(accounting_code_gchar);

    if (record_set!=NULL)
	gda_recordset_free(record_set);



    igalia_command_free(command);

    return result;
}

/**
 * m_accounting_period_exists:
 * @accounting_period: Perodo de contabilidad del que se quiere comprobar su existencia.
 *
 * Comprueba la existencia de un perdo de contabilidad en la BD para una empresa en particular.
 * 
 * Returns: %TRUE si existe, %FALSE en otro caso.
 */
gboolean m_accounting_period_exists (AccountingPeriod * accounting_period) {
    gboolean result = TRUE;
    gulong code = d_accounting_get_accounting_code (accounting_period);
    AccountingPeriod * accounting_period_exist=NULL;
    ConditionAccountingPeriod * condition = (ConditionAccountingPeriod *) g_malloc0(sizeof(ConditionAccountingPeriod));
    time_t initial_date = d_accounting_get_initial_date (accounting_period);
    time_t ending_date = d_accounting_get_ending_date (accounting_period);
    condition->enterprise_code=d_accounting_get_enterprise_code (accounting_period);
    condition->initial_date=g_strdup(ctime(&initial_date));
    condition->ending_date=g_strdup(ctime(&ending_date));

    accounting_period_exist = m_accounting_period_find_by_condition (condition);
    g_free(condition->initial_date);
    g_free(condition->ending_date);
    g_free(condition);
    if (accounting_period_exist==NULL) {
	gchar *query = "SELECT codigoContabilidad FROM periodoContabilidad WHERE codigoContabilidad = ";
        gchar *more_query = "and codigoEmpresa =";
	gchar *code_gchar = g_strdup_printf("%lu",code);
	gchar *enterprise_code = g_strdup_printf("%lu",d_accounting_get_enterprise_code (accounting_period));
	GdaRecordset *rs = NULL;
	GdaConnection * connection = m_model_get_connection ();
	GdaCommand * command = gda_command_new();
	gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

	query = g_strconcat(query, code_gchar, more_query,enterprise_code,NULL);

	gda_command_set_connection(command,connection);


	printf(_("\nQuery: %s\n"),query);

	m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);

	if (*tuples==1) {
            result = TRUE;
	} else {
            result = FALSE;
	}

	g_free(query);
	g_free(code_gchar);
        g_free(enterprise_code);
    }
    return result;
}

/**
 * m_accounting_period_is_closed: 
 * @accounting_period: Perodo de contabilidad del que se quiere contrastar si est cerrado.
 * 
 *
 * Comprueba si un perodo de contabilidad de una empresa est ya cerrado, ya que un
 * perodo de contabilidad cerrado implica que no se pueden insertar ms facturas de pago en ese perodo.
 *
 * Returns: %TRUE si est cerrado.
 */

gboolean m_accounting_period_is_closed (AccountingPeriod * accounting_period) {
    gboolean result = FALSE;
    gulong code = d_accounting_get_accounting_code (accounting_period);

    if (code!=ULONG_MAX) {
	gchar *query = "SELECT codigoContabilidad FROM periodoContabilidad WHERE cerrado=TRUE and codigoContabilidad = ";
	gchar *more_query = "and codigoEmpresa =";
	gchar *code_gchar = g_strdup_printf("%lu",code);
	gchar *enterprise_code = g_strdup_printf("%lu",d_accounting_get_enterprise_code (accounting_period));
	GdaRecordset *rs = NULL;
	GdaConnection * connection = m_model_get_connection ();
	GdaCommand * command = gda_command_new();
	gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

	query = g_strconcat(query, code_gchar, more_query,enterprise_code,NULL);

	gda_command_set_connection(command,connection);


	printf(_("\nQuery: %s\n"),query);

	m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);

	if (*tuples==1) {
            result = TRUE;
	} else {
            result = FALSE;
	}

	g_free(query);
	g_free(code_gchar);
        g_free(enterprise_code);
    }
    return result;
}

/**
 * m_accounting_period_clear_array:
 * @accounting_periods: El array de perodos de contabilidad a liberar
 *
 * Libera la memoria ocupada por el array de perodos de contabilidad.
 *
 */
void m_accounting_period_clear_array (AccountingPeriod ** accounting_periods) {
    if (accounting_periods!=NULL) {
	gint index = 0;
	while (accounting_periods[index]!=NULL) {
	    d_accounting_period_clear (accounting_periods[index]);
	    index++;
	}
    }
    g_free(accounting_periods);
    accounting_periods=NULL;
}
/**
 * m_accounting_period_find_by_condition:
 * @condition: Condicin por la que se quiere buscar el perodo de contabilidad. Esta condicin es un
 * "struct" con los campos fecha_desde,fecha_hasta y codigoEmpresa
 * 
 * Busca el perodo de contabilidad que cumple la condicin.
 * Antes de obtener un perodo de contabilidad ser necesario buscarlo en la BD. No pueden existir
 * perodos de contabilidad con las mismas caractersticas para la misma empres, con lo que la condicin
 * la satisfar un perodo de contabilidad como mximo.
 *
 * Returns: El perodo de contabilidad de una empresa con la fecha de inicio y fin de la condicin
 * si existe en la BD. %NULL en caso de no existir.
 */
AccountingPeriod * m_accounting_period_find_by_condition (ConditionAccountingPeriod * condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_where_aux=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise_code = FALSE;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    AccountingPeriod * accounting_period;
    gint index;

    my_command = gda_command_new();


    gda_command_set_connection(my_command,my_connection);

    if ((condition->enterprise_code!=ULONG_MAX) && (condition->enterprise_code!=0))
        enterprise_code = g_strdup_printf("%lu",condition->enterprise_code);


    gchar_select=g_strconcat("SELECT codigoContabilidad as codigoContabilidad,"
                             "fechaInicio as fechaInicioContabilidad,",
                             "fechaFin as fechaFinContabilidad,",
                             "codigoEmpresa as codigoEmpresaContabilidad,",
                             "cerrado as cerradoContabilidad "
			     ,NULL);


    gchar_from= g_strconcat(" FROM PeriodoContabilidad "
			    ,NULL);

    gchar_where=g_strconcat(" WHERE codigoEmpresa=",enterprise_code,
			    NULL);

    if ((condition->initial_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (fechaInicio='",
				      condition->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (fechaFin='",
				      condition->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples==1) {
	    index = gda_recordset_move_first(record_set);
	    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	    accounting_period = m_accounting_period_create (record_set);
	} else {
            accounting_period = NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
        accounting_period=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(enterprise_code);

    return accounting_period;

}

/**
 * m_accounting_get_day_after_to_last_closed:
 * @enterprise_code: Cdigo de la empresa para la cul se va a buscar el da adecuado.
 * @invoice_issue_date:Fecha de emisin propuesta para almacenar la factura.
 * 
 *
 * Obtiene el primer da disponible para insertar una factura de
 * pago. Si una factura de pago nos llega una vez hemos cerrado la contabilidad del perodo anterior, no podremos
 * insertar la factura con fecha de emisin la original, ya que quedara sis declarar. As, si el perodo en
 * el que nos han emitido la factura est cerrado, es necesario insertarlo con el siguiente da disponible dentro
 * del perodo de contabilidad abierto.
 *
 * Returns: Devuelve la misma fecha que se ha pasado por parmetro si esta fecha est dentro de un perodo
 * de contabilidad abierto para esa empresa, en otro caso devolver el primer da disponible despues del ltimo
 * da del perodo de contabilidad cerrado.
 */
time_t * m_accounting_get_day_after_to_last_closed (gulong enterprise_code,time_t * invoice_issue_date) {
    gint result = -1;
    gchar *query = "SELECT MAX(fechaFin) FROM periodoContabilidad WHERE cerrado=TRUE and codigoEmpresa = ";
    gchar *enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    GdaRecordset *record_set = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    time_t * last_day=NULL;
    time_t * next_day =(time_t *) g_malloc(sizeof(time_t));

    query = g_strconcat(query, enterprise_code_char,NULL);

    gda_command_set_connection(command,connection);


    printf(_("\nQuery: %s\n"),query);

    m_model_command_execute (&record_set, query, connection, command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
	    struct tm * time_tm;
	    GDate * date;
            GdaField * field;
	    result = TRUE;
	    gda_recordset_move_first(record_set);
	    field = gda_recordset_field_idx(record_set,0);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    temp=gda_stringify_value(NULL, 0, field);
		    last_day = u_parse_date (temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	    gda_recordset_close(record_set);
	    gda_recordset_free(record_set);

	    time_tm = gmtime(last_day);
            time_tm ->tm_isdst = 1;
            date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
	    g_date_add_days(date,2);
	    time_tm ->tm_mday = g_date_day(date);
	    time_tm ->tm_mon= g_date_month(date)-1;
	    time_tm ->tm_year = g_date_year(date)-1900;
	    (*next_day) = mktime(time_tm);

	} else {
	    (*next_day)=(*invoice_issue_date);
	}
    } else {
        next_day=NULL;
    }


    g_free(query);
    g_free(enterprise_code_char);

    igalia_command_free(command);
    return next_day;
}
/**
 * m_accounting_get_date_in_closed:
 * @enterprise_code: Cdigo de la empresa para la cul se va a comprobar la fecha.
 * @date:Fecha que se quiere comprobar.
 *
 * Comprueba si la fecha indicada est dentro de un perodo de contabilidad
 * cerrado. Al igual que la funcin m_accounting_get_day_after_to_last_closed () esta se utiliza para comprobar si las
 * fechas de emisin de las facturas estn en perodos de contabilidad cerrados.
 *
 * Returns: %TRUE si la fehca cuadra en un perodo de contabilidad ya cerrado.
 */
gboolean m_accounting_get_date_in_closed (gulong enterprise_code,time_t date) {
    gint result = -1;

    if (enterprise_code!=ULONG_MAX) {
	gchar *query = "SELECT fechafin FROM periodoContabilidad WHERE cerrado=TRUE and codigoEmpresa =";
	gchar *code_gchar = g_strdup_printf("%lu",enterprise_code);
        gchar *dates = g_strconcat(" and fechainicio<='",ctime(&date),"' and fechafin>='",ctime(&date),"' ",NULL);
	GdaRecordset *rs = NULL;
	GdaConnection * connection = m_model_get_connection ();
	GdaCommand * command = gda_command_new();
	gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

	query = g_strconcat(query, code_gchar,dates,NULL);

	gda_command_set_connection(command,connection);


	printf(_("\nQuery: %s\n"),query);

	m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);

	if (*tuples==1) {
            result = TRUE;
	} else {
            result = FALSE;
	}

	g_free(query);
	g_free(code_gchar);
        g_free(dates);
    }
    return result;
}


