/* cash_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "cash_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "cash_operation_model.h"

#define TABLE         "Caja "
#define ATTRIBUTES    "Caja.codigoEmpresa as codigoEmpresaCash , \
                       Caja.codigoTienda as codigoTiendaCash , \
                       Caja.saldoInicial as saldoInicialCash , \
                       Caja.codigoCaja as codigoCajaCash ";


/**
 * get_d_cash_get_cash_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_cash_get_table () {
    return TABLE;
}

/**
 * d_cash_get_atributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_cash_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_cash_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Cash con los campos rellenados con los datos de la base de datos.
 */
Cash *m_cash_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    Cash *cash = d_cash_create_empty();

    field = gda_recordset_field_name(rs,"codigocajacash");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    cash->cash_code=igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);

        cash->enterprise = m_enterprise_create (rs);
	cash->shop = m_shop_create (rs);

	field = gda_recordset_field_name(rs, "saldoinicialcash");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp=gda_stringify_value(NULL, 0, field);
		cash->initial_balance = atof(temp);
		g_free(temp);
	    }
	    gda_field_free(field);
	}
	field = gda_recordset_field_name(rs, "saldoactualcash");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp=gda_stringify_value(NULL, 0, field);
		cash->current_balance = atof(temp);
		g_free(temp);
	    }
	    gda_field_free(field);
	}
    }

    return cash;
}



/**
 * m_cash_find_by_shop: 
 * @enterprise_code: Cdigo de la empresa.
 * @shop_code: Cdigo de la tienda.
 * 
 * Busca la caja de una tienda por su cdigo de tienda y cdigo de empresa.
 * Los objetos caja representan las cajas reales de las tiendas, as que para buscarlas es necesario
 * obtenerlas por cgigo de tienda y cdigo de empresa.
 *
 * Returns: Devuelve el objeto caja de la tienda en cuestin.
 */
Cash * m_cash_find_by_shop (gulong enterprise_code,gulong shop_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise = g_strdup_printf("%lu",enterprise_code);
    gchar * shop = g_strdup_printf("%lu",shop_code);
    Cash * cash;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat(" SELECT codigoCaja as codigocajacash, ",
			      " caja.codigoTienda as codigoTienda, ",
			      "participanteEnter.codigoParty as codigoPartyParticipantEnter,",
			      "participanteEnter.nombre as nombreParticipantEnter,",
			      "participanteEnter.cif as cifParticipantEnter,",
			      "empresa.perteneceGrupo as perteneceGrupoEnterprise,",
			      "participanteShop.codigoParty as codigoPartyParticipantShop,",
			      "participanteShop.nombre as nombreParticipantShop,",
			      "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop,",
			      " caja.codigoEmpresa as codigoEmpresa, ",
                              " to_char(saldoInicial,'99999999D99') as saldoInicialCash, ",
                              " to_char(saldoActual,'99999999D99') as saldoActualCash ",
			      " from caja,participante participanteEnter,",
			      " participante participanteShop,empresa,tienda where caja.codigoEmpresa=",enterprise,
                              " and caja.codigoTienda=",shop,
                              " and caja.codigoEmpresa=participanteEnter.codigoParty",
                              " and empresa.codigoParty=participanteEnter.codigoParty",
                              " and caja.codigoTienda=participanteShop.codigoParty",
                              " and tienda.codigoTienda=participanteShop.codigoParty",
                              " and empresa.codigoParty=tienda.codigoEmpresa",
			      NULL );

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0,TRUE);

    if (record_set!=NULL) {
	gda_recordset_move_first(record_set);
	if (*tuples==1) {
            cash = m_cash_create (record_set);
	} else {
	    cash=NULL;
	}
	gda_recordset_close(record_set);
    } else {
        cash = NULL;
    }


    g_free(gchar_query);
    g_free(tuples);
    g_free(enterprise);
    g_free(shop);

    igalia_command_free(my_command);
    gda_recordset_free(record_set);
    g_print(_("Freeing database completed\n"));

    return cash;
}



/**
 * m_cash_find_by_period:
 * @initial_date: Fecha de comienzo del intervalo para buscar movimientos de caja.
 * @ending_date: Fecha de fin del intervalo para buscar movimientos de caja.
 * @cash: Caja de la cul deseamos consultar los movimientos.
 * 
 * Devuelve el movimiento de caja entre dos fechas.
 * Funcin necesaria para poder comprobar los movimientos producidos en una caja en
 * un perodo de tiempo, comprendidos entre %initial_date y %ending_date.
 *
 * Returns: Devuelve una lista de movimientos de caja.
 */
CashOperation ** m_cash_find_by_period (time_t initial_date, time_t ending_date, Cash *cash) {
  CashOperation **list_of_cash_operations=NULL;
  gint length_of_list=0;
  GdaCommand         * my_command;
  GdaConnection      * my_connection = m_model_get_connection ();
  GdaRecordset       * record_set = NULL;
  CashOperation * cash_operation;
  gchar * gchar_query;
  gchar *enterprise, *shop, *initial_date_char, *ending_date_char;
  gulong * tuples=NULL;
  gint index;

  my_command = gda_command_new();
  tuples = (gulong *) g_malloc0(sizeof(gulong));
  gda_command_set_connection(my_command,my_connection);
  enterprise=g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_cash_get_enterprise (cash))));
  shop=g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_cash_get_shop (cash))));
  initial_date_char = g_strconcat("'",ctime(&initial_date),"'",NULL);
  ending_date_char = g_strconcat("'",ctime(&ending_date),"'",NULL);
  if (cash!=NULL) {
    gchar_query = g_strconcat("SELECT ",
			      " movimientoCaja.codigoCaja as codigoCajaCash, ",
			      " movimientoCaja.codigoEmpresa as codigoPartyParticipantEnter, ",
			      " movimientoCaja.codigoTienda as codigoPartyParticipantShop, ",
			      " movimientoCaja.codigoMovimiento as codigomovimientocashoperation, ",
			      " movimientoCaja.codigoPago as codigopagocashoperation, ",
			      " to_char(movimientoCaja.cantidad,'9999999D99') as cantidadCashOperation, ",
			      " movimientoCaja.fecha as fechaCashOperation, ",
			      " movimientoCaja.concepto as conceptooperation ",
			      " FROM movimientoCaja ",
			      " WHERE (movimientoCaja.codigoEmpresa=",enterprise,") ",
			      " AND (movimientoCaja.codigoTienda=",shop,") ",
			      " AND (movimientoCaja.fecha>=",initial_date_char,") ",
			      " AND (movimientoCaja.fecha<=",ending_date_char,") ",
			      NULL
			      );

  } else {
    return NULL;
  }
  g_print(_("\nQuery: %s\n"),gchar_query);

  m_model_command_execute (&record_set, gchar_query, my_connection,
			my_command,tuples,0,TRUE);
  length_of_list=*tuples;

  if (record_set!=NULL) {
    if (*tuples!=0) {
      list_of_cash_operations = (CashOperation **) g_malloc0(sizeof(CashOperation*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	cash_operation = m_cash_operation_create (record_set);
	list_of_cash_operations[index-1] = cash_operation;
	index = gda_recordset_move_next(record_set);
      }
      list_of_cash_operations[length_of_list]=NULL;
    } else {
      list_of_cash_operations=NULL;
    }
    gda_recordset_close(record_set);
  } else {
    list_of_cash_operations=NULL;
  }

    
  g_free(gchar_query);
  g_free(tuples);
  g_free(enterprise);
  g_free(shop);
  g_free(initial_date_char);
  g_free(ending_date_char);

  igalia_command_free(my_command);
  gda_recordset_free(record_set);
  g_print(_("Freeing database completed\n"));

  return list_of_cash_operations;

  
}
