/* cash_operation_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "cash_operation_model.h"
#include "cash_model.h"
#include "payment_model.h"

#define TABLE         "MovimientoCaja "
#define ATTRIBUTES    "MovimientoCaja.codigoEmpresa as codigoEmpresaCashOperation , \
                       MovimientoCaja.codigoTienda as codigoTiendaCashOperation , \
                       MovimientoCaja.codigoPago as codigoPagoCashOperation , \
                       MovimientoCaja.codigoMovimiento as codigoMovimientoCashOperation , \
                       MovimientoCaja.codigoCaja as codigoCajaCashOperation ";



/**
 * m_cash_operation_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_cash_operation_get_table () {
    return TABLE;
}

/**
 * m_cash_operation_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_cash_operation_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_cash_operation_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns: Un objeto #CashOperation con los campos rellenados con los datos de la base de datos.
 */
CashOperation *m_cash_operation_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    CashOperation *cash_operation = d_cash_operation_create_empty();

    field = gda_recordset_field_name(rs,"codigoMovimientoCashOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    cash_operation->operation_code=igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs, "cantidadCashOperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    cash_operation->amount = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaCashOperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    cash_operation->date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"conceptooperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    cash_operation->section = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    cash_operation->cash =m_cash_create (rs);
	    cash_operation->payment = m_payment_create (rs);
	} else {
	    gda_field_free(field);
	}
    }
    return cash_operation;
}



/**
 * m_cash_operation_insert: 
 * @cash_operation: Movimiento de caja que ya incluye la caja en la cual se quiere introducir el movimiento.
 * 
 * Inserta un movimiento de caja en una caja en particular. El objeto cash_operation ya contiene una referencia
 * al objeto cash (caja).
 *
 * Returns: %TRUE si se ha insertado correctamente, si no %FALSE.
 */
gboolean m_cash_operation_insert (CashOperation * cash_operation) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gchar * query2=NULL;
    gchar * query3=NULL;
    gchar * query_aux=NULL;
    gchar * query_payment=NULL;
    gchar * values=NULL;
    gchar * payment_code_aux=NULL;
    gchar * values_aux;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    time_t fecha;
    GdaRecordset *record_set=NULL;
    GdaRecordset *record_set_payment=NULL;
    GdaRecordset *record_set2=NULL;
    GdaRecordset *record_set3=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong cash_code=d_cash_get_cash_code (d_cash_operation_get_cash (cash_operation));
    gulong enteprise_code=d_participant_get_code (d_enterprise_get_participant (d_cash_get_enterprise (d_cash_operation_get_cash (cash_operation))));
    gulong shop_code=d_participant_get_code (d_shop_get_participant (d_cash_get_shop (d_cash_operation_get_cash (cash_operation))));
    //    gulong enteprise_code = d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (d_cash_operation_get_payment (cash_operation))));
    //    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_payment_get_shop (d_cash_operation_get_payment (cash_operation))));
    gda_command_set_connection(command,connection);

    if ((cash_code!=ULONG_MAX) && (enteprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
        gchar * temp;
	gchar * amount_char=NULL;
	gchar * cash_code_char=NULL;
	gdouble saldo=0.0;

	m_model_connection_begin_mutex_transaction_logged (connection,enteprise_code,shop_code,ULONG_MAX);

	query = g_strconcat(" INSERT INTO MOVIMIENTOCAJA VALUES ",NULL);
	values = g_strconcat(" (",NULL);


	temp = g_strdup_printf("%lu",enteprise_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;

	temp = g_strdup_printf("%lu",shop_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;

	cash_code_char = g_strdup_printf("%lu",d_cash_get_cash_code (d_cash_operation_get_cash (cash_operation)));
	values_aux = g_strconcat(values,cash_code_char,",",NULL);
	g_free(values);
	values = values_aux;


	temp = m_model_nextval_sequence ("seqMovimientoCaja",connection);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;


	if ((d_cash_operation_get_payment (cash_operation)!=NULL) && (d_payment_get_payment_code (d_cash_operation_get_payment (cash_operation))!=ULONG_MAX)) {
	    temp = g_strdup_printf("%lu",d_payment_get_payment_code (d_cash_operation_get_payment (cash_operation)));
	    values_aux = g_strconcat(values,temp,",",NULL);
	    g_free(values);
	    g_free(temp);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}

	if (d_cash_operation_get_amount (cash_operation)!=G_MINDOUBLE) {
	    amount_char = g_strdup_printf("%f",d_cash_operation_get_amount (cash_operation));
	    values_aux = g_strconcat(values,"to_number('",amount_char,"','99999999D99'),",NULL);
	    g_free(values);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}

	fecha = d_cash_operation_get_date (cash_operation);
	values_aux = g_strconcat(values,"'",ctime(&fecha),"',",NULL);
	g_free(values);
	values = values_aux;

        temp = d_cash_operation_get_section (cash_operation);
	if (temp!=NULL) {
	    values_aux = g_strconcat(values,"'",temp,"'",NULL);
	    g_free(values);
	    g_free(temp);                         
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}

	query_aux=g_strconcat(query,values,")",NULL);
	g_free(query);
	query=query_aux;
	query_aux=NULL;


	if (m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE)==FALSE) {
	    result = FALSE;
	}                            

	if (d_cash_operation_get_payment (cash_operation)!=NULL) {
	    if (d_payment_get_payment_code (d_cash_operation_get_payment (cash_operation))!=ULONG_MAX) {
		time_t fechaRealizado=time(NULL);
		payment_code_aux = g_strdup_printf("%lu",d_payment_get_payment_code (d_cash_operation_get_payment (cash_operation)));
		query_payment = g_strconcat("update pago set fecharealizado='",ctime(&fechaRealizado),"' where codigoPago=",payment_code_aux,NULL);
		g_print("\n%s\n",query_payment);
		g_free(payment_code_aux);
	    }
	}

	if (query_payment!=NULL) {
	    result = m_model_command_execute (&record_set_payment, query_payment, connection, command,tuples,0, FALSE) && result;
	}

        query2 = g_strconcat("select to_char(saldoActual,'99999999D99') from caja where codigoCaja=",cash_code_char,NULL);
	g_print(query2);

/*	if (m_model_command_execute (&record_set2, query2, connection,command,tuples,0,TRUE)==FALSE) {
	    result = FALSE;
	}                            
*/

/*	if (record_set2!=NULL) {
	    if (*tuples!=0) {
		GdaField * field;
		gda_recordset_move_first(record_set2);
		field  = gda_recordset_field_idx(record_set2,0);
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			gchar * saldoActual;
			saldoActual = gda_stringify_value(NULL, 0, field);
			saldo=atof(saldoActual);
		    }
		    gda_field_free(field);
		} else {
                    saldo = 0.0;
		}
	    } else {
		result = FALSE;
	    }
	} else {
	    result=FALSE;
	}
*/	if (result != FALSE) {
    //	    saldo = saldo + (d_cash_operation_get_amount (cash_operation));
            saldo = d_cash_operation_get_amount (cash_operation);
	    if (amount_char!=NULL)
		g_free(amount_char);
	    amount_char = g_strdup_printf("%.2f",saldo);
	    query3 = g_strconcat("update caja set saldoActual=saldoActual+(to_number('",amount_char,"','99999999D99')",") where codigocaja=",cash_code_char,NULL);
            g_print(query3);
	    g_free(amount_char);
	    g_free(cash_code_char);
	    if (m_model_command_execute (&record_set3, query3, connection,command,tuples,0,FALSE)==FALSE) {
		result = FALSE;
	    }
            g_free(query3);
	}
	if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
	    result = FALSE;
	}
    }

    if (query_payment!=NULL) g_free(query_payment);
    if (query!=NULL) g_free(query);
    if (query2!=NULL) g_free(query2);
    if (tuples!=NULL) g_free(tuples);

    if (record_set!=NULL)
	gda_recordset_free(record_set);
    if (record_set_payment!=NULL)
	gda_recordset_free(record_set_payment);
    if (record_set2!=NULL)
	gda_recordset_free(record_set2);
    if (record_set3!=NULL)
	gda_recordset_free(record_set3);

    igalia_command_free(command);

    return result;
}

/**
 * m_cash_operation_update_in_transaction:
 * @payment_code: Pago al que se refiere este movimiento.
 * @shop_code: Cdigo de la tienda en la que se realiz en movimiento.
 * @enteprise_code: Cdigo de la empresa a la que pertenece la tienda.
 * @amount: Cantidad nueva que se meter.
 * @connection: Conexin a la base de datos.
 *
 * Modifica la amount de un movimiento en caja.
 *
 * Returns:
 * %TRUE si se modific correctamente, %FALSE en otro caso.
 */
gboolean m_cash_operation_update_in_transaction (gulong payment_code,gulong shop_code,gulong enteprise_code,gdouble amount,GdaConnection * connection) {
    gboolean result=TRUE;
    GdaCommand *command  = gda_command_new();
    gchar * query = NULL;
    GdaRecordset * record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * payment_code_char = g_strdup_printf("%lu",payment_code);
    gchar * shop_code_char = g_strdup_printf("%lu",shop_code);
    gchar * enteprise_code_char = g_strdup_printf("%lu",enteprise_code);
    gchar * amount_char = g_strdup_printf("%.2f",u_round_to_decimal (amount,2));
    gchar * amount_aux=NULL;
    gchar * sig=NULL;
    gchar * dif_char=NULL;
    gchar * cash_code=NULL;
    gdouble amount_aux_double = G_MINDOUBLE;
    gdouble dif;
    gda_command_set_connection(command,connection);

    u_commas_to_points (amount_char);

    query = g_strconcat(" SELECT to_char(cantidad,'99999999D99'),codigoCaja FROM MovimientoCaja WHERE ",
			" codigoPago=",payment_code_char,
			" and codigoTienda=",shop_code_char,
			" and codigoEmpresa=",enteprise_code_char,NULL);

    g_print(_("\nQuery: %s\n"),query);
    result = m_model_command_execute (&record_set, query, connection, command,tuples,0, TRUE);

    if (record_set!=NULL) {
	    if (*tuples!=0) {
		GdaField * field;
		gda_recordset_move_first(record_set);
		field  = gda_recordset_field_idx(record_set,0);
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {

		        amount_aux = gda_stringify_value(NULL, 0, field);
                        amount_aux_double = atof(amount_aux);
			g_print(_("\nDouble: %f\n"),amount_aux_double);
		    }
		    gda_field_free(field);
		} 
		field  = gda_recordset_field_idx(record_set,1);
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
		        cash_code = gda_stringify_value(NULL, 0, field);
		    }
		    gda_field_free(field);
		} 
	    }
    }
    g_free(query);


    if (amount_aux_double!=G_MINDOUBLE) {
	query = g_strconcat(" UPDATE MOVIMIENTOCAJA SET cantidad=",amount_char," WHERE ",
			    " codigoPago=",payment_code_char,
			    " and codigoTienda=",shop_code_char,
			    " and codigoEmpresa=",enteprise_code_char,NULL);

	g_print(_("\nQuery: %s\n"),query);
	result = m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);

	if (record_set!=NULL) {
	    if (GDA_IS_RECORDSET(record_set)) {
		gda_recordset_close(record_set);
	    }
	}
	g_free(query);

	dif = u_round_to_decimal ((amount - amount_aux_double ),2);
	if (dif >= 0) {
	    sig = "+";
	} else {
	    sig = "-";
	    dif = dif*(-1);
	}

	dif_char = g_strdup_printf("%.2f",dif);
	u_commas_to_points (dif_char);
	query = g_strconcat(" UPDATE CAJA SET saldoactual= saldoactual " ,sig ," CAST(",dif_char,"as DECIMAL(10,2)) WHERE ",
			    " codigoCaja=",cash_code,
			    " and codigoTienda=",shop_code_char,
			    " and codigoEmpresa=",enteprise_code_char,NULL);

	g_print(_("\nQuery: %s\n"),query);
	result = m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);
	g_free(query);

    }

    if (payment_code_char!=NULL)
	g_free(payment_code_char);
    if (enteprise_code_char!=NULL)
    g_free(enteprise_code_char);
    if (shop_code_char!=NULL)
	g_free(shop_code_char);
    g_free(tuples);
    igalia_command_free(command);
    return result;
}



/**
 * m_cash_operation_delete:
 * @selected: Movimientos seleccionados
 * @operation_list: Lista de movimientos de caja.
 * @shop_code: Cdigo de la tienda en la que se realiz en movimiento.
 * @enteprise_code: Cdigo de la empresa a la que pertenece la tienda.
 *
 * Borra los movimientos de caja 
 *
 * Returns:
 * %TRUE si se modific correctamente, %FALSE en otro caso.
 */
gboolean m_cash_operation_delete (GArray * selected, CashOperation ** operation_list, gulong shop_code,gulong enteprise_code) {

    GdaConnection * connection = m_model_get_connection ();

    return m_cash_operation_delete_in_transaction (selected, operation_list, shop_code, enteprise_code, connection);
}	


/**
 * d_cash_operation_clear:
 * @cash_operation: Movimiento de caja a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint m_cash_operation_clear_array (CashOperation **cash_operations) {

    CashOperation **aux = cash_operations;
    	
    while (*aux != NULL) {
			
	d_cash_operation_clear (*aux);
	aux++;
    }	
	
    g_free(cash_operations);
    
  return 0;
}


/**
 * m_cash_operation_delete_in_transaction:
 * @selected: Operaciones seleccionadas.
 * @operation_list: Lista de movimientos entre las que escoger las seleccionadas.
 * @shop_code: Cdigo de la tienda.
 * @enteprise_code: Cdigo de la empresa a la que pertenece la tienda.
 * @connection: Conexin a la base de datos.
 *
 * Borra los movimientos de caja dentro de una transaccin.
 *
 * Returns:
 * %TRUE si se modific correctamente, %FALSE en otro caso.
 */
gboolean m_cash_operation_delete_in_transaction (GArray * selected, CashOperation ** operation_list, gulong shop_code,gulong enteprise_code,GdaConnection * connection) {
    gboolean result=TRUE;
    GdaCommand *command  = gda_command_new();
    gchar * query = NULL;
    GdaRecordset * record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * shop_code_char = g_strdup_printf("%lu",shop_code);
    gchar * enteprise_code_char = g_strdup_printf("%lu",enteprise_code);
    gchar * codigoMov_char;
    CashOperation * operation;
    gint i, index, limit;
    gulong codigoMov; 
    gchar * amount_char;
    gchar * cash_code;
    gchar * sig=NULL;
    GdaField * payment_code;
    
        
    m_model_connection_begin_mutex_transaction_logged (connection,enteprise_code,shop_code,ULONG_MAX);
        	
    gda_command_set_connection(command,connection);

    
    limit = selected->len;
    i = limit-1;
    while (i >= 0) {

	index = g_array_index(selected, gint , i); 
	operation = operation_list[index];

        cash_code = g_strdup_printf("%lu",d_cash_get_cash_code (d_cash_operation_get_cash (operation)));
	g_print(_("\nIndex: %d | i : %d\n"),index,i);
			
	codigoMov = d_cash_operation_get_operation_code (operation);
	codigoMov_char =  g_strdup_printf("%lu",codigoMov);

    	query = g_strconcat(" SELECT codigoPago FROM MovimientoCaja WHERE ",
			    " codigoMovimiento=", codigoMov_char,
                            " and codigoTienda=", shop_code_char,
                            " and codigoEmpresa=", enteprise_code_char,
			    NULL);

   	g_print(_("\nQuery: %s\n"),query);
    
    	result = result &&m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);
	g_free(query);
	
	gda_recordset_move_first(record_set);
	payment_code = gda_recordset_field_name(record_set, "codigopago"); 
 	
	if (gda_field_is_null(payment_code)) {
      		
		query = g_strconcat(" DELETE FROM MOVIMIENTOCAJA WHERE ",
					" codigoMovimiento=", codigoMov_char, 
					" and codigoTienda=", shop_code_char,
					" and codigoEmpresa=", enteprise_code_char,
					" and codigoPago is NULL", NULL);

	    	g_print(_("\nQuery: %s\n"),query);
    
    		result = result && m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);

		g_free(query);

		if (d_cash_operation_get_amount (operation)>=0)
		    sig = "-";
		else
                    sig = "+";

		amount_char = g_strdup_printf("%.2f",d_cash_operation_get_amount (operation));
		u_commas_to_points (amount_char);

		query = g_strconcat(" UPDATE CAJA SET saldoactual= saldoactual " ,sig ," CAST(",amount_char,"as DECIMAL(10,2)) WHERE ",
				    " codigoCaja=",cash_code,
				    " and codigoTienda=",shop_code_char,
				    " and codigoEmpresa=",enteprise_code_char,NULL);

    		result = result && m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);
                g_free(query);

		g_array_remove_index(selected, i);
    	}	
	    	
	i--;
	g_free(codigoMov_char);
        g_free(cash_code);
	gda_field_free(payment_code);
    
    	if (record_set!=NULL) {
		if (GDA_IS_RECORDSET(record_set)) {
	    		gda_recordset_close(record_set);
		}
    	}
    }
            
    g_free(enteprise_code_char);
    g_free(shop_code_char);
    g_free(tuples);
    igalia_command_free(command);
  

    if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
	   
	    result = FALSE;
    }  
    

    return result;
}
