/* charge_invoice_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "util.h"
#include "charge_invoice_model.h"
#include "charge_invoice_line_model.h"
#include "enterprise_model.h"
#include "supplier_model.h"
#include "shop_model.h"


#define TABLE         "FacturaCargo"
#define ATTRIBUTES    "codigoProveedor as codigoProveedorFacturaCargo, \
    numeroFactura as numeroFacturaFacturaCargo, \
    fecha as fechaFacturaCargo, \
    to_char(total,99999999D99) as totalFacturaCargo, \
    concepto as conceptoFacturaCargo";

static gboolean insert_charge_lines(ChargeInvoiceLine **charge_lines, GdaCommand * command, GdaConnection *connection);


/**
 * m_charge_invoice_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_charge_invoice_get_table () {
    return TABLE;
}

/**
 * m_charge_invoice_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_charge_invoice_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_charge_invoice_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #ChargeInvoice con los campos rellenados con los datos de la base de datos.
 */
ChargeInvoice * m_charge_invoice_create (GdaRecordset *rs) {

    GdaField * field;
    ChargeInvoice * charge_invoice;

    charge_invoice = d_charge_invoice_create_empty();

    field = gda_recordset_field_name(rs,"numeroFacturaFacturaCargo" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp_number=gda_stringify_value(NULL, 0, field);
	    charge_invoice->invoice_number = g_strdup(temp_number);
	    g_free(temp_number);

	    field = gda_recordset_field_name(rs,"fechaFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    charge_invoice->date = (*my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "conceptoFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    charge_invoice->section = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "codigoFacturaFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    charge_invoice->code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "estadoFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    charge_invoice->state = gda_field_get_integer_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "totalFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    charge_invoice->total = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	    charge_invoice -> shop = m_shop_create (rs);
	    charge_invoice -> supplier = m_supplier_create (rs);
	    charge_invoice -> payment = m_payment_create (rs);
	    charge_invoice -> enterprise = m_enterprise_create (rs);
	} else {
	    gda_field_free(field);
	}

    }
    return charge_invoice;

}



/** 
 * m_charge_invoice_insert: 
 * @charge_invoice: Factura de cargo en cuestin.
 * 
 * Funcin que inserta en la base de datos una factura, con un pago asociado. Es
 * necesario que tenga un pago asociado ya que despus no se podra insertar un
 * pago asociado a la factura.
 *
 * Returns: Nmero de la factura insertada. %NULL si no se ah podido
 * insertar.
 */
gchar * m_charge_invoice_insert (ChargeInvoice * charge_invoice) {
    gulong result=0;
    gchar * result_gchar=NULL;
    gchar * query1=NULL;
    gchar * query2=NULL;
    gchar * query4=NULL;
    gchar * query5=NULL;
    gchar * query=NULL;
    gchar * query_aux;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * invoice_number=NULL;
    gchar * supplier;
    gchar * invoice;
    gchar * charge_invoice_temp;
    gulong shop_code=ULONG_MAX;
    gchar * invoice_code_char;
    gulong invoice_code_gulong;
    gchar ** stop = (gchar**) g_malloc0(sizeof(gchar *));
    gulong supplier_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong payment_code = ULONG_MAX;
    gdouble total = d_charge_invoice_get_total (charge_invoice);
    ChargeInvoiceLine ** charge_lines = d_charge_invoice_get_charge_invoice_lines (charge_invoice);
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;
    GdaRecordset *record_set3=NULL;
    GdaRecordset *record_set4=NULL;
    GdaRecordset *record_set5=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    time_t date=d_charge_invoice_get_date (charge_invoice);
    gchar * section = d_charge_invoice_get_section (charge_invoice);
    gint state = d_charge_invoice_get_state (charge_invoice);
    gboolean with_payment;
    gboolean with_vat;

    gda_command_set_connection(command,connection);

    with_vat =d_charge_invoice_get_vat (charge_invoice);

    shop_code=d_participant_get_code (d_shop_get_participant (d_charge_invoice_get_shop (
										     charge_invoice
										    )));
    enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_charge_invoice_get_enterprise (
												  charge_invoice
												 )));
    supplier_code = d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice)));

    supplier = g_strdup_printf("%lu",supplier_code);
    invoice = d_charge_invoice_get_invoice_number (charge_invoice);

    if (with_vat!=FALSE) {
        gchar * date_temp=g_strdup(ctime(&date));
	query4 = g_strconcat("select numeroFactura from facturaCargo where numeroFactura='",invoice,
			     "' and codigoProveedor=",supplier," and fecha='",date_temp,"'",NULL);
        g_free(date_temp);
    } else {
        gchar * date_temp=g_strdup(ctime(&date));
	query4 = g_strconcat("select codigoNotaCargo from notaCargo where codigoNotaCargo='",invoice,
			     "' and codigoProveedor=",supplier," and fecha='",date_temp,"'",NULL);
	g_free(date_temp);
    }

    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);

    if (m_model_command_execute (&record_set4, query4, connection, command, tuples, 0, TRUE)==FALSE) {
        result = ULONG_MAX;
    }

    if ((record_set4!=NULL) && (*tuples==0)) {
	if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX) && (supplier_code!=ULONG_MAX)) {
	    Payment * payment = d_charge_invoice_get_payment (charge_invoice);
	    if (d_charge_invoice_get_payment (charge_invoice)!=NULL) {
		/* The payment has data but it can already exists or no  */
		/* If the payment already exists codigoPago != ULONG_MAX */


		query1 = g_strconcat(" INSERT INTO PAGO VALUES ",NULL);
		values = g_strconcat(" (",NULL);
		payment_code = d_payment_get_payment_code (payment);

		if ((payment_code==0) || (payment_code==ULONG_MAX)) {
		    time_t time_aux;
                    gchar * temp_seq;
		    if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
			gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(values);
			g_free(temp);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
		    if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
			gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(values);
			g_free(temp);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
                    temp_seq=m_model_nextval_sequence ("seqPago",connection);
		    values_aux = g_strconcat(values,temp_seq,",",NULL);
		    g_free(values);
		    values = values_aux;
		    g_free(temp_seq);
		    if (with_vat!=FALSE) {
			if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
			    gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
			    values_aux = g_strconcat(values,temp,",",NULL);
			    g_free(temp);
			    g_free(values);
			    values = values_aux;
			} else {
			    values_aux = g_strconcat(values,"NULL,",NULL);
			    g_free(values);
			    values = values_aux;
			}
		    } else {
                        /* A NOTE IS PAID*/
			    gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
			    values_aux = g_strconcat(values,temp,",",NULL);
			    g_free(temp);
			    g_free(values);
			    values = values_aux;
		    }

		    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
			time_t temp = d_payment_get_achieved_date (payment);
			values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }


		    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    time_aux = d_payment_get_init_date (payment);
		    values_aux = g_strconcat(values,"'",ctime(&time_aux),"',",NULL);
		    g_free(values);
		    values = values_aux;


		    if (d_payment_get_entry_percentage (payment)!=-1) {
			gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
                        u_commas_to_points(temp);
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }
		    if (d_payment_get_number_expiries (payment)!=-1) {
			gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_interval_days (payment)!=-1) {
			gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_direct_debit (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_bank_code (payment)!=-1) {
			gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_office_code (payment)!=-1) {
			gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_account_number (payment)!=NULL) {
			gchar * temp = d_payment_get_account_number (payment);
			values_aux = g_strconcat(values,temp,",",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }


		    if (d_payment_get_is_transfer (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_is_cheque (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }


		    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
			gchar * temp = d_payment_c_util_get_cheque_number (payment);
			values_aux = g_strconcat(values,"'",temp,"',",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_credit_card (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE,",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE,",NULL);
			g_free(values);
			values = values_aux;
		    }

		    if (d_payment_get_credit_card_number (payment)!=NULL) {
			gchar * temp = d_payment_get_credit_card_number (payment);
			values_aux = g_strconcat(values,"'",temp,"',",NULL);
			g_free(temp);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);;
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_credit_card (payment)==TRUE) {
			time_t temp = d_payment_get_expiry_date (payment);
			values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
			g_free(values);
			values = values_aux;

		    } else {
			values_aux = g_strconcat(values,"NULL,",NULL);;
			g_free(values);
			values = values_aux;

		    }

		    if (d_payment_get_is_paid_cash (payment)==TRUE) {
			values_aux = g_strconcat(values,"TRUE",NULL);
			g_free(values);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"FALSE",NULL);
			g_free(values);
			values = values_aux;
		    }

		    query_aux=g_strconcat(query1,values,")",NULL);
		    g_free(query1);
		    query1=query_aux;
		    query_aux=NULL;
		}
		if (m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE)==FALSE) {
                    result = ULONG_MAX;
		}

		query5 = g_strconcat("select currval('seqPago')",NULL);

		if (m_model_command_execute (&record_set5, query5, connection, command,tuples,0, TRUE)==FALSE) {
		    result = ULONG_MAX;
		}

		if (record_set5!=NULL) {
		    if (*tuples==1) {
			GdaField * field;
			gda_recordset_move_first(record_set5);
			field = gda_recordset_field_idx(record_set5,0);
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				gulong cP;
				gchar ** stop = (gchar**)g_malloc0(sizeof(gchar*));
				gchar * cP_gchar = gda_stringify_value(NULL,0,field);
				cP = strtoul(cP_gchar,stop,10);
				g_free(cP_gchar);
				d_payment_set_payment_code (payment,cP);
			    }
			    gda_field_free(field);
			}
		    } else {
		    }
		    gda_recordset_close(record_set5);
		} else {
                    result = ULONG_MAX;
		}
		with_payment=TRUE;
	    } else {
		with_payment=FALSE;
	    }

	    invoice_number = d_charge_invoice_get_invoice_number (charge_invoice);
	    if  (result == 0) {
		result_gchar = d_charge_invoice_get_invoice_number (charge_invoice);
	    }
	    g_free(values);
	    values = g_strconcat(" ",NULL);
	    if (with_vat!=FALSE) {
		query2 = g_strconcat("INSERT INTO FACTURACARGO VALUES (",NULL);
	    } else {
		query2 = g_strconcat("INSERT INTO NOTACARGO VALUES (",NULL);
	    }

	    charge_invoice_temp = g_strdup_printf("%lu",supplier_code);
	    values_aux = g_strconcat(values,charge_invoice_temp,",",NULL);
	    g_free(values);
	    g_free(charge_invoice_temp);
	    values = values_aux;

	    charge_invoice_temp = g_strdup_printf("%lu",enterprise_code);
	    values_aux = g_strconcat(values,charge_invoice_temp,",",NULL);
	    g_free(values);
	    g_free(charge_invoice_temp);
	    values = values_aux;

	    charge_invoice_temp = g_strdup_printf("%lu",shop_code);
	    values_aux = g_strconcat(values,charge_invoice_temp,",",NULL);
	    g_free(values);
	    g_free(charge_invoice_temp);
	    values = values_aux;

	    values_aux = g_strconcat(values,"'",invoice_number,"',",NULL);
	    g_free(values);
	    values = values_aux;

	    if (with_payment==TRUE) {
                gchar * temp_currval = m_model_currval_sequence ("seqPago",connection);
		values_aux = g_strconcat(values,temp_currval,",'",ctime(&date),"',",NULL);
		g_free(values);
		values = values_aux;
		g_free(temp_currval);
	    } else {
		values_aux = g_strconcat(values,"NULL",",'",ctime(&date),"',",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (section!=NULL) {
		values_aux = g_strconcat(values,"'",section,"',",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (total != G_MINDOUBLE) {
		charge_invoice_temp = g_strdup_printf("%10.2f",total);
		g_strstrip(charge_invoice_temp);
		values_aux = g_strconcat(values,"to_number('",charge_invoice_temp,"','99999999D99')",NULL);
		g_free(values);
		g_free(charge_invoice_temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (with_vat!=FALSE) {
		charge_invoice_temp = g_strdup_printf("%d",state);
		values_aux = g_strconcat(values,",",charge_invoice_temp,NULL);
		g_free(values);
		g_free(charge_invoice_temp);
		values = values_aux;
	    }

	    if (with_vat!=FALSE) {
		charge_invoice_temp = m_model_nextval_sequence ("seqFacturaCargo",connection);
		values_aux = g_strconcat(values,",NULL,",charge_invoice_temp,NULL);
	    } else {
		charge_invoice_temp = m_model_nextval_sequence ("seqNotaCargo",connection);
		values_aux = g_strconcat(values,",",charge_invoice_temp,NULL);
	    }
	    g_free(values);
	    g_free(charge_invoice_temp);
	    values = values_aux;

	    query_aux=g_strconcat(query2,values,")",NULL);
	    g_free(query2);
	    query2=query_aux;
	    g_free(values);
	    query_aux=NULL;


	    if (m_model_command_execute (&record_set2, query2, connection, command, tuples, 0, FALSE)==FALSE) {
		result = ULONG_MAX;
	    }



	    if (query1!=NULL) {
		query = g_strconcat (query1,";\n",query2,";\n",query4,";\n",query5,";\n",NULL);
		g_free(query1);
	    } else {
		query = g_strconcat (query2,";\n",query4,";\n",NULL);
	    }

	    g_free(query2);
	    g_free(query);
	    g_free(tuples);
	    g_free(invoice_number);



	    if (record_set1!=NULL)
		gda_recordset_free(record_set1);
	    if (record_set2!=NULL)
		gda_recordset_free(record_set2);
	    if (record_set3!=NULL)
		gda_recordset_free(record_set3);

	}
    } else {
	result = ULONG_MAX-1;
        result_gchar = g_strdup("");
    }

    if (with_vat!=FALSE)
	invoice_code_char = m_model_currval_sequence ("seqFacturaCargo",connection);
    else
	invoice_code_char = m_model_currval_sequence ("seqNotaCargo",connection);

    invoice_code_gulong = strtoul(invoice_code_char,stop,10);

    if (result<ULONG_MAX-1)
	d_charge_invoice_set_code (charge_invoice,invoice_code_gulong);

    if (with_vat!=FALSE) {
	if (result<ULONG_MAX-1) {
	    m_charge_invoice_line_associate_charge_invoice (charge_lines, invoice_code_gulong, supplier_code);
     
	}
	if (!insert_charge_lines(charge_lines, command, connection)) {
	    result = ULONG_MAX;
	    result_gchar = g_strdup("");
	}
    }

    if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	result = ULONG_MAX;
    }

    if (result == ULONG_MAX) {
	if (result_gchar!=NULL) {
	    g_free(result_gchar);
	}
	result_gchar = NULL;
    }
    igalia_command_free(command);

    return result_gchar;
}

/** 
 * m_charge_invoice_update_state:
 * @charge_invoice: Factura de cargo en cuestin.
 * @state: Nuevo estado de la factura de cargo.
 * 
 * Modifica el estado de la factura de cargo.
 *
 * Returns: %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_charge_invoice_update_state (ChargeInvoice * charge_invoice,gint state) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * invoice_number=NULL;
    gchar * supplier_code=NULL;
    gchar * state_char = NULL;
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gboolean with_vat;


    with_vat =d_charge_invoice_get_vat (charge_invoice);

    if ( with_vat != FALSE ) {
	if (charge_invoice !=NULL) {
	    invoice_number = g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoice));
	    supplier_code = g_strdup_printf("%lu",d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice))));

	    if (invoice_number!=NULL) {
	      gulong codigo_participant_shop=ULONG_MAX;
	      gulong codigo_participant_enterprise=ULONG_MAX;
	      if (d_charge_invoice_get_enterprise (charge_invoice)!=NULL)
		codigo_participant_enterprise = d_participant_get_code (d_enterprise_get_participant (d_charge_invoice_get_enterprise (charge_invoice)));
	      if (d_charge_invoice_get_shop (charge_invoice)!=NULL)
		codigo_participant_shop = d_participant_get_code (d_shop_get_participant (d_charge_invoice_get_shop (charge_invoice)));
	      
	      gda_command_set_connection(command,connection);
	      m_model_connection_begin_mutex_transaction_logged (connection,
							      codigo_participant_enterprise,
							      codigo_participant_shop,
							      ULONG_MAX);
	      
	      state_char = g_strdup_printf("%d",state);
	      query = g_strconcat("update facturacargo set estado=",state_char,
				  " where (codigoFactura=",invoice_number,
				  ") and (codigoProveedor=",supplier_code,
				  ")",NULL);
	      if (m_model_command_execute (&record_set, query, connection, command, tuples, 0, FALSE)==FALSE) {
		result = FALSE;
	      }	      
	      
	      if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
		result = FALSE;
	      }
	      
	      g_free(invoice_number);
	      
	    }
	    if (record_set!=NULL)
		gda_recordset_free(record_set);

	    g_free(query);
	    g_free(tuples);
	    g_free(supplier_code);
	    g_free(state_char);

	}
    } else {
        result = FALSE;
    }
    igalia_command_free(command);

    return result;
}

/** 
 * m_charge_invoice_find_by_condition:
 * @condition: Estructura que permite elegir una serie de filtros por las que buscar las facturas.
 * 
 * Realiza la bsqueda de facturas a partir de unas condiciones. Dichas condiciones pueden ser:
 * tipo de factura, cdigo de la tienda, cdigo de proveedor, numero de la factura, estado de la
 * factura, perodo de fecha de vencimiento, tipo de pago con la que se paga la factura, catidad
 * total de la factura. Estas condiciones pueden ser todas o solo
 * un grupo de ellas.
 *
 * Returns: Devuelve un array de facturas de cargo que cumplen los filtros de bsqueda.
 */
ChargeInvoice ** m_charge_invoice_find_by_condition (ConditionFindChargeInvoice * condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_order=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set1 = NULL;
    gulong index_of_charge_invoices=0;
    gboolean with_vat;

    ChargeInvoice ** charge_invoices=NULL;
    ChargeInvoice * charge_invoice;
    gint index;
    gulong * tuples1;

    my_command = gda_command_new();
    tuples1 = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);
    with_vat =condition->vat;

    if (with_vat!=FALSE) {
	gchar_select = g_strconcat("(SELECT ","facturacargo.numeroFactura as numeroFacturaFacturaCargo,",
				   "facturacargo.codigoFactura as codigoFacturaFacturaCargo,",
				   "facturacargo.codigoEmpresa as codigoEmpresaFacturaCargo,",
				   "facturacargo.codigoTienda as codigoTiendaFacturaCargo,",
				   "facturacargo.estado as estadoFacturaCargo,",
				   "facturacargo.concepto as conceptoFacturaCargo,",
				   "to_char(facturacargo.total,'99999999D99') as totalFacturaCargo,",
				   "participanteShop.codigoParty as codigoPartyParticipantShop,",
				   "participanteShop.nombre as nombreParticipantShop,",
				   "participanteProveedor.codigoParty as codigoPartyParticipantSupplier,",
				   "participanteProveedor.nick as nickParticipantSupplier,",
				   "facturacargo.codigoEmpresa as codigoPartyParticipantEnter,",
				   "pago.codigoPago as codigoPagoPago, ",
				   "pago.fechaInicio as fechaInicioPago,",
				   "pago.pagosRealizados as pagosRealizadosPago,",
				   "pago.esTransferencia as esTransferenciaPago,",
				   "pago.esTalon as esTalonPago,",
				   "pago.esDomiciliacion as esDomiciliacionPago,",
				   "pago.esTarjeta as esTarjetaPago",
				   NULL );
	gchar_from = g_strconcat(" from ",
				 " participante participanteProveedor,",
				 " participante participanteShop,",
				 " facturacargo left outer join pago on",
                                 " (facturacargo.codigoPago=pago.codigoPago) ",
				 NULL);

	gchar_where=g_strconcat(" where ",
                                " facturacargo.codigoTienda=participanteShop.codigoParty",
				" and facturacargo.codigoProveedor = participanteProveedor.codigoParty ",
				NULL);
	gchar_order=g_strconcat(" order by facturacargo.fecha) ",NULL);
	if (((condition->shop_code) != ULONG_MAX) && ((condition->shop_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->shop_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.codigoTienda=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if (((condition->supplier_code) != ULONG_MAX) && ((condition->supplier_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->supplier_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.codigoProveedor=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if (((condition->invoice_number) != NULL) && (strlen(condition->invoice_number) != 0))  {
	    gchar * temp = g_strdup(condition->invoice_number);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.numeroFactura='",
					  temp,"')",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;

	}

	if ((condition->invoice_state) >= 0)  {
	    gchar * temp = g_strdup_printf("%d",condition->invoice_state);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.estado=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->initial_expire_date)!= NULL)  {
	    gchar * temp = ctime(condition->initial_expire_date);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.fecha>='",
					  temp,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->ending_expire_date)!= NULL)  {
	    gchar * temp = ctime(condition->ending_expire_date);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.fecha<='",
					  temp,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->amount) > 0)  {
            gdouble total_real;
	    gchar * temp;

	    if ((condition->vat)==FALSE) {
                total_real = (condition->amount / 16.0);
	    } else {
                total_real = condition->amount;
	    }
            temp = g_strdup_printf("%10.2f",total_real);
            g_strstrip(temp);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.total=to_number('",temp,"','999999D99')",
					  ")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->payment_type)!= ULONG_MAX)  {
	    gchar * conditionPayment;
	    if (condition->payment_type==D_PAYMENT_TRANSFER_CODE) {
                conditionPayment = g_strconcat(" and (pago.esTransferencia=true) ",NULL);
	    } else if (condition->payment_type==D_PAYMENT_IN_CASH) {
		conditionPayment = g_strconcat(" and (pago.esTransferencia=false)",
					       " and (pago.esDomiciliacion=false)",
					       " and (pago.esTarjeta=false)",
					       " and (pago.esTalon=false) ",
					       NULL);
	    } else if (condition->payment_type==D_PAYMENT_DIRECT_DEBIT_CODE) {
                conditionPayment = g_strconcat(" and (pago.esDomiciliacion=true)",NULL);
	    } else if (condition->payment_type==D_PAYMENT_CREDIT_CARD) {
                conditionPayment = g_strconcat(" and (pago.esTarjeta=true)",NULL);
	    } else if (condition->payment_type==D_PAYMENT_CHEQUE) {
                conditionPayment = g_strconcat(" and (pago.esTalon=true)",NULL);
	    } else {
                conditionPayment = g_strconcat(" ",NULL);
	    }
	    gchar_where_aux = g_strconcat(gchar_where,conditionPayment,NULL);
            g_free(conditionPayment);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

    } else {
	gchar_select = g_strconcat("(SELECT ","notacargo.codigoNotaCargo as numeroFacturaFacturaCargo,",
				   "notacargo.codigoNota as codigoFacturaFacturaCargo,",
				   "notacargo.codigoEmpresa as codigoEmpresaFacturaCargo,",
				   "notacargo.codigoTienda as codigoTiendaFacturaCargo,",
				   "notacargo.concepto as conceptoFacturaCargo,",
				   "to_char(notacargo.total,'99999999D99') as totalFacturaCargo,",
				   "participanteShop.codigoParty as codigoPartyParticipantShop,",
				   "participanteShop.nombre as nombreParticipantShop,",
				   "participanteProveedor.codigoParty as codigoPartyParticipantSupplier,",
				   "participanteProveedor.nick as nickParticipantSupplier,",
				   "notacargo.codigoEmpresa as codigoPartyParticipantEnter,",
				   "pago.codigoPago as codigoPagoPago, ",
				   "pago.fechaInicio as fechaInicioPago,",
				   "pago.pagosRealizados as pagosRealizadosPago,",
				   "pago.esTransferencia as esTransferenciaPago,",
				   "pago.esTalon as esTalonPago,",
				   "pago.esDomiciliacion as esDomiciliacionPago,",
				   "pago.esTarjeta as esTarjetaPago",
				   NULL );
	gchar_from = g_strconcat(" from ",
				 " participante participanteProveedor,",
				 " participante participanteShop,",
				 " notacargo left outer join pago on",
                                 " (notacargo.codigoPago=pago.codigoPago) ",
				 NULL);

	gchar_where=g_strconcat(" where ",
                                " notacargo.codigoTienda=participanteShop.codigoParty",
				" and notacargo.codigoProveedor = participanteProveedor.codigoParty ",
				NULL);

	gchar_order=g_strconcat(" order by notacargo.fecha) ",NULL);
	if (((condition->shop_code) != ULONG_MAX) && ((condition->shop_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->shop_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (notacargo.codigoTienda=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if (((condition->supplier_code) != ULONG_MAX) && ((condition->supplier_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->supplier_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (notacargo.codigoProveedor=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if (((condition->invoice_number) != NULL) && (strlen(condition->invoice_number) != 0))  {
	    gchar * temp = g_strdup(condition->invoice_number);
	    gchar_where_aux = g_strconcat(gchar_where, " and (notacargo.codigoNotaCargo='",
					  temp,"')",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;

	}

	if ((condition->initial_expire_date)!= NULL)  {
	    gchar * temp = ctime(condition->initial_expire_date);
	    gchar_where_aux = g_strconcat(gchar_where, " and (pago.fechaInicio>='",
					  temp,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->ending_expire_date)!= NULL)  {
	    gchar * temp = ctime(condition->ending_expire_date);
	    gchar_where_aux = g_strconcat(gchar_where, " and (pago.fechaInicio<='",
					  temp,"')",NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->amount) > 0)  {
            gdouble total_real;
	    gchar * temp;

	    if ((condition->vat)==FALSE) {
                total_real = (condition->amount / 16.0);
	    } else {
                total_real = condition->amount;
	    }
            temp = g_strdup_printf("%10.2f",total_real);
            g_strstrip(temp);
	    gchar_where_aux = g_strconcat(gchar_where, " and (notacargo.total=to_number('",temp,"','999999D99')",
					  ")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	if ((condition->payment_type)!= ULONG_MAX)  {
	    gchar * conditionPayment;
	    if (condition->payment_type==D_PAYMENT_TRANSFER_CODE) {
                conditionPayment = g_strconcat(" and (pago.esTransferencia=true) ",NULL);
	    } else if (condition->payment_type==D_PAYMENT_IN_CASH) {
		conditionPayment = g_strconcat(" and (pago.esTransferencia=false)",
					       " and (pago.esDomiciliacion=false)",
					       " and (pago.esTarjeta=false)",
					       " and (pago.esTalon=false) ",
					       NULL);
	    } else if (condition->payment_type==D_PAYMENT_DIRECT_DEBIT_CODE) {
                conditionPayment = g_strconcat(" and (pago.esDomiciliacion=true)",NULL);
	    } else if (condition->payment_type==D_PAYMENT_CREDIT_CARD) {
                conditionPayment = g_strconcat(" and (pago.esTarjeta=true)",NULL);
	    } else if (condition->payment_type==D_PAYMENT_CHEQUE) {
                conditionPayment = g_strconcat(" and (pago.esTalon=true)",NULL);
	    } else {
                conditionPayment = g_strconcat(" ",NULL);
	    }
	    gchar_where_aux = g_strconcat(gchar_where,conditionPayment,NULL);
            g_free(conditionPayment);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}


    }





    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order,
			      NULL);



    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set1, gchar_query, my_connection, my_command,tuples1,0, TRUE);



    if (record_set1!=NULL) {
	if (*tuples1!=0) {
	    charge_invoices = (ChargeInvoice **) g_malloc0(sizeof(ChargeInvoice*)*(*tuples1+1));
	    index = gda_recordset_move_first(record_set1);
	    while (!gda_recordset_eof(record_set1)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		charge_invoice = m_charge_invoice_create (record_set1);
                d_charge_invoice_set_vat (charge_invoice,with_vat);
		charge_invoices[index_of_charge_invoices] = charge_invoice;
		index = gda_recordset_move_next(record_set1);
		index_of_charge_invoices ++;
	    }
	} else {
	    charge_invoices=NULL;
	}
	gda_recordset_close(record_set1);
    } else {
	charge_invoices=NULL;
    }

    if (charge_invoices!=NULL) {
	charge_invoices[index_of_charge_invoices] = NULL;
    }

    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_from);
    g_free(gchar_order);
    g_free(gchar_query);
    g_free(tuples1);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return charge_invoices;

}



/** 
 * m_charge_invoice_clear_array:
 * @charge_invoices:Array de facturas de cargo a liberar.
 *
 * Libera la memoria ocupada por un array de facturas de cargo.
 *
 */
void m_charge_invoice_clear_array (ChargeInvoice ** charge_invoices) {
    if (charge_invoices!=NULL) {
	gint index = 0;
	while (charge_invoices[index]!=NULL) {
	    d_charge_invoice_clear (charge_invoices[index]);
	    index++;
	}
    }
    g_free(charge_invoices);
    charge_invoices=NULL;
}

/** 
 * m_charge_invoice_find_by_code:
 * @charge_invoice_id: Identificador de la factura. %ChargeInvoiceId es una estructura que contiene los
 * campos que identifican la factura.
 * @vat: Permite elegir si se buscan notas o facturas de cargo.
 *
 * Obtiene una factura de cargo de la base de datos a partir del
 * identificador de la misma.
 *
 * Returns: La factura de cargo en cuestin.
 */
ChargeInvoice * m_charge_invoice_find_by_code (ChargeInvoiceId * charge_invoice_id,gboolean vat) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar * gchar_from;
    gchar * gchar_where;
    gchar * gchar_where_aux=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    GdaField * field;

    ChargeInvoice * charge_invoice=NULL;
    gint index;
    gulong shop_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong payment_code=ULONG_MAX;
    gulong * tuples;
    gchar * table = NULL;
    gchar * state_char=NULL;
    gchar * code_char;
    gchar * previous_code_char;
    gint error=0;

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    if (vat!=FALSE) {
	table = g_strdup(" facturacargo");
        previous_code_char = "numeroFactura as numeroFacturaFacturaCargo,";
	code_char = g_strdup(" codigofactura");
        state_char = g_strdup(" estado as estadoFacturaCargo,");

    } else {
	table = g_strdup(" notacargo");
        previous_code_char = "codigoNota as numeroFacturaFacturaCargo,";
	code_char = g_strdup("codigoNota");
        state_char = g_strdup(" ");

    }

    gchar_select=g_strconcat("select ",previous_code_char, code_char ," as codigoFacturaFacturaCargo,",
			      table,".codigoTienda as codigoTiendaFacturaCargo,",
			      table,".codigoEmpresa as codigoEmpresaFacturaCargo,",
			     "codigoProveedor as codigoPartyParticipantSupplier,",
			     "fecha as fechaFacturaCargo,",
			     "codigoPago as codigoPagoPago,",
			     "codigoPago as codigoPagoFacturaCargo,",
			     "concepto as conceptoFacturaCargo,",
			     "to_char(total,'99999999D99') as totalFacturaCargo,",
                             state_char,
			     "participanteShop.codigoParty as codigoPartyParticipantShop,",
			     "participanteShop.cif as cifParticipantShop,",
			     "participanteShop.nick as nickParticipantShop,",
			     "participanteShop.nombre as nombreParticipantShop,",
			     "participanteShop.cif as cifParticipantShop,",
			     "participanteShop.codigoParty as codigoPartyParticipantShop,",
			     "participanteShop.email as emailParticipantShop,",
			     "participanteShop.telefono as telefonoParticipantShop,",
			     "participanteShop.telefono2 as telefono2ParticipantShop,",
			     "participanteShop.fax as faxParticipantShop,",
			     "participanteShop.calle as calleParticipantShop,",
			     "participanteShop.numero as numeroParticipantShop,",
			     "participanteShop.piso as pisoParticipantShop,",
			     "participanteShop.cp as cpParticipantShop,",
			     "participanteShop.municipio as municipioParticipantShop,",
			     "participanteShop.provincia as provinciaParticipantShop,",
			     "participanteSupplier.nick as nickParticipantSupplier,",
			     "participanteSupplier.nombre as nombreParticipantSupplier,",
			     "participanteSupplier.cif as cifParticipantSupplier,",
			     "participanteSupplier.codigoParty as codigoPartyParticipantSupplier,",
			     "participanteSupplier.email as emailParticipantSupplier,",
			     "participanteSupplier.telefono as telefonoParticipantSupplier,",
			     "participanteSupplier.telefono2 as telefono2ParticipantSupplier,",
			     "participanteSupplier.fax as faxParticipantSupplier,",
			     "participanteSupplier.calle as calleParticipantSupplier,",
			     "participanteSupplier.numero as numeroParticipantSupplier,",
			     "participanteSupplier.piso as pisoParticipantSupplier,",
			     "participanteSupplier.cp as cpParticipantSupplier,",
			     "participanteSupplier.municipio as municipioParticipantSupplier,",
			     "participanteSupplier.provincia as provinciaParticipantSupplier,",
			     "participanteEmpresa.nombre as nombreParticipantEnter,",
			     "participanteEmpresa.cif as cifParticipantEnter,",
			     "participanteEmpresa.codigoParty as codigoPartyParticipantEnter,",
			     "participanteEmpresa.email as emailParticipantEnter,",
			     "participanteEmpresa.telefono as telefonoParticipantEnter,",
			     "participanteEmpresa.telefono2 as telefono2ParticipantEnter,",
			     "participanteEmpresa.fax as faxParticipantEnter,",
			     "participanteEmpresa.calle as calleParticipantEnter,",
			     "participanteEmpresa.numero as numeroParticipantEnter,",
			     "participanteEmpresa.piso as pisoParticipantEnter,",
			     "participanteEmpresa.cp as cpParticipantEnter,",
			     "participanteEmpresa.municipio as municipioParticipantEnter,",
			     "participanteEmpresa.provincia as provinciaParticipantEnter,",
			     "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop",
			     NULL);
    gchar_from = g_strconcat(" from ",table,",participante participanteShop,",
			     " participante participanteEmpresa,"
			     " participante participanteSupplier,tienda",NULL);
    gchar_where = g_strconcat(" where ",
			      table,".codigoTienda=participanteShop.codigoParty ",
			      "and",table ,".codigoEmpresa=participanteEmpresa.codigoParty ",
			      "and",table ,".codigoProveedor=participanteSupplier.codigoParty ",
			      "and",table ,".codigoTienda=tienda.codigoTienda ",
			      NULL);

    if (((charge_invoice_id->invoice_number) != NULL) && (strlen(charge_invoice_id->invoice_number)!=0))  {
	gchar_where_aux = g_strconcat(gchar_where, " and (",table,".",code_char,"='",
				      charge_invoice_id->invoice_number,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if (((charge_invoice_id->supplier_code) != NULL) && (strlen(charge_invoice_id->supplier_code)!=0))  {
	gchar_where_aux = g_strconcat(gchar_where, " and (",table,".codigoProveedor=",
				      charge_invoice_id->supplier_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);
    g_print("Query: %s\n",gchar_query);
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
	    index = gda_recordset_move_first(record_set);
	    field = gda_recordset_field_name(record_set,"codigoTiendaFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    shop_code=igalia_field_get_bigint_value (field);
		} else {
		    shop_code=ULONG_MAX;
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(record_set,"codigoEmpresaFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    enterprise_code=igalia_field_get_bigint_value (field);
		} else {
		    enterprise_code=ULONG_MAX;
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(record_set,"codigoPagoFacturaCargo");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment_code=igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
	    charge_invoice = m_charge_invoice_create (record_set);
	} else {
	    error =1;
	}
	gda_recordset_close(record_set);
	record_set=NULL;
    }
    g_free(gchar_query);
    g_free(gchar_select);
    g_free(gchar_where);
    g_free(gchar_from);


    if (error!=1) {
	PaymentId * payment_id = (PaymentId *) g_malloc0(sizeof(PaymentId));
	//	ChargeDeliveryPayment ** charge_delivery_payments;
	//	ChargeDeliveryPayment * charge_delivery_payment;
	payment_id->payment_code=g_strdup_printf("%lu",payment_code);
	payment_id->enterprise_code=g_strdup_printf("%lu",enterprise_code);
	payment_id->shop_code=g_strdup_printf("%lu",shop_code);
	d_charge_invoice_set_payment (charge_invoice,m_payment_find_by_id (payment_id));


 /*       gchar_query = g_strconcat("select numeroAlbaran,enterprise_code,shop_code,supplier_code from albarancargo where albarancargo.invoice_number=",
				  charge_invoice_id->invoice_number,NULL);
	g_print("Query: %s\n",gchar_query);
	record_set =gda_command_execute(my_command,tuples,0);
	if (record_set!=NULL) {
	    if (*tuples!=0) {
		Order** order_lines;
                Order * order;
		charge_delivery_payments = (ChargeDeliveryPayment **) g_malloc0(sizeof(ChargeDeliveryPayment *)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
			OrderId * orderId=(OrderId *) g_malloc0(sizeof(OrderId));
                        orderId->enterprise_code=NULL;
                        orderId->order_code=NULL;
                        orderId->warehouse_code=NULL;
			if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
			field = gda_recordset_field_name(record_set,"codigoEmpresa");
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				orderId->enterprise_code=g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
			    }
			    gda_field_free(field);
			}
			field = gda_recordset_field_name(record_set,"codigoAlmacen");
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				    orderId->warehouse_code=g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
			    }
			    gda_field_free(field);
			}
			field = gda_recordset_field_name(record_set,"codigoPedido");
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				    orderId->order_code=g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
			    }
			    gda_field_free(field);
			}
			order_lines = model_find_order_lines_by_order_id(orderId);

			order = model_find_order_object_by_id(orderId,order_lines);

			charge_delivery_payment = model_find_charge_delivery_payment_by_order(orderId);

			if (order!=NULL) {
			    d_order_clear (get_order_detail_charge_delivery_payment(charge_delivery_payment));
			    set_order_detail_charge_delivery_payment(charge_delivery_payment,order);
			}

			charge_delivery_payments[index-1] = charge_delivery_payment;
			order_lines =NULL;
			order=NULL;
                        charge_delivery_payment=NULL;
			index = gda_recordset_move_next(record_set);
			if (orderId!=NULL) {
			    if (orderId->enterprise_code!=NULL) g_free(orderId->enterprise_code);
			    if (orderId->warehouse_code!=NULL) g_free(orderId->warehouse_code);
			    if (orderId->order_code!=NULL) g_free(orderId->order_code);
			    g_free(orderId);
			}
		    }
		    charge_delivery_payments[*tuples+1]=NULL;
		} else {
		    charge_delivery_payments=NULL;
		}
		gda_recordset_close(record_set);
	    } else {
		charge_delivery_payments=NULL;
	    }
	    if (get_charge_delivery_payments_charge_invoice(charge_invoice)!=NULL)
		model_charge_delivery_payment_array_clear(get_charge_delivery_payments_charge_invoice(charge_invoice));
            set_charge_delivery_payments_charge_invoice(charge_invoice,charge_delivery_payments);
	    }
            */
    } else {
	charge_invoice =NULL;
    }

    g_free(tuples);
    return charge_invoice;
    return NULL;
}

/** 
 * m_charge_invoice_update:
 * @charge_invoice: Factura de cargo a modificar.
 * @note: Booleano que indica si es una nota o factura.
 *
 * Modifica la factura de cargo.
 *
 * Returns: %TRUE si se ha modificado, %FALSE en caso contrario.
 */
gboolean m_charge_invoice_update (ChargeInvoice * charge_invoice,gboolean note) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * invoice_number=NULL;
    gchar * supplier_code=NULL;
    gchar * section=NULL;
    gchar * total=NULL;
    gchar * date=NULL;
    gchar * enterprise_code=NULL;
    gchar * shop_code=NULL;
    GdaRecordset *record_set=NULL;
    time_t date_aux;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    if (charge_invoice !=NULL) {
	invoice_number = g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoice));
	supplier_code = g_strdup_printf("%lu",d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice))));

	enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_charge_invoice_get_enterprise (charge_invoice))));
	shop_code = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_charge_invoice_get_shop (charge_invoice))));

	total=g_strdup_printf("%.2f",d_charge_invoice_get_total (charge_invoice));

	u_commas_to_points (total);

	date_aux = d_charge_invoice_get_date (charge_invoice);
	date = g_strdup(ctime(&date_aux));
	section = d_charge_invoice_get_section (charge_invoice);
	if (section==NULL)
            section = g_strdup("");

	if (invoice_number!=NULL) {
	  ChargeInvoiceLine ** charge_lines = d_charge_invoice_get_charge_invoice_lines (charge_invoice);
	  gda_command_set_connection(command,connection);
	  m_model_connection_begin_mutex_transaction_logged (connection,
							  d_participant_get_code (d_enterprise_get_participant (d_charge_invoice_get_enterprise (charge_invoice))),
							  d_participant_get_code (d_shop_get_participant (d_charge_invoice_get_shop (charge_invoice))),
							  ULONG_MAX
							  );
	  if (note==FALSE) {
	    query = g_strconcat("update facturacargo set ",
				"codigoEmpresa=",enterprise_code,
				",codigoTienda=",shop_code,
				",total=",total,
				",fecha='",date,"'"
				",concepto='",section,
				"' where (codigoFactura='",invoice_number,
				"') and (codigoProveedor=",supplier_code,
				")",NULL);
	    } else {
		query = g_strconcat("update notacargo set ",
				    "codigoEmpresa=",enterprise_code,
				    ",codigoTienda=",shop_code,
				    ",total=",total,
				    ",fecha='",date,"'"
				    ",concepto='",section,
				    "' where (codigoNota='",invoice_number,
				    "') and (codigoProveedor=",supplier_code,
				    ")",NULL);

	    }
	    result = m_model_command_execute (&record_set, query, connection,
					   command,tuples,0, FALSE);


	    result = result && insert_charge_lines(charge_lines, command, connection);

	    if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
		result = TRUE;
	    }


	}
	if (record_set!=NULL)
	    gda_recordset_free(record_set);

	g_print(query);

	g_free(query);
	g_free(tuples);
	g_free(shop_code);
	g_free(total);
        g_free(section);
	g_free(enterprise_code);
	g_free(supplier_code);
	g_free(invoice_number);
        g_free(date);
    }
	igalia_command_free(command);

    return result;


}

/** 
 * m_charge_invoice_delete_deprecated:
 * @charge_invoice: Factura de cargo a borrar.
 * @note: Booleano que indica si es una nota o factura.
 *
 * Borra una factura de cargo de la base de datos.
 *
 * Returns: %TRUE si se ha modificado, %FALSE en caso contrario.
 */
/*gboolean m_charge_invoice_delete_deprecated (ChargeInvoice * charge_invoice,gboolean note) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * invoice_number=NULL;
    gchar * supplier_code=NULL;
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    if (charge_invoice !=NULL) {
	invoice_number = g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoice));
	supplier_code = g_strdup_printf("%lu",d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice))));


	if (invoice_number!=NULL) {
	    gda_command_set_connection(command,connection);
	    m_model_connection_begin_mutex_transaction_logged (connection,
						     d_participant_get_code (d_enterprise_get_participant (d_charge_invoice_get_enterprise (charge_invoice))),
						     d_participant_get_code (d_shop_get_participant (d_charge_invoice_get_shop (charge_invoice))),
						     ULONG_MAX);

	    if (note==FALSE) {
		query = g_strconcat("delete from facturacargo ",
				    " where (codigoFactura='",invoice_number,
				    "') and (codigoProveedor=",supplier_code,
				    ")",NULL);
	    } else {
		query = g_strconcat("delete from notacargo ",
				    " where (codigoNota='",invoice_number,
				    "') and (codigoProveedor=",supplier_code,
				    ")",NULL);

	    }
	    result=m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);


	    if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
		result = TRUE;
	    }


	}
	if (record_set!=NULL)
	    gda_recordset_free(record_set);

	g_free(query);
	g_free(tuples);
	g_free(supplier_code);
	g_free(invoice_number);
    }
	igalia_command_free(command);

    return result;


} */


/** 
 * insert_charge_lines:
 * @charge_lines: Lneas de factura de cargo a insertar.
 * @command: Comando en la que se ejecutarn las sentencias sql.
 * @connection: Conexin a la base de datos.
 *
 * Inserta las lneas de factura de cargo en la base de datos, dentro de una transaccin.
 *
 * Returns: %TRUE si se han insertado, %FALSE en caso contrario.
 */
static gboolean insert_charge_lines(ChargeInvoiceLine **charge_lines, GdaCommand * command, GdaConnection *connection) {
  
  gboolean result = TRUE;

  if (charge_lines != NULL){
    gint index = 0;
    while (charge_lines[index]!=NULL) {
      gulong invoice_number = d_charge_invoice_line_get_invoice_number (charge_lines[index]);
      gulong invoice_line_number_gulong = d_charge_invoice_line_get_line_number (charge_lines[index]);
      gchar *invoice_line_number = NULL;    
      gulong supplier_code_gulong = d_charge_invoice_line_get_supplier_code (charge_lines[index]);
      gchar *supplier_code = NULL;
      
      if (invoice_line_number_gulong != ULONG_MAX)
	invoice_line_number = g_strdup_printf("%lu",invoice_line_number_gulong);
      else invoice_line_number = g_strdup("NULL");
      
      if (supplier_code_gulong != ULONG_MAX)
	supplier_code = g_strdup_printf("%lu",supplier_code_gulong);
      else supplier_code = g_strdup("NULL");
      
      if (m_charge_invoice_line_is_new (invoice_number, supplier_code, invoice_line_number))
	result = m_charge_invoice_line_insert (charge_lines[index],command,connection,FALSE);
      else
	result = m_charge_invoice_line_insert (charge_lines[index],command,connection,TRUE);
      
      index ++;

      g_free(invoice_line_number);
      g_free(supplier_code);
    }
  }    
  
    return result;
}


/** 
 * m_charge_invoice_find_last_from_supplier_by_enterprise:
 * @supplier_code: Cdigo de proveedor del que se buscar el pago.
 * @enterprise_code: Cdigo de empresa que realiz el pago.
 *
 * Busca el ltimo pago realizado al
 * proveedor por la empresa identificada por %enterprise_code.
 *
 * Returns: Identificador del pago.
 */
PaymentId * m_charge_invoice_find_last_from_supplier_by_enterprise (gulong supplier_code,gulong enterprise_code) {
    gchar * query;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    gchar * supplier_code_char=NULL;
    gchar * enterprise_code_char=NULL;
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    PaymentId * result=NULL;


    gda_command_set_connection(command,connection);

    if (supplier_code !=ULONG_MAX) {
        supplier_code_char = g_strdup_printf("%lu",supplier_code);
        enterprise_code_char = g_strdup_printf("%lu",enterprise_code);

	query = g_strconcat(" select ",
			    " facturacargo.codigoEmpresa as empresa, ",
			    " facturacargo.codigoTienda as tienda, ",
			    " facturacargo.codigoFactura as factura,",
			    " facturacargo.codigoPago as pago,fecha from facturacargo,pago",
			    " where ",
			    " facturacargo.codigoPago = pago.codigoPAgo and ",
			    " facturacargo.codigoTienda=pago.codigoTienda ",
			    " and facturacargo.codigoEmpresa=pago.codigoEmpresa and ",
			    " facturacargo.codigoProveedor=",
                            supplier_code_char,
			    " and facturaCargo.codigoEmpresa=",enterprise_code_char,
			    " order by fecha desc,codigoFactura desc ",
			    NULL);


	g_free(supplier_code_char);
	g_free(enterprise_code_char);
	g_print(_("\nQuery: %s\n"),query);
	if (m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE)==FALSE) {
	    result = NULL;
	}

        g_free(query);
	if (record_set!=NULL) {
	    if (*tuples>=1) {
		GdaField * field;
		gda_recordset_move_first(record_set);
//		gda_recordset_move_next(record_set);
		result = (PaymentId *) g_malloc0(sizeof(PaymentId));
		field  = gda_recordset_field_name(record_set,"tienda");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> shop_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field  = gda_recordset_field_name(record_set,"empresa");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> enterprise_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field  = gda_recordset_field_name(record_set,"pago");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> payment_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}

	    } else {
		result = NULL;
	    }
	} else {
	    result = NULL;
	}

    }
    if (record_set!=NULL)
	gda_recordset_free(record_set);
    g_free(tuples);

    igalia_command_free(command);

    return result;

}

/** 
 * m_charge_invoice_find_last_from_supplier:
 * @supplier_code: Cdigo de proveedor del que se buscar el pago.
 *
 * Busca el ltimo pago realizado al
 * proveedor.
 *
 * Returns: Identificador del pago.
 */
PaymentId * m_charge_invoice_find_last_from_supplier (gulong supplier_code) {
    gchar * query;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    gchar * supplier_code_char=NULL;
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    PaymentId * result=NULL;


    gda_command_set_connection(command,connection);

    if (supplier_code !=ULONG_MAX) {
        supplier_code_char = g_strdup_printf("%lu",supplier_code);

	query = g_strconcat(" select ",
			    " facturacargo.codigoEmpresa as empresa, ",
			    " facturacargo.codigoTienda as tienda, ",
			    " facturacargo.codigoFactura as factura,",
			    " facturacargo.codigoPago as pago,fecha from facturacargo,pago",
			    " where ",
			    " facturacargo.codigoPago = pago.codigoPAgo and ",
			    " facturacargo.codigoTienda=pago.codigoTienda ",
			    " and facturacargo.codigoEmpresa=pago.codigoEmpresa and ",
			    " facturacargo.codigoProveedor=",
			    supplier_code_char,
			    " order by fecha desc,codigoFactura desc ",
			    NULL);


	g_free(supplier_code_char);
	g_print(_("\nQuery: %s\n"),query);
	if (m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE)==FALSE) {
	    result = NULL;
	}

        g_free(query);
	if (record_set!=NULL) {
	    if (*tuples>=1) {
		GdaField * field;
		gda_recordset_move_first(record_set);
//		gda_recordset_move_next(record_set);
		result = (PaymentId *) g_malloc0(sizeof(PaymentId));
		field  = gda_recordset_field_name(record_set,"tienda");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> shop_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field  = gda_recordset_field_name(record_set,"empresa");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> enterprise_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field  = gda_recordset_field_name(record_set,"pago");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> payment_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}

	    } else {
		result = NULL;
	    }
	} else {
	    result = NULL;
	}

    }
    if (record_set!=NULL)
	gda_recordset_free(record_set);
    g_free(tuples);

    igalia_command_free(command);

    return result;

}

/**
 * m_charge_invoice_delete:
 * @charge_invoice: Factura a borrar.
 *
 * Borra una factura de pago en la base de datos.
 *
 * Returns:
 * %NULL si se ha borrado correctamente, un mensaje de error en otro caso.
 */
gchar *m_charge_invoice_delete (ChargeInvoice * charge_invoice) {
    gchar * result = NULL;
    gchar * query;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong invoice_code = d_charge_invoice_get_code (charge_invoice);
    gulong supplier_code = d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice)));
    gulong enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_charge_invoice_get_enterprise (charge_invoice)));
    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_charge_invoice_get_shop (charge_invoice)));
    gulong payment_code = d_payment_get_payment_code (d_charge_invoice_get_payment (charge_invoice));
    gchar * payment_code_char = NULL; 
    gchar * invoice_code_char = NULL;
    gchar * shop_code_char = NULL;
    gchar * enterprise_code_char = NULL;
    gchar * supplier_code_char = NULL;
    gint invoice_state=D_CHARGE_INVOICE_PAID;
    gboolean without_error=TRUE;

    gda_command_set_connection(command,connection);

    invoice_code_char = g_strdup_printf("%lu",invoice_code);
    enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    shop_code_char = g_strdup_printf("%lu",shop_code);
    payment_code_char = g_strdup_printf("%lu",payment_code);
    supplier_code_char = g_strdup_printf("%lu",supplier_code);

    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);

    query = g_strconcat(" select estado from facturacargo ",
			" WHERE CODIGOFACTURA=",invoice_code_char,
			" AND CODIGOEMPRESA=",enterprise_code_char,
			" AND CODIGOTIENDA=",shop_code_char,NULL);
    without_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE);
    g_print(_("\nInvoice Delete. State : %s\n"),query);

    if (without_error!=FALSE) {
	if (record_set!=NULL) {
	    if (GDA_IS_RECORDSET(record_set)) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set);
		    field  = gda_recordset_field_name(record_set,"estado");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    invoice_state= gda_field_get_smallint_value(field);;
			}
		    }
		}
		gda_recordset_free(record_set);
		record_set=NULL;
	    }
	}
    }
    if (query!=NULL)
	g_free(query);
    query=NULL;

    if (invoice_state>=D_CHARGE_INVOICE_PAID) {
	result = g_strdup(_("Paid Invoice.\nPaid Invoices can't be deleted."));
    } 

    if (result==NULL) {


	if (result == NULL) {
	    query = g_strconcat(" SELECT * FROM LINEAPEDIDO ",
				" WHERE NUMEROFACTURACARGO=",invoice_code_char,
				" AND CODIGOEMPRESA=",enterprise_code_char,
				NULL);
	    without_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);

	    g_print(_("\nDeleting invoices. Checking Order Line: %s\n"),query);

	    if (query!=NULL)
		g_free(query);
	    query=NULL;

	    gda_recordset_free(record_set);
	    record_set=NULL;

	    if (*tuples!=0) {
		query = g_strconcat("UPDATE LINEAPEDIDO SET numeroFacturaCargo=NULL,numeroLineaCargo=NULL,codigoProveedorFacturaCargo=NULL",
				    " WHERE numerofacturaCargo = ", invoice_code_char,
				    " AND codigoproveedorFacturaCargo = ", supplier_code_char,
				    NULL);

		g_print(_("\nDeleting invoices. Modifing Order Line: %s\n"),query);

		if (query!=NULL)
		    g_free(query);
		query=NULL;

		gda_recordset_free(record_set);
		record_set=NULL;

	    }



	    if (result == NULL) {
		query = g_strconcat(" DELETE FROM PAGO ",
				    " WHERE CODIGOPAGO=",payment_code_char,
				    " AND CODIGOEMPRESA=",enterprise_code_char,
				    " AND CODIGOTIENDA=",shop_code_char,NULL);

		without_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);
		g_print(_("\nDeleting invoices. Invoice payment: %s\n"),query);

		if (query!=NULL)
		    g_free(query);
		query=NULL;

		gda_recordset_free(record_set);
		record_set=NULL;

	    }
	}
    }
    m_model_execute_mutex_modification_logged (connection);

    g_free(tuples);

    if (invoice_code_char!=NULL)
	g_free(invoice_code_char);
    if (enterprise_code_char!=NULL)
	g_free(enterprise_code_char);
    if (shop_code_char!=NULL)
	g_free(shop_code_char);
    if (payment_code_char!=NULL)
	g_free(payment_code_char);

    return result;

}



