/* client_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "client_model.h"
#include "participant_model.h"
#include "particular_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "supplier_model.h"
#include "company_model.h"
#include "garage_model.h"
#include "consortium_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"
#include "account.h"
#include "account_model.h"

#define TABLE         "Cliente"
#define ATTRIBUTES    "dtocliente as dtoclienteclient"

static gchar *query_value(gchar *data);





/**
 * m_client_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_client_get_table () {
    return TABLE;
}

/**
 * m_client_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_client_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_client_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Client con los campos rellenados con los datos de la base de datos.
 */
Client * m_client_create (GdaRecordset *rs)
{
  GdaField * field;
  Client *client = d_client_create_empty();

  client->participant = m_participant_create (rs,"client");
  
  if (d_participant_get_code (d_client_get_participant (client)) != ULONG_MAX) {
     field = gda_recordset_field_name(rs,"dtoclienteclient");
     if (field!=NULL) {
	 if (!gda_field_is_null(field)) {
	     client->client_discount = igalia_field_get_double_value (field);
	 }
	 gda_field_free(field);
     }

     field = gda_recordset_field_name(rs,"agenteclient");
     if (field!=NULL) {
	 if (!gda_field_is_null(field)) {
	     client->agent = gda_field_get_boolean_value(field);
	 }
	 gda_field_free(field);
     }

  }

  return client;  
}



/**
 * m_client_delete:
 * @participant: Objeto %Participant que se desea borrar. Dicho objeto es lo que tienen en comn todos los tipos de clientes existentes.
 * @type: Tipo de cliente.
 *
 * Borra un cliente de la base de datos. No es recomendable utilizar esta funcin de borrado, ya que nos
 * podra llevar a prdidas de datos importantes, tal como sus albaranes y facturas.
 *
 * Returns: %TRUE si se ha realizado la operacin, %FALSE en otro caso.
 *
 */
gboolean m_client_delete (Participant *participant, gulong type) {
  gboolean result = FALSE;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);
  
  if (participant!=NULL) {
    gulong code_gulong = d_participant_get_code (participant);

    if (code_gulong != ULONG_MAX){
    gchar * query;
    gchar * query_aux;
    gchar * code;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    if (code_gulong != ULONG_MAX)
      code = g_strdup_printf("%lu",code_gulong);
    else code = g_strdup("NULL");

    gda_command_set_connection(command,connection);

    query = g_strconcat("DELETE FROM ",m_participant_get_table ()," WHERE ",
			      "codigoparty = ",code,
			      NULL);
    g_free(code);

  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
      query_aux = g_strconcat(query, " AND (particular = true)",NULL);
      g_free(query);
      query = query_aux;
      query_aux = NULL;
      break;}
    case M_CLIENT_ENTERPRISE_CODE:{
      query_aux = g_strconcat(query, " AND (empresa = true)",NULL);
      g_free(query);
      query = query_aux;
      query_aux = NULL;
      break;}
    case M_CLIENT_SUPPLIER_CODE:{
      query_aux = g_strconcat(query, " AND (proveedor = true)",NULL);
      g_free(query);
      query = query_aux;
      query_aux = NULL;
      break;}
    case M_CLIENT_COMPANY_CODE:{
      query_aux = g_strconcat(query, " AND (compania = true)",NULL);
      g_free(query);
      query = query_aux;
      query_aux = NULL;
      break;}
    case M_CLIENT_CONSORTIUM_CODE:{
      query_aux = g_strconcat(query, " AND (consorcio = true)",NULL);
      g_free(query);
      query = query_aux;
      query_aux = NULL;
      break;}
    case M_CLIENT_GARAGE_CODE:{
      query_aux = g_strconcat(query, " AND (taller = true)",NULL);
      g_free(query);
      query = query_aux;
      query_aux = NULL;
      break;}
    case M_CLIENT_ALL_PARTICIPANT_CODE:{
    }
    }
    
  result = m_model_mutex_transactional_command_execute_logged (&record_set, query, connection, command,tuples,0, FALSE,
						    logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    if (record_set != NULL) gda_recordset_free(record_set);

    g_print(query);
    
    g_free(query);
    g_free(tuples);

    igalia_command_free(command);
   
    }
  }

  return result;
}


/** 
 * m_client_clear_array:
 * @clients: Array de clientes a liberar.
 *
 * Libera la memoria utilizada por el array de clientes.
 *
 */
void m_client_clear_array (Client ** clients) {
    if (clients!=NULL) {
	gint index = 0;
	while (clients[index]!=NULL) {
	    d_client_clear (clients[index]);
	    index++;
	}
    }
    g_free(clients);

}



/** 
 * m_client_find_by_delivery_payment_condition: 
 * @condition: Filtro de bsqueda.
 *
 * Funcin utilizada para buscar los clientes que tienen algn albarn de abono y que cumplen una
 * serie de condiciones. Estas condiciones  pueden ser: el cdigo de empresa del albarn, el cdigo
 * de tienda del albarn, el cif/nif del cliente, el nombre completo del cliente, el tipo de bsqueda
 * que puede tomar los valores M_CLIENT_PERSON_SEARCH, M_CLIENT_ENTERPRISE_SEARCH o C_NO_SPECIFIED_SEARCH
 * y si los albaranes que se buscan son con seguro o sin seguro.
 *
 * Returns: Lista de clientes que cumplen la condicin de bsqueda que adems tienen albaranes de abono.
 */
Client **  m_client_find_by_delivery_payment_condition (ConditionFindClients *condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_order=NULL;
    gchar              *gchar_where_aux=NULL;
#ifndef IS_SHOP
    gchar              *gchar_select_garage=NULL;
    gchar              *gchar_where_garage=NULL;
    gchar              *where_garage_enterprise=NULL;
    gchar              *where_garage_shop=NULL;
    gchar              *gchar_where_aux_garage=NULL;
    gchar              *gchar_union=NULL;
#endif
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    Client ** clients;
    gulong * tuples;
    Client * client;
    gint index;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));

    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat("(SELECT DISTINCT",
			       " Participante.codigoParty as codigopartyparticipantclient, ",
			       " Participante.nombre as nombreparticipantclient, ",
			       " Participante.nombrecorto as shortnameparticipantclient, ",
			       " Participante.cif as cifparticipantclient, ",
			       " Participante.nick as nickparticipantclient, ",
			       " Participante.telefono as telefonoparticipantclient, ",
			       " Participante.telefono2 as telefono2participantclient, ",
			       " Participante.fax as faxparticipantclient, ",
			       " Participante.email as emailparticipantclient, ",
			       " Participante.calle as calleparticipantclient, ",
			       " Participante.numero as numeroparticipantclient, ",
			       " Participante.piso as pisoparticipantclient, ",
			       " Participante.CP as cpparticipantclient, ",
			       " Participante.municipio as municipioparticipantclient, ",
			       " to_char(Cliente.dtocliente,'999D99') as dtoclienteclient ",
			       " from cliente,participante,albaranabono,reparacion left outer join nota",
			       " on (reparacion.codigoReparacion= nota.codigoReparacion  and ",

			       // the aplication do not show some albaranabono,
			       // without this two conditions
			       // a reparacion-nota with the same code in other shop

			       " nota.codigoTienda=reparacion.codigoTienda and " ,
			       " nota.codigoEmpresa=reparacion.codigoEmpresa) ",
			       NULL );

#ifndef IS_SHOP
    gchar_union = g_strconcat(" union ",NULL);

    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
        where_garage_enterprise = g_strconcat("facturataller.codigoEmpresaAlbaran = ",condition->enterprise_code," ",NULL);
    } else {
        where_garage_enterprise = g_strdup("");
    }

    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
        where_garage_shop = g_strconcat("and facturataller.codigoTiendaAlbaran = ",condition->shop_code," ",NULL);
    } else {
        where_garage_shop = g_strdup("");
    }

    gchar_select_garage = g_strconcat("(SELECT ",
			       " Participante.codigoParty as codigopartyparticipantclient, ",
			       " Participante.nombre as nombreparticipantclient, ",
			       " Participante.nombrecorto as shortnameparticipantclient, ",
			       " Participante.cif as cifparticipantclient, ",
			       " Participante.nick as nickparticipantclient, ",
			       " Participante.telefono as telefonoparticipantclient, ",
			       " Participante.telefono2 as telefono2participantclient, ",
			       " Participante.fax as faxparticipantclient, ",
			       " Participante.email as emailparticipantclient, ",
			       " Participante.calle as calleparticipantclient, ",
			       " Participante.numero as numeroparticipantclient, ",
			       " Participante.piso as pisoparticipantclient, ",
			       " Participante.CP as cpparticipantclient, ",
			       " Participante.municipio as municipioparticipantclient, ",
			       " to_char(Cliente.dtocliente,'999D99') as dtoclienteclient ",
			       " from cliente,participante,reparacion,albaranabono",
			       " left outer join (select numerofacturaTaller,numeroalbaran ",
        		       " from facturataller where ",where_garage_enterprise,where_garage_shop,
			       " ) as albaranesTaller  on (albaranabono.numeroAlbaran=albaranesTaller.numeroAlbaran)",
      			      NULL );
#endif
    gchar_where=g_strconcat(" where ","(albaranabono.numeroFactura is null) and ",
			    " (reparacion.codigoReparacion=albaranabono.codigoReparacion) and ",
			    " (participante.codigoParty=cliente.codigoParty) ",
			    " and reparacion.codigoTienda = albaranabono.codigoTienda ",
                            " and reparacion.codigoEmpresa = albaranabono.codigoEmpresa ",
			    " and (cliente.codigoParty=reparacion.codigoCliente) and nota.codigoReparacion is null ",NULL);
//			    " and (cliente.codigoParty=reparacion.codigoCliente)  ",NULL);

#ifndef IS_SHOP
    gchar_where_garage=g_strconcat(" where ",
			    " (reparacion.codigoReparacion=albaranabono.codigoReparacion) and ",
			    " (participante.codigoParty=cliente.codigoParty) ",
			    " and reparacion.codigoTienda = albaranabono.codigoTienda ",
                            " and reparacion.codigoEmpresa = albaranabono.codigoEmpresa ",
			    " and (cliente.codigoParty=reparacion.codigoTaller) and (albaranabono.totalTaller>0) and albaranesTaller.numeroAlbaran is null",NULL);
#endif
    if (condition->insurance==TRUE) {
	gchar_where_aux=g_strconcat(gchar_where," and (reparacion.codigoCompania is not null) ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;

    } else {
	gchar_where_aux=g_strconcat(gchar_where," and (reparacion.codigoCompania is null) ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;

    }


    if (((condition->initial_date) != NULL) && (strlen(condition->initial_date)!=0)) {
        gchar_where_aux = g_strconcat(gchar_where," and albaranabono.fechaEmision >= '",condition->initial_date,"' ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;

#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage," and albaranabono.fechaEmision >= '",condition->initial_date,"' ",NULL);    

	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
	gchar_where_aux_garage=NULL;
#endif
    }

	
    if (((condition->ending_date) != NULL) && (strlen(condition->ending_date)!=0)) {
        gchar_where_aux = g_strconcat(gchar_where," and albaranabono.fechaEmision <= '",condition->ending_date,"' ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;

#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage," and albaranabono.fechaEmision <= '",condition->ending_date,"' ",NULL);    

	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
	gchar_where_aux_garage=NULL;
#endif    
    }        


    if (((condition->code) != NULL) && (strlen(condition->code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (participante.cif='",
				      condition->code,"')",NULL);
#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (participante.cif='",
				      condition->code,"')",NULL);
	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
	gchar_where_aux_garage=NULL;
#endif

	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    if (((condition->complete_name) != NULL) && (strlen(condition->complete_name)!=0)) {
        u_blanks_to_percentage (condition->complete_name);
	gchar_where_aux = g_strconcat(gchar_where, " and (participante.nick ilike '%",
				  condition->complete_name,"%')",NULL);
#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (participante.nick ilike '%",
				  condition->complete_name,"%')",NULL);
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
	gchar_where_aux_garage=NULL;
#endif

    }

    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoEmpresa=",
				      condition->enterprise_code,")",NULL);
#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (albaranabono.codigoEmpresa=",
				      condition->enterprise_code,")",NULL);
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
	gchar_where_aux_garage=NULL;
#endif

    }

    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoTienda=",
				      condition->shop_code,")",NULL);
#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (albaranabono.codigoTienda=",
				      condition->shop_code,")",NULL);
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
	gchar_where_aux_garage=NULL;
#endif

    }

    g_print("Tipo: %d\n",condition->search_type);
    if (condition->search_type==M_CLIENT_PERSON_SEARCH) {
	gchar_where_aux = g_strconcat(gchar_where, " and (participante.particular=TRUE) ",NULL);
#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (participante.particular=TRUE) ",NULL);
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
        gchar_where_aux=NULL;
#ifndef IS_SHOP
	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
        gchar_where_aux_garage=NULL;
#endif

    } else if (condition->search_type==M_CLIENT_ENTERPRISE_SEARCH) {
	gchar_where_aux = g_strconcat(gchar_where, " and (participante.particular=FALSE) and ",
				  "(participante.tienda=FALSE)",NULL);
#ifndef IS_SHOP
	gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (participante.particular=FALSE) and ",
				  "(participante.tienda=FALSE)",NULL);
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
        gchar_where_aux=NULL;
#ifndef IS_SHOP
	g_free(gchar_where_garage);
	gchar_where_garage = gchar_where_aux_garage;
        gchar_where_aux_garage=NULL;
#endif

    } else {

    }


    gchar_order = g_strconcat(" ORDER BY PARTICIPANTE.NOMBRE )",NULL);

#ifndef IS_SHOP
    if (condition->insurance!=FALSE) {
	gchar_query = g_strconcat(gchar_select, gchar_where, gchar_order,
				  NULL);
    } else {
	gchar_query = g_strconcat(gchar_select, gchar_where, gchar_order,gchar_union,
				  gchar_select_garage,gchar_where_garage,gchar_order,NULL);
    }
#else
    gchar_query = g_strconcat(gchar_select, gchar_where, gchar_order,
			      NULL);
#endif





    g_print(_("\nQuery: %s\n"),gchar_query);


    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {
	    clients = (Client **) g_malloc0(sizeof(Client*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		client = m_client_create (record_set);
		clients[index-1] = client;
		index = gda_recordset_move_next(record_set);
	    }
	    clients[*tuples]=NULL;

	} else {
	    clients=NULL;
	}
	gda_recordset_free(record_set);
    } else {
	clients=NULL;
    }
    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(tuples);

#ifndef IS_SHOP
    if (gchar_where_garage!=NULL)
	g_free(gchar_where_garage);
    if (gchar_select_garage!=NULL)
	g_free(gchar_select_garage);
#endif

    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_order);

return clients;

}
/** 
 * m_client_insert: 
 * @client_data: Datos particulares de cada cliente, el tipo del puntero difiere segn el tipo de cliente.
 * @client: Cliente que se desea insertar.
 * @type:Tipo del cliente. Esta variable puede tomar los valores: M_CLIENT_PERSON_CODE, M_CLIENT_ENTERPRISE_CODE,
 * M_CLIENT_SUPPLIER_CODE, M_CLIENT_COMPANY_CODE, M_CLIENT_CONSORTIUM_CODE y M_CLIENT_GARAGE_CODE.
 * @accounts: Cuentas corrientes del cliente que se desean insertar.
 *
 * Inserta un cliente en la base de datos. Adems de insertar los datos del cliente tambin inserta
 * los datos segn el tipo de cliente. Es decir, si es una empresa introduce los datos necesarios en la
 * base de datos. Adems, inserta las cuentas corrientes del cliente en cuestin.
 *
 * Returns: %TRUE si se ha realizado la operacin, %FALSE en otro caso.
 */
gboolean m_client_insert (gpointer *client_data, Client *client, gulong type,GList * accounts) {

  gboolean result = FALSE;
  
  if (client_data!=NULL) {
    Participant *participant = d_particular_get_participant ((ParticularObject *) client);
    gchar *enterprise_legal_id = d_participant_get_legal_id (participant);
    gchar * temp_nextval=NULL;
    gchar * temp_currval=NULL;
    gchar * query=NULL;
    gchar * gchar_query1 = NULL;
    gchar * gchar_query1_aux;
    gchar * gchar_query2 = NULL;
    gchar * gchar_query3;
    gchar * gchar_query4 = NULL;
    gchar * gchar_query5 = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1 = NULL;
    GdaRecordset *record_set2 = NULL;
    GdaRecordset *record_set3 = NULL;
    GdaRecordset *record_set4 = NULL;
    GdaRecordset *record_set5 = NULL;

    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    gulong postal_code_gulong = d_participant_get_postal_code (participant);
    gchar * postal_code = NULL;
    gdouble client_discount_gint = d_client_get_discount (client);
    gchar *client_discount = NULL;    
    gboolean agent_gboolean = d_client_get_agent (client);
    gchar *agent = FALSE;
    gchar *name = d_participant_get_name (participant);
    gchar *nick = d_participant_get_nick (participant);
    gchar *short_name = d_participant_get_short_name (participant);    
    gchar *phone = d_participant_get_phone (participant);
    gchar *phone2 = d_participant_get_phone2 (participant);
    gchar *fax = d_participant_get_fax (participant);
    gchar *email = d_participant_get_email (participant);
    gchar *street = d_participant_get_street (participant);
    gchar *number = d_participant_get_number (participant);
    gchar *floor = d_participant_get_floor (participant);
    gchar *town = d_participant_get_town (participant);
    gchar *provincia = d_participant_get_province (participant);
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;
    gboolean belong_to_principal=FALSE;
    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    if (postal_code_gulong != ULONG_MAX)
      postal_code = g_strdup_printf("%lu",postal_code_gulong);
    else postal_code = g_strdup("NULL");
    
    if (client_discount_gint != -1)
      client_discount = g_strdup_printf("to_number('%.2f','999D99')",client_discount_gint);
    else client_discount = g_strdup("NULL");
    
    if (agent_gboolean) agent = g_strdup("TRUE");
    else agent = g_strdup("FALSE");

    name = query_value(name);
    short_name = query_value(short_name);
    nick = query_value(nick);
    enterprise_legal_id = query_value(enterprise_legal_id);
    phone = query_value(phone);
    phone2 = query_value(phone2);
    fax = query_value(fax);
    email = query_value(email);
    street = query_value(street);
    number = query_value(number);
    floor = query_value(floor);
    town = query_value(town);
    provincia = query_value(provincia);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    temp_nextval = m_model_nextval_sequence ("seqParticipante",connection);
    if (temp_nextval!=NULL) {
	gchar_query1_aux = g_strconcat("INSERT INTO ",m_participant_get_table ()," VALUES (",temp_nextval,",",
				       name,",",
				       enterprise_legal_id,",",
				       nick,",",
				       phone,",",
				       phone2,",",
				       fax,",",
				       email,",",
				       street,",",
				       number,",",
				       floor,",",
				       postal_code,",",
				       town,",",
				       provincia,",",
				       NULL);
	g_free(temp_nextval);
	temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	gchar_query3 = g_strconcat("INSERT INTO ",m_client_get_table ()," VALUES (",temp_currval,",",
				   client_discount,",",agent,")",
				   NULL);
	g_free(temp_currval);

	switch (type)
	{
	case M_CLIENT_PERSON_CODE:{
	    gchar *first_surname = d_particular_get_first_surname ((ParticularObject *) client_data);
	    gchar *second_surname = d_particular_get_second_surname ((ParticularObject *) client_data);

	    first_surname = query_value(first_surname);
	    second_surname = query_value(second_surname);

	    gchar_query1 = g_strconcat(gchar_query1_aux,"FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,",short_name,")",NULL);
	    temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	    gchar_query2 = g_strconcat("INSERT INTO ",d_particular_get_table ()," VALUES (",temp_currval,",",
				       first_surname,",",
				       second_surname,")",
				       NULL);
	    g_free(temp_currval);
	    g_free(first_surname);
	    g_free(second_surname);
	    break;}
	case M_CLIENT_ENTERPRISE_CODE:{
	    gchar *belong_to_principal_char = NULL;

	    belong_to_principal = d_enterprise_get_belong_to_principal ((Enterprise *) client_data);
	    belong_to_principal_char = g_strdup_printf("%d",belong_to_principal);
	    gchar_query1 = g_strconcat(gchar_query1_aux,"FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,FALSE, ",short_name,")",NULL);
	    temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	    gchar_query2 = g_strconcat("INSERT INTO ",m_enterprise_get_table ()," VALUES (",temp_currval,",",
				       "'",belong_to_principal_char,"')",
				       NULL);
	    g_free(temp_currval);
	    g_free(belong_to_principal_char);
	    break;}
	case M_CLIENT_SUPPLIER_CODE:{
	    gchar_query1 = g_strconcat(gchar_query1_aux,"TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE, ",short_name, ")",NULL);
	    temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	    gchar_query2 = g_strconcat("INSERT INTO ",m_supplier_get_table ()," VALUES (",temp_currval,")",
				       NULL);
	    g_free(temp_currval);
	    break;}

	case M_CLIENT_COMPANY_CODE:{
	    gdouble dtocompany_gint = d_company_get_discount ((Company *) client_data);
	    gchar *dtocompany = NULL;
	    gulong consortium_code_gulong = d_company_get_consortium_code ((Company *) client_data);
	    gchar * consortium_code = NULL;
    	    gint template_int = d_company_get_template ((Company *) client_data);
	    gchar *template = g_strdup_printf("%i", template_int);
	    
	    if (dtocompany_gint != -1)
		dtocompany = g_strdup_printf("to_number('%.2f','999D99')",dtocompany_gint);
	    else dtocompany = g_strdup("NULL");

	    if ((consortium_code_gulong != ULONG_MAX) && (consortium_code_gulong != 0))
		consortium_code = g_strdup_printf("%lu",consortium_code_gulong);
	    else consortium_code = g_strdup("NULL");

	    gchar_query1 = g_strconcat(gchar_query1_aux,"FALSE,FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,",short_name,")",NULL);

	    temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	    gchar_query2 = g_strconcat("INSERT INTO ",m_company_get_table ()," VALUES (",temp_currval,",",
				       consortium_code,",",
				       dtocompany, ",",
				       template, ")",
				       NULL);

	    g_free(dtocompany);
	    g_free(temp_currval);
	    g_free(consortium_code);
	    g_free(template);
	    break;}
	case M_CLIENT_CONSORTIUM_CODE:{
	    gchar_query1 = g_strconcat(gchar_query1_aux,"FALSE,FALSE,FALSE,TRUE,FALSE,FALSE,FALSE,",short_name,")",NULL);
	    temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	    gchar_query2 = g_strconcat("INSERT INTO ",m_consortium_get_table ()," VALUES (",temp_currval,")",
				       NULL);
	    g_free(temp_currval);
	    break;}
	case M_CLIENT_GARAGE_CODE:{
	    gdouble commission_gint = d_garage_get_commission ((Garage *) client_data);
	    gchar *commission = NULL;

	    if (commission_gint != -1)
		commission = g_strdup_printf("to_number('%.2f','999D99')",commission_gint);
	    else commission = g_strdup("NULL");

	    temp_currval = m_model_currval_sequence ("seqParticipante",connection);
	    gchar_query1 = g_strconcat(gchar_query1_aux,"FALSE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,",short_name,")",NULL);
	    gchar_query2 = g_strconcat("INSERT INTO ",m_garage_get_table ()," VALUES (",temp_currval,",",
				       commission,")",
				       NULL);
	    g_free(temp_currval);
	    g_free(commission);
	    break;}
	}

	result = m_model_command_execute (&record_set1, gchar_query1, connection, command,tuples,0, FALSE);
	result = result && m_model_command_execute (&record_set2, gchar_query2, connection, command,tuples,0, FALSE);
	result = result && m_model_command_execute (&record_set3, gchar_query3, connection, command,tuples,0, FALSE);

	g_print(_("\nParty Insert: %s\n"),gchar_query1);
	g_print(_("\nClient Insert: %s\n"),gchar_query2);
	g_print(_("\nParticular Data Insert: %s\n"),gchar_query3);
	if (gchar_query4 != NULL) {
		result = result && m_model_command_execute (&record_set3, gchar_query4, connection, command,tuples,0, FALSE);
        	g_print(_("\nCK Data Insert: %s\n"),gchar_query4);
        	g_free(gchar_query4);
		gchar_query4=NULL;
	}

	if (result!=FALSE) {
	    GList * actual_list ;
	    gchar * client_code=NULL;
	    gchar_query4 = g_strdup("select currval('seqParticipante')");
	    result = m_model_command_execute (&record_set4, gchar_query4, connection, command,tuples,0, TRUE);
	    if (record_set4!=NULL) {
		if (*tuples == 1) {
		    GdaField * field;
		    gint index = gda_recordset_move_first(record_set4);
		    if (index==GDA_RECORDSET_INVALID_POSITION) result = FALSE;
		    field = gda_recordset_field_idx(record_set4,0);
		    client_code = gda_stringify_value(NULL, 0, field);
		} else {
		    result = FALSE;
		}
	    } else {
		result = FALSE;
	    }
	    if ((result != FALSE) && (accounts!=NULL) && (client_code!=NULL)) {
		actual_list = g_list_first(accounts);
		while (actual_list!=NULL) {
		    Account * account = (Account *) actual_list->data;
		    gchar * bank_code = g_strdup_printf("%d",d_office_get_bank_code (d_account_get_office (account)));
		    gchar * office_code = g_strdup_printf("%d",d_office_get_office_code (d_account_get_office (account)));
		    gchar * account_code = d_account_get_account_number (account);


		    gchar_query5 = g_strconcat(" insert into ccparty values(",
					       bank_code,",",
					       office_code,",",
					       account_code,",",
					       client_code,")",
					       NULL);

		    g_print("%s",gchar_query5);
		    result = result && m_model_command_execute (&record_set5, gchar_query5, connection, command,tuples,0, FALSE);

		    if (m_client_get_type_in_transaction (d_participant_get_code (participant),connection)==M_CLIENT_ENTERPRISE_CODE) {
			if (belong_to_principal!=FALSE)
			    result = result && (m_account_insert_in_transaction (account,connection));
		    }

		    if (record_set5!=NULL) {
			if GDA_IS_RECORDSET(record_set5) {
			    gda_recordset_close(record_set5);
			    gda_recordset_free(record_set5);
			}
		    }
		    record_set5=NULL;
		    actual_list = g_list_next(actual_list);
		    g_free(bank_code);
		    g_free(office_code);
		    g_free(account_code);
		}

	    }
            if (client_code!=NULL)
		g_free(client_code);

	}
	query = g_strconcat (gchar_query1,";\n",gchar_query2,";\n",gchar_query3,";\n",NULL);
	if (gchar_query1_aux!=NULL)
		g_free(gchar_query1_aux);
	if (gchar_query1!=NULL)
		g_free(gchar_query1);	
	if (gchar_query4!=NULL)
	g_free(gchar_query4);
	if (gchar_query2!=NULL)
	g_free(gchar_query2);
	if (gchar_query3!=NULL)
	g_free(gchar_query3);
    } else {
        result=FALSE;
    }
    if (postal_code!=NULL)
	g_free(postal_code);
    if (client_discount!=NULL)
	g_free(client_discount);
    if (agent!=NULL)
	g_free(agent);
    if (name!=NULL)
	g_free(name);
    if (enterprise_legal_id!=NULL)
	g_free(enterprise_legal_id);
    if (nick!=NULL)
	g_free(nick);
    if (phone!=NULL)
	g_free(phone);
    if (phone2!=NULL)
	g_free(phone2);
    if (fax!=NULL)
	g_free(fax);
    if (email!=NULL)
	g_free(email);
    if (street!=NULL)
	g_free(street);
    if (number!=NULL)
	g_free(number);
    if (floor!=NULL)
	g_free(floor);
    if (town!=NULL)
	g_free(town);
    if (provincia!=NULL)
	g_free(provincia);

    result = m_model_execute_mutex_modification_logged (connection) && result;

    gda_recordset_close(record_set1);
    gda_recordset_free(record_set1);
    gda_recordset_close(record_set2);
    gda_recordset_free(record_set2);
    gda_recordset_close(record_set3);
    gda_recordset_free(record_set3);
    if (record_set4!=NULL) {
	gda_recordset_close(record_set4);
	gda_recordset_free(record_set4);
    }
    if (query !=NULL) g_free(query);
    g_free(tuples);

    igalia_command_free(command);

  }

  return result;
}


/** 
 * m_client_find_participants: 
 * @participant: Objeto %Participant utilizado como condicin de bsqueda. Los campos de objeto
 * son utilizados como filtros de bsqueda.
 * @type: Tipo de cliente.
 * @data: Datos particulares de cada tipo de participante.
 *
 * Busca los participantes (clientes,empresas,compaas de seguros) por condicin.
 *
 * Returns: Array de participantes que cumplen la condicin.
 */
Participant **  m_client_find_participants (Participant *participant, gulong type, gpointer data){

  Participant ** parties = NULL;

  if (participant!=NULL) {
    gchar *street = d_participant_get_street (participant);
    gchar *number = d_participant_get_number (participant);
    gchar *floor = d_participant_get_floor (participant);
    gulong postal_code_gulong = d_participant_get_postal_code (participant);
    gchar *postal_code = NULL;
    gchar *town = d_participant_get_town (participant);
    gchar *province = d_participant_get_province (participant);
    gchar *phone = d_participant_get_phone (participant);
    gchar *phone2 = d_participant_get_phone2 (participant);
    gchar *fax = d_participant_get_fax (participant);
    gchar *email = d_participant_get_email (participant);
    gchar *enterprise_legal_id = d_participant_get_legal_id (participant);
    gchar *nick = d_participant_get_nick (participant);
    gchar *name = d_participant_get_name (participant);
    gchar *short_name = d_participant_get_short_name (participant);

    gchar *query = NULL;
    gchar *query_from = NULL;
    gchar *query_where = NULL;
    gchar *query_aux = NULL;
    gchar *order = " order by nick ";
    gboolean first = TRUE;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    

    if (postal_code_gulong!=ULONG_MAX)
      postal_code = g_strdup_printf("%lu",postal_code_gulong);

    gda_command_set_connection(command,connection);
	
    query = g_strconcat("SELECT participante.codigoParty as codigopartyparticipant, "
			"nick as nickparticipant, "
			"telefono as telefonoparticipant, "
			"telefono2 as telefono2participant, "
			"fax as faxparticipant, "
			"email as emailparticipant, "
			"calle as calleparticipant, "
			"numero as numeroparticipant, "
			"piso as pisoparticipant, "
			"CP as cpparticipant, "
			"municipio as municipioparticipant, "
			"participante.nombre as nombreparticipant, "
			"cif as cifparticipant, "
			"provincia as provinciaparticipant, "
			"nombrecorto as shortnameparticipant",NULL); 
			
    query_from = g_strconcat(" FROM ",m_participant_get_table (),NULL);
    query_where = " WHERE participante.tienda = false AND";

     
    if ((name != NULL) && (strlen(name)!=0)) {
      query_aux = g_strconcat(query_where, " (participante.nombre ILIKE '%",name,"%')",NULL);
      query_where = query_aux; 
      query_aux = NULL;
      first = FALSE;
    }
    
    if ((short_name != NULL) && (strlen(short_name)!=0)) {
	if (first) query_aux = g_strconcat(query_where, " (participante.nombrecorto ILIKE '%",short_name,"%')",NULL);
   	else {    
      		query_aux = g_strconcat(query_where, " AND (participante.nombrecorto ILIKE '%",short_name,"%')",NULL);
      		g_free(query_where);
	}

	query_where = query_aux; 
      	query_aux = NULL;
      	first = FALSE;
    }    

    if ((street != NULL) && (strlen(street)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.calle ILIKE '%",street,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.calle ILIKE '%",street,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((number != NULL) && (strlen(number)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.numero ILIKE '%",number,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.numero ILIKE '%",number,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((floor != NULL) && (strlen(floor)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.piso ILIKE '%",floor,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.piso ILIKE '%",floor,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((postal_code != NULL) && (strlen(postal_code)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.cp ILIKE '%",postal_code,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.cp ILIKE '%",postal_code,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((province != NULL) && (strlen(province)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.provincia ILIKE '%",province,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.provincia ILIKE '%",province,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((phone != NULL) && (strlen(phone)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.telefono ILIKE '%",phone,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.telefono ILIKE '%",phone,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((phone2 != NULL) && (strlen(phone2)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.telefono2 ILIKE '%",phone2,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.telefono2 ILIKE '%",phone2,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((fax != NULL) && (strlen(fax)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.fax ILIKE '%",fax,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.fax ILIKE '%",fax,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((email != NULL) && (strlen(email)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.email ILIKE '%",email,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.email ILIKE '%",email,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((enterprise_legal_id != NULL) && (strlen(enterprise_legal_id)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.cif ILIKE '",enterprise_legal_id,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.cif ILIKE '",enterprise_legal_id,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

    if ((nick != NULL) && (strlen(nick)!=0)) {
	u_blanks_to_percentage (nick);

	if (first) query_aux = g_strconcat(query_where, " (participante.nick ILIKE '%",nick,"%')",NULL);
	else {
	    query_aux = g_strconcat(query_where, " AND (participante.nick ILIKE '%",nick,"%')",NULL);
	    g_free(query_where);
	}
	query_where = query_aux;
	query_aux = NULL;
	first = FALSE;
    }

    if ((town != NULL) && (strlen(town)!=0)) {
      if (first) query_aux = g_strconcat(query_where, " (participante.municipio ILIKE '%",town,"%')",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.municipio ILIKE '%",town,"%')",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
    }

  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
      gchar * first_surname = d_particular_get_first_surname ((ParticularObject *) data);
      gchar * second_surname = d_particular_get_second_surname ((ParticularObject *) data);

      query_aux = g_strconcat(query_from,", ", d_particular_get_table (),NULL);
      g_free(query_from);
      query_from = query_aux;
      query_aux = NULL;

      if (first) query_aux = g_strconcat(query_where, " (participante.codigoParty = particular.codigoParty)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.codigoParty = particular.codigoParty)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;

      if (first) query_aux = g_strconcat(query_where, " (participante.particular = true)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.particular = true)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;

      if ((first_surname != NULL) && (strlen(first_surname)!=0)) {
	query_aux = g_strconcat(query_where, " AND (particular.primerapellido ILIKE '",first_surname,"%')",NULL);
	if (!first) g_free(query_where);
	query_where = query_aux;
	query_aux = NULL;
      }

      if ((second_surname != NULL) && (strlen(second_surname)!=0)) {
	query_aux = g_strconcat(query_where, " AND (particular.segundoapellido ILIKE '",second_surname,"%')",NULL);
	if (!first) g_free(query_where);
	query_where = query_aux;
	query_aux = NULL;
      }

      g_free(first_surname);
      g_free(second_surname);

      break;}
    case M_CLIENT_ENTERPRISE_CODE:{
      if (first) query_aux = g_strconcat(query_where, " (participante.empresa = true)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.empresa = true)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
      break;}
    case M_CLIENT_SUPPLIER_CODE:{
      if (first) query_aux = g_strconcat(query_where, " (participante.proveedor = true)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.proveedor = true)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;      
      break;}
    case M_CLIENT_COMPANY_CODE:{
      if (first) query_aux = g_strconcat(query_where, " (participante.compania = true)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.compania = true)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;      
      break;}
    case M_CLIENT_CONSORTIUM_CODE:{
      if (first) query_aux = g_strconcat(query_where, " (participante.consorcio = true)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.consorcio = true)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
      break;}
    case M_CLIENT_GARAGE_CODE:{
      if (first) query_aux = g_strconcat(query_where, " (participante.taller = true)",NULL);
      else {
	query_aux = g_strconcat(query_where, " AND (participante.taller = true)",NULL);
	g_free(query_where);
      }
      query_where = query_aux;
      query_aux = NULL;
      first = FALSE;
      break;}
    case M_CLIENT_ALL_PARTICIPANT_CODE:{
    }
    }
  if (!first) {
    query_aux = g_strconcat(query, query_from, query_where,order,NULL);
    g_free(query_where);
  }
  else query_aux = g_strconcat(query, query_from, " WHERE participante.tienda = false",order,NULL);

  g_free(query);
  g_free(query_from);
  query = query_aux;
  query_aux = NULL;


  printf(_("\nModel find participant: %s\n"),query);

    /* FIXME: Memory leak??????? */
    m_model_command_execute (&record_set, query, connection, command,tuples,0, TRUE);

    if (record_set!=NULL) {
      if (*tuples!=0) {
	gint index;
	Participant *participant_aux=NULL;

	parties = (Participant **) g_malloc0 (sizeof(Participant *)*(*tuples+1));
	index = gda_recordset_move_first(record_set);
	while (!gda_recordset_eof(record_set)) {
	  if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	  participant_aux = m_participant_create (record_set,"");
	  parties[index-1] = participant_aux;
	  participant_aux = NULL;
	  index = gda_recordset_move_next(record_set);
	}
	parties[*tuples]=NULL;
	
      } else {
	parties=NULL;
      }

      gda_recordset_move_first(record_set);
      gda_recordset_free(record_set);
    } else {
      parties=NULL;
    }

/*     g_print(g_strdup_printf("%lu",*tuples)); */

    g_free(street);
    g_free(number);
    g_free(email);
    g_free(fax);
    g_free(phone2);
    g_free(phone);
    g_free(province);
    g_free(enterprise_legal_id);
    g_free(town);
    g_free(name);
    g_free(floor);
    g_free(nick);
    g_free(short_name);    
    g_free(postal_code);
    g_free(query);
    g_free(tuples);
    
    igalia_command_free(command);
  }
  return parties;
}

/** 
 * m_client_get_type: 
 * @code: Cdigo del participante.
 *
 * Obtiene el tipo de un participante identificado por el cdigo.
 *
 * Returns: Tipo del participante.
 */
gint m_client_get_type (gulong code)
{
  gint result = -1;
  
  if (code!=ULONG_MAX) {
    gchar *query = "SELECT proveedor, taller, particular, consorcio, compania, empresa, tienda FROM participante WHERE codigoparty = ";
    gchar *code_gchar = g_strdup_printf("%lu",code);
    GdaRecordset *rs = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

    query = g_strconcat(query, code_gchar, NULL);

    gda_command_set_connection(command,connection);


    printf(_("\nQuery: %s\n"),query);

    m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);

    if ((rs!=NULL)&&(*tuples!=0)) {
	gint index;
	GdaField *field;
	gboolean type_aux = FALSE;

	index = gda_recordset_move_first(rs);

	if (!type_aux){
	field = gda_recordset_field_name(rs,"proveedor");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_SUPPLIER_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"taller");	
	type_aux = gda_field_get_boolean_value(field);	
	if (type_aux) result = M_CLIENT_GARAGE_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"particular");
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_PERSON_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"consorcio");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_CONSORTIUM_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"compania");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_COMPANY_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"empresa");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_ENTERPRISE_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"tienda");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_SHOP_CODE;
	gda_field_free(field);
	}

	gda_recordset_move_first(rs);
	gda_recordset_free(rs);
    }

    igalia_command_free(command);    
    g_free(code_gchar);
    g_free(tuples);
    g_free(query);
    
  }
      
  return result;
			     
}

/** 
 * m_client_get_type_in_transaction:
 * @code: Cdigo del participante.
 * @connection: Conexin a la base de datos.
 *
 * Obtiene el tipo de un participante identificado por el cdigo dentro de una transaccin.
 *
 * Returns: Tipo del participante.
 */
gint m_client_get_type_in_transaction (gulong code,GdaConnection * connection)
{
  gint result = -1;
  
  if (code!=ULONG_MAX) {
    gchar *query = "SELECT proveedor, taller, particular, consorcio, compania, empresa, tienda FROM participante WHERE codigoparty = ";
    gchar *code_gchar = g_strdup_printf("%lu",code);
    GdaRecordset *rs = NULL;
    GdaCommand * command = gda_command_new();
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

    query = g_strconcat(query, code_gchar, NULL);

    gda_command_set_connection(command,connection);


    printf(_("\nQuery: %s\n"),query);

    m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);

    if ((rs!=NULL)&&(*tuples!=0)) {
	gint index;
	GdaField *field;
	gboolean type_aux = FALSE;

	index = gda_recordset_move_first(rs);

	if (!type_aux){
	field = gda_recordset_field_name(rs,"proveedor");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_SUPPLIER_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"taller");	
	type_aux = gda_field_get_boolean_value(field);	
	if (type_aux) result = M_CLIENT_GARAGE_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"particular");
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_PERSON_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"consorcio");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_CONSORTIUM_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"compania");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_COMPANY_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"empresa");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_ENTERPRISE_CODE;
	gda_field_free(field);
	}

	if (!type_aux){
	field = gda_recordset_field_name(rs,"tienda");	
	type_aux = gda_field_get_boolean_value(field);
	if (type_aux) result = M_CLIENT_SHOP_CODE;
	gda_field_free(field);
	}

	gda_recordset_move_first(rs);
	gda_recordset_free(rs);
    }

    igalia_command_free(command);    
    g_free(code_gchar);
    g_free(tuples);
    g_free(query);
    
  }
      
  return result;
			     
}


/** 
 * m_client_get_by_code:
 * @code: Cdigo del cliente.
 *
 * Obtiene los datos del cliente a partir del identificador.
 *
 * Returns: Datos del cliente.
 */
Client *m_client_get_by_code (gulong code)
{
  Client *result = NULL;
  
  if (code!=ULONG_MAX) {
    GdaField *field;
    gchar *query = "SELECT to_char(dtocliente,'999D99') as dtoclienteclient, agente as agenteclient FROM cliente WHERE codigoparty = ";
    gchar *code_gchar = g_strdup_printf("%lu",code);
    GdaRecordset *rs = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

    query = g_strconcat(query, code_gchar, NULL);

    gda_command_set_connection(command,connection);


    printf(_("\nQuery: %s\n"),query);

    m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);

    if ((rs!=NULL)&&(*tuples!=0)) {
      
      result = d_client_create_empty ();

      gda_recordset_move_first(rs);
      
      field = gda_recordset_field_name(rs,"dtoclienteclient");
      if ((field!=NULL) && (!gda_field_is_null(field))) {
	d_client_set_discount (result, igalia_field_get_double_value (field));
	gda_field_free(field);
      }      

      field = gda_recordset_field_name(rs,"agenteclient");
      if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	  d_client_set_agent (result, gda_field_get_boolean_value(field));
	}
	gda_field_free(field);
      }

      gda_recordset_free(rs);
    }

    igalia_command_free(command);    
    g_free(code_gchar);
    g_free(tuples);
    g_free(query);
    
  }
      
  return result;
			     
}

/** 
 * m_client_get_participant_by_name:
 * @name: Nombre a partir del cul queremos obtener el participante.
 * @type: Tipo de participante que se desea buscar.
 * @belong_to_principal: En caso de empresa permite elegir si pertenece al grupo de empresas.
 *
 * Obtiene el participante a partir del nombre.
 *
 * Returns: Objeto %Participante que se desea buscar.
 */
Participant *m_client_get_participant_by_name (gchar * name,gint type,gboolean belong_to_principal)
{
  Participant *  condition;
  Participant ** participants=NULL;
  Participant * participant=NULL;
  gint index=0;

  condition = d_participant_create_empty ();
  d_participant_set_nick (condition,name);

  participants = m_client_find_participants (condition, type , NULL);

  if (belong_to_principal!=FALSE) {
      if (participants!=NULL)
	  while (participants[index]!=NULL) {
	      Enterprise * enter = (Enterprise *) m_client_get_data (M_CLIENT_ENTERPRISE_CODE,d_participant_get_code (participants[index]));
	      if (d_enterprise_get_belong_to_principal (enter)!=FALSE) {
                  break;
	      }
	      index++;
	  }
  }

  participant = participants[index];

  g_free(participants);
  return participant;
}


/** 
 * m_client_get_data: 
 * @type: Tipo de participante que se desea buscar.
 * @code: Identificador del participante.
 *
 * Obtiene los datos del cliente con identificador %code y de tipo %type.
 *
 * Returns: Datos particulares del participante segn el tipo del mismo.
 */
gpointer *m_client_get_data (gint type, gulong code)
{
  gpointer *result = NULL;
  
  if (code!=ULONG_MAX) {
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    GdaRecordset *rs = NULL;
    GdaRecordset *rs1 = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar *query = NULL;
    gchar *query1 = NULL;
    gchar *code_gchar = g_strdup_printf("%lu",code);    

    switch (type)
      {
      case M_CLIENT_PERSON_CODE: query = "SELECT primerApellido as primerapellidopartic, \
	  segundoApellido as segundoapellidopartic FROM particular WHERE codigoparty = ";break;
      case M_CLIENT_ENTERPRISE_CODE: query = "SELECT perteneceGrupo as perteneceGrupoenter FROM empresa WHERE codigoparty = ";break;
      case M_CLIENT_SUPPLIER_CODE:break;
      case M_CLIENT_COMPANY_CODE: {query = "SELECT codigoConsorcio as codigoconsorciocompany, to_char(dtocompania,'999D99') as dtocompany, plantilla as plantillacompany FROM compania WHERE codigoparty = ";
      			    query1 = "SELECT codigoCK as codigoCKcompany FROM DatosCK WHERE codigoparty = ";break;} 
      case M_CLIENT_CONSORTIUM_CODE: break;
      case M_CLIENT_GARAGE_CODE: query = "SELECT to_char(comision,'999D99') as comisiongarage FROM taller WHERE codigoparty = ";break;
      }

    if (query!=NULL){

	gda_command_set_connection(command,connection);

	if (query1 != NULL) {

	    query1 = g_strconcat(query1, code_gchar, NULL);

	    printf(_("\nQuery: %s\n"),query1);

	    m_model_command_execute(&rs1, query1, connection, command,tuples,0, TRUE);
	}

	query = g_strconcat(query, code_gchar, NULL);


	printf(_("\nQuery: %s\n"),query);

	m_model_command_execute (&rs, query, connection, command,tuples,0, TRUE);



    }

    switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
	if ((rs!=NULL)&&(*tuples!=0)){
	  GdaField *field;
	  gda_recordset_move_first(rs);

	  result = (gpointer) d_particular_create_empty ();
	  
	  field = gda_recordset_field_name(rs,"primerapellidopartic");
	  if ((field!=NULL) && (!gda_field_is_null(field))) {
	    gchar *content = gda_stringify_value(NULL, 0, field);
	    d_particular_set_first_surname ((ParticularObject *) result,content);
	    g_free(content);
	    gda_field_free(field);	    
	  }      


	  field = gda_recordset_field_name(rs,"segundoapellidopartic");
	  if ((field!=NULL) && (!gda_field_is_null(field))) {
	    gchar *content = gda_stringify_value(NULL, 0, field);
	    d_particular_set_second_surname ((ParticularObject *) result,content);
	    g_free(content);
	    gda_field_free(field);	    
	  }      

	  gda_recordset_free(rs);

	};
	break;}
      case M_CLIENT_ENTERPRISE_CODE:{
	if ((rs!=NULL)&&(*tuples!=0)){
	  GdaField *field;
	  gda_recordset_move_first(rs);

	  result = (gpointer) d_enterprise_create_empty ();

	  field = gda_recordset_field_name(rs,"perteneceGrupoenter");
	  if ((field!=NULL) && (!gda_field_is_null(field))) {
	    d_enterprise_set_belong_to_principal ((Enterprise *)result, gda_field_get_boolean_value(field));
	    gda_field_free(field);	    
	  }      

	  gda_recordset_free(rs);

	};
	break;}
      case M_CLIENT_SUPPLIER_CODE:{
	result = (gpointer) d_supplier_create_empty ();
      }break;
      case M_CLIENT_COMPANY_CODE:{
	if ((rs!=NULL)&&(*tuples!=0)){
	  GdaField *field;
	  gda_recordset_move_first(rs);
	  
	  result = (gpointer) d_company_create_empty ();

	  field = gda_recordset_field_name(rs,"codigoconsorciocompany");
	  if ((field!=NULL) && (!gda_field_is_null(field))){
	    d_company_set_consortium_code ((Company *) result, igalia_field_get_bigint_value (field));
	    gda_field_free(field);
	  }
	  
	  field = gda_recordset_field_name(rs,"dtocompany");
	  if ((field!=NULL) && (!gda_field_is_null(field))){
	   d_company_set_discount ((Company *) result, igalia_field_get_double_value (field));
	   gda_field_free(field);
	  }

	  field = gda_recordset_field_name(rs,"plantillacompany");
	  if ((field!=NULL) && (!gda_field_is_null(field))){
	   d_company_set_template ((Company *) result, gda_field_get_integer_value(field));
	   gda_field_free(field);
	  }

	  gda_recordset_move_first(rs1);

	  field = gda_recordset_field_name(rs1,"codigoCKcompany");
	  if ((field!=NULL) && (!gda_field_is_null(field))){
	   d_company_set_ck_code ((Company *) result, gda_stringify_value(NULL, 0, field));
	   gda_field_free(field);
	  }


	gda_recordset_free(rs);
        gda_recordset_free(rs1);

      
	}	
	break;};
      case M_CLIENT_CONSORTIUM_CODE:{
	result = (gpointer) d_consortium_create_empty ();
      }break;
      case M_CLIENT_GARAGE_CODE:{
	if ((rs!=NULL)&&(*tuples!=0)){
	  GdaField *field;
	  gda_recordset_move_first(rs);
	  
	  result = (gpointer) d_garage_create_empty ();

	  field = gda_recordset_field_name(rs,"comisiongarage");
	  if ((field!=NULL) && (!gda_field_is_null(field))){
	    d_garage_set_commission ((Garage *) result, igalia_field_get_double_value (field));
	    gda_field_free(field);

	  }

	  gda_recordset_free(rs);
      
	};break;
      }
	
      }

      igalia_command_free(command);    
      g_free(code_gchar);
      g_free(tuples);
      if (query!=NULL) g_free(query);
      if (query1!=NULL) g_free(query1);

      
  }
  
  return result;			        
}


/** 
 * m_client_clear_participant: 
 * @participants: Array de participantes del que se quiere liberar memoria.
 *
 * Libera la memoria utilizada por el array de clientes.
 *
 */
void m_client_clear_participant (Participant ** participants) {
    if (participants!=NULL) {
	gint index = 0;

	while (participants[index]!=NULL) {
	    d_participant_clear (participants[index]);
	    index++;
	}
    }
    g_free(participants);

}


/** 
 * m_client_update: 
 * @client_data: Datos particulares del cliente segn el tipo del mismo.
 * @client: Cliente que se desea modificar.
 * @type: Tipo del cliente;
 * @accounts: Cuentas corrientes de cliente. Si existe una cuenta corriente que antes no exista en
 * la base de datos se inserta.
 *
 * Modifica un cliente en la base de datos.
 *
 * Returns: %TRUE si se ha realizado la operacin, %FALSE en otro caso.
 */
gboolean m_client_update (gpointer *client_data, Client *client, gulong type,GList * accounts) {

  gboolean result = FALSE;
  
  if (client_data!=NULL) {
    Participant *participant = d_particular_get_participant ((ParticularObject *) client);
    gulong code_gulong = d_participant_get_code (participant);
    if (code_gulong!=ULONG_MAX) {
      gchar * query;
      gchar * gchar_query1 = NULL;
      gchar * gchar_query2 = NULL;
      gchar * gchar_query3;
      gchar * gchar_query4 = NULL;
      gchar * gchar_query5 = NULL;
      gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
      GdaRecordset *record_set1 = NULL;   
      GdaRecordset *record_set2 = NULL;   
      GdaRecordset *record_set3 = NULL;   
      GdaRecordset *record_set4 = NULL;
      GdaRecordset *record_set5 = NULL;
      GdaRecordset *record_set6 = NULL;
//      gulong code_gulong = d_participant_get_code (participant);
      gchar *code = NULL;
      gulong postal_code_gulong = d_participant_get_postal_code (participant);
      gchar * postal_code = NULL;
      gdouble client_discount_gint = d_client_get_discount (client);
      gchar *client_discount = NULL;
      gboolean agent_gboolean = d_client_get_agent (client);
      gchar *agent = FALSE;
      gchar *name = d_participant_get_name (participant);
      gchar *nick = d_participant_get_nick (participant);
      gchar *short_name = d_participant_get_short_name (participant);      
      gchar *phone = d_participant_get_phone (participant);
      gchar *phone2 = d_participant_get_phone2 (participant);
      gchar *fax = d_participant_get_fax (participant);
      gchar *email = d_participant_get_email (participant);
      gchar *enterprise_legal_id = d_participant_get_legal_id (participant);
      gchar *street = d_participant_get_street (participant);
      gchar *number = d_participant_get_number (participant);
      gchar *floor = d_participant_get_floor (participant);
      gchar *town = d_participant_get_town (participant);
      gchar *province = d_participant_get_province (participant);
      GdaConnection * connection = m_model_get_connection ();
      GdaCommand * command = gda_command_new();
      gulong logged_shop_code;
      gulong logged_enterprise_code;
      gulong logged_warehouse_code;
      gchar * temp_name;
      gboolean belong_to_principal = FALSE;
      temp_name = u_parameter_manager_get_parameter ("empresa");
      logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
      g_free(temp_name);
      temp_name = u_parameter_manager_get_parameter ("tienda");
      logged_shop_code = m_shop_get_code_from_name (temp_name);
      g_free(temp_name);
      temp_name = u_parameter_manager_get_parameter ("almacen");
      logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

      g_free(temp_name);

      if (postal_code_gulong != ULONG_MAX)
	postal_code = g_strdup_printf("%lu",postal_code_gulong);
      else postal_code = g_strdup("NULL");

      if (client_discount_gint != -1)
	client_discount = g_strdup_printf("to_number('%.2f','999D99')",client_discount_gint);
      else client_discount = g_strdup("NULL");

      if (agent_gboolean) agent = g_strdup("TRUE");
      else agent = g_strdup("FALSE");

      if (code_gulong != ULONG_MAX)
	code = g_strdup_printf("%lu",code_gulong);
      else code = g_strdup("NULL");

      name = query_value(name);
      nick = query_value(nick);
      short_name = query_value(short_name);
      enterprise_legal_id = query_value(enterprise_legal_id);
      phone = query_value(phone);
      phone2 = query_value(phone2);
      fax = query_value(fax);
      email = query_value(email);
      street = query_value(street);
      number = query_value(number);
      floor = query_value(floor);
      town = query_value(town);
      province = query_value(province);
      
      gda_command_set_connection(command,connection);
      m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

      
      gchar_query1 = g_strconcat("UPDATE ",m_participant_get_table ()," SET ",
				 " nombre = ",name,",",
				 " cif = ",enterprise_legal_id,",",
				 " nombrecorto = ", short_name,",",
				 " nick = ",nick,",",
				 " telefono = ",phone,",",
				 " telefono2 = ",phone2,",",
				 " fax = ",fax,",",
				 " email = ",email,",",
				 " calle = ",street,",",
				 " numero = ",number,",",
				 " piso = ",floor,",",
				 " cp = ",postal_code,",",
				 " municipio = ",town,",",
				 " provincia = ",province,
				 " WHERE codigoparty = ",code,
				 NULL);  
      
      gchar_query3 = g_strconcat("UPDATE ",m_client_get_table ()," SET ",
				 " dtocliente = ",client_discount,",",
				 " agente = ",agent,
				 " WHERE codigoparty = ",code,			 
				 NULL);
    
    switch (type)
    {
    case M_CLIENT_PERSON_CODE:{  
      gchar *first_surname = d_particular_get_first_surname ((ParticularObject *) client_data);
      gchar *second_surname = d_particular_get_second_surname ((ParticularObject *) client_data);

      first_surname = query_value(first_surname);
      second_surname = query_value(second_surname);

      gchar_query2 = g_strconcat("UPDATE ",d_particular_get_table ()," SET ",
				 " primerapellido = ",first_surname,",",
				 " segundoapellido = ",second_surname,
				 " WHERE codigoparty = ",code,
				 NULL);

      result = m_model_command_execute (&record_set2, gchar_query2, connection,
					      command,tuples,0, FALSE);

      g_free(first_surname);
      g_free(second_surname);
      break;}
    case M_CLIENT_ENTERPRISE_CODE:{
      gboolean belong_to_principal_gboolean = d_enterprise_get_belong_to_principal ((Enterprise *) client_data);
      gchar *belong_to_principal_char = NULL;

      belong_to_principal = belong_to_principal_gboolean;
      if (belong_to_principal_gboolean != -1)
	belong_to_principal_char = g_strdup_printf("%d",belong_to_principal_gboolean);
      else belong_to_principal_char = g_strdup("NULL");

      belong_to_principal_char = query_value(belong_to_principal_char);
    
      gchar_query2 = g_strconcat("UPDATE ",m_enterprise_get_table ()," SET ",
				 " perteneceGrupo = ",belong_to_principal_char,
				 " WHERE codigoparty = ",code,
				 NULL);

      result = m_model_command_execute (&record_set2, gchar_query2, connection,
					      command,tuples,0, FALSE);

      g_free(belong_to_principal_char);
      break;}
    case M_CLIENT_SUPPLIER_CODE: { result = TRUE; break;}
    case M_CLIENT_COMPANY_CODE:{
      gdouble company_discount_gint = d_company_get_discount ((Company *) client_data);
      gchar *company_discount = NULL;
      gchar *gchar_query = NULL;
      gulong consortium_code_gulong = d_company_get_consortium_code ((Company *) client_data);
      gchar *consortium_code = NULL;
      gint template_int = d_company_get_template ((Company *) client_data);
      gchar *template = g_strdup_printf("%d", template_int);

      if (company_discount_gint != -1)
	company_discount = g_strdup_printf("to_number('%.2f','999D99')",company_discount_gint);
      else company_discount = g_strdup("NULL");
      

      if ((consortium_code_gulong != ULONG_MAX) && (consortium_code_gulong != 0))
	consortium_code = g_strdup_printf("%lu",consortium_code_gulong);
      else consortium_code = g_strdup("NULL");
      
      
      
      gchar_query2 = g_strconcat("UPDATE ",m_company_get_table ()," SET ",
				 " codigoconsorcio = ",consortium_code,",",
				 " dtocompania = ",company_discount, ",",
				 " plantilla = ", template,				 
				 " WHERE codigoparty = ",code,
				 NULL);

      result = m_model_command_execute (&record_set2, gchar_query2, connection,command,tuples,0,FALSE);



      if (gchar_query != NULL) {
      			  
	    g_print(_("\nQuery: %s\n"), gchar_query);

            result = result && m_model_command_execute (&record_set2, gchar_query, connection,command,tuples,0,FALSE);
	    
	    g_print(_("\nResult: %d\n"), result);

	    g_free(gchar_query);
	
      }
	

      g_free(company_discount);
      g_free(template);		
      g_free(consortium_code);
      break;}
    case M_CLIENT_CONSORTIUM_CODE: {result = TRUE; break;}
    case M_CLIENT_GARAGE_CODE:{
      gdouble commission_gint = d_garage_get_commission ((Garage *) client_data);
      gchar *commission = NULL;

      if (commission_gint != -1)
	commission = g_strdup_printf("to_number('%.2f','999D99')",commission_gint);
      else commission = g_strdup("NULL");

      gchar_query2 = g_strconcat("UPDATE ",m_garage_get_table ()," SET ",
				 " comision = ",commission,
				 " WHERE codigoparty = ",code,
				 NULL);

      result = m_model_command_execute (&record_set2, gchar_query2, connection,
					      command,tuples,0, FALSE);

      g_free(commission);
      break;}
    default : result = TRUE; 
    }

    result = result && m_model_command_execute (&record_set1, gchar_query1, connection,
					    command,tuples,0,FALSE);

    result = result && m_model_command_execute (&record_set3, gchar_query3, connection,
					    command,tuples,0,FALSE);


    if ((result != FALSE) && (accounts!=NULL) && (code_gulong!=ULONG_MAX)) {
        GList * actual_list;
	actual_list = g_list_first(accounts);
	while (actual_list!=NULL) {
	    Account * account = (Account *) actual_list->data;
	    gchar * bank_code = g_strdup_printf("%d",d_office_get_bank_code (d_account_get_office (account)));
	    gchar * office_code = g_strdup_printf("%d",d_office_get_office_code (d_account_get_office (account)));
	    gchar * account_code = d_account_get_account_number (account);
	    gchar * client_code = g_strdup_printf("%lu",code_gulong);
            gboolean insert= FALSE;

	    gchar_query4 = g_strconcat(" select codigoParty from CCParty where ",
				       " codigoBanco=",bank_code,
				       " and codigoOficina=",office_code,
				       " and numeroCuenta=",account_code,
				       " and codigoParty=",client_code,
				       NULL);

	    result = result && m_model_command_execute (&record_set4, gchar_query4, connection, command,tuples,0, TRUE);
	    g_print(_("\nQuery: %s\n"),gchar_query4);

	    if (record_set4 != NULL) {
		if (*tuples==0) {
                    insert=TRUE;
		} else {
		    insert=FALSE;
		}
	    }
	    if (insert) {
		gchar_query5 = g_strconcat(" insert into ccparty values(",
					   bank_code,",",
					   office_code,",",
					   account_code,",",
					   client_code,")",
					   NULL);

		if (m_client_get_type_in_transaction (d_participant_get_code (participant),connection)==M_CLIENT_ENTERPRISE_CODE) {
                    if (belong_to_principal!=FALSE)
			result = result && (m_account_insert_in_transaction (account,connection));
		}

		g_print(_("\nInsert: %s\n"),gchar_query5);
		result = result && m_model_command_execute (&record_set5, gchar_query5, connection, command,tuples,0, FALSE);

		if (record_set5!=NULL) {
		    if GDA_IS_RECORDSET(record_set5)
			gda_recordset_free(record_set5);
		}
		record_set5=NULL;
	    }
	    if (record_set4!=NULL) {
		if GDA_IS_RECORDSET(record_set4)
		    gda_recordset_free(record_set4);
	    }
	    record_set4=NULL;

	    actual_list = g_list_next(actual_list);
	    g_free(bank_code);
	    g_free(office_code);
	    g_free(account_code);
	    g_free(client_code);
	}

    }


    if (gchar_query2 != NULL)
	query = g_strconcat (gchar_query1,";\n",gchar_query2,";\n",gchar_query3,";\n",NULL);
    else query = g_strconcat (gchar_query1,";\n",gchar_query3,";\n",NULL);

    g_print(query);

    if (gchar_query1 != NULL) g_free(gchar_query1);
    if (gchar_query3 != NULL) g_free(gchar_query3);
    g_free(code);
    g_free(postal_code);
    g_free(nick);
    g_free(client_discount);
    g_free(short_name);    
    g_free(agent);
    g_free(name);
    g_free(enterprise_legal_id);
    g_free(phone);
    g_free(phone2);
    g_free(fax);
    g_free(email);
    g_free(street);
    g_free(number);
    g_free(floor);
    g_free(town);
    g_free(province);

    if (gchar_query2 != NULL) g_free(gchar_query2);


    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (record_set1 != NULL) gda_recordset_free(record_set1);
    if (record_set2 != NULL) gda_recordset_free(record_set2);
    if (record_set3 != NULL) gda_recordset_free(record_set3);
    if (record_set6 != NULL) gda_recordset_free(record_set6);
    g_free(query);
    g_free(tuples);
    
    igalia_command_free(command);
    }
  }

  return result;
}


/** 
 * query_value: 
 * @data: Datos a que utilizarn en la query.
 *
 * Devuelve la cadena de caracteres para meter en la query a partir de los datos.
 *
 * Returns: Si data es %NULL devuelve la cadena "NULL", en otro caso, devuelve la cadena
 * apuntada por %data entre comillas simples.
 */
static gchar *query_value(gchar *data) {

  gchar * result = NULL;

  if (data == NULL) result = g_strdup("NULL");
  else {
    result = g_strconcat("'",data,"'",NULL);

    g_free(data);
  }

  return result;

}

/** 
 * m_client_get_participant_by_code:
 * @code:Identificador del participante.
 *
 * Devuelve un participante a partir del cdigo que lo identifica.
 *
 * Returns: Devuelve el participante identificado por el cdigo.
 */
Participant *m_client_get_participant_by_code (gulong code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * code_char = FALSE;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    Participant * participant=NULL;
    gint index;

    if (code!=ULONG_MAX) {
	my_command = gda_command_new();


	gda_command_set_connection(my_command,my_connection);

	code_char = g_strdup_printf("%lu",code);


	gchar_select=g_strconcat("SELECT ",
				 "participante.nick as nickParticipant,",
				 "participante.nombre as nombreParticipant,",
				 "participante.nombrecorto as shortnameParticipant,",
				 "participante.cif as cifParticipant,",
				 "participante.codigoParty as codigoPartyParticipant,",
				 "participante.email as emailParticipant,",
				 "participante.telefono as telefonoParticipant,",
				 "participante.telefono2 as telefono2Participant,",
				 "participante.fax as faxParticipant,",
				 "participante.calle as calleParticipant,",
				 "participante.numero as numeroParticipant,",
				 "participante.piso as pisoParticipant,",
				 "participante.cp as cpParticipant,",
				 "participante.municipio as municipioParticipant,",
				 "participante.provincia as provinciaParticipant  "
				 ,NULL);


	gchar_from= g_strconcat(" FROM Participante "
				,NULL);

	gchar_where=g_strconcat(" WHERE codigoParty=",code_char,
				NULL);

	gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where,NULL);

	g_print(_("\nQuery: %s\n"),gchar_query);

	m_model_command_execute (&record_set, gchar_query, my_connection,
			      my_command,tuples,0, TRUE);


	if (record_set!=NULL) {
	    if (*tuples==1) {
		index = gda_recordset_move_first(record_set);
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		participant = m_participant_create (record_set,"");
	    } else {
		participant = NULL;
	    }
	    gda_recordset_close(record_set);
	    gda_recordset_free(record_set);
	} else {
	    participant=NULL;
	}

	igalia_command_free(my_command);
	g_print(_("Freeing database completed\n"));

	g_free(gchar_where);
	g_free(gchar_select);
	g_free(gchar_query);
        g_free(code_char);
    }
    
    return participant;
}

/** 
 * get_participant_by_cif:
 * @enterprise_legal_id:Cif del participante
 *
 * Devuelve un participante a partir del cif.
 *
 * Returns: Devuelve el participante identificado por el cdigo.
 */
Participant **m_client_get_participants_by_legal_id (gchar ** enterprise_legal_id_tokens) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gint indice=0;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    Participant ** parties=NULL;

    if (enterprise_legal_id_tokens!=NULL) {
	my_command = gda_command_new();

	gda_command_set_connection(my_command,my_connection);

	gchar_select=g_strconcat("SELECT ",
				 "participante.nick as nickParticipant,",
				 "participante.nombre as nombreParticipant,",
				 "participante.nombrecorto as shortnameParticipant,",
				 "participante.cif as cifParticipant,",
				 "participante.codigoParty as codigoPartyParticipant,",
				 "participante.email as emailParticipant,",
				 "participante.telefono as telefonoParticipant,",
				 "participante.telefono2 as telefono2Participant,",
				 "participante.fax as faxParticipant,",
				 "participante.calle as calleParticipant,",
				 "participante.numero as numeroParticipant,",
				 "participante.piso as pisoParticipant,",
				 "participante.cp as cpParticipant,",
				 "participante.municipio as municipioParticipant,",
				 "participante.provincia as provinciaParticipant "
				 ,NULL);


	gchar_from= g_strconcat(" FROM Participante "
				,NULL);

        gchar_where=g_strdup(" WHERE cif ilike '%");
	while (enterprise_legal_id_tokens[indice]!=NULL) {
	    gchar_where_aux=g_strconcat(gchar_where,enterprise_legal_id_tokens[indice],"%",
					NULL);
            g_free(gchar_where);
	    gchar_where=gchar_where_aux;
            indice++;
	}
	gchar_where_aux=g_strconcat(gchar_where,"'",NULL);
	g_free(gchar_where);
	gchar_where=gchar_where_aux;

	gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where,NULL);

	g_print(_("\nQuery: %s\n"),gchar_query);

	m_model_command_execute (&record_set, gchar_query, my_connection,
			      my_command,tuples,0, TRUE);



	if (record_set!=NULL) {
	    if (*tuples!=0) {
		gint index=0;
		Participant *participant_aux=NULL;

		parties = (Participant **) g_malloc0 (sizeof(Participant *)*(*tuples+1));
		gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    participant_aux = m_participant_create (record_set,"");
		    parties[index] = participant_aux;
		    participant_aux = NULL;
		    gda_recordset_move_next(record_set);
                    index++;
		}
		parties[*tuples]=NULL;

	    } else {
		parties=NULL;
	    }

	    gda_recordset_move_first(record_set);
	    gda_recordset_free(record_set);
	} else {
	    parties=NULL;
	}


	igalia_command_free(my_command);
	g_print(_("Freeing database completed\n"));

	g_free(gchar_where);
	g_free(gchar_select);
	if (gchar_from!=NULL) g_free(gchar_from);
	g_free(gchar_query);
    }
    return parties;


}

