
/* consortium_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "consortium_model.h"
#include "participant_model.h"

#include "util.h"

Consortium ** consortiums_list = NULL;
gint length_of_list;

#define TABLE         "Consorcio"
#define IDENTIFICATOR "codigoParty"
   


/**
 * m_consortium_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_consortium_get_table () 
{  
       return TABLE;
}


/**
 * m_consortium_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Consortium con los campos rellenados con los datos de la base de datos.
 */
Consortium * m_consortium_create (GdaRecordset *rs) {

    Consortium * consortium;
    GdaField * field;

    consortium = d_consortium_create_empty();


    field = gda_recordset_field_name(rs,"descuentoconsortium");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    consortium->discount = igalia_field_get_double_value (field);
	}
	gda_field_free(field);
    }

    consortium->participant = m_participant_create (rs,"consort");

    return consortium;
}




/** 
 * m_consortium_find_by_nick:
 * @nick: Nombre completo del consorcio buscado.
 *
 *  Busca un consorcio por nombre.
 *
 * Returns: Consorcio con nombre completo %nick.
 */
Consortium * m_consortium_find_by_nick (gchar *nick) {
    gint index = 0;
    Participant *participant = NULL;
    if (consortiums_list==NULL) {
	m_consortium_find_by_code (ULONG_MAX);
    }

    while(consortiums_list[index]!=NULL) {
	participant = d_consortium_get_participant (consortiums_list[index]);
	if(!strcmp(nick, d_participant_get_nick (participant))) {
            return consortiums_list[index];
	}
        index++;
    }

    return NULL;

}

/** 
 * m_consortium_find_by_code:
 * @code: Identificador del consorcio. Si se desean todos los consorcios %ULONG_MAX.
 *
 * Busca un consorcio por cdigo de participante o
 * todos los consorcios si el cdigo es %ULONG_MAX.
 *
 * Returns: Array de todos los consorcios o una lista con el consorcio solicitado.
 */
Consortium ** m_consortium_find_by_code (gulong code) {

  if ((consortiums_list==NULL)||(code!=ULONG_MAX)) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Consortium * consortium = NULL;
    gchar * gchar_query;
    gulong * tuples;
    gint * column;
    gint index;
    
    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    
    gchar_query = g_strconcat("SELECT ",
			      " Participante.codigoParty as codigopartyparticipantconsort, ",
			      " Participante.nombre as nombreparticipantconsort, ",
			      " Participante.cif as cifparticipantconsort, ",
			      " Participante.nick as nickparticipantconsort, ",
			      " Participante.telefono as telefonoparticipantconsort, ",
			      " Participante.telefono2 as telefono2participantconsort, ",
			      " Participante.fax as faxparticipantconsort, ",
			      " Participante.email as emailparticipantconsort, ",
			      " Participante.calle as calleparticipantconsort, ",
			      " Participante.numero as numeroparticipantconsort, ",
			      " Participante.piso as pisoparticipantconsort, ",
			      " Participante.CP as cpparticipantconsort, ",
			      " Participante.municipio as municipioparticipantconsort, ",
			      " Participante.provincia as proviciaparticipantconsort, "
			      " to_char(consorcio.dtoconsorcio,'999D99') as descuentoconsortium",
			      " FROM participante, consorcio "
			      " WHERE participante.consorcio=true",
			      " AND consorcio.codigoparty = participante.codigoparty ",
			      NULL);
    
    if (code != ULONG_MAX){
      gchar * gchar_query_aux = NULL;
      gchar *code_gchar = g_strdup_printf("%lu", code);

      gchar_query_aux = g_strconcat(gchar_query," AND participante.codigoparty='",code_gchar,"'",NULL);
      g_free(gchar_query);
      g_free(code_gchar);
      gchar_query =  gchar_query_aux;
      gchar_query_aux = NULL;
    }
    else {
      gchar * gchar_query_aux = NULL;

      gchar_query_aux = g_strconcat(gchar_query," ORDER BY participante.nombre",NULL);
      g_free(gchar_query);
      gchar_query =  gchar_query_aux;
      gchar_query_aux = NULL;
    }

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
    g_free(gchar_query);
    
    length_of_list = *tuples;
    column = (gint *) g_malloc0(sizeof(gint));
    
    if (record_set!=NULL) {
      if (*tuples!=0) {
	if (code == ULONG_MAX){
	  consortiums_list = (Consortium **) g_malloc0(sizeof(Consortium*)*(*tuples+1));
	  index = gda_recordset_move_first(record_set);
	  while (!gda_recordset_eof(record_set)) {
	    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	    (*column)=0;
	    consortium = m_consortium_create (record_set);
	    consortiums_list[index-1] = consortium;
	    index = gda_recordset_move_next(record_set);
	  }
	  consortiums_list[length_of_list]=NULL;	
	} else{
	  index = gda_recordset_move_first(record_set);
	  consortium = m_consortium_create (record_set);
	}
      } else {
	if (code == ULONG_MAX) consortiums_list=NULL;
      }
      gda_recordset_close(record_set);
    } else {
      if (code == ULONG_MAX) consortiums_list=NULL;
    }
    
    igalia_command_free(my_command);
    gda_recordset_free(record_set);
    
    g_free(tuples);
    g_free(column);
 
    if (code != ULONG_MAX) {
      Consortium **consortiums = (Consortium **) g_malloc0(sizeof(Consortium*)*(2));
      consortiums[0]=consortium;
      consortiums[1]=NULL;
      return consortiums;
    }  
  }

  return consortiums_list;
}

/** 
 * m_consortium_clear_array: 
 * 
 * Funcin que libera la memoria utilizada por la variable global @consortiums_list.
 *
 */
void m_consortium_clear_array () {
  
    if (consortiums_list!=NULL) {
	gint index = 0;
	while (consortiums_list[index]!=NULL) {
	  d_consortium_clear (consortiums_list[index]);
	  index++;
	}

    g_free(consortiums_list);   
    
    consortiums_list = NULL;

    }
    
}
