
/* company.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "company.h"
static gint company_init(Company *company);


/**
 d_company_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Company vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Company *d_company_create_empty () {
    Company * company = (Company *) g_malloc (sizeof(Company));

    company_init(company);

    return company;
}

/**
 * d_company_clear:
 * @company: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_company_clear (Company *company) {

    if (company!=NULL) {
	d_participant_clear (company->participant);
	
	if (company->ck_code != NULL) g_free(company->ck_code);
	g_free(company);
    }

  return 0;
}


/**
 * d_company_set_participant:
 * @company: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica todo lo relativo al participante que referencia la compaa de seguros.
 *
 */
void d_company_set_participant (Company *company, Participant * participant){
  company->participant = participant;
}

/**
 * d_company_set_consortium_code:
 * @company: Objeto en el que se establece el atributo.
 * @consortium_code: Atributo a insertar.
 *
 * Establece el atributo @consortium_code en el objeto. El atributo @consortium_code
 * indica el cdigo del consorcio al que pertenece la compaa de seguros.
 *
 */
void d_company_set_consortium_code (Company *company, gulong consortium_code){
  company->consortium_code=consortium_code;
}

     
/**
 * d_company_set_discount:
 * @company: Objeto en el que se establece el atributo.
 * @discount: Atributo a insertar.
 *
 * Establece el atributo @discount en el objeto. El atributo @discount
 * indica el descuento que se le aplicara a una reparacin de tener como
 * seguro dicha compaa.
 *
 */
void d_company_set_discount (Company *company, gdouble discount){
    if (discount>100)
	company->discount=100;
    else if (discount<0)
	company->discount=0;
    else
	company->discount=discount;
}

/**
 * d_company_set_template:
 * @company: Objeto en el que se establece el atributo.
 * @ck_code: Atributo a insertar.
 *
 * Establece el atributo @ck_code en el objeto. El atributo @ck_code
 * indica el codigo CK asciado a dicha compaa.
 *
 */

void d_company_set_ck_code (Company *company, gchar *ck_code){
  company->ck_code = g_strdup(ck_code);
}


/**
 * d_company_set_template:
 * @company: Objeto en el que se establece el atributo.
 * @template: Atributo a insertar.
 *
 * Establece el atributo @template en el objeto. El atributo @template
 * indica la plantilla con la que se debe imprimir el albarn.
 *
 */
void d_company_set_template (Company *company, gint template){
  company->template = template;
}

/**
 * d_company_get_participant:
 * @company: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_company_get_participant (Company *company){
  return company->participant;
}

/**    
 * d_company_get_consortium_code:
 * @company: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_company_get_consortium_code (Company *company){
  return company->consortium_code;
}


/**    
 * d_company_get_consortium_code:
 * @company: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_company_get_ck_code (Company *company){
  return g_strdup(company->ck_code);
}

/**
 * d_company_get_discount:
 * @company: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_company_get_discount (Company *company){
  return company->discount;
}

/**
 * d_company_get_template:
 * @company: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_company_get_template (Company *company){
  return company->template;
}

/**
 * d_company_duplicate:
 * @company: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el un objeto #company.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Company *d_company_duplicate (Company *company){

  Company *result = d_company_create_empty ();
  Participant *participant = d_participant_duplicate (d_company_get_participant (company));
  gulong consortium_code = d_company_get_consortium_code (company);
  gdouble discount = d_company_get_discount (company);
  gint template = d_company_get_template (company);
  gchar * ck_code = d_company_get_ck_code (company);

  d_company_set_participant (result, participant);
  d_company_set_consortium_code (result, consortium_code);
  d_company_set_discount (result, discount);
  d_company_set_template (result, template);
  d_company_set_ck_code (company,ck_code);

  if (ck_code!=NULL)
      g_free(ck_code);
  return result;
}


/* Private functions*/

static gint company_init(Company *company)
{
  company->participant = NULL;  
  company->consortium_code = ULONG_MAX;
  company->ck_code = NULL;  
  company->discount = -1;
  company->template = -1;
  
  return 0;
}

