
/* document.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "document.h"


gint document_init(Document *document);

/**
 * d_document_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Document vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Document *d_document_create_empty () {
    Document *document = (Document *) g_malloc (sizeof(Document));

    document_init(document);
    
    return document;
}



/**
 * d_document_clear:
 * @document: Documento a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_document_clear (Document *document) {

  if (document!=NULL){
      if (document->name!=NULL)
	  g_free(document->name);
      if (document->description!=NULL)
	  g_free(document->description);

      g_free(document);
  }

  return 0;
}


/**
 * d_document_set_code:
 * @document: Objeto en el que se establece el atributo.
 * @document_code: Atributo a insertar.
 *
 * Establece el atributo @document_code en el objeto. El atributo @document_code
 * indica el cdigo interno con el que se identifica el documento
 *
 *
 */
void d_document_set_code (Document *document, gulong document_code){
  document->document_code = document_code;
}

/**
 * d_document_set_description:
 * @document: Objeto en el que se establece el atributo.
 * @description: Atributo a insertar.
 *
 * Establece el atributo @description en el objeto. El atributo @description
 * indica la descripcin del documento.
 *
 */
void d_document_set_description (Document *document, gchar *description){
  if (document->description != NULL) g_free(document->description);
  document->description = g_strdup(description);
}

/**
 * d_document_set_name:
 * @document: Objeto en el que se establece el atributo.
 * @name: Atributo a insertar.
 *
 * Establece el atributo @ en el objeto. El atributo @
 * indica el nombre documento.
 *
 */
void d_document_set_name (Document *document, gchar *name){
  if (document->name != NULL) g_free(document->name);
  document->name = g_strdup(name);
}

/**
 * d_document_set_from_repair:
 * @document: Objeto en el que se establece el atributo.
 * @from_repair: Atributo a insertar.
 *
 * Establece el atributo @from_repair en el objeto. El atributo @from_repair
 * indica si el documento es para una reparacin.
 *
 */
void d_document_set_from_repair (Document *document, gboolean from_repair){
    document->from_repair = from_repair;
}

/**
 * d_document_set_from_invoice:
 * @document: Objeto en el que se establece el atributo.
 * @from_invoice: Atributo a insertar.
 *
 * Establece el atributo @from_invoice en el objeto. El atributo @from_invoice
 * indica si el documento es para una factura.
 *
 */
void d_document_set_from_invoice (Document *document, gboolean from_invoice){
    document->from_invoice = from_invoice;
}

/**
 * d_document_get_code:
 * @document: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_document_get_code (Document *document){
  return document->document_code;
}

/**
 * d_document_get_name:
 * @document: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_document_get_name (Document *document){
  return g_strdup(document->name);
}

/**
 * d_document_get_description:
 * @document: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_document_get_description (Document *document){
  return g_strdup(document->description);
}

/**
 * d_document_get_from_repair:
 * @document: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_document_get_from_repair (Document *document){
  return document->from_repair;
}

/**
 * d_document_get_from_invoice:
 * @document: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_document_get_from_invoice (Document *document){
  return document->from_invoice;
}

/**
 * d_document_duplicate:
 * @document: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el duplicado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Document *d_document_duplicate (Document *document){

    if(document!=NULL) {
	Document *result = d_document_create_empty ();
	gulong document_code = d_document_get_code (document);
	gchar *name = d_document_get_name (document);
	gchar *description = d_document_get_description (document);

	d_document_set_code (result, document_code);
	d_document_set_name (result, name);
	d_document_set_description (result, description);
	d_document_set_from_invoice (result, d_document_get_from_invoice (document));
	d_document_set_from_repair (result, d_document_get_from_repair (document));

	g_free(name);
	g_free(description);

	return result;
    } 

    return NULL;
  
}


/*Private functions*/

gint document_init(Document *document) {

	document->document_code = ULONG_MAX;
	document->from_repair   = FALSE;
	document->from_invoice  = FALSE;
	document->name = NULL;
	document->description = NULL;

        return 0;
}
