 /* incoming_book.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "incoming_book.h"
#include "participant.h"
#include "util.h"


#define D_EXPENSE_CONTENT_SIZE 7


static gint incoming_book_init(IncomingBook *incoming_book);


/**
 * d_incoming_book_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #IncomingBook vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
IncomingBook * d_incoming_book_create_empty () {

    IncomingBook * incoming_book;

    incoming_book = (IncomingBook *) g_malloc0(sizeof(IncomingBook));

    incoming_book_init(incoming_book);

    return incoming_book;
}



/**
 * d_incoming_book_set_participant:
 * @incoming_book: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el participante al que pertene la factura recibida.
 *
 */
void d_incoming_book_set_participant (IncomingBook * incoming_book, Participant * participant) {
    if (incoming_book->participant!= NULL) {
        d_participant_clear (incoming_book->participant);
    }
    incoming_book->participant = participant;
}

/**
 * d_incoming_book_set_total:
 * @incoming_book: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de las lneas de una factura agrupadas por el mismo tipo de Iva.
 *
 */
void d_incoming_book_set_total (IncomingBook * incoming_book, gdouble total) {
    incoming_book->total = total;
}

/**
 * d_incoming_book_set_total_summary:
 * @incoming_book: Objeto en el que se establece el atributo.
 * @total_summary: Atributo a insertar.
 *
 * Establece el atributo @total_summary en el objeto. El atributo @total_summary
 * indica si el objeto se utiliza para el resumen de totales de los gastos.
 *
 */
void d_incoming_book_set_total_summary (IncomingBook * incoming_book, gboolean total_summary) {
    incoming_book->total_summary =total_summary;
}


/**
 * d_incoming_book_get_total_summary:
 * @incoming_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_incoming_book_get_total_summary (IncomingBook * incoming_book) {
    return incoming_book->total_summary;
}


/**
 * d_incoming_book_get_participant:
 * @incoming_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_incoming_book_get_participant (IncomingBook * incoming_book) {
    return (incoming_book->participant);
}

/**
 * d_incoming_book_get_total:
 * @incoming_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_incoming_book_get_total (IncomingBook * incoming_book) {
    return (incoming_book->total);

}

/**
 * d_incoming_book_clear:
 * @incoming_book: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_incoming_book_clear (IncomingBook * incoming_book) {

    if (incoming_book!=NULL) {

	if (incoming_book->participant!=NULL) {
	    d_participant_clear (incoming_book->participant);
	}
	g_free(incoming_book);
	incoming_book = NULL;
    }

    return 0;
}


/**
 * d_incoming_book_array_to_content:
 * @array: Array de agrupaciones de facturas de abono que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar el total de objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_incoming_book_array_to_content (IncomingBook **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_incoming_book_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_incoming_book_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto libro de ingresos. Utiliza
 * los mtodos del participante.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_incoming_book_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_incoming_book_get_content_size ()+1);
    gint index = 0;

    d_participant_get_content_title (content, &index);
    index--;
    content[index++] = g_strdup_printf(_("COST"));
    content[index++] = NULL;
    return content;
}

/**
 * d_incoming_book_get_listing_title:
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_incoming_book_get_listing_title () {
    gchar *content = g_strdup_printf(_("INCOME DECLARATION"));
    return content;
}

/**
 * d_incoming_book_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados del libro de gastos.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_incoming_book_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_incoming_book_get_content_size ()+1);
    gint index = 0;

    d_participant_get_listing_columns_width (widths, &index);
    index--;
    widths[index++] = 7;
    widths[index++] = -1;

    return widths;
}

/**
 * d_incoming_book_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_incoming_book_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_incoming_book_get_content_size ()+1);
    gint index = 0;

    d_participant_get_listing_columns_alignment (orientation, &index);
    index--;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_incoming_book_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_incoming_book_get_content_size () {
    return (D_EXPENSE_CONTENT_SIZE+ d_participant_get_content_size ());
}



/**
 * d_incoming_book_get_content:
 * @incoming_book: Objeto #IncomingBook a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un objeto #IncomingBook en un array de cadenas de caracteres para aadir al listado
 * del libro de gastos.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_incoming_book_get_content (IncomingBook *incoming_book) {
    gchar **content = (gchar **)malloc(sizeof(gchar*) *
				       d_incoming_book_get_content_size ()+1);
    gint    index   = 0;

    if (d_incoming_book_get_total_summary (incoming_book)==FALSE) {
	content = d_participant_get_content (d_incoming_book_get_participant (incoming_book),
					 content, &index);
    } else {
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
        content[index++]=g_strdup("");
    }
    index--;
    if (d_incoming_book_get_total_summary (incoming_book)==FALSE)
	content[index++] = g_strdup_printf("%.2f", d_incoming_book_get_total (incoming_book));
    else {
	content[index++] = g_strdup_printf(_("TOTAL: %.2f"), d_incoming_book_get_total (incoming_book));
        content[index++]=g_strdup("");
    }
    content[index++] = NULL;

    return content;
}

/**
 * d_incoming_book_calculate_total_from_list:
 * @incoming_books: Array de objetos #IncomingBook del que calcularn los totales.
 *
 * Calcula la suma total de todos los elementos del array.
 *
 * Returns:
 * Una lista con un elemento que contiene el resumen de totales.
 */
IncomingBook **  d_incoming_book_calculate_total_from_list (IncomingBook ** incoming_books) {
    IncomingBook * incoming_book=NULL;
    IncomingBook ** result=(IncomingBook **) g_malloc0((sizeof(IncomingBook*)*2));
    gulong index=0;
    gdouble total_sum = 0;

    if (incoming_books!=NULL) {
	while (incoming_books[index]!=NULL) {
	    gdouble total=0;

	    total=d_incoming_book_get_total (incoming_books[index]);

            total_sum = total_sum + total;

	    index++;
	}
    }
    incoming_book = d_incoming_book_create_empty ();
    d_incoming_book_set_total_summary (incoming_book,TRUE);
    d_incoming_book_set_total (incoming_book,total_sum);
    result[0]=incoming_book;
    return result;
    
}

/**
 * d_incoming_book_concat:
 * @incoming_books: Primer array a concatenar.
 * @incoming_books2: Segundo array a concatenar.
 *
 * Concatena dos arrays de objetos gasto. #incoming_book2 ir en segundo lugar en la concatenacin.
 *
 * Returns:
 * EL nuevo array que contiene los dos arrays pasados como parmetros.
 */
IncomingBook ** d_incoming_book_concat (IncomingBook ** incoming_books,IncomingBook ** incoming_books2) {
    gulong size=0;
    gulong size2=0;
    gulong index=0;
    gulong index2=0;
    IncomingBook ** ib=NULL;
    if (incoming_books!=NULL) {
	while (incoming_books[size]!=NULL) {
	    size++;
	}


	if (incoming_books2!=NULL) {
	    while (incoming_books2[size2]!=NULL) {
		size2++;
	    }
	}
	ib = (IncomingBook **) g_malloc0(sizeof(IncomingBook*)*(size+size2+1));
	index = 0;
	while (index<(size)) {
	    ib[index]=incoming_books[index];
	    incoming_books[index]=NULL;
	    index++;
	    
	}

	index=size;
	while (index<(size+size2)) {
	    ib[index]=incoming_books2[index2];
	    incoming_books2[index2]=NULL;
	    index++;
	    index2++;
	}
        g_print("\n%lu\n",size+size2+1);

    }
    return ib;
}


/* Private functions */

static gint incoming_book_init(IncomingBook *incoming_book) {
    incoming_book->participant=NULL;
    incoming_book->total=-1;
    incoming_book->total_summary=FALSE;

    return 0;
}

