/* invoice.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "invoice.h"
#include "util.h"


#define D_INVOICE_CONTENT_SIZE 5

static gint invoice_init(Invoice *invoice);



/**
 * d_invoice_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Invoice vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Invoice * d_invoice_create_empty () {

    Invoice * invoice;

    invoice = (Invoice *) g_malloc0(sizeof(Invoice));

    invoice_init(invoice);

    return invoice;
}


/**
 * d_invoice_get_template_name:
 * @template_id: Identificador de plantilla.
 *
 * Devuelve el nombre de la plantilla, en base al cdigo pasdo como parametro.
 *
 * Returns:
 * Un objeto #(char *) con el nombe de la plantilla.
 */
gchar * d_invoice_get_template_name (gint template_id) {

	if (template_id == D_INVOICE_NORMAL_CODE) return g_strdup(D_INVOICE_MESSAGE_NORMAL);

	return g_strdup(D_INVOICE_MESSAGE_NORMAL);
}


/**
 * d_invoice_set_enterprise:
 * @invoice: Objeto en el que se establece el atributo.
 * @enterprise: Atributo a insertar.
 *
 * Establece el atributo @enterprise en el objeto. El atributo @enterprise
 * indica la empresa a la que pertenece la tienda en la que se
 * emite la factura.
 *
 */
void d_invoice_set_enterprise (Invoice * invoice,Enterprise * enterprise){
    invoice->enterprise = enterprise;
}

/**
 * d_invoice_set_shop:
 * @invoice: Objeto en el que se establece el atributo.
 * @shop: Atributo a insertar.
 *
 * Establece el atributo @shop en el objeto. El atributo @shop
 * indica la tienda en la que se emite la factura
 *
 */
void d_invoice_set_shop (Invoice * invoice,Shop * shop){
    invoice->shop=shop;
}
/**
 * d_invoice_set_invoice_number:
 * @invoice: Objeto en el que se establece el atributo.
 * @invoice_number: Atributo a insertar.
 *
 * Establece el atributo @invoice_number en el objeto. El atributo @invoice_number
 * indica el nmero de la factura. El nmero de la factura es siempre el anterior ms uno.
 * Los nmeros de factura se almacenan en la base de datos en una tabla llamada NumerosCorrelativosFactura
 * que indica los valores actuales de las facturas, y en cada insercin de facturas se le suma uno
 * (en caso de insercin correcta).
 *
 */
void d_invoice_set_invoice_number (Invoice * invoice,gulong invoice_number){
    invoice->invoice_number=invoice_number;
}

/**
 * d_invoice_set_invoice_discount:
 * @invoice: Objeto en el que se establece el atributo.
 * @invoice_discount: Atributo a insertar.
 *
 * Establece el atributo @invoice_discount en el objeto. El atributo @invoice_discount
 * indica el descuento que se aplicar a la base imponible de la factura.
 *
 */
void d_invoice_set_invoice_discount (Invoice * invoice,gdouble invoice_discount){
    if (invoice_discount>100)
	invoice->invoice_discount=100;
    else if (invoice_discount<0)
	invoice->invoice_discount=0;
    else
	invoice->invoice_discount=invoice_discount;

}

/**
 * d_invoice_set_participant:
 * @invoice: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el pagador de la factura.
 *
 */
void d_invoice_set_participant (Invoice * invoice,Participant * participant){
    invoice->participant=participant;
}
/**
 * d_invoice_set_payment:
 * @invoice: Objeto en el que se establece el atributo.
 * @payment: Atributo a insertar.
 *
 * Establece el atributo @payment en el objeto. El atributo @payment
 * indica el pago relacionado con la factura de abono.
 *
 */
void d_invoice_set_payment (Invoice * invoice,Payment * payment){
    if (invoice->payment!=NULL) {
	d_payment_clear (invoice->payment);
        invoice->payment=NULL;
    }
    invoice->payment=payment;
}
/**
 * d_invoice_set_total:
 * @invoice: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de la factura con IVA INCLUIODO. OJO!! se mantiene este nombre por
 * compatibilidad con versiones antiguas que mantenan en este campo la base imponible.
 *
 */
void d_invoice_set_total (Invoice * invoice,gdouble total){
    invoice->total = total;
}

/**
 * d_invoice_set_modified_total:
 * @invoice: Objeto en el que se establece el atributo.
 * @modified_total: Atributo a insertar.
 *
 * Establece el atributo @modified_total en el objeto. El atributo @modified_total
 * indica si la factura se ha realizado por mdulo pactado.
 *
 */
void d_invoice_set_modified_total (Invoice * invoice,gboolean modified_total){
    invoice->modified_total = modified_total;
}

/**
 * d_invoice_set_vat_type:
 * @invoice: Objeto en el que se establece el atributo.
 * @vat_type: Atributo a insertar.
 *
 * Establece el atributo @vat_type en el objeto. El atributo @vat_type
 * se utiliza para indicar si la factura e con iva o es sin iva.
 *
 */
void d_invoice_set_vat_type (Invoice * invoice,gdouble vat_type) {
    invoice->vat_type = vat_type;
}
/**
 * d_invoice_set_invoice_code:
 * @invoice: Objeto en el que se establece el atributo.
 * @invoice_code: Atributo a insertar.
 *
 * Establece el atributo @invoice_code en el objeto. El atributo @invoice_code
 * indica el cdigo interno que identifica la factura.
 *
 */
void d_invoice_set_invoice_code (Invoice * invoice,gulong invoice_code) {
    invoice->invoice_code = invoice_code;
}


/**
 * d_invoice_set_issue_date:
 * @invoice: Objeto en el que se establece el atributo.
 * @issue_date: Atributo a insertar.
 *
 * Establece el atributo @issue_date en el objeto. El atributo @issue_date
 * indica la fecha en la que se emiti la factura.
 *
 */
void d_invoice_set_issue_date (Invoice * invoice,time_t issue_date) {
    invoice->issue_date=issue_date;
}

/**
 * d_invoice_set_state:
 * @invoice: Objeto en el que se establece el atributo.
 * @state: Atributo a insertar.
 *
 * Establece el atributo @state en el objeto. El atributo @state
 * indica el estado actual de la factura: D_INVOICE_NOT_ISSUED, D_INVOICE_NOT_PAID, PAGADA o
 * D_INVOICE_MANAGING (estado especial para las facturas enviadas a ck o pagadas por domiciliacin).
 *
 *
 */
void d_invoice_set_state (Invoice * invoice,gint state) {
    invoice->state=state;
}

/**
 * d_invoice_set_repair:
 * @invoice: Objeto en el que se establece el atributo.
 * @repair: Atributo a insertar.
 * @free_repair: Indica si queremos liberar previamente la reparacin.
 *
 * Establece el atributo @ en el objeto. El atributo @repair
 * indica la reparacin para la que se realiz la factura. Si
 * la reparacacin es %NULL entonces la factura se cre a partir
 * de albaranes.
 *
 */
void d_invoice_set_repair (Invoice * invoice,Repair * repair,gboolean free_repair) {
    if (free_repair!=FALSE) {
	if (invoice->repair!=NULL) {
	    d_repair_clear (invoice->repair);
	    invoice->repair=NULL;
	}
    }
    invoice->repair=repair;
}


/**
 * d_invoice_set_delivery_payments:
 * @invoice: Objeto en el que se establece el atributo.
 * @delivery_payments: Atributo a insertar.
 *
 * Establece el atributo @delivery_payments en el objeto. El atributo @delivery_payments
 * indica los albaranes a partir de los cuales se gener la factura. Si este atributo
 * es %NULL entonces la factura se cre directamente a partir de una reparacin.
 *
 */
void d_invoice_set_delivery_payments (Invoice * invoice,DeliveryPayment ** delivery_payments){
    invoice->delivery_payments = delivery_payments;
}
/**
 * d_invoice_set_enterprise_code_split_invoice:
 * @invoice: Objeto en el que se establece el atributo.
 * @enterprise_code_split_invoice: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code_split_invoice en el objeto. El atributo @enterprise_code_split_invoice
 * indica la empresa a la que pertenece la tienda en la que se desea emitir la refactura.
 *
 */
void d_invoice_set_enterprise_code_split_invoice (Invoice * invoice,gulong enterprise_code_split_invoice) {
    invoice->enterprise_code_split_invoice = enterprise_code_split_invoice;
}

/**
 * d_invoice_set_repair_code_split_invoice:
 * @invoice: Objeto en el que se establece el atributo.
 * @repair_code_split_invoice: Atributo a insertar.
 *
 * Establece el atributo @repair_code_split_invoice en el objeto. El atributo @repair_code_split_invoice
 * indica el cdigo de la reparacin que referencia la factura en caso de que la factura cargada
 * sea una refactura.
 *
 */
void d_invoice_set_repair_code_split_invoice (Invoice * invoice,gulong repair_code_split_invoice) {
    invoice->repair_code_split_invoice = repair_code_split_invoice;
}

/**
 * d_invoice_set_split_invoice:
 * @invoice: Objeto en el que se establece el atributo.
 * @split_invoice: Atributo a insertar.
 *
 * Establece el atributo @split_invoice en el objeto. El atributo @split_invoice
 * indica si la factura es una refactura o no.
 *
 */
void d_invoice_set_split_invoice (Invoice * invoice,gboolean split_invoice) {
    invoice->split_invoice = split_invoice;
}

/**
 * d_invoice_set_garage_is_receiver:
 * @invoice: Objeto en el que se establece el atributo.
 * @garage_is_receiver: Atributo a insertar.
 *
 * Establece el atributo @garage_is_receiver en el objeto. El atributo @garage_is_receiver
 * indica si la factura se realiz para un taller o no. Es importante
 * distinguir estas facturas ya que podran tener un trato diferente
 * respecto a las lneas de reparacin.
 *
 */
void d_invoice_set_garage_is_receiver (Invoice * invoice,gboolean garage_is_receiver) {
    invoice->garage_is_receiver = garage_is_receiver;
}

/**
 * d_invoice_set_shop_code_split_invoice:
 * @invoice: Objeto en el que se establece el atributo.
 * @shop_code_split_invoice: Atributo a insertar.
 *
 * Establece el atributo @shop_code_split_invoice en el objeto. El atributo @shop_code_split_invoice
 * indica el cdigo de la tienda en la que se quiere emitir la refactura.
 *
 */
void d_invoice_set_shop_code_split_invoice (Invoice * invoice,gulong shop_code_split_invoice) {
    invoice->shop_code_split_invoice = shop_code_split_invoice;
}

/**
 * d_invoice_get_enterprise:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Enterprise * d_invoice_get_enterprise (Invoice * invoice){
    return invoice->enterprise;
}

/**
 * d_invoice_get_shop:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Shop * d_invoice_get_shop (Invoice * invoice){
    return invoice->shop;
}

/**
 * d_invoice_get_invoice_number:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_invoice_get_invoice_number (Invoice * invoice){
    return invoice->invoice_number;
}

/**
 * d_invoice_get_invoice_number:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el nmero de la factura completo.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_invoice_get_invoice_number_with_group_code (Invoice * invoice) {
    gchar * numero =NULL;

    if (d_invoice_get_invoice_number(invoice)!=ULONG_MAX) {
	gchar * group_code = d_shop_get_group_code (d_invoice_get_shop (invoice));
	gchar * invoice_number_char = g_strdup_printf("%lu",d_invoice_get_invoice_number(invoice));


	if (group_code!=NULL) {
	    numero = g_strconcat(group_code," ",invoice_number_char,NULL);
	    g_free(invoice_number_char);
	} else {
	    numero = invoice_number_char;
	    invoice_number_char = NULL;
	}


	g_free(group_code);
    }
    
    return numero;

}

/**
 * d_invoice_get_invoice_discount:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_invoice_get_invoice_discount (Invoice * invoice){
    return invoice->invoice_discount;
}

/**
 * d_invoice_get_state:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_invoice_get_state (Invoice * invoice){
    return invoice->state;
}

/**
 * d_invoice_get_participant:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_invoice_get_participant (Invoice * invoice){
    return invoice->participant;
}

/**
 * d_invoice_get_payment:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Payment * d_invoice_get_payment (Invoice * invoice){
    return invoice->payment;
}

/**
 * d_invoice_get_repair:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Repair * d_invoice_get_repair (Invoice * invoice){
    return invoice->repair;
}


/**
 * d_invoice_get_repair:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve la reparacin del primer albarn o, si no existe, la reparacin de la factura.
 *
 * Returns: El valor del atributo solicitado.
 */
Repair * d_invoice_get_first_repair (Invoice * invoice) {

    DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(invoice);
    Repair * repair = NULL;
    if (delivery_payments!=NULL) {
	if (delivery_payments[0]!=NULL) {
	    repair = d_delivery_payment_get_repair (delivery_payments[0]);
	}
    } else if (d_invoice_get_repair(invoice)!=NULL) {
	repair = d_invoice_get_repair (invoice);
    }

    return repair;
}


/**
 * d_invoice_get_issue_date:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_invoice_get_issue_date (Invoice * invoice){
    return invoice->issue_date;

}

/**
 * d_invoice_get_delivery_payments:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
DeliveryPayment ** d_invoice_get_delivery_payments (Invoice * invoice){
    return invoice->delivery_payments;
}

/**
 * d_invoice_get_total:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_invoice_get_total (Invoice * invoice) {
    return invoice->total;
}

/**
 * d_invoice_get_modified_total:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_invoice_get_modified_total (Invoice * invoice){
    return invoice->modified_total;
}

/**
 * d_invoice_get_vat_type:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_invoice_get_vat_type (Invoice * invoice) {
    return invoice->vat_type;
}


/**
 * d_invoice_get_invoice_code:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_invoice_get_invoice_code (Invoice * invoice) {
    return invoice->invoice_code;
}



/**
 * d_invoice_get_shop_code_split_invoice:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_invoice_get_shop_code_split_invoice (Invoice * invoice) {
    return invoice->shop_code_split_invoice;
}


/**
 * d_invoice_get_enterprise_code_split_invoice:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_invoice_get_enterprise_code_split_invoice (Invoice * invoice) {
    return invoice->enterprise_code_split_invoice;
}

/**
 * d_invoice_get_repair_code_split_invoice:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_invoice_get_repair_code_split_invoice (Invoice * invoice) {
    return invoice->repair_code_split_invoice;
}

/**
 * d_invoice_get_split_invoice:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_invoice_get_split_invoice (Invoice * invoice) {
    return invoice->split_invoice;
}


/**
 * d_invoice_get_garage_is_receiver:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_invoice_get_garage_is_receiver (Invoice * invoice) {
    return invoice->garage_is_receiver;
}


gboolean d_invoice_get_is_from_garage (Invoice * invoice) {
    gboolean result = FALSE ;
    Repair * repair = d_invoice_get_first_repair(invoice);

    if (d_repair_get_garage (repair)!=NULL) {
	Garage * garage = d_repair_get_garage(repair);
	if (garage!=NULL) {
	    Participant * participant = d_garage_get_participant(garage);
	    if (d_participant_get_code(participant)!=ULONG_MAX)
                result=TRUE;
	}
    }

    return result;
}

/**
 * d_invoice_get_with_insurance:
 * @invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve si la factura se realiz a partir de una reparacin con seguro o de un albarn que contenga
 * una reparacin con seguro.
 *
 * Returns:
 * %TRUE si es una factura con un seguro en la reparacin, %FALSE en otro caso.
  */
gboolean d_invoice_get_with_insurance (Invoice * invoice) {
    Repair * repair = NULL;
    gboolean result = FALSE;
    DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(invoice);
    if (delivery_payments!=NULL) {
	if (delivery_payments[0]!=NULL) {
	    repair = d_delivery_payment_get_repair(delivery_payments[0]);
	}
    } else {
        repair = d_invoice_get_repair(invoice);
    }

    result = d_repair_is_with_insurance(repair);

    return result;
}


/**
 * d_invoice_clear:
 * @invoice: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_invoice_clear (Invoice * invoice) {

    gint index=0;

    if (invoice!=NULL) {
	d_enterprise_clear (invoice->enterprise);
	d_shop_clear (invoice->shop);
	d_participant_clear (invoice->participant);
	d_payment_clear (invoice->payment);

	if (invoice->delivery_payments!=NULL) {
	    while (invoice->delivery_payments[index]!=NULL)
	    d_delivery_payment_clear (invoice->delivery_payments[index++]);
	}

	d_repair_clear (invoice->repair);
	g_free(invoice);

    }
    return 0;
}

/**
 * d_invoice_to_payment_note:
 * @invoice: Factura a transformar.
 *
 * Transforma una factura de abono en una nota de abono.
 *
 * Returns:
 * Una nota con los datos de la factura. Si la factura se obtuvo a partir de una reparacin la nota la referenciar
 * y si la factura se realiz a partir de albaranes solo se obtendr la primera reparacin de la factura.
 */
PaymentNote * d_invoice_to_payment_note (Invoice * invoice) {
    PaymentNote * payment_note = d_payment_note_create_empty ();
    if (d_invoice_get_repair (invoice)!=NULL) {
	d_payment_note_set_repair_detail (payment_note,d_invoice_get_repair (invoice));
	invoice->repair=NULL;
    }
    else
	if (invoice->delivery_payments!=NULL) {
	    if (invoice->delivery_payments[1]==NULL) {
		d_payment_note_set_repair_detail (payment_note,d_delivery_payment_get_repair (invoice->delivery_payments[0]));
                invoice->delivery_payments[0]->repair=NULL;
	    }
	}
    d_payment_note_set_payment(payment_note,d_invoice_get_payment (invoice));
    d_payment_note_set_emission_date(payment_note,d_invoice_get_issue_date (invoice));

    invoice->payment=NULL;
    d_invoice_clear (invoice);
    return payment_note;
}

/**
 * d_invoice_calculate_total:
 * @base: Puntero a gdouble que almacenar la base imponible calculada.
 * @vat: Puntero a gdouble que almacenar el total de iva calculado de la factura.
 * @total: Puntero a gdouble que almacenar el total calculado de la factura.
 * @invoice: Factura a partir de la cual se realizar el clculo.
 * @set_on_invoice: Decide si guarda en la factura los nuevos datos calculados o solo son de consulta.
 *
 * Realiza el clculo de los totales de una factura. 
 * Para realizar los clculos se comprueba primero si la factura es una factura que se deba cobrar por
 * mdulo pactado (depende de los seguros) o no. Si no es una factura por mdulo pactado se comprueba
 * si la factura referencia a una reparacin o a varios albaranes. En el caso de una reparacin se recorren
 * las lneas de la reparacin comprobando si existe un total de lnea o un precio del producto para calcular
 * el total de la lnea final y se calcula la suma de todas las lneas. En el caso de una lista de albaranes
 * se recorren los albarares para obtener las reparaciones, de las cuales se calculan los
 * totales por lnea de la reparacin del albaran como antes. Por otro lado si la factura es por mdulo pactado se comprueba si la reparacin
 * de la que proviene la factura es sin seguro o si proviene de varias reparaciones (implican reparacin sin seguro)
 * para ver si se le aplica el descuento de la factura o no. El modulo pactado solo aplica el iva sobre una
 * base imponible introducida de antemano, aplicandole el descuento o no segn lo anterior.
 *
 */
void d_invoice_calculate_total (gdouble * base,gdouble * vat,
					      gdouble * total,Invoice * invoice,gboolean set_on_invoice) {

    gint aux_count=0;
//    gdouble discount=0;
    gdouble invoice_discount=0;
//    gdouble price=0;
    gdouble vat_type=0;
    gdouble invoice_vat=0;
    gdouble total_invoice=0;
    gdouble subtotal_invoice=0;
    gdouble line_total=0;
    gdouble vat_total=0;

    /*Obtenemos el descuento de la factura*/
    invoice_discount = d_invoice_get_invoice_discount (invoice);

    /*Utilizado para comprobar si la factura es con o sin iva*/
    if (d_invoice_get_vat_type (invoice)==-1) invoice_vat=1.0;

    /*No puede existir un descuento mayor del 100% o menor del 0%*/
    if ((invoice_discount>100) || (invoice_discount<0)) invoice_discount=0;

    /*Comprobacin de factura por modulo pactado*/
    if (d_invoice_get_modified_total (invoice)==FALSE) {
	if (invoice->delivery_payments!=NULL) {
	    /*CASO 1: La factura proviene de al menos un albarn y recorremos la lista de albaranes*/
	    while (invoice->delivery_payments[aux_count]!=NULL) {
		DeliveryPayment * dP = invoice->delivery_payments[aux_count];
		if (dP->repair->repair_line!=NULL) {
		    gint aux_count2=0;
		    /*Se recorren la lneas de la reparacin calculando el total por lnea*/
		    while (dP->repair->repair_line[aux_count2]) {

			Repair_line * repair_line = dP->repair->repair_line[aux_count2];

			vat_type = invoice_vat * (d_repair_line_get_historical_vat_type (repair_line));

			line_total = d_repair_line_calculate_total(repair_line);
			if (line_total==G_MINDOUBLE) {
                            line_total = 0;
			}
			line_total = u_round_to_decimal (line_total,2);

			/*Se calcula el total por iva (si es sin iva la factura tipoIVA va a ser 0)*/
			vat_total = vat_total + (((vat_type*1.0)/100)*line_total);

			subtotal_invoice = subtotal_invoice + line_total;
			total_invoice = total_invoice + line_total + (((vat_type*1.0)/100)*line_total);
			aux_count2++;
		    }
		}
		aux_count++;
	    }

	} else {
	    /*CASO 2: La factura proviene de una reparacin directamente. Se acta como en el caso de
	     que no haya totalTaller en el albarn*/
            Repair * repair = d_invoice_get_repair (invoice);
	    gint aux_count2=0;
	    while (repair->repair_line[aux_count2]!=NULL) {
		Repair_line * repair_line = repair->repair_line[aux_count2];

		vat_type = invoice_vat *  (d_repair_line_get_historical_vat_type(repair_line));

		line_total = d_repair_line_calculate_total(repair_line);
		if (line_total==G_MINDOUBLE) {
		    line_total = 0;
		}
		line_total = u_round_to_decimal (line_total,2);

		/*Se calcula el total por iva (si es sin iva la factura tipoIVA va a ser 0)*/
		vat_total = vat_total + (((vat_type*1.0)/100)*line_total);



		subtotal_invoice = subtotal_invoice + line_total;
		total_invoice = total_invoice + line_total + (((vat_type*1.0)/100)*line_total);
		aux_count2++;
	    }

	}

 
        /*Se calculan los totales*/
	*base = subtotal_invoice-(((1.0*invoice_discount)/100)*subtotal_invoice);
        *base = u_round_to_decimal (*base,2);
	*vat = vat_total-(((1.0*invoice_discount)/100)*vat_total);
        *vat = u_round_to_decimal (*vat,2);
        *total = *base + *vat;

        if (set_on_invoice!=FALSE)
	    d_invoice_set_total (invoice,*total);
    } else {
	/*CASO 3: La factura es mediante modulo pactado*/
        /*Se comprueba si es con seguro o no*/
        gboolean con_seguro = d_invoice_get_with_insurance (invoice);

	/*Se obtiene el total almacenado, pero es con iva y descuentos aplicados as que hai que realizar el
	  clculo a la inversa.*/
	*total = d_invoice_get_total (invoice);

	g_print(_("\nINSURANCE: %d\n"),con_seguro);
	if (con_seguro!=FALSE) {
	    /*Con seguro se obvia el descuento y la base imponible se calcula solo quitando el IVA*/
	    if (invoice_vat!=0) {
		*base = (*total)/1.16;
		*base = u_round_to_decimal (*base,2);
		*vat =  (*base)*((16.0)/100.0) ;
		*vat = u_round_to_decimal (*vat,2);
	    } else {
		*base = *total;
		*vat = 0.0;
	    }
	} else {
	    /*Sin seguro*/
	    /*Sin seguro se obtiene el descuento y la base imponible se calcula quitando el IVA y sumando el
	     descuento que se le quit a la base imponible*/
	    gdouble dto = d_invoice_get_invoice_discount (invoice);
	    if ((dto>=100) || (dto<0))
		dto = 0;
	    if (invoice_vat!=0) {
                gdouble total_dto;
		*base = ((*total)/1.16) ;
		*base = u_round_to_decimal (*base,2);
		g_print(_("\nBASE: %.2f\n"),*base);
		total_dto= ((*base)/(1.0-(dto/100.0)));
		g_print(_("\nDiscount: %.2f\n"),total_dto);
		//      	*base = u_round_to_decimal (*base,2) + u_round_to_decimal (total_dto,2);
		*base = u_round_to_decimal (total_dto,2);
		*vat =  (*base)*((16.0)/100.0) ;
		*vat = u_round_to_decimal (*vat,2);
	    } else {
                gdouble total_dto = ((*total)/(1.0-(dto/100.0)));
		*base = u_round_to_decimal (total_dto,2);
		*vat = 0.0;
	    }
	}
    }

    *base = u_round_to_decimal (*base,2);
    *vat = u_round_to_decimal (*vat,2);
    *total = u_round_to_decimal (*total,2);

}

/**
 * d_invoice_calculate_total_to_garage:
 * @base: Puntero a gdouble que almacenar la base imponible calculada.
 * @vat: Puntero a gdouble que almacenar el total de iva calculado de la factura.
 * @total: Puntero a gdouble que almacenar el total calculado de la factura.
 * @invoice: Factura a partir de la cual se realizar el clculo.
 * @set_on_invoice: Decide si guarda en la factura los nuevos datos calculados o solo son de consulta.
 *
 * Realiza el clculo de los totales de la factura para un taller. Este clculo es diferente del clculo de
 * las facturas normales ya que los albaranes pueden tener dos tratos diferentes, segn el taller
 * aparezca como cliente de la reparacin o como taller de la misma al que hai que pagarle una comisin.
 * Para realizar los clculos se comprueba primero si la factura es una factura que se deba cobrar por
 * mdulo pactado (depende de los seguros) o no. Si no es una factura por mdulo pactado se comprueba
 * si la factura referencia a una reparacin o a varios albaranes. En el caso de una reparacin se recorren
 * las lneas de la reparacin comprobando si existe un total de lnea o un precio del producto para calcular
 * el total de la lnea final y se calcula la suma de todas las lneas. En el caso de una lista de albaranes
 * se buscan albaranes en los que el taller no sea el cliente. Si el taller es el cliente se calculan los
 * totales por lnea de la reparacin del albaran como antes. Se el taller actu como taller en la reparacin
 * se comprueba el total del albarn que se le pagar al taller y se pone como total por dicho albarn, y se
 * pasa al siguiente. Por otro lado si la factura es por mdulo pactado se comprueba si la reparacin
 * de la que proviene la factura es sin seguro o si proviene de varias reparaciones (implican reparacin sin seguro)
 * para ver si se le aplica el descuento de la factura o no. El modulo pactado solo aplica el iva sobre una
 * base imponible introducida de antemano, aplicandole el descuento o no segn lo anterior.
 *
 */
void d_invoice_calculate_total_to_garage (gdouble * base,gdouble * vat,
					      gdouble * total,Invoice * invoice,gboolean set_on_invoice) {

    gint aux_count=0;
//    gdouble discount;
    gdouble invoice_discount;
//    gdouble price;
    gdouble vat_type=0;
    gdouble total_invoice=0;
    gdouble subtotal_invoice=0;
    gdouble totalTaller=0;
    gdouble line_total=0;
    gdouble vat_total=0;
    gdouble invoice_vat=0;

    /*Obtenemos el descuento de la factura*/
    invoice_discount = d_invoice_get_invoice_discount (invoice);
    /*Utilizado para comprobar si la factura es con o sin iva*/
    if (d_invoice_get_vat_type (invoice)==-1) invoice_vat=1.0;
    /*No puede existir un descuento mayor del 100% o menor del 0%*/
    if ((invoice_discount>100) || (invoice_discount<0)) invoice_discount=0;
    /*Comprobacin de factura por modulo pactado*/
    if (d_invoice_get_modified_total (invoice)==FALSE) {
	if (invoice->delivery_payments!=NULL) {
	    /*CASO 1: La factura proviene de al menos un albarn y recorremos la lista de albaranes*/
	    while (invoice->delivery_payments[aux_count]!=NULL) {
		DeliveryPayment * dP = invoice->delivery_payments[aux_count];
		/*Comisin que se le pagar al taller como comisin. El hecho de que exista un total taller
		  implica que el albarn tiene como cliente alguien distinto del taller */
                totalTaller = d_delivery_payment_get_garage_total (invoice->delivery_payments[aux_count]);
		if (totalTaller<=0) {
                    /*La reparacion del albarn tiene como cliente al taller*/
		    if (dP->repair->repair_line!=NULL) {
			gint aux_count2=0;
                        /*Se recorren la lneas de la reparacin calculando el total por lnea*/
			while (dP->repair->repair_line[aux_count2]) {
			    Repair_line * repair_line = dP->repair->repair_line[aux_count2];

			    /*Se calcula iva en caso de que la factura tenga en cuenta el iva*/
			    vat_type = invoice_vat * (d_repair_line_get_historical_vat_type(repair_line));

         		    line_total = d_repair_line_calculate_total(repair_line);
			    if (line_total==G_MINDOUBLE) {
				line_total = 0;
			    }
			    line_total = u_round_to_decimal (line_total,2);

			    vat_total = vat_total + (((vat_type*1.0)/100)*line_total);


			    subtotal_invoice = subtotal_invoice + line_total;
			    total_invoice = total_invoice + line_total + (((vat_type*1.0)/100)*line_total);
			    aux_count2++;
			}
		    }
		} else {
		    /*Si posteriormente se cambia, e totalTaller o mete en negativo hai que cambiar o signo*/
		    /*Resta a los totales de factura el total taller como si fuese ese el total del albarn*/
                    /*Ya que el total de taller se le paga al taller*/
		    subtotal_invoice = subtotal_invoice - totalTaller;
		    if (invoice_vat!=0) {
			vat_total = vat_total - (((16*1.0)/100)*totalTaller);
			total_invoice = total_invoice - totalTaller - (((16*1.0)/100)*totalTaller);
		    } else {
			total_invoice = total_invoice - totalTaller;
		    }
		}
		aux_count++;
	    }
	} else {
	    /*CASO 2: La factura proviene de una reparacin directamente. Se acta como en el caso de
	     que no haya totalTaller en el albarn*/
            Repair * repair = d_invoice_get_repair (invoice);
	    if (repair->repair_line!=NULL) {
		gint aux_count2=0;
		/*Se recorren la lneas de la reparacin calculando el total por lnea*/
		while (repair->repair_line[aux_count2]) {
 		    Repair_line * repair_line = repair->repair_line[aux_count2];
		    /*se obtiene el precio de la lnea*/
		    line_total = d_repair_line_calculate_total(repair_line);
		    if (line_total==G_MINDOUBLE) {
			line_total = 0;
		    }
		    line_total = u_round_to_decimal (line_total,2);

		    /*Se calcula el total por iva (si es sin iva la factura tipoIVA va a ser 0)*/
		    vat_total = vat_total + (((vat_type*1.0)/100)*line_total);

		    subtotal_invoice = subtotal_invoice + line_total;
		    total_invoice = total_invoice + line_total + (((vat_type*1.0)/100)*line_total);
		    aux_count2++;
		}
	    }
	}
        /*Se calculan los totales*/
	*base = subtotal_invoice-(((1.0*invoice_discount)/100)*subtotal_invoice);
        *base = u_round_to_decimal (*base,2);
	*vat = vat_total-(((1.0*invoice_discount)/100)*vat_total);
        *vat = u_round_to_decimal (*vat,2);
        *total = *base + *vat;
        if (set_on_invoice!=FALSE)
	    d_invoice_set_total (invoice,*total);
    } else {
	/*CASO 3: La factura es mediante modulo pactado*/
        /*Se comprueba si es con seguro o no*/
	gboolean con_seguro = d_invoice_get_with_insurance (invoice);


	/*Se obtiene el total almacenado, pero es con iva y descuentos aplicados as que hai que realizar el
	  clculo a la inversa.*/
	*total = d_invoice_get_total (invoice);

	if (con_seguro!=FALSE) {
	    /*Con seguro se obvia el descuento y la base imponible se calcula solo quitando el IVA*/
	    if (invoice_vat!=0) {
		*base = (*total)/1.16;
		*base = u_round_to_decimal (*base,2);
		*vat =  (*base)*((16.0)/100.0) ;
		*vat = u_round_to_decimal (*vat,2);
	    } else {
		*base = *total;
		*vat = 0;
	    }
	} else {
	    /*Sin seguro*/
	    /*Sin seguro se obtiene el descuento y la base imponible se calcula quitando el IVA y sumando el
	     descuento que se le quit a la base imponible*/
	    gdouble dto = d_invoice_get_invoice_discount (invoice);
	    if ((dto>=100) || (dto<0))
		dto = 0;
	    if (invoice_vat!=0) {
                gdouble total_dto;
		*base = ((*total)/1.16) ;
		*base = u_round_to_decimal (*base,2);
		g_print(_("\nBASE: %.2f\n"),*base);
		total_dto= ((*base)/(1.0-(dto/100.0)));
		g_print(_("\nDiscount: %.2f\n"),total_dto);
		//      	*base = u_round_to_decimal (*base,2) + u_round_to_decimal (total_dto,2);
		*base = u_round_to_decimal (total_dto,2);
		*vat =  (*base)*((16.0)/100.0) ;
		*vat = u_round_to_decimal (*vat,2);
	    } else {
                gdouble total_dto = ((*total)/(1.0-(dto/100.0)));
		*base = u_round_to_decimal (total_dto,2);
		*vat = 0.0;
	    }
	}

    }
    *base = u_round_to_decimal (*base,2);
    *vat = u_round_to_decimal (*vat,2);
    *total = u_round_to_decimal (*total,2);

}

/**
 * d_invoice_array_to_content:
 * @array: Array de facturas de abono que se transformarn en listables.
 * @invoice_total: Factura vaca que contendr el total del listado.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_invoice_array_to_content (Invoice **array,Invoice * invoice_total, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) g_malloc0(sizeof(gchar **)*array_size+2);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_invoice_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = d_invoice_get_content (invoice_total);
    (*reccount)++;

    content[*reccount] = NULL;

    return content;

}

/**
 * d_invoice_array_to_content_by_company:
 * @array: Array de facturas de abono que se transformarn en listables.
 * @invoice_total: Factura vaca que contendr el total del listado.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel. Este listado
 * posee la caracterstica que saca una lnea por compaa de seguros, de modo que segn la compaa de
 * seguros imprime el total de la factura.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_invoice_array_to_content_by_company (Invoice **array,Invoice * invoice_total, gulong *reccount) {
    gint     array_size = 0;
    gint number_of_companies=0;
    gulong next_company_code=ULONG_MAX;
    gulong actual_company_code=ULONG_MAX;
    gulong previous_company_code=ULONG_MAX;
    gchar ***content = NULL;
    gint index_of_content=0;
    gdouble * total_by_company=(gdouble *)g_malloc0(sizeof(gdouble));

    *reccount = 0;
    while(array[array_size]!=NULL) {
        actual_company_code=ULONG_MAX;
	if (array[array_size]->delivery_payments!=NULL) {
	    Company * company=d_repair_get_company (d_delivery_payment_get_repair (array[array_size]->delivery_payments[0]));
	    if (company!=NULL) {
		if (d_company_get_participant (company)!=NULL) {
		    actual_company_code = d_participant_get_code (d_company_get_participant (company));
		}
	    }
	} else {
	    Company * company=d_repair_get_company (array[array_size]->repair);
	    if (company!=NULL) {
		if (d_company_get_participant (company)!=NULL) {
		    actual_company_code = d_participant_get_code (d_company_get_participant (company));
		}
	    }
	}
	if (actual_company_code!=previous_company_code)
	    number_of_companies++;

        previous_company_code = actual_company_code;
	array_size++;
    }

    content = (gchar ***) g_malloc0(sizeof(gchar **)*(array_size+number_of_companies+2));

    previous_company_code=ULONG_MAX;
    index_of_content=0;
    while(array[*reccount] != NULL) {
	actual_company_code=next_company_code;
	next_company_code=ULONG_MAX;


	if (array[*reccount]!=NULL) {
	    if (array[*reccount]->delivery_payments!=NULL) {
		Company * company=d_repair_get_company (d_delivery_payment_get_repair (array[*reccount]->delivery_payments[0]));
		if (company!=NULL) {
		    if (d_company_get_participant (company)!=NULL) {
			next_company_code = d_participant_get_code (d_company_get_participant (company));
		    }
		}
	    } else {
		Company * company=d_repair_get_company (array[*reccount]->repair);
		if (company!=NULL) {
		    if (d_company_get_participant (company)!=NULL) {
			next_company_code = d_participant_get_code (d_company_get_participant (company));
		    }
		}
	    }



	    if ((actual_company_code!=next_company_code) && (*reccount!=0)) {
		content[index_of_content] = d_invoice_to_content_by_company (array[*reccount-1],total_by_company,TRUE);
		*total_by_company = 0;
                index_of_content++;
		
	    }
	    content[index_of_content] = d_invoice_to_content_by_company (array[*reccount],total_by_company,FALSE);
	    (*reccount)++;
            index_of_content++;
	}


    }

    content[index_of_content] = d_invoice_to_content_by_company (array[*reccount-1],total_by_company,TRUE);
    *total_by_company = 0;
    index_of_content++;
    content[index_of_content] = d_invoice_to_content_by_company (invoice_total,total_by_company,FALSE);
    (*reccount)++;
//    index_of_content++;
//    content[index_of_content] = NULL;

    return content;

}

/**
 * d_invoice_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto factura emitida. 
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_invoice_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_invoice_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup_printf(_("ISSUE DATE"));
    content[index++] = g_strdup_printf(_("INVOICE No."));
    content[index++] = g_strdup_printf(_("CLIENT"));
    content[index++] = g_strdup_printf(_("COMPANY"));
    content[index++] = g_strdup_printf(_("TOTAL"));
    content[index++] = NULL;
    return content;
}

/**
 * d_invoice_get_listing_title:
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_invoice_get_listing_title () {
    gchar *content = g_strdup_printf(_("ISSUED INVOICE LISTING"));
    return content;
}

/**
 * d_invoice_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados facturas emitidas.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_invoice_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_invoice_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 15;
    widths[index++] = 10;
    widths[index++] = 35;
    widths[index++] = 25;
    widths[index++] = 15;
    widths[index++] = -1;

    return widths;
}

/**
 * d_invoice_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_invoice_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_invoice_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_invoice_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_invoice_get_content_size () {
    return D_INVOICE_CONTENT_SIZE;
}

/**
 * d_invoice_get_content:
 * @invoice: Objeto factura a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un objeto factura en un array de cadenas de caracteres para aadir al listado
 * del facturas emitidas.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_invoice_get_content (Invoice *invoice) {
    time_t  date =  d_invoice_get_issue_date (invoice);
    gchar  *temp = u_get_parsed_date (&date);
    gchar **content = (gchar **) g_malloc(sizeof(gchar *)*d_invoice_get_content_size ()+1);
    gint    index = 0;
    gchar *name_tmp = NULL;
    gulong code_tmp = ULONG_MAX;
    gchar * company_name = NULL;
//    gchar * company_name_tmp = NULL;
    gulong company_code_tmp = ULONG_MAX;
    gchar * group_code=NULL;
    if (d_invoice_get_invoice_code (invoice)==ULONG_MAX) {
	content[index++] = g_strdup("");
	content[index++] = g_strdup(_("INVOICE LISTING TOTAL:"));
	content[index++] = g_strdup("");
	content[index++] = g_strdup("");
	content[index++] = g_strdup_printf("%.2f", d_invoice_get_total (invoice));
	content[index++] = NULL;

    } else {
	if (invoice->delivery_payments!=NULL) {
	    Company * company=d_repair_get_company (d_delivery_payment_get_repair (invoice->delivery_payments[0]));
	    if (company!=NULL) {
		if (d_company_get_participant (company)!=NULL) {
		    //		    company_name = d_participant_get_name (d_company_get_participant (company));
		    company_name = d_participant_get_short_name (d_company_get_participant (company));
		    company_code_tmp = d_participant_get_code (d_company_get_participant (company));
		}
	    }
	} else {
	    Company * company=d_repair_get_company (invoice->repair);
	    if (company!=NULL) {
		if (d_company_get_participant (company)!=NULL) {
//		    company_name = d_participant_get_name (d_company_get_participant (company));
		    company_name = d_participant_get_short_name (d_company_get_participant (company));
		    company_code_tmp = d_participant_get_code (d_company_get_participant (company));
		}
	    }
	}
//	if (company_name==NULL) {
//	    company_name_tmp = g_strdup(" ");
//	} else {
//	    company_name_tmp = d_company_get_short_name (NULL,company_code_tmp,company_name);
//	}

//	name_tmp = d_participant_get_nick (d_invoice_get_participant (invoice));
	name_tmp = d_participant_get_short_name (d_invoice_get_participant (invoice));
	code_tmp = d_participant_get_code (d_invoice_get_participant (invoice));

	group_code = d_shop_get_group_code (d_invoice_get_shop (invoice));

	content[index++] = temp;
	content[index++] = g_strdup_printf("%s %lu",group_code, d_invoice_get_invoice_number (invoice));
	content[index++] = name_tmp; //d_company_get_short_name (NULL,code_tmp,name_tmp);
	//	content[index++] = company_name_tmp;
	content[index++] = company_name;
	content[index++] = g_strdup_printf("%.2f   ", d_invoice_get_total (invoice));
	content[index++] = NULL;
        g_free(group_code);
    }
    return content;
}

/**
 * d_invoice_to_content_by_company:
 * @invoice: Objeto factura a partir de la que se obtendrn los valores para el listado.
 * @total_by_company: Total acumulado para una compaa de seguros.
 * @resumen: Indica si la lnea actual es un total por compaa.
 *
 * Transforma un objeto factura en un array de cadenas de caracteres para aadir al listado
 * del facturas emitidas. Se puede transforma la factura actual o bien un total por compaa, donde
 * el total es total_by_company. Si resument es %FALSE lo que se hace es imprimir una lnea con la
 * factura y sumar el total de la factura al #total_by_company.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar ** d_invoice_to_content_by_company (Invoice * invoice,gdouble * total_by_company,gboolean resumen) {
    time_t  date =  d_invoice_get_issue_date (invoice);
    gchar  *temp = u_get_parsed_date (&date);
    gchar **content = (gchar **) g_malloc(sizeof(gchar *)*d_invoice_get_content_size ()+1);
    gint    index = 0;
    gchar *name_tmp = NULL;
    gulong code_tmp = ULONG_MAX;
    gchar * company_name = NULL;
    gulong company_code_tmp = ULONG_MAX;
//    gchar * company_name_tmp = NULL;
    gchar * group_code=NULL;

    if (d_invoice_get_invoice_code (invoice)==ULONG_MAX) {
	content[index++] = g_strdup("");
	content[index++] = g_strdup(_("INVOICE LISTING TOTAL:"));
	content[index++] = g_strdup("");
	content[index++] = g_strdup("");
	content[index++] = g_strdup_printf("%.2f", d_invoice_get_total (invoice));
	content[index++] = NULL;

    } else {
	    if (invoice->delivery_payments!=NULL) {
		Company * company=d_repair_get_company (d_delivery_payment_get_repair (invoice->delivery_payments[0]));
		if (company!=NULL) {
		    if (d_company_get_participant (company)!=NULL) {
//			company_name = d_participant_get_name (d_company_get_participant (company));
			company_name = d_participant_get_short_name (d_company_get_participant (company));
			company_code_tmp = d_participant_get_code (d_company_get_participant (company));
		    }
		}
	    } else {
		Company * company=d_repair_get_company (invoice->repair);
		if (company!=NULL) {
		    if (d_company_get_participant (company)!=NULL) {
//			company_name = d_participant_get_name (d_company_get_participant (company));
			company_name = d_participant_get_short_name (d_company_get_participant (company));
			company_code_tmp = d_participant_get_code (d_company_get_participant (company));
		    }
		}
	    }
/*	    if (company_name==NULL) {
		company_name_tmp = g_strdup(" ");
	    } else {
		company_name_tmp = d_company_get_short_name (NULL,company_code_tmp,company_name);
	    }
*/
	    if (resumen==FALSE) {
//		name_tmp = d_participant_get_nick (d_invoice_get_participant (invoice));
		name_tmp = d_participant_get_short_name (d_invoice_get_participant (invoice));
		code_tmp = d_participant_get_code (d_invoice_get_participant (invoice));
		group_code = d_shop_get_group_code (d_invoice_get_shop (invoice));

		content[index++] = temp;
		content[index++] = g_strdup_printf("%s %lu",group_code, d_invoice_get_invoice_number (invoice));
		content[index++] = name_tmp; //d_company_get_short_name (NULL,code_tmp,name_tmp);
//		content[index++] = company_name_tmp;
		content[index++] = company_name;
		content[index++] = g_strdup_printf("%.2f   ", d_invoice_get_total (invoice));
		content[index++] = NULL;
		g_free(group_code);
		*total_by_company = *total_by_company + d_invoice_get_total (invoice);
	    } else {
                if (company_name !=NULL)
		    //		    content[index++] = g_strconcat("TOTAL PARA ",company_name_tmp,":",NULL);
		    content[index++] = g_strconcat(_("TOTAL FOR "),company_name,":",NULL);
		else
                    content[index++] = g_strconcat(_("TOTAL INVOICES WITHOUT COMPANY"),NULL);
		content[index++] = g_strdup("");
		content[index++] = g_strdup("");
		content[index++] = g_strdup("");
		content[index++] = g_strdup_printf("%.2f", *total_by_company);
		content[index++] = NULL;
                *total_by_company=0;
	}
    }
	return content;
}

/**
 * m_invoice_obtain_data: 
 * @delivery_payments_client: Lista de identificadores de albaranes de abono.
 *
 * Esta funcin crea, a partir de los identificadores de los albaranes de abono de un cliente,
 * una factura con los datos de los albaranes y reparaciones asociadas.
 *
 * Returns:
 * Devuelve los datos de los albaranes con sus reparaciones respectivas a partir de los identificadores en forma
 * de factura.
 */
gint d_invoice_get_number_of_delivery_payments (Invoice * invoice) {

    DeliveryPayment **delivery_payments = d_invoice_get_delivery_payments (invoice);
    gint index = 0;
    
    if (delivery_payments!=NULL) {
	while ((delivery_payments[index])!=NULL) {
	    index++;
	}
    }
    
    return index;	
}



/**
 * d_invoice_calculate_total_discount:
 * @invoice: Factura a partir de la cual se realizar el clculo.
 *
 * Realiza el clculo de los totales de descuentos para una factura.
 * Para el clculo de los descuentos se realiza el recorrido que se
 * explica en la funcin d_invoice_calculate_total ().
 *
 * Returns:
 * La suma de todos los descuentos de las lneas de una factura.
 */
gdouble d_invoice_calculate_total_discount (Invoice * invoice) {

    gint aux_count=0;
    gdouble discount=0;
    gdouble price=0;
    gdouble total_discount=0;

    if (invoice->delivery_payments!=NULL) {
	/*CASO 1: La factura proviene de al menos un albarn y recorremos la lista de albaranes*/
	while (invoice->delivery_payments[aux_count]!=NULL) {
	    DeliveryPayment * dP = invoice->delivery_payments[aux_count];
	    if (dP->repair->repair_line!=NULL) {
		gint aux_count2=0;
		/*Se recorren la lneas de la reparacin calculando el total por lnea*/
		while (dP->repair->repair_line[aux_count2]) {
 		    Repair_line * repair_line = dP->repair->repair_line[aux_count2];
		    /*se obtiene el precio de la lnea*/
		    gdouble line_price = d_repair_line_get_price (repair_line);
		    gdouble aux_price=G_MINDOUBLE;
		    /*Si no hay precio por lnea se obtiene el precio del producto*/
		    if (line_price==G_MINDOUBLE) {
			/*Comprobamos si existe un precio del producto, si no existe el precio por
			 lnea es cero*/
			if (d_product_get_price (d_repair_line_get_product(repair_line))!=G_MINDOUBLE)
			    aux_price = d_product_get_price (d_repair_line_get_product(repair_line));

			else {
			    aux_price= 0;
			}
		    } else {
			/*Al existir un precio de la lnea es el que se tiene en cuenta y
			 se multiplica por el nmero de unidades de la lnea. No se tiene
			 en cuenta el descuento en este caso*/
			line_price = line_price * d_repair_line_get_quantity (repair_line);

		    }
		    /*Se obtiene el descuento de la lnea*/

		    discount = d_repair_line_get_discount (repair_line);
		    if ((discount>=100) || (discount<0)) {
			discount = 0;
		    } else {
                        discount = (discount*1.0)/100.0;
		    }

		    /*aux_price distinto de cero solo si exista un precio de producto
		     y no de lnea de reparacin, por lo que se multiplica por las unidaes*/
		    price = 1.0 * aux_price * d_repair_line_get_quantity (repair_line);


		    /*Se calcula el total de descuento. Si existe precio de la linea se calcula sobre line_price,
                     si existe el precio del producto se calcula
		     sobre el precio calculado a partir del precio del producto y en otro caso vale 0*/
		    if (line_price!=G_MINDOUBLE) {
			total_discount = total_discount + (line_price*discount);
		    } else {
			if (price!=G_MINDOUBLE)
			    total_discount = total_discount + (price*discount);
			else
			    total_discount=0;
		    }
		    aux_count2++;
		}
	    }
	    aux_count++;
	}

    } else {
	Repair * repair = d_invoice_get_repair (invoice);
	gint aux_count2=0;
	while (repair->repair_line[aux_count2]!=NULL) {
	    Repair_line * repair_line = repair->repair_line[aux_count2];
	    /*se obtiene el precio de la lnea*/
	    gdouble line_price = d_repair_line_get_price (repair_line);
	    gdouble aux_price=G_MINDOUBLE;
	    /*Si no hay precio por lnea se obtiene el precio del producto*/
	    if (line_price==G_MINDOUBLE) {
		/*Comprobamos si existe un precio del producto, si no existe el precio por
		 lnea es cero*/
		if (d_product_get_price (d_repair_line_get_product(repair_line))!=G_MINDOUBLE)
		    aux_price = d_product_get_price (d_repair_line_get_product(repair_line));

		else
		    aux_price=0;
	    } else {
		/*Al existir un precio de la lnea es el que se tiene en cuenta y
		 se multiplica por el nmero de unidades de la lnea. No se tiene
		 en cuenta el descuento en este caso*/
		line_price = line_price * d_repair_line_get_quantity (repair_line);

	    }

	    /*Se obtiene el descuento de la lnea*/
	    discount = d_repair_line_get_discount (repair_line);
	    if ((discount>=100) || (discount<0)) {
		discount = 0;
	    } else {
		discount = (discount*1.0)/100.0;
	    }
	    /*aux_price distinto de cero solo si exista un precio de producto
	     y no de lnea de reparacin, por lo que se multiplica por las unidaes*/
	    price = 1.0 * aux_price * d_repair_line_get_quantity (repair_line);

	    /*Se calcula el total de descuento. Si existe precio de la linea se calcula sobre line_price,
	     si existe el precio del producto se calcula
	     sobre el precio calculado a partir del precio del producto y en otro caso vale 0*/
	    if (line_price==G_MINDOUBLE) {
		if (price!=G_MINDOUBLE)
		    total_discount = total_discount + price*discount;
		else
		    total_discount = 0;
	    } else {
		total_discount = total_discount + line_price*discount;
	    }

	    aux_count2++;
	}

    }
    if (total_discount<=0) total_discount=G_MINDOUBLE;
    return total_discount;
}


/* Private functions */

static gint invoice_init(Invoice *invoice) {

    invoice->invoice_number=ULONG_MAX;
    invoice->invoice_code=ULONG_MAX;
    invoice->enterprise=NULL;
    invoice->invoice_discount=-1;
    invoice->shop=NULL;
    invoice->participant=NULL;
    invoice->payment=NULL;
    invoice->repair=NULL;
    invoice->issue_date = time(NULL);
    invoice->delivery_payments = NULL;
    invoice->state=-1;
    invoice->vat_type=-1;
    invoice->total=G_MINDOUBLE;
    invoice->shop_code_split_invoice=ULONG_MAX;
    invoice->enterprise_code_split_invoice=ULONG_MAX;
    invoice->repair_code_split_invoice=ULONG_MAX;
    invoice->split_invoice=FALSE;
    invoice->modified_total=FALSE;
    invoice->garage_is_receiver=FALSE;
    return 0;
}
