/* operation.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "operation.h"
#include "stdlib.h"
#include "time.h"
#include "util.h"


#define D_OPERATION_CONTENT_SIZE 4

static gint operation_init(Operation *operation);




/**
 * d_operation_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Operation vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Operation *d_operation_create_empty () {
  Operation *operation = (Operation *) g_malloc (sizeof(Operation));

  operation_init(operation);
  
  return operation;
}



/**
 * d_operation_set_account:
 * @operation: Objeto en el que se establece el atributo.
 * @account: Atributo a insertar.
 *
 * Establece el atributo @account en el objeto. El atributo @account
 * indica la cuenta corriente en la que realiza el movimiento bancario.
 *
 */
void d_operation_set_account (Operation *operation, Account * account){
    operation->account = account;
}

/**
 * d_operation_set_payment:
 * @operation: Objeto en el que se establece el atributo.
 * @payment: Atributo a insertar.
 *
 * Establece el atributo @payment en el objeto. El atributo @payment
 * indica pago con el que est relacionado el movimiento.
 *
 */
void d_operation_set_payment (Operation *operation, Payment * payment){
    operation->payment = payment;
}

/**
 * d_operation_set_amount:
 * @operation: Objeto en el que se establece el atributo.
 * @amount: Atributo a insertar.
 *
 * Establece el atributo @amount en el objeto. El atributo @amount
 * indica la cantidad del mivimento bancario.
 *
 */
void d_operation_set_amount (Operation *operation, gdouble amount){
    operation->amount = amount;
}
/**
 * d_operation_set_section:
 * @operation: Objeto en el que se establece el atributo.
 * @section: Atributo a insertar.
 *
 * Establece el atributo @section en el objeto. El atributo @section
 * indica la descripcin del movimiento bancario.
 *
 */
void d_operation_set_section (Operation *operation, gchar * section){
    if(operation->section!=NULL) {
        g_free(operation->section);
    }
    operation->section = g_strdup(section);
}
/**
 * d_operation_set_receipt_reference:
 * @operation: Objeto en el que se establece el atributo.
 * @receipt_reference: Atributo a insertar.
 *
 * Establece el atributo @receipt_reference en el objeto. El atributo @receipt_reference
 * indica la referencia del recibo bancario del movimiento.
 *
 */
void d_operation_set_receipt_reference (Operation *operation, gchar * receipt_reference){
    if(operation->receipt_reference!=NULL) {
        g_free(operation->receipt_reference);
    }
    operation->receipt_reference = g_strdup(receipt_reference);
}

/**
 * d_operation_set_date:
 * @operation: Objeto en el que se establece el atributo.
 * @date: Atributo a insertar.
 *
 * Establece el atributo @date en el objeto. El atributo @date
 * indica la fecha de realizacin del movimiento.
 *
 */
void d_operation_set_date (Operation *operation, time_t date){
    operation->date = date;
}
/**
 * d_operation_set_notification_date:
 * @operation: Objeto en el que se establece el atributo.
 * @notification_date: Atributo a insertar.
 *
 * Establece el atributo @notification_date en el objeto. El atributo @notification_date
 * indica la fecha de notificacin del movimiento bancario.
 *
 */
void d_operation_set_notification_date (Operation *operation, time_t notification_date){
    operation->notification_date = notification_date;
}
/**
 * d_operation_set_code:
 * @operation: Objeto en el que se establece el atributo.
 * @operation_code: Atributo a insertar.
 *
 * Establece el atributo @operation_code en el objeto. El atributo @operation_code
 * indica el cdigo del movimiento bancario.
 *
 */
void d_operation_set_code (Operation *operation, gulong operation_code){
    operation->operation_code=operation_code;
}

/**
 * d_operation_get_account:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Account * d_operation_get_account (Operation *operation){
    return operation->account;
}
/**
 * d_operation_get_payment:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Payment * d_operation_get_payment (Operation *operation){
    return operation->payment;
}
/**
 * d_operation_get_amount:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_operation_get_amount (Operation *operation){
    return operation->amount;
}
/**
 * d_operation_get_section:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_operation_get_section (Operation *operation){
    return g_strdup(operation->section);
}

/**
 * d_operation_get_receipt_reference:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_operation_get_receipt_reference (Operation *operation){
    return g_strdup(operation->receipt_reference);
}

/**
 * d_operation_get_code:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_operation_get_code (Operation *operation){
    return operation->operation_code;
}
/**
 * d_operation_get_date:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_operation_get_date (Operation *operation){
    return operation->date;
}
/**
 * d_operation_get_notification_date:
 * @operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_operation_get_notification_date (Operation *operation){
    return operation->notification_date;
}


/**
 * d_operation_clear:
 * @operation: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_operation_clear (Operation *operation) {

    if (operation!=NULL) {
	if (operation->account!=NULL)
	    d_account_clear (operation->account);
	if (operation->payment!=NULL)
	    d_payment_clear (operation->payment);
	if (operation->section!=NULL)
	    g_free(operation->section);
	if (operation->receipt_reference!=NULL)
	    g_free(operation->receipt_reference);
	g_free(operation);
    }

  return 0;
}



/**
 * d_operation_array_to_content:
 * @array: Array de movimientos que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_operation_array_to_content (Operation **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_operation_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_operation_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados de movimientos. Utiliza
 * los mtodos del participante.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_operation_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_operation_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup_printf(_("NOTIF. DATE"));
    content[index++] = g_strdup_printf(_("COMPANY"));
    content[index++] = g_strdup_printf(_("AMOUNT"));
    content[index++] = g_strdup_printf(_("INCOME"));
    content[index++] = NULL;
    return content;
}

/**
 * d_operation_get_listing_title:
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_operation_get_listing_title () {
    gchar *content = g_strdup_printf(_("OPERATIONS MOVEMENT"));
    return content;
}

/**
 * d_operation_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados de movimientos.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_operation_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_operation_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 15;
    widths[index++] = 40;
    widths[index++] = 30;
    widths[index++] = 15;
    widths[index++] = -1;

    return widths;
}

/**
 * d_operation_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_operation_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_operation_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_LEFT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_operation_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_operation_get_content_size () {
    return D_OPERATION_CONTENT_SIZE;
}

/**
 * d_operation_get_content:
 * @operation: Movimiento a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un movimiento en un array de cadenas de caracteres para aadir al listado
 * de operaciones.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_operation_get_content (Operation *operation) {
    time_t  date = d_operation_get_date (operation);
    time_t  notification_date = d_operation_get_notification_date (operation);
    gchar  *temp = u_get_parsed_date (&date);
    gchar  *notification_temp = u_get_parsed_date (&notification_date);
    gchar **content = (gchar **) g_malloc0(sizeof(gchar *)*d_operation_get_content_size ()+1);
    gint    index   = 0;
    gchar *temp_company=NULL;

    u_delete_intros (temp);
    u_delete_intros (notification_temp);
    temp_company = "";
    content[index++] = temp;
    content[index++] = g_strdup(temp_company);
    content[index++] = g_strdup_printf("%.2f",d_operation_get_amount (operation));
    content[index++] = notification_temp;
    content[index++] = NULL;
    g_free(temp_company);
    return content;
}


/* Private functions*/

static gint operation_init(Operation *operation) {

    operation->account = NULL ;
    operation->payment = NULL ;
    operation->amount = -1;
    operation->operation_code = ULONG_MAX;
    operation->section = NULL;
    operation->receipt_reference = NULL;
    operation->date = time(NULL);
    operation->notification_date = time(NULL);
    return 0;
}
