/* order.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "order.h"
#include "util.h"

static gint order_init(Order *order);
static void free_order_lines(Order *order);

/**
 * d_order_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Order vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Order  *d_order_create_empty (){
    Order *order = (Order *) g_malloc(sizeof(Order));

    order_init(order);

    return order;
}


/**
 * d_order_clear:
 * @order: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_order_clear (Order *order) {

    if (order!=NULL) {
	free_order_lines(order);
	g_free(order);
	order=NULL;
    }
    return 0;

}

/**
 * d_order_set_code:
 * @order: Objeto en el que se establece el atributo.
 * @order_code: Atributo a insertar.
 *
 * Establece el atributo @order_code en el objeto. El atributo @order_code
 * indica el cdigo interno para identificar el pedido.
 *
 */
void d_order_set_code (Order * order, gulong order_code) {
    order->order_code = order_code;
}

/**
 * d_order_set_enterprise_code:
 * @order: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa que ha solicitado el pedido.
 *
 */
void d_order_set_enterprise_code (Order * order, gulong enterprise_code) {
    order->enterprise_code = enterprise_code;
}

/**
 * d_order_set_warehouse_code:
 * @order: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica el cdigo del almacn para el que se realiz el pedido.
 *
 */
void d_order_set_warehouse_code (Order * order, gulong warehouse_code) {
    order->warehouse_code = warehouse_code;
}

/**
 * d_order_set_supplier_code:
 * @order: Objeto en el que se establece el atributo.
 * @supplier_code: Atributo a insertar.
 *
 * Establece el atributo @supplier_code en el objeto. El atributo @supplier_code
 * indica el cdigo del proveedor al que se realiz el pedido.
 *
 */
void d_order_set_supplier_code (Order * order, gulong supplier_code) {
    order->supplier_code = supplier_code;
}


/**
 * d_order_set_requested:
 * @order: Objeto en el que se establece el atributo.
 * @requested: Atributo a insertar.
 *
 * Establece el atributo @requested en el objeto. El atributo @requested
 * indica si el pedido ya se ha solicitado.
 *
 */
void d_order_set_requested (Order * order, gboolean requested) {
    order->requested = requested;
}

/**
 * d_order_set_order_lines:
 * @order: Objeto en el que se establece el atributo.
 * @order_lines: Atributo a insertar.
 *
 * Establece el atributo @order_lines en el objeto. El atributo @order_lines
 * indica las lneas del pedido creado.
 *
 */
void d_order_set_order_lines (Order * order, OrderLine **order_lines) {
    free_order_lines(order);
    order->order_lines = order_lines;
}


/**
 * d_order_get_code:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_get_code (Order * order) {
    return(order->order_code);
}


/**
 * d_order_get_enterprise_code:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_get_enterprise_code (Order * order) {
    return(order->enterprise_code);
}


/**
 * d_order_get_warehouse_code:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_get_warehouse_code (Order * order) {
    return(order->warehouse_code);
}


/**
 * d_order_get_supplier_code:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_get_supplier_code (Order * order) {
    return(order->supplier_code);
}



/**
 * d_order_get_requested:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_order_get_requested (Order * order) {
    return(order->requested);
}


/**
 * d_order_get_date:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_order_get_date (Order * order) {
    return(order->date);
}


/**
 * d_order_get_order_lines:
 * @order: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
OrderLine **d_order_get_order_lines (Order * order) {
    return(order->order_lines);
}

/**
 * d_order_duplicate:
 * @order: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto pedido.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Order *d_order_duplicate (Order *order) {
    Order *result = d_order_create_empty ();
    OrderLine **lines = d_order_line_duplicate (d_order_get_order_lines (order));
    d_order_set_code (result, d_order_get_code (order));
    d_order_set_enterprise_code (result, d_order_get_enterprise_code (order));
    d_order_set_warehouse_code (result, d_order_get_warehouse_code (order));
    d_order_set_supplier_code (result, d_order_get_supplier_code (order));
    d_order_set_requested (result, d_order_get_requested (order));

    d_order_set_order_lines (result, lines);

    return result;
}


/* Private functions */

static gint order_init(Order *order) {
    order->order_code    = ULONG_MAX;
    order->enterprise_code   = ULONG_MAX;
    order->warehouse_code   = ULONG_MAX;
    order->supplier_code = ULONG_MAX;
    order->requested      = FALSE;
    order->date           = time(NULL);
    order->order_lines    = NULL;
    return 0;
}

/**
 * free_order_lines:
 * @order: Pedido al que se liberar la memoria de las lneas de pedido.
 *
 * Libera la memoria utilizada por las lneas de pedido
 *
*/
static void free_order_lines(Order *order) {
    gint index = 0;

    if (order->order_lines!=NULL) {
	while(order->order_lines[index] != NULL) {
	    d_order_line_clear (order->order_lines[index]);
	    index++;
	}
	g_free(order->order_lines);
	order->order_lines = NULL;
    }
}

