/* participant.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "participant.h"
#include "util.h"

#define D_PARTICIPANT_CONTENT_SIZE 6

static gint participant_init(Participant *participant);

/**
 * d_participant_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Participant vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Participant * d_participant_create_empty ()
{
    Participant * participant = (Participant *) g_malloc (sizeof(Participant));

    participant_init(participant);

    return participant;
}


/**
 * d_participant_clear:
 * @participant: El objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_participant_clear (Participant *participant)
{
    if (participant!=NULL) {
	if (participant->nick!=NULL) {
	    g_free(participant->nick);
	    participant->nick = NULL;
	}
	if (participant->name!=NULL) {
	    g_free(participant->name);
	    participant->name = NULL;
	}
	if (participant->short_name!=NULL) {
	    g_free(participant->short_name);
	    participant->short_name = NULL;
	}
	if (participant->email!=NULL) {
	    g_free(participant->email);
	    participant->email = NULL;
	}
	if (participant->phone!=NULL) {
	    g_free(participant->phone);
	    participant->phone = NULL;
	}
	if (participant->phone2!=NULL) {
	    g_free(participant->phone2);
	    participant->phone2 = NULL;
	}
	if (participant->nick!=NULL) {
	    g_free(participant->nick);
	    participant->nick = NULL;
	}
	if (participant->fax!=NULL) {
	    g_free(participant->fax);
	    participant->fax = NULL;
	}
	if (participant->street!=NULL) {
	    g_free(participant->street);
	    participant->street = NULL;
	}
	if (participant->number!=NULL) {
	    g_free(participant->number);
	    participant->number = NULL;
	}
	if (participant->floor!=NULL) {
	    g_free(participant->floor);
	    participant->floor = NULL;
	}
	if (participant->town!=NULL) {
	    g_free(participant->town);
	    participant->town = NULL;
	}
	if (participant->province!=NULL) {
	    g_free(participant->province);
	    participant->province = NULL;
	}
	if (participant->legal_id!=NULL) {
	    g_free(participant->legal_id);
	    participant->legal_id = NULL;
	}
	if (participant->post_address!=NULL) {
	    gint index=0;
	    while (participant->post_address[index]) {
		d_post_address_clear (participant->post_address[index]);
	    }
	    g_free(participant->post_address);
	    participant->post_address = NULL;
	}

	g_free(participant);
	participant = NULL;
    }

    return 0;
}

/**
 * d_participant_set_code:
 * @participant: Objeto en el que se establece el atributo.
 * @code: Atributo a insertar.
 *
 * Establece el atributo @code en el objeto. El atributo @code
 * indica el cdigo interno que identifica el participante.
 *
 */
void d_participant_set_code (Participant * participant,gulong code){
    participant->code = code;
}

/**
 * d_participant_set_email:
 * @participant: Objeto en el que se establece el atributo.
 * @email: Atributo a insertar.
 *
 * Establece el atributo @email en el objeto. El atributo @email
 * indica la direccin de email del participante.
 *
 */
void d_participant_set_email (Participant * participant,gchar * email){
    if (participant->email!=NULL) {
	g_free(participant->email);
    }
    participant->email = g_strdup (email);
}

/**
 * d_participant_set_phone:
 * @participant: Objeto en el que se establece el atributo.
 * @phone: Atributo a insertar.
 *
 * Establece el atributo @phone en el objeto. El atributo @phone
 * indica un telfono principal del cliente.
 *
 */
void d_participant_set_phone (Participant * participant,gchar * phone){
    if (participant->phone!=NULL) {
	g_free(participant->phone);
    }
    participant->phone = g_strdup (phone);
}

/**
 * d_participant_set_phone2:
 * @participant: Objeto en el que se establece el atributo.
 * @phone2: Atributo a insertar.
 *
 * Establece el atributo @phone2 en el objeto. El atributo @phone2
 * indica un segundo telfono del participante.
 *
 */
void d_participant_set_phone2 (Participant * participant,gchar * phone2){
    if (participant->phone2!=NULL) {
	g_free(participant->phone2);
    }
    participant->phone2 = g_strdup (phone2);
}

/**
 * d_participant_set_fax:
 * @participant: Objeto en el que se establece el atributo.
 * @fax: Atributo a insertar.
 *
 * Establece el atributo @fax en el objeto. El atributo @fax
 * indica el nmero de fax del participante.
 *
 */
void d_participant_set_fax (Participant * participant,gchar * fax){
    if (participant->fax!=NULL) {
	g_free(participant->fax);
    }
    participant->fax = g_strdup (fax);
}

/**
 * d_participant_set_street:
 * @participant: Objeto en el que se establece el atributo.
 * @street: Atributo a insertar.
 *
 * Establece el atributo @street en el objeto. El atributo @street
 * indica la calle en la que reside el
 *
 */
void d_participant_set_street (Participant * participant,gchar * street){
    if (participant->street!=NULL) {
	g_free(participant->street);
    }
    participant->street = g_strdup (street);
}

/**
 * d_participant_set_number:
 * @participant: Objeto en el que se establece el atributo.
 * @number: Atributo a insertar.
 *
 * Establece el atributo @number en el objeto. El atributo @number
 * indica el nmero de la direccin del participante.
 *
 */
void d_participant_set_number (Participant * participant,gchar * number){
    if (participant->number!=NULL) {
	g_free(participant->number);
    }
    participant->number = g_strdup (number);
}

/**
 * d_participant_set_floor:
 * @participant: Objeto en el que se establece el atributo.
 * @floor: Atributo a insertar.
 *
 * Establece el atributo @floor en el objeto. El atributo @floor
 * indica el piso del participante.
 *
 */
void d_participant_set_floor (Participant * participant,gchar * floor){
    if (participant->floor!=NULL) {
	g_free(participant->floor);
    }
    participant->floor = g_strdup (floor);
}

/**
 * d_participant_set_nick:
 * @participant: Objeto en el que se establece el atributo.
 * @nick: Atributo a insertar.
 *
 * Establece el atributo @nick en el objeto. El atributo @nick
 * indica 
 *
 */
void d_participant_set_nick (Participant * participant,gchar * nick){
    if (participant->nick!=NULL) {
	g_free(participant->nick);
    }
    participant->nick = g_strdup (nick);
}

/**
 * d_participant_set_postal_code:
 * @participant: Objeto en el que se establece el atributo.
 * @postal_code: Atributo a insertar.
 *
 * Establece el atributo @postal_code en el objeto. El atributo @postal_code
 * indica el cdigo postal de direccin.
 *
 */
void d_participant_set_postal_code (Participant * participant,gulong postal_code){
    participant->postal_code = postal_code;
}

/**
 * d_participant_set_town:
 * @participant: Objeto en el que se establece el atributo.
 * @town: Atributo a insertar.
 *
 * Establece el atributo @town en el objeto. El atributo @town
 * indica el municipio del participante.
 *
 */
void d_participant_set_town (Participant * participant,gchar * town){
    if (participant->town!=NULL) {
	g_free(participant->town);
    }
    participant->town = g_strdup (town);
}

/**
 * d_participant_set_post_addresses:
 * @participant: Objeto en el que se establece el atributo.
 * @post_address: Atributo a insertar.
 *
 * Establece el atributo @post_address en el objeto. El atributo @post_address
 * indica las direcciones de envo del participante.
 *
 */
void d_participant_set_post_addresses (Participant * participant,PostAddress ** post_address) {
    if (participant->post_address!=NULL) {
	//m_post_address_clear_array (participant->post_address);
    }
    participant->post_address=post_address;
}


/**
 * d_participant_set_province:
 * @participant: Objeto en el que se establece el atributo.
 * @province: Atributo a insertar.
 *
 * Establece el atributo @province en el objeto. El atributo @province
 * indica la provincia del participante.
 *
 */
void d_participant_set_province (Participant * participant,gchar * province){
    if (participant->province!=NULL) {
	g_free(participant->province);
    }
    participant->province = g_strdup (province);
}

/**
 * d_participant_set_name:
 * @participant: Objeto en el que se establece el atributo.
 * @name: Atributo a insertar.
 *
 * Establece el atributo @name en el objeto. El atributo @name
 * indica el nombre de pila del participante en caso de ser un particular.
 *
 */
void d_participant_set_name (Participant * participant,gchar * name){
    if (participant->name!=NULL) {
	g_free(participant->name);
    }
    participant->name = g_strdup (name);
}

/**
 * d_participant_set_legal_id:
 * @participant: Objeto en el que se establece el atributo.
 * @legal_id: Atributo a insertar.
 *
 * Establece el atributo @legal_id en el objeto. El atributo @legal_id
 * indica cif/nif del participante.
 *
 */
void d_participant_set_legal_id (Participant * participant,gchar * legal_id){
    if (participant->legal_id!=NULL) {
	g_free(participant->legal_id);
    }
    participant->legal_id = g_strdup (legal_id);
}


/**
 * d_participant_set_legal_id:
 * @participant: Objeto en el que se establece el atributo.
 * @legal_id: Atributo a insertar.
 *
 * Establece el atributo @legal_id en el objeto. El atributo @legal_id
 * indica cif/nif del participante.
 *
 */
void d_participant_set_short_name (Participant * participant,gchar * short_name){
    if (participant->short_name!=NULL) {
	g_free(participant->short_name);
    }
    participant->short_name = g_strdup (short_name);
}


/**
 * d_participant_get_code:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_participant_get_code (Participant * participant){
    return participant->code;
}


/**
 * d_participant_get_short_name:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar* d_participant_get_short_name (Participant * participant){
    return g_strdup(participant->short_name);
}


/**
 * d_participant_get_email:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_email (Participant * participant){
    return g_strdup(participant->email);
}

/**
 * d_participant_get_phone:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_phone (Participant * participant){
    return g_strdup(participant->phone);
}

/**
 * d_participant_get_phone2:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_phone2 (Participant * participant){
    return g_strdup(participant->phone2);
}

/**
 * d_participant_get_fax:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_fax (Participant * participant){
    return g_strdup(participant->fax);
}

/**
 * d_participant_get_street:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_street (Participant * participant){
    return g_strdup(participant->street);
}

/**
 * d_participant_get_number:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_number (Participant * participant){
    return g_strdup(participant->number);
}

/**
 * d_participant_get_floor:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_floor (Participant * participant){
    return g_strdup(participant->floor);
}
/**
 * d_participant_get_postal_code:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_participant_get_postal_code (Participant * participant){
    return participant->postal_code;
}

/**
 * d_participant_get_nick:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_nick (Participant * participant){
    return g_strdup(participant->nick);
}

/**
 * d_participant_get_town:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_town (Participant * participant){
    return g_strdup(participant->town);
}

/**
 * d_participant_get_province:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_province (Participant * participant){
    return g_strdup(participant->province);
}

/**
 * d_participant_get_name:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_name (Participant * participant){
    return g_strdup(participant->name);
}

/**
 * d_participant_get_legal_id:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_participant_get_legal_id (Participant * participant){
    return g_strdup(participant->legal_id);
}

/**
 * d_participant_get_post_addresses:
 * @participant: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
PostAddress ** d_participant_get_post_addresses (Participant * participant){
    return participant->post_address;
}


/**
 * d_participant_duplicate:
 * @participant: Objeto del que se desea obtener el duplicado.
 *
 * Duplica un participante.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Participant *d_participant_duplicate (Participant *participant){
    Participant *result = NULL;

    if (participant!=NULL) {

	gulong code = d_participant_get_code (participant);
	gchar *legal_id = d_participant_get_legal_id (participant);
	gchar *nick = d_participant_get_nick (participant);
	gchar *street = d_participant_get_street (participant);
	gchar *number = d_participant_get_number (participant);
	gchar *floor = d_participant_get_floor (participant);
	gulong postal_code = d_participant_get_postal_code (participant);
	gchar *town = d_participant_get_town (participant);
	gchar *province = d_participant_get_province (participant);
	gchar *phone1 = d_participant_get_phone (participant);
	gchar *phone2 = d_participant_get_phone2 (participant);
	gchar *fax = d_participant_get_fax (participant);
	gchar *email = d_participant_get_email (participant);
	gchar *name = d_participant_get_name (participant);
	gchar *short_name = d_participant_get_short_name (participant);

	result = d_participant_create_empty ();

	d_participant_set_code (result, code);
	d_participant_set_email (result, email);
	d_participant_set_phone (result, phone1);
	d_participant_set_phone2 (result,phone2);
	d_participant_set_fax (result, fax);
	d_participant_set_street (result, street);
	d_participant_set_number (result, number);
	d_participant_set_floor (result, floor);
	d_participant_set_nick (result, nick);
	d_participant_set_postal_code (result, postal_code);
	d_participant_set_town (result, town);
	d_participant_set_province (result, province);
	d_participant_set_name (result, name);
	d_participant_set_short_name (result, short_name);
	d_participant_set_legal_id (result, legal_id);
	if (participant->post_address!=NULL) {
	    gint index_address=0;
	    while (participant->post_address[index_address]!=NULL) {
		index_address++;
	    }
	    (PostAddress **) result->post_address = (PostAddress **) g_malloc0((index_address+1)*sizeof(PostAddress *));
	    index_address=0;
	    while (participant->post_address[index_address]!=NULL) {
		result->post_address[index_address] = d_post_address_duplicate (participant->post_address[index_address]);
		index_address++;
	    }
	}
	g_free(nick);
	g_free(legal_id);
	g_free(street);
	g_free(number);
	g_free(floor);
	g_free(town);
	g_free(province);
	g_free(phone1);
	g_free(phone2);
	g_free(fax);
	g_free(email);
	g_free(name);
	g_free(short_name);
    }
    return result;
}

/**
 * d_participant_array_to_content:
 * @array: Array de participantes que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar el total de objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_participant_array_to_content (Participant **array, gulong *reccount) {
    gint     array_size = 0;
    gint     index      =  0;
    gchar ***content = NULL;
    gchar  **data = NULL;


    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	data = (gchar **) malloc(sizeof(gchar *) * d_participant_get_content_size ());
        index = 0;
	content[*reccount] = d_participant_get_content (array[*reccount], data, &index);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_participant_get_content_title:
 * @content: Contenido al que se aade la cabecera del participante.
 * @index: Puntero a entero que indica la posicin del array en la que va cada cabecera y
 * guarda el nuevo ndice en el que se debe meter cabecera.
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto participante.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_participant_get_content_title (gchar **content, gint *index) {
    content[(*index)++] = g_strdup_printf(_("NAME"));
    content[(*index)++] = g_strdup_printf(_("TC/LI")); // Tax Code / Legal Id
    content[(*index)++] = g_strdup_printf(_("ADDRESS"));
    content[(*index)++] = g_strdup_printf(_("POSTAL CODE"));  // Postal Code
    content[(*index)++] = g_strdup_printf(_("TOWN"));
    content[(*index)++] = g_strdup_printf(_("STATE"));
    content[(*index)++] = NULL;
    return content;
}

/**
 * d_participant_get_listing_title:
 *
 * Obtiene el ttulo del listado de participantes.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_participant_get_listing_title () {
    gchar *content = g_strdup_printf(_("PEOPLE LISTING"));
    return content;
}


/**
 * d_participant_get_listing_columns_width:
 * @widths: Array de anchuras al que se aadirn las
 * @index: Puntero a entero que indica la posicin del array en la que va cada cabecera y
 * guarda el nuevo ndice en el que se debe meter cabecera.
 *
 * Indica los anchos de columna de los listados de de participantes.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_participant_get_listing_columns_width (gint *widths, gint *index) {
    widths[(*index)++] = 29;
    widths[(*index)++] = 7;
    widths[(*index)++] = 27;
    widths[(*index)++] = 5;
    widths[(*index)++] = 12;
    widths[(*index)++] = 12;
    widths[(*index)++] = -1;

    return widths;
}


/**
 * d_participant_get_listing_columns_alignment:
 * @orientation: Orientaciones a los que se aaden los del participante.
 * @index: Puntero a entero que indica la posicin del array en la que va cada cabecera y
 * guarda el nuevo ndice en el que se debe meter cabecera.
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_participant_get_listing_columns_alignment (gint *orientation, gint *index) {
    orientation[(*index)++] = U_LEFT;
    orientation[(*index)++] = U_LEFT;
    orientation[(*index)++] = U_LEFT;
    orientation[(*index)++] = U_LEFT;
    orientation[(*index)++] = U_LEFT;
    orientation[(*index)++] = U_LEFT;
    orientation[(*index)++] = -1;

    return orientation;
}

/**
 * d_participant_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas
 */
gint d_participant_get_content_size () {
    return D_PARTICIPANT_CONTENT_SIZE;
}


/**
 * d_participant_get_content:
 * @participant: Participante a partir de la que se obtendrn los valores para el listado.
 * @content:
 * @index:
 *
 * Transforma un participante en un array de cadenas de caracteres para aadir al listado
 * de participantes.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_participant_get_content (Participant *participant, gchar **content, gint *index) {
    
    gchar * street = d_participant_get_street (participant);
    gchar * number =  d_participant_get_number (participant);
    gchar * floor = d_participant_get_floor (participant);
    gchar * legal_id = d_participant_get_legal_id (participant);
    gchar * town=d_participant_get_town (participant);
    gchar * short_name = d_participant_get_short_name (participant);
    gchar * province = d_participant_get_province (participant);
    gchar * address = NULL;
    gchar * postal_code = NULL;
        
    gchar * temp_nick = NULL;
    gchar * temp_address = NULL;
    gchar * temp_town=NULL;
    gchar * temp_province = NULL;

    if (number==NULL) number = g_strdup(" ");     
    if (floor==NULL) floor = g_strdup(" ");    
    if (street==NULL) street = g_strdup(" ");    

    address = g_strdup_printf("%s, %s-%s", street, number, floor);
    
    if (d_participant_get_postal_code (participant)!=ULONG_MAX)
	postal_code = g_strdup_printf("%lu",d_participant_get_postal_code (participant));    
      
    if (short_name==NULL) short_name=g_strdup("");
    if (legal_id==NULL) legal_id=g_strdup("");
    if (postal_code==NULL) postal_code=g_strdup("");
    if (address==NULL) address=g_strdup("");
    if (town==NULL) town=g_strdup("");
    if (province==NULL) province=g_strdup("");
	
    g_strup(short_name);
    g_strup(legal_id);
    g_strup(address);
    g_strup(postal_code);
    g_strup(town);
    g_strup(province);


    temp_nick = g_strndup(short_name,50);
    temp_address = g_strndup(address,43);
    temp_town = g_strndup(town,20);
    temp_province = g_strndup(province,20);

    content[(*index)++] = temp_nick;
    content[(*index)++] = legal_id;
    content[(*index)++] = temp_address;
    content[(*index)++] = postal_code;
    content[(*index)++] = temp_town;
    content[(*index)++] = temp_province;
    content[(*index)++] = NULL;

    g_free(short_name);
    g_free(street);
    g_free(number);
    g_free(floor);
    g_free(address);
    g_free(town);
    g_free(province);
    
    return content;
}


/* Private functions*/

static gint participant_init(Participant *participant)
{
    participant->code = ULONG_MAX;
    participant->email = NULL;
    participant->phone = NULL;
    participant->phone2 = NULL;
    participant->nick = NULL;
    participant->fax = NULL;
    participant->street = NULL;
    participant->number = NULL;
    participant->floor = NULL;
    participant->postal_code = ULONG_MAX;
    participant->town = NULL;
    participant->province = NULL;
    participant->name = NULL;
    participant->legal_id = NULL;
    participant->post_address=NULL;
    participant->short_name=NULL;    
    
    return 0;
}

