/* piece_history.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "piece_history.h"
#include "delivery_payment_movement.h"
#include "repair_movement.h"
#include "exchange.h"
#include "regularization.h"
#include "util.h"
#include "warehouse.h"
#include "enterprise.h"

#define D_HISTORY_PIECE_CONTENT_SIZE 4
static gint piece_history_init(PieceHistory *piece_history);



/**
 * d_piece_history_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #PieceHistory vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
PieceHistory * d_piece_history_create_empty () {
    PieceHistory *piece_history = (PieceHistory *) g_malloc0(sizeof(PieceHistory));

    piece_history_init(piece_history);

    return piece_history;                           
}

/**
 * d_piece_history_clear:
 * @piece_history: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_piece_history_clear (PieceHistory *piece_history) {
    if (piece_history!=NULL) {
	switch (piece_history->movement_type) {
	case D_PIECE_HISTORY_REGULARIZATION_MOVEMENT:{
	    piece_history -> movement = (gpointer) d_regularization_clear ((Regularization *) piece_history -> movement);
	    break;
	}
	case D_PIECE_HISTORY_INTERCHANGE_MOVEMENT:{
	    piece_history -> movement = (gpointer) d_exchange_clear ((Exchange *) piece_history -> movement);
	    break;
	}
	case D_MOVEMENT_REPAIR:{
	    piece_history -> movement = (gpointer) d_repair_movement_clear ((RepairMovement *) piece_history -> movement);
	    break;
	}
	case D_MOVEMENT_DELIVERY_PAYMENT:{
	    piece_history -> movement = (gpointer) d_delivery_payment_movement_clear ((DeliveryPaymentMovement *) piece_history -> movement);
	    break;
	} default: {

	    piece_history->movement=NULL;
	}
	}
	g_free(piece_history);
    }
    return 0;
}

/**
 * d_piece_history_set_movement_type:
 * @piece_history: Objeto en el que se establece el atributo.
 * @type: Atributo a insertar.
 *
 * Establece el atributo @movement_type en el objeto. El atributo @movement_type
 * indica el tipo del movimiento.
 *
 */
void d_piece_history_set_movement_type (PieceHistory *piece_history, gint type) {
    piece_history->movement_type = type;
}

/**
 * d_piece_history_set_movement:
 * @piece_history: Objeto en el que se establece el atributo.
 * @movement: Atributo a insertar.
 *
 * Establece el atributo @movement en el objeto. El atributo @movement
 * indica el movimiento de piezas.
 *
 */
void d_piece_history_set_movement (PieceHistory *piece_history, gpointer movement) {
    piece_history->movement = movement;
}

/**
 * d_piece_history_get_movement_type:
 * @piece_history: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_piece_history_get_movement_type (PieceHistory *piece_history) {
    return piece_history->movement_type;
}

/**
 * get_Movimiento_piece_history:
 * @piece_history: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gpointer d_piece_history_get_movement (PieceHistory *piece_history) {
    return piece_history->movement;
}

/* Private functions*/

static gint piece_history_init(PieceHistory *piece_history) {
    piece_history->movement_type = -1;
    piece_history->movement = NULL;

    return 0;
}

/**
 * d_piece_history_array_to_content:
 * @array: Array de piezas de abono que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_piece_history_array_to_content (PieceHistory **array,gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) g_malloc0(sizeof(gchar **)*array_size+2);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_piece_history_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}


/**
 * d_piece_history_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto pieza emitida. 
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_piece_history_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_piece_history_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup_printf(_("TYPE"));
    content[index++] = g_strdup_printf(_("DATE"));
    content[index++] = g_strdup_printf(_("ADDITONAL DATA"));
    content[index++] = g_strdup_printf(_("UNITS"));
    content[index++] = NULL;
    return content;
}

/**
 * d_piece_history_get_listing_title:
 * @eurocode: Eurocode de la pieza.
 * @warehouse: Almacn.
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_piece_history_get_listing_title (gchar * eurocode,gchar * warehouse) {
    gchar *content = NULL;
    g_strstrip(eurocode);
    g_strstrip(warehouse);
    content = g_strdup_printf(_("Operations from \"%s\" to \"%s\""),eurocode,warehouse);
    return content;
}

/**
 * d_piece_history_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados piezas emitidas.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_piece_history_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_piece_history_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 35;
    widths[index++] = 10;
    widths[index++] = 45;
    widths[index++] = 10;
    widths[index++] = -1;

    return widths;
}

/**
 * d_piece_history_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_piece_history_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_piece_history_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_piece_history_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_piece_history_get_content_size () {
    return D_HISTORY_PIECE_CONTENT_SIZE;
}

/**
 * d_piece_history_get_content:
 * @piece_history: Objeto pieza a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un objeto pieza en un array de cadenas de caracteres para aadir al listado
 * del piezas emitidas.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_piece_history_get_content (PieceHistory *piece_history) {
    gint  type = d_piece_history_get_movement_type (piece_history);
    gchar **content = (gchar **) g_malloc0(sizeof(gchar *)*d_piece_history_get_content_size ()+1);
    gint    index = 0;
    gchar * movement_type = NULL;
    gchar * unidades = NULL;
    gchar * fecha_char = NULL;
    gchar * datos = NULL;
    time_t fecha;

    switch (type) {
    case D_PIECE_HISTORY_REGULARIZATION_MOVEMENT:{
	Regularization * reg = (Regularization *) d_piece_history_get_movement (piece_history);
	movement_type=g_strdup(_("Stock regularization"));
	unidades = g_strdup_printf("%.3f",d_regularization_get_quantity (reg));
	fecha = d_regularization_get_date (reg);
	fecha_char = u_get_parsed_date (&fecha);
	datos = g_strdup("");
	break;
    }
    case D_PIECE_HISTORY_INTERCHANGE_MOVEMENT:{
	Exchange * ex = (Exchange *) d_piece_history_get_movement (piece_history);
        gchar * temp_name = NULL;

        temp_name = d_exchange_get_consignee_warehouse_name (ex);
	if (d_exchange_get_in (ex)!=FALSE) {
	    movement_type=g_strdup(_("Input change"));
	    datos = g_strconcat(_("From \""),temp_name,"\"",NULL);
	} else {
	    movement_type=g_strdup(_("Output Change"));
            datos = g_strconcat(_("To \""),temp_name,"\"",NULL);
	}
	unidades = g_strdup_printf("%.3f",d_exchange_get_quantity (ex));
        fecha = d_exchange_get_date (ex);
	fecha_char = u_get_parsed_date (&fecha);
	break;
    }
    case D_MOVEMENT_REPAIR:{
        RepairMovement * rm = (RepairMovement *) d_piece_history_get_movement (piece_history);
        gchar * temp_name=NULL;

	movement_type=g_strdup(_("Movement due to repair"));
	unidades = g_strdup_printf("%.3f",(-1)*d_repair_movement_get_quantity (rm));
        fecha = d_repair_movement_get_date (rm);
	fecha_char = u_get_parsed_date (&fecha);
	temp_name = d_repair_movement_get_license_number (rm);
	datos = g_strconcat(_("License No.: \""),temp_name,"\"",NULL);
	if (temp_name!=NULL)
	    g_free(temp_name);
	break;
    }
    case D_MOVEMENT_DELIVERY_PAYMENT:{
        DeliveryPaymentMovement * dpm = (DeliveryPaymentMovement *) d_piece_history_get_movement (piece_history);
	gchar * temp_name=NULL;

        if (d_delivery_payment_movement_get_pending (dpm)==FALSE)
	    movement_type=g_strdup(_("Operations due to order arrival"));
	else
	    movement_type=g_strdup(_("Order pending for reception"));

	unidades = g_strdup_printf("%.3f",d_delivery_payment_movement_get_quantity (dpm));
        fecha = d_delivery_payment_movement_get_date (dpm);
	fecha_char = u_get_parsed_date (&fecha);
	if (d_delivery_payment_movement_get_pending (dpm)==FALSE) {
	    temp_name = d_delivery_payment_movement_get_delivery_payment_number (dpm);
	    datos = g_strconcat(_("Delivery payment number: \""),temp_name,"\"",NULL);
	} else {
	    temp_name = g_strdup_printf("%lu",d_delivery_payment_movement_get_order_code (dpm));
	    datos = g_strconcat(_("Order Number "),temp_name,NULL);
	}
	if (temp_name!=NULL)
	    g_free(temp_name);

	break;
    } default: {
        movement_type=g_strdup("");
        unidades=g_strdup("");
        fecha_char=g_strdup("");
        datos=g_strdup("");
    }
    }


    content[index++] = movement_type;
    content[index++] = fecha_char;
    content[index++] = datos;
    content[index++] = unidades;
    content[index++] = NULL;
    

    return content;
}


