/* supplier_price.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "supplier_price.h"
#include "util.h"

static gint supplier_price_init(SupplierPrice *supplier_price);

/**
 * d_supplier_price_create_empty:
 *
 * Crea un objeto vaco, inicializado a unos valores dependientes del tipo.
 *
 * Returns: Un objeto #SupplierPrice con los campos vacos.
 */
SupplierPrice * d_supplier_price_create_empty () {
    SupplierPrice *supplier_price = (SupplierPrice *) g_malloc0(sizeof(SupplierPrice));

    supplier_price_init(supplier_price);

    return supplier_price;
}


/**
 * d_supplier_price_clear:
 * @supplier_price: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_supplier_price_clear (SupplierPrice *supplier_price) {
    if (supplier_price!=NULL) {
        if (supplier_price->participant!=NULL)
	    d_participant_clear (supplier_price->participant);
        if (supplier_price->piece!=NULL)
	    d_piece_clear (supplier_price->piece);
	g_free(supplier_price);
        supplier_price=NULL;
    }
    return 0;
}


/**
 * d_supplier_price_set_participant:
 * @supplier_price: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el proveedor que impone el precio de coste.
 *
 */
void  d_supplier_price_set_participant (SupplierPrice *supplier_price, Participant * participant) {
    if (supplier_price->participant!=NULL)
        d_participant_clear (supplier_price->participant);
    supplier_price->participant = participant;
}

/**
 * d_supplier_price_set_cost_price:
 * @supplier_price: Objeto en el que se establece el atributo.
 * @cost_price: Atributo a insertar.
 *
 * Establece el atributo @cost_price en el objeto. El atributo @cost_price
 * indica el precio de coste de la pieza.
 *
 */
void d_supplier_price_set_cost_price (SupplierPrice *supplier_price, gdouble cost_price){
    supplier_price->cost_price = cost_price;
}

/**
 * d_supplier_price_set_piece:
 * @supplier_price: Objeto en el que se establece el atributo.
 * @piece: Atributo a insertar.
 *
 * Establece el atributo @piece en el objeto. El atributo @piece
 * indica el cdigo del pieza para el que el proveedor impone un precio de coste.
 *
 */
void  d_supplier_price_set_piece (SupplierPrice *supplier_price, Piece * piece) {
    if (supplier_price->piece!=NULL)
        d_piece_clear (supplier_price->piece);
    supplier_price->piece = piece;
}

/**
 * d_supplier_price_get_participant:
 * @supplier_price: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_supplier_price_get_participant (SupplierPrice *supplier_price) {
    return supplier_price->participant;
}

/**
 * d_supplier_price_get_cost_price:
 * @supplier_price: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_supplier_price_get_cost_price (SupplierPrice *supplier_price){
    return (supplier_price->cost_price);
}

/**
 * d_supplier_price_get_piece:
 * @supplier_price: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Piece * d_supplier_price_get_piece (SupplierPrice *supplier_price) {
    return supplier_price->piece;
}



/**
 * d_supplier_price_duplicate:
 * @supplier_price: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
SupplierPrice *d_supplier_price_duplicate (SupplierPrice *supplier_price) {
  
  SupplierPrice *result = d_supplier_price_create_empty ();

  d_supplier_price_set_cost_price (result, d_supplier_price_get_cost_price (supplier_price));

  d_supplier_price_set_piece (result, d_piece_duplicate(d_supplier_price_get_piece (supplier_price)));
  d_supplier_price_set_participant (result, d_participant_duplicate (d_supplier_price_get_participant (supplier_price)));

  return result;
}


/* Private functions*/

static gint supplier_price_init(SupplierPrice *supplier_price) {
    supplier_price->piece = NULL;
    supplier_price->participant = NULL;
    supplier_price->cost_price = G_MINDOUBLE;

    return 0;
}

