/* direct_debit_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "direct_debit_model.h"
#include "parameter_manager.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "invoice_model.h"
#include "warehouse_model.h"


#define TABLE         "Domiciliacion "
#define ATTRIBUTES    "Domiciliacion.codigoDomiciliacion as codigoDomiciliacionDirectDebit , \
                       Domiciliacion.codigoEmpresa as codigoEmpresaDirectDebit , \
                       Domiciliacion.codigoTienda as codigoTiendaDirectDebit , \
                       Domiciliacion.codigoFactura as codigoFacturaDirectDebit , \
                       Domiciliacion.estado as estadoDirectDebit "



/**
 * m_direct_debit_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_direct_debit_get_table () {
    return TABLE;
}

/**
 * m_direct_debit_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_direct_debit_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_direct_debit_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #DirectDebit con los campos rellenados con los datos de la base de datos.
 */
DirectDebit *m_direct_debit_create (GdaRecordset *rs) {
    GdaField *field;
    DirectDebit  *direct_debit = d_direct_debit_create_empty();

    field = gda_recordset_field_name(rs,"codigoDomiciliacionDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->direct_debit_code = igalia_field_get_bigint_value (field);
	} 
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoEmpresaDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->enterprise_code = igalia_field_get_bigint_value (field);
	} 
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs,"codigoTiendaDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->shop_code = igalia_field_get_bigint_value (field);
	} 
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs,"codigoFacturaDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->invoice_code = igalia_field_get_bigint_value (field);
	} 
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"estadoDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->state = gda_field_get_integer_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"esAPlazosDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->is_hire_purchase = gda_field_get_integer_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"plazoActualDirectDebit");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    direct_debit->current_instalment = gda_field_get_integer_value(field);
	}
	gda_field_free(field);
    }

    return direct_debit;
}

/**
 * m_direct_debit_insert:
 * @direct_debit: Objeto que representa la domiciliacion asociada a una factura.
 * 
 * Inserta el objeto direct_debit en la BD. Cada domiciliacin puede estar asociada a varias facturas, con lo que es necesario
 * asociarlas en la Bd.
 *
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_direct_debit_insert (DirectDebit * direct_debit) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    gulong logged_shop_code = d_direct_debit_get_shop_code (direct_debit);
    gulong logged_enterprise_code=d_direct_debit_get_enterprise_code (direct_debit);
    gchar * invoice_code_gchar = g_strdup_printf("%lu",d_direct_debit_get_invoice_code (direct_debit));
    gchar * direct_debit_code_gchar = g_strdup_printf("%lu",d_direct_debit_get_code (direct_debit));
    gchar * shop_code_gchar=g_strdup_printf("%lu",logged_shop_code);
    gchar * enterprise_code_gchar=g_strdup_printf("%lu",logged_enterprise_code);
    gchar * state_gchar;
    gchar * is_hire_purchase_gchar;
    gchar * current_instalment_gchar;
    gulong state;

    if (d_direct_debit_get_state (direct_debit)!=-1) {
        state_gchar = g_strdup_printf("%d",d_direct_debit_get_state (direct_debit));
    } else {
        state_gchar = g_strdup("NULL");
    }
    if (d_direct_debit_get_is_hire_purchase (direct_debit)!=FALSE) {
        is_hire_purchase_gchar = g_strdup("TRUE");
        current_instalment_gchar = g_strdup_printf("%d",d_direct_debit_get_current_instalment (direct_debit));
    } else {
        is_hire_purchase_gchar = g_strdup("FALSE");
        current_instalment_gchar = g_strdup("NULL");
    }

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,ULONG_MAX);

    state = m_direct_debit_get_state_from_invoice (d_direct_debit_get_invoice_code (direct_debit)
						 ,d_direct_debit_get_enterprise_code (direct_debit)
						 ,d_direct_debit_get_shop_code (direct_debit));

    if (state ==ULONG_MAX)
	query = g_strconcat(" INSERT INTO DOMICILIACION VALUES (",
			    direct_debit_code_gchar,",",
			    invoice_code_gchar,",",
			    enterprise_code_gchar,",",
			    shop_code_gchar,",",
			    state_gchar,",",
			    is_hire_purchase_gchar,",",
			    current_instalment_gchar,")",
			    NULL);
    else if (state < D_DIRECT_DEBIT_PAID)
	query = g_strconcat(" UPDATE DOMICILIACION SET CODIGODOMICILIACION=",direct_debit_code_gchar,
			    " ,ESTADO = ",state_gchar,
			    " ,ESAPLAZOS = ",is_hire_purchase_gchar,
			    " ,PLAZOACTUAL = ",current_instalment_gchar,
			    " WHERE CODIGOFACTURA=",invoice_code_gchar,
			    " AND CODIGOEMPRESA=",enterprise_code_gchar,
			    " AND CODIGOTIENDA=",shop_code_gchar,NULL
			   );
    else
        query = g_strconcat(" INSERT FALSE;",NULL);


    result = result && m_model_command_execute (&record_set, query, connection,
				   command,tuples,0, FALSE);


    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);

    if (record_set!=NULL)
	gda_recordset_free(record_set);


    if (invoice_code_gchar!=NULL)
        g_free(invoice_code_gchar);
    if (direct_debit_code_gchar!=NULL)
        g_free(direct_debit_code_gchar);
    if (shop_code_gchar!=NULL)
        g_free(shop_code_gchar);
    if (enterprise_code_gchar!=NULL)
	g_free(enterprise_code_gchar);
    if (state_gchar!=NULL)
	g_free(state_gchar);
    if (is_hire_purchase_gchar!=NULL)
	g_free(is_hire_purchase_gchar);
    if (current_instalment_gchar!=NULL)
	g_free(current_instalment_gchar);


    igalia_command_free(command);

    return result;
}

/**
 * m_direct_debit_insert_in_transaction:
 * @direct_debit: Objeto que representa la domiciliacin
 * @connection: Conexin en la que se ejecuta la actual transaccin.
 * 
 * Funcin que se utiliza para insertar en la base de datos una domiciliacin,
 * cuando sea necesario una transaccin.
 *
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_direct_debit_insert_in_transaction (DirectDebit * direct_debit,GdaConnection * connection) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaCommand * command = gda_command_new();
    gulong state;

    gulong logged_shop_code = d_direct_debit_get_shop_code (direct_debit);
    gulong logged_enterprise_code=d_direct_debit_get_enterprise_code (direct_debit);
    gchar * invoice_code_gchar = g_strdup_printf("%lu",d_direct_debit_get_invoice_code (direct_debit));
    gchar * direct_debit_code_gchar = g_strdup_printf("%lu",d_direct_debit_get_code (direct_debit));
    gchar * shop_code_gchar=g_strdup_printf("%lu",logged_shop_code);
    gchar * enterprise_code_gchar=g_strdup_printf("%lu",logged_enterprise_code);
    gchar * state_gchar;
    gchar * is_hire_purchase_gchar;
    gchar * current_instalment_gchar;

    if (d_direct_debit_get_state (direct_debit)!=-1) {
        state_gchar = g_strdup_printf("%d",d_direct_debit_get_state (direct_debit));
    } else {
        state_gchar = g_strdup("NULL");
    }
    if (d_direct_debit_get_is_hire_purchase (direct_debit)!=FALSE) {
        is_hire_purchase_gchar = g_strdup("TRUE");
        current_instalment_gchar = g_strdup_printf("%d",d_direct_debit_get_current_instalment (direct_debit));
    } else {
        is_hire_purchase_gchar = g_strdup("FALSE");
        current_instalment_gchar = g_strdup("NULL");
    }

    gda_command_set_connection(command,connection);

    state = m_direct_debit_get_state_from_invoice (d_direct_debit_get_invoice_code (direct_debit)
						 ,d_direct_debit_get_enterprise_code (direct_debit)
						 ,d_direct_debit_get_shop_code (direct_debit));


    if (state ==ULONG_MAX)
	query = g_strconcat(" INSERT INTO DOMICILIACION VALUES (",
			    direct_debit_code_gchar,",",
			    invoice_code_gchar,",",
			    enterprise_code_gchar,",",
			    shop_code_gchar,",",
			    state_gchar,",",
			    is_hire_purchase_gchar,",",
			    current_instalment_gchar,")",
			    NULL);
    else if (state < D_DIRECT_DEBIT_PAID)
	query = g_strconcat(" UPDATE DOMICILIACION SET CODIGODOMICILIACION=",direct_debit_code_gchar,
			    " ,ESTADO = ",state_gchar,
			    " ,ESAPLAZOS = ",is_hire_purchase_gchar,
			    " ,PLAZOACTUAL = ",current_instalment_gchar,
			    " WHERE CODIGOFACTURA=",invoice_code_gchar,
			    " AND CODIGOEMPRESA=",enterprise_code_gchar,
			    " AND CODIGOTIENDA=",shop_code_gchar,NULL
			   );
    else
        /*PARA PROVOCAR EL ROLLBACK*/
        query = g_strconcat(" INSERT FALSE;",NULL);

    g_print(_("\nUpdate: %s\n"),query);
    result = m_model_command_execute (&record_set, query, connection,
				   command,tuples,0, FALSE);



    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (invoice_code_gchar!=NULL)
        g_free(invoice_code_gchar);
    if (direct_debit_code_gchar!=NULL)
        g_free(direct_debit_code_gchar);
    if (shop_code_gchar!=NULL)
        g_free(shop_code_gchar);
    if (enterprise_code_gchar!=NULL)
	g_free(enterprise_code_gchar);
    if (state_gchar!=NULL)
	g_free(state_gchar);
    if (is_hire_purchase_gchar!=NULL)
	g_free(is_hire_purchase_gchar);
    if (current_instalment_gchar!=NULL)
	g_free(current_instalment_gchar);

    if (record_set!=NULL)
	gda_recordset_free(record_set);




    igalia_command_free(command);

    return result;
}


/**
 * m_direct_debit_insert_array:
 * @direct_debits: Lista de objetos que representan la domiciliacion asociada a una factura.
 * 
 * Inserta una lista de objetos direct_debit en la BD. Cada domiciliacin puede estar asociada a varias facturas, con lo que es necesario
 * asociarlas en la Bd.
 *
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_direct_debit_insert_array (DirectDebit ** direct_debits) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;
    gint index=0;
    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,ULONG_MAX);


    if (direct_debits!=NULL) {
	while (direct_debits[index]!=NULL) {
	    result  = result && m_direct_debit_insert_in_transaction (direct_debits[index],connection);
            index++;
	}
    }
    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);

    if (record_set!=NULL)
	gda_recordset_free(record_set);




    igalia_command_free(command);

    return result;
}




/**
 * m_direct_debit_return_in_transaction:
 * @direct_debit: Objeto que representa la domiciliacin
 * @connection: Conexin en la que se ejecuta la actual transaccin.
 * 
 * Funcin que se utiliza para modificar en la base de datos el estade de una domiciliacin,
 * cuando sea necesario realizarlo en una transaccin.
 * 
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_direct_debit_return_in_transaction (DirectDebit * direct_debit,GdaConnection * connection) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    DirectDebit ** direct_debits_aux=NULL;
    GdaCommand * command = gda_command_new();
    gulong state;
    gint index=0;
    gulong logged_enterprise_code=d_direct_debit_get_enterprise_code (direct_debit);
    gchar * direct_debit_code_gchar = g_strdup_printf("%lu",d_direct_debit_get_code (direct_debit));
    gchar * enterprise_code_gchar=g_strdup_printf("%lu",logged_enterprise_code);
    gchar * state_gchar;


    if (d_direct_debit_get_state (direct_debit)!=-1) {
        state_gchar = g_strdup_printf("%d",d_direct_debit_get_state (direct_debit));
    } else {
        state_gchar = g_strdup("NULL");
    }
    gda_command_set_connection(command,connection);

    state = m_direct_debit_get_state_from_invoice (d_direct_debit_get_invoice_code (direct_debit)
						 ,d_direct_debit_get_enterprise_code (direct_debit)
						 ,d_direct_debit_get_shop_code (direct_debit));

    query = g_strconcat(" UPDATE DOMICILIACION SET ESTADO = ",state_gchar,
                        ",PLAZOACTUAL=(PLAZOACTUAL-1) ",
			" WHERE CODIGODOMICILIACION=",direct_debit_code_gchar,
                        " AND CODIGOEMPRESA=",enterprise_code_gchar,
			NULL
		       );


    g_print(_("\nUpdate: %s\n"),query);
    result = result && m_model_command_execute (&record_set, query, connection,
				   command,tuples,0, FALSE);

    gda_recordset_close(record_set);
    record_set=NULL;

    if (query!=NULL) g_free(query);
    query = g_strconcat(" SELECT codigoDomiciliacion as codigoDomiciliacionDirectDebit, ",
                        " codigoEmpresa as codigoEmpresaDirectDebit,",
                        " codigoTienda as codigoTiendaDirectDebit,",
                        " codigoFactura as codigoFacturaDirectDebit ",
			" FROM DOMICILIACION WHERE CODIGODOMICILIACION=",direct_debit_code_gchar,
			" AND CODIGOEMPRESA=",enterprise_code_gchar,
			NULL
		       );

    g_print(_("\nQuery: %s\n"),query);
    result = result && m_model_command_execute (&record_set, query, connection,
				   command,tuples,0, FALSE);

    if (query!=NULL) g_free(query);
    if (record_set!=NULL) {
	if (*tuples>=1) {
	    gda_recordset_move_first(record_set);
	    direct_debits_aux = (DirectDebit **) g_malloc0(sizeof(DirectDebit *)*(*tuples+1));
	    while (!gda_recordset_eof(record_set)) {
                direct_debits_aux[index]=m_direct_debit_create (record_set);
		gda_recordset_move_next(record_set);
		index++;
	    }
	}
	gda_recordset_close(record_set);
    }

    if (direct_debits_aux!=NULL) {
	result = result && m_invoice_get_state_from_direct_debit (direct_debits_aux,connection);
    } else {
	result = FALSE;
    }
    if (tuples!=NULL) g_free(tuples);
    if (direct_debit_code_gchar!=NULL)
        g_free(direct_debit_code_gchar);
    if (enterprise_code_gchar!=NULL)
	g_free(enterprise_code_gchar);
    if (state_gchar!=NULL)
	g_free(state_gchar);

    if (record_set!=NULL)
	gda_recordset_free(record_set);




    igalia_command_free(command);

    return result;
}


/**
 * m_direct_debit_return_array:
 * @direct_debits: Lista de objetos que representan la domiciliacion asociada a una factura.
 *
 *  Modifica el estado de una lista de objetos direct_debit en la BD.
 *
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_direct_debit_return_array (GList * direct_debits) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gchar * query_error=g_strconcat("INSERT INTO FALSE",NULL);
    GList * element=NULL;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;
    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,ULONG_MAX);

    if (direct_debits!=NULL) {
	element = g_list_first(direct_debits);
	while (element!=NULL) {
	    result  = result && m_direct_debit_return_in_transaction ((DirectDebit *) element->data,connection);
            element=g_list_next(element);
	}
    }
    if (result == FALSE) {
	m_model_command_execute (&record_set, query_error, connection,command,tuples,0,TRUE);
    }

    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);

    if (record_set!=NULL)
	gda_recordset_free(record_set);


    igalia_command_free(command);

    return result;
}


/**
 * m_direct_debit_find_by_code:
 * @direct_debit_code: Cdigo de la domiciliacin.
 * 
 * Busca las facturas asociadas a una domiciliacin en particular.
 *
 * Returns: Lista de objetos DirectDebit, que representan la asociacin de una domiciliacin con
 * con una factura. %NULL si no existe ninguna.
 */
DirectDebit ** m_direct_debit_find_by_code (gulong direct_debit_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gint index;
    DirectDebit ** result;
    gchar * code_char = g_strdup_printf("%lu",direct_debit_code);
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat(" SELECT * from domiciliacion where codigoDomiciliacion=",code_char,
			       NULL );

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples>0) {
	    DirectDebit * direct_debit;
            result = (DirectDebit **)g_malloc0(sizeof(DirectDebit *)*(*tuples+1));
	    gda_recordset_move_first(record_set);
            index=0;
	    while (!gda_recordset_eof(record_set)) {
		direct_debit = m_direct_debit_create (record_set);
		result[index] = direct_debit;
		gda_recordset_move_next(record_set);
		index ++;
	    }
	} else {
	    result=NULL;
	}
	gda_recordset_close(record_set);
    } else {
        result  = NULL;
    }


    g_free(gchar_query);
    g_free(tuples);


    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;
}

/**
 * m_direct_debit_clear_array: 
 * @direct_debits: El array de domiciliaciones a liberar
 * 
 * Liberacin de memoria de los objetos direct_debits
 *
 */
void m_direct_debit_clear_array (DirectDebit ** direct_debits) {
    if (direct_debits!=NULL) {
	gint index = 0;
	while (direct_debits[index]!=NULL) {
	    d_direct_debit_clear (direct_debits[index]);
	    index++;
	}
    }
    g_free(direct_debits);
    direct_debits=NULL;
}

/**
 * m_direct_debit_get_next_code:
 *
 * Devuelve el prximo cdigo de domiciliacin
 *
 * Returns: El prximo cdigo de domiciliacin
 */
gchar * m_direct_debit_get_next_code () {
    GdaConnection      *my_connection = m_model_get_connection ();
    gchar * result;

    result = m_model_nextval_sequence ("seqDomiciliacion",my_connection);

    return result;
}
/**
 * m_direct_debit_get_next_code_from_invoice:
 * @invoice_code: Cdigo de la factura para la cual se obtendr el cdigo de domiciliacin adecuado
 * @enterprise_code: Cdigo de la empresa que forma parte de la clave de factura
 * @shop_code: Cdigo de la tienda que forma parte de la clave de factura
 *
 *  Devuelve el prximo cdigo de domiciliacin.
 *
 * Returns: El prximo cdigo de domiciliacin
 */
gchar * m_direct_debit_get_next_code_from_invoice (gulong invoice_code,gulong enterprise_code,gulong shop_code) {
    GdaCommand         *my_command;
    GdaConnection      *my_connection = m_model_get_connection ();
    gchar * invoice_code_gchar = g_strdup_printf("%lu",invoice_code);
    gchar * enterprise_code_gchar = g_strdup_printf("%lu",enterprise_code);
    gchar * shop_code_gchar = g_strdup_printf("%lu",shop_code);
    gchar * result=NULL;
    gulong *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset       *record_set = NULL;
    gchar * gchar_query =g_strconcat(" select codigoDomiciliacion from domiciliacion where codigoFactura=",invoice_code_gchar,
                               " and codigoEmpresa=",enterprise_code_gchar,
                               " and codigoTienda=",shop_code_gchar,
			       NULL
			      );


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    g_print(_("\nQuery: %s\n"),gchar_query);
    if (record_set!=NULL) {
	if (*tuples==1) {
            GdaField * field =NULL;
	    gda_recordset_move_first(record_set);
	    field = gda_recordset_field_idx(record_set,0);
	    result = gda_stringify_value(NULL, 0, field);
            if (field!=NULL)
		gda_field_free(field);
	} 
	gda_recordset_close(record_set);
    } else

    g_free(tuples);
    g_free(gchar_query);
    g_free(invoice_code_gchar);
    g_free(shop_code_gchar);
    g_free(enterprise_code_gchar);
    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;
}


/**
 * m_direct_debit_generate_from_invoice_array:
 * @direct_debit_code: Cdigo de la domiciliacin.
 * @invoices: Lista de facturas a partir de las cules se generarn las domiciliaciones.
 * @new_state: Estado de la que se asociar a las domiciliaciones.
 *
 * Genera una lista de domiciliaciones a partir de una lista de facturas
 * un cdigo de domiciliacin y del estado. Esta funcin que asocia a un mismo cdigo
 * de domiciliacin una lista de facturas, ya que varias facturas pueden
 * formar parte de una sola domiciliacin.
 *
 * Returns: Lista de domiciliaciones
 */
DirectDebit ** m_direct_debit_generate_from_invoice_array (gchar * direct_debit_code,GList * invoices,gint new_state) {
    DirectDebit ** result = NULL;

    if (invoices!=NULL) {
	guint length = g_list_length(invoices);
	gint index=0;
	GList * element;
	gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
	result = (DirectDebit**) g_malloc0(sizeof(DirectDebit *)*(length+1));
	element = g_list_first(invoices);
	while (element!=NULL) {
	    Invoice * invoice = (Invoice *)element->data;
	    gboolean is_hire_purchase = d_payment_get_is_hire_purchase (d_invoice_get_payment (invoice));
	    DirectDebit * direct_debit = d_direct_debit_create_empty ();
	    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice)));
	    gulong enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
	    gulong invoice_code = d_invoice_get_invoice_code (invoice);
	    gulong direct_debit_code_gulong = strtoul(direct_debit_code,stop,10);


	    d_direct_debit_set_shop_code (direct_debit,shop_code);
	    d_direct_debit_set_enterprise_code (direct_debit,enterprise_code);
	    d_direct_debit_set_invoice_code (direct_debit,invoice_code);
	    d_direct_debit_set_code (direct_debit,direct_debit_code_gulong);
	    d_direct_debit_set_state (direct_debit,new_state);
	    d_direct_debit_set_is_hire_purchase (direct_debit,is_hire_purchase);
	    if (is_hire_purchase!=FALSE) {
                d_direct_debit_set_current_instalment (direct_debit,d_payment_get_achieved_payments (d_invoice_get_payment (invoice)));
	    }
	    result[index] = direct_debit;
	    element = g_list_next(element);
	    index++;
	}
	g_free(stop);
    }

    return result;

}

/**
 * m_direct_debit_get_state_from_invoice:
 * @invoice_code: Cdigo de la factura.
 * @enterprise_code: Empresa en la que se emiti la factura
 * @shop_code: Tienda en la que se emiti la factura.
 *
 * Funcin que devuelve el estado de la domiciliacin de una factura.
 * Una factura puede ya estar en proceso de domiciliacin, con lo que la prxima vez que se domicilie puede
 * estar ya en la base de datos con un cdigo domiciliacin. Esto implica que no se debe insertar de nuevo la
 * domiciliacin
 *
 * Returns: El estado de la domiciliacin. %ULONG_MAX si no existe entrada en la tabla para dicha factura.
 */
gulong  m_direct_debit_get_state_from_invoice (gulong invoice_code,gulong enterprise_code,gulong shop_code) {
    gulong result = ULONG_MAX;
    GdaCommand         *my_command;
    GdaConnection      *my_connection = m_model_get_connection ();
    gchar * invoice_code_gchar = g_strdup_printf("%lu",invoice_code);
    gchar * enterprise_code_gchar = g_strdup_printf("%lu",enterprise_code);
    gchar * shop_code_gchar = g_strdup_printf("%lu",shop_code);
    gulong *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset       *record_set = NULL;
    gchar * gchar_query =g_strconcat(" select estado from domiciliacion where codigoFactura=",invoice_code_gchar,
                               " and codigoEmpresa=",enterprise_code_gchar,
                               " and codigoTienda=",shop_code_gchar,
			       NULL
			      );


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
	    GdaField * field =NULL;
	    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
            gchar * temp;
	    gda_recordset_move_first(record_set);
	    field = gda_recordset_field_idx(record_set,0);
	    temp = gda_stringify_value(NULL, 0, field);
	    result = strtoul(temp,stop,10);
            g_free(stop);
	    g_free(temp);
            if (field!=NULL)
		gda_field_free(field);
	} 
	gda_recordset_close(record_set);
    } 

    g_free(tuples);
    g_free(gchar_query);
    g_free(invoice_code_gchar);
    g_free(shop_code_gchar);
    g_free(enterprise_code_gchar);
    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;
}

/**
 * m_direct_debit_set_state_from_invoice:
 * @invoice_code: Cdigo de la factura.
 * @enterprise_code: Empresa en la que se emiti la factura
 * @shop_code: Tienda en la que se emiti la factura.
 * @new_state: Estado en el que se encuentra la domiciliacin.
 *
 * Funcin que establece el estado de la domiciliacin de una factura. Se utiliza para establecer como pagada
 * una domiciliacin.
 *
 * Returns:
 * %TRUE si se ha realizado correctamente.
 */
gboolean m_direct_debit_set_state_from_invoice (gulong invoice_code,gulong enterprise_code,gulong shop_code,gint new_state) {
    GdaCommand         *my_command;
    GdaConnection      *my_connection = m_model_get_connection ();
    gchar * invoice_code_gchar = g_strdup_printf("%lu",invoice_code);
    gchar * enterprise_code_gchar = g_strdup_printf("%lu",enterprise_code);
    gchar * shop_code_gchar = g_strdup_printf("%lu",shop_code);
    gchar * state = g_strdup_printf("%d",new_state);
    gulong *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset       *record_set = NULL;
    gboolean result=FALSE;
    gchar * gchar_query =g_strconcat(" update domiciliacion set estado =",state," where codigoFactura=",invoice_code_gchar,
                               " and codigoEmpresa=",enterprise_code_gchar,
                               " and codigoTienda=",shop_code_gchar,
			       NULL
			      );


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    result = m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	gda_recordset_close(record_set);
    } 

    g_free(tuples);
    g_free(gchar_query);
    g_free(invoice_code_gchar);
    g_free(state);
    g_free(shop_code_gchar);
    g_free(enterprise_code_gchar);
    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;
}



/**
 * m_direct_debit_find_number_of_instalment:
 * @invoice_code: Cdigo de factura.
 * @shop_code: Cdigo de tienda.
 * @enterprise_code: Cdigo de la empresa.
 *
 * Busca el nmero de plazos de una factura.
 *
 * Returns:
 * Nmero de plazos.
 */
gint m_direct_debit_find_number_of_instalment (gulong invoice_code,gulong shop_code,gulong enterprise_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_query_payment=NULL;
    gchar              *gchar_select_payment=NULL;
    gchar              *gchar_where_payment=NULL;
    gchar              *gchar_from_payment=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    GdaRecordset       *record_set_payment = NULL;
    gchar * invoice_code_char = g_strdup_printf("%lu",invoice_code);
    gchar * enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    gchar * shop_code_char = g_strdup_printf("%lu",shop_code);
    gint result=0;
    gulong * tuples;
    gulong * tuples_payment;
    gulong index;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    tuples_payment = (gulong *) g_malloc0(sizeof(gulong));

    gchar_select = g_strconcat("select plazoActual",NULL);
    gchar_from= g_strconcat(" from domiciliacion ",NULL);
    gchar_where = g_strconcat("where ",
                              " codigoFactura=",invoice_code_char,
                              " and codigoEmpresa=",enterprise_code_char,
			      " and codigoTienda=",shop_code_char,NULL);

    gchar_select_payment = g_strconcat("select CAST(pagosRealizados as integer)",NULL);
    gchar_from_payment= g_strconcat(" from facturaabono,pago ",NULL);
    gchar_where_payment = g_strconcat("where pago.esAPlazos = TRUE and facturaabono.codigoPago=pago.codigoPago",
                              " and facturaabono.codigoTienda=pago.codigoTienda ",
                              " and facturaabono.codigoEmpresa=pago.codigoEmpresa ",
                              " and facturaabono.codigoFactura=",invoice_code_char,
                              " and facturaabono.codigoEmpresa=",enterprise_code_char,
			      " and facturaabono.codigoTienda=",shop_code_char,NULL);

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where, NULL);
    gchar_query_payment = g_strconcat(gchar_select_payment,gchar_from_payment,gchar_where_payment, NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    g_print(_("\nQuery: %s\n"),gchar_query_payment);
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0,TRUE);

    m_model_command_execute (&record_set_payment, gchar_query_payment, my_connection, my_command,tuples_payment,0,TRUE);


    if (record_set!=NULL) {
	if (*tuples==1) {
	    GdaField * field;
	    index = gda_recordset_move_first(record_set);
	    if (index==GDA_RECORDSET_INVALID_POSITION) return -1;
	    field = gda_recordset_field_idx(record_set,0);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    result = igalia_field_get_bigint_value (field);
		} else {
		    result=0;
		}
	    }
	} else {
	    if (*tuples_payment==1) {
		GdaField * field;
		index = gda_recordset_move_first(record_set_payment);
		if (index==GDA_RECORDSET_INVALID_POSITION) return -1;
		field = gda_recordset_field_idx(record_set_payment,0);
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result = gda_field_get_integer_value(field);
		    } else {
			result=0;
		    }
		}
	    }
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    }

    g_free(invoice_code_char);
    g_free(enterprise_code_char);
    g_free(shop_code_char);
    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_from);
    g_free(gchar_query);
    g_free(gchar_where_payment);
    g_free(gchar_select_payment);
    g_free(gchar_from_payment);
    g_free(gchar_query_payment);
    g_free(tuples);
    g_free(tuples_payment);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;

}
