/* enterprise_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "enterprise_model.h"
#include "participant_model.h"

#include "util.h"

Enterprise ** enterprises_list = NULL;
gint length_of_list;



#define TABLE         "Empresa"
#define IDENTIFICATOR "codigoParty"
#define ATTRIBUTES    "codigoParty as codigoempresaenterprise, \
                       pertenecegrupo as pertenecegrupoenterprise"

/**
 * m_enterprise_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_enterprise_get_table () {
    return TABLE;
}

/**
 * m_enterprise_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_enterprise_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_enterprise_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Enterprise con los campos rellenados con los datos de la base de datos.
 */
Enterprise * m_enterprise_create (GdaRecordset *rs) {
    GdaField * field;
    Enterprise * enterprise = d_enterprise_create_empty();

    enterprise->participant = m_participant_create (rs,"enter");
    
    if (d_participant_get_code (d_enterprise_get_participant (enterprise)) != ULONG_MAX) {
       field = gda_recordset_field_name(rs,"pertenecegrupoenterprise");
       if (field!=NULL) {
	   if (!gda_field_is_null(field)) {
	       enterprise->belong_to_principal = gda_field_get_boolean_value(field);
	   }
	   gda_field_free(field);
       }
    }

    return enterprise;
}


/**
 * m_enterprise_find_by_shop:
 * @shop_code:Cdigo de la tienda.
 * 
 * Busca una empresa a la que pertenece la tienda con cdigo de tienda @shop_code.
 *
 * Returns:
 * Empresa a la que pertenece la tienda.
 */
Enterprise * m_enterprise_find_by_shop (gint shop_code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Enterprise * enterprise;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",shop_code);
     gchar_query = g_strconcat("SELECT ",
			       " Participante.codigoParty as codigopartyparticipantenter, ",
			       " Participante.nombre as nombreparticipantenter, ",
			       " Participante.cif as cifparticipantenter, ",
			       " Participante.nick as nickparticipantenter, ",
			       " Participante.telefono as telefonoparticipantenter, ",
			       " Participante.telefono2 as telefono2participantenter, ",
			       " Participante.fax as faxparticipantenter, ",
			       " Participante.email as emailparticipantenter, ",
			       " Participante.calle as calleparticipantenter, ",
			       " Participante.numero as numeroparticipantenter, ",
			       " Participante.piso as pisoparticipantenter, ",
			       " Participante.CP as cpparticipantenter, ",
			       " Participante.municipio as municipioparticipantenter",
			      " FROM EMPRESA,PARTICIPANTE,TIENDA WHERE ",
			      "(EMPRESA.CODIGOPARTY=TIENDA.CODIGOEMPRESA) AND ",
			      "(PARTICIPANTE.CODIGOPARTY=EMPRESA.CODIGOPARTY) ",
			      "AND (TIENDA.CODIGOTIENDA=",aux,")",NULL);


    g_free(aux);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    g_free(gchar_query);
    
    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if ((*tuples)==1) {
   	    gda_recordset_move_first(record_set);
	    enterprise = m_enterprise_create (record_set);
	} else {
	    enterprise = NULL;
	}
        gda_recordset_close(record_set);
    } else {
        enterprise = NULL;
    }

    igalia_command_free(my_command);

    g_free(tuples);
    g_free(column);

    return enterprise;
}

/**
 * model_find_enterprise_by_code 
 * @enterprise_code:Cdigo de la empresa que se desea obtener.
 * 
 * Busca una empresa por su identificador.
 *
 * Returns:
 * Empresa solicitada.
 */
Enterprise * m_enterprise_find_by_code (gint enterprise_code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Enterprise * enterprise;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",enterprise_code);
     gchar_query = g_strconcat("SELECT ",
			       " Participante.codigoParty as codigopartyparticipantenter, ",
			       " Participante.nombre as nombreparticipantenter, ",
			       " Participante.cif as cifparticipantenter, ",
			       " Participante.nick as nickparticipantenter, ",
			       " Participante.telefono as telefonoparticipantenter, ",
			       " Participante.telefono2 as telefono2participantenter, ",
			       " Participante.fax as faxparticipantenter, ",
			       " Participante.email as emailparticipantenter, ",
			       " Participante.calle as calleparticipantenter, ",
			       " Participante.numero as numeroparticipantenter, ",
			       " Participante.piso as pisoparticipantenter, ",
			       " Participante.CP as cpparticipantenter, ",
			       " Participante.municipio as municipioparticipantenter ",
			      " FROM EMPRESA,PARTICIPANTE WHERE ",
			      " (PARTICIPANTE.CODIGOPARTY=EMPRESA.CODIGOPARTY) ",
			      "AND (PARTICIPANTE.CODIGOPARTY=",aux,")",NULL);


    g_free(aux);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    g_free(gchar_query);
    
    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if ((*tuples)==1) {
   	    gda_recordset_move_first(record_set);
	    enterprise = m_enterprise_create (record_set);
	} else {
	    enterprise = NULL;
	}
        gda_recordset_close(record_set);
    } else {
        enterprise = NULL;
    }

    igalia_command_free(my_command);

    g_free(tuples);
    g_free(column);

    return enterprise;
}


/**
 * m_enterprise_find_all:
 * 
 * Busca todas las empresas del grupo
 *
 * Returns:
 * Array con las empresas del grupo.
 */
Enterprise**m_enterprise_find_all () {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Enterprise * enterprise;
    gchar * gchar_query;
    gulong * tuples;
    gint * column;
    gint index;


    if (enterprises_list==NULL) {
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

	gchar_query = g_strconcat("SELECT ",
				  " Participante.codigoParty as codigopartyparticipantenter, ",
				  " Participante.nombre as nombreparticipantenter, ",
				  " Participante.cif as cifparticipantenter, ",
				  " Participante.nick as nickparticipantenter, ",
				  " Participante.telefono as telefonoparticipantenter, ",
				  " Participante.telefono2 as telefono2participantenter, ",
				  " Participante.fax as faxparticipantenter, ",
				  " Participante.email as emailparticipantenter, ",
				  " Participante.calle as calleparticipantenter, ",
				  " Participante.numero as numeroparticipantenter, ",
				  " Participante.piso as pisoparticipantenter, ",
				  " Participante.CP as cpparticipantenter, ",
				  " Participante.municipio as municipioparticipantenter ",
				  " FROM ",m_enterprise_get_table (),",",m_participant_get_table (),
                                  " WHERE EMPRESA.CODIGOPARTY=PARTICIPANTE.CODIGOPARTY AND ",
                                  " EMPRESA.PERTENECEGRUPO=TRUE",
                                  NULL);

	tuples = (gulong *) g_malloc0(sizeof(gulong));
	m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
	g_free(gchar_query);

        length_of_list = *tuples;
	column = (gint *) g_malloc0(sizeof(gint));

	if (record_set!=NULL) {
	    if (*tuples!=0) {
		enterprises_list = (Enterprise **) g_malloc0(sizeof(Enterprise*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    (*column)=0;
		    enterprise = m_enterprise_create (record_set);
		    enterprises_list[index-1] = enterprise;
		    index = gda_recordset_move_next(record_set);
		}
		enterprises_list[length_of_list]=NULL;

	    } else {
		enterprises_list=NULL;
	    }
	} else {
	    enterprises_list=NULL;
	}

	gda_recordset_free(record_set);
	igalia_command_free(my_command);

	g_free(tuples);
	g_free(column);


    }
    return enterprises_list;
}

/**
 * m_enterprise_get_name_from_code:
 * @code:Cdigo de la empresa de la que se desea obtener el nombre.
 * 
 * Funcin que devuelve el nombre de la empresa con el identificador @code de la lista de
 * empresas.
 *
 * Returns:
 * Nombre de la empresa solicitada.
 */
gchar *m_enterprise_get_name_from_code (gulong code) {
    gchar *name = NULL;
    gint   index=0;

    if (enterprises_list==NULL) {
	m_enterprise_find_all ();
    }

    while(enterprises_list[index]!=NULL) {
	if (code == d_participant_get_code (d_enterprise_get_participant (enterprises_list[index]))) {
            name = d_participant_get_nick (d_enterprise_get_participant (enterprises_list[index]));
	}
        index++;
    }

    return name;
}

/**
 * m_enterprise_get_code_from_name:
 * @name:Nombre de la empresa de la que se desea obtener el cdigo.
 * 
 * Funcin que devuelve el cdigo de la empresa con el nombre @name de la lista de
 * empresas.
 *
 * Returns:
 * Cdigo de la empresa solicitada.
 */
gulong m_enterprise_get_code_from_name (gchar *name) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (enterprises_list==NULL) {
	m_enterprise_find_all ();
    }

    while(enterprises_list[index]!=NULL) {
        temp = d_participant_get_nick (d_enterprise_get_participant (enterprises_list[index]));
	if (!strcmp(name, temp)) {
	    code = d_participant_get_code (d_enterprise_get_participant (enterprises_list[index]));
	    g_free(temp);
           break;
	}
        g_free(temp);
        index++;
    }

    return code;
}


/**
 * m_enterprise_find_by_participant_code:
 * @my_command: Comando en el que se ejecutar la sentencia sql.
 * @my_connection: Conexin a la base de datos.
 * @party_code: Cdigo de participante por el que deseamos buscar la empresa.
 * 
 * Busca una empresa por el cdigo de participante en transaccin.
 *
 * Returns:
 * Empresa solicitada.
 */
Enterprise * m_enterprise_find_by_participant_code (GdaCommand * my_command,GdaConnection * my_connection,gint party_code) {
    
    GdaRecordset       * record_set = NULL;
    Enterprise * enterprise;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    aux = g_strdup_printf("%d",party_code);
     gchar_query = g_strconcat("SELECT ",
			       " Participante.codigoParty as codigopartyparticipantenter, ",
			       " Participante.nombre as nombreparticipantenter, ",
			       " Participante.cif as cifparticipantenter, ",
			       " Participante.nick as nickparticipantenter, ",
			       " Participante.telefono as telefonoparticipantenter, ",
			       " Participante.telefono2 as telefono2participantenter, ",
			       " Participante.fax as faxparticipantenter, ",
			       " Participante.email as emailparticipantenter, ",
			       " Participante.calle as calleparticipantenter, ",
			       " Participante.numero as numeroparticipantenter, ",
			       " Participante.piso as pisoparticipantenter, ",
			       " Participante.CP as cpparticipantenter, ",
			       " Participante.municipio as municipioparticipantenter",
			       " Participante.nombreCorto as shortNameparticipantenter",
			       " FROM PARTICIPANTE,EMPRESA WHERE ",
                               " PARTICIPANTE.CODIGOPARTY = EMPRESA.CODIGOPARTY",
			      " (PARTICIPANTE.CODIGOPARTY=",aux,")",NULL);


    g_free(aux);

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    g_free(gchar_query);


    
    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if ((*tuples)==1) {
   	    gda_recordset_move_first(record_set);
	    enterprise = m_enterprise_create (record_set);
	} else {
	    enterprise = NULL;
	}
        gda_recordset_close(record_set);
    } else {
        enterprise = NULL;
    }


    g_free(tuples);
    g_free(column);

    return enterprise;
}

/**
 * m_enterprise_get_code_from_legal_id:
 * @legal_id: Cif de la empresa que se desea buscar.
 * 
 * Busca el cdigo de una empresa por cif.
 *
 * Returns:
 * Cdigo de la empresa deseada.
 */
gulong m_enterprise_get_code_from_legal_id (gchar *legal_id) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (enterprises_list==NULL) {
	m_enterprise_find_all ();
    }

    while(enterprises_list[index]!=NULL) {
	temp = d_participant_get_legal_id (d_enterprise_get_participant (enterprises_list[index]));
	if (!u_cif_strcmp (legal_id, temp)) {
	    code = d_participant_get_code (d_enterprise_get_participant (enterprises_list[index]));
	    g_free(temp);
           break;
	}
        g_free(temp);
        index++;
    }

    return code;
}



/**
 * m_enterprise_find_by_shop_in_transaction:
 * @shop_code:Cdigo de la tienda.
 * @my_command:Comando en el que se ejecutar la/s sentencia/s sql.
 * @my_connection:Conexin a la base de datos.
 *
 * Busca una empresa a la que pertenece la tienda con cdigo de tienda @shop_code
 * dentro de una transaccin.
 *
 * Returns:
 * Empresa a la que pertenece la tienda.
 */
Enterprise * m_enterprise_find_by_shop_in_transaction (GdaCommand * my_command,GdaConnection * my_connection,gint shop_code) {
    GdaRecordset       * record_set = NULL;
    Enterprise * enterprise;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    aux = g_strdup_printf("%d",shop_code);

    gchar_query = g_strconcat("SELECT ",
			       " Participante.codigoParty as codigopartyparticipantenter, ",
			       " Participante.nombre as nombreparticipantenter, ",
			       " Participante.cif as cifparticipantenter, ",
			       " Participante.nick as nickparticipantenter, ",
			       " Participante.telefono as telefonoparticipantenter, ",
			       " Participante.telefono2 as telefono2participantenter, ",
			       " Participante.fax as faxparticipantenter, ",
			       " Participante.email as emailparticipantenter, ",
			       " Participante.calle as calleparticipantenter, ",
			       " Participante.numero as numeroparticipantenter, ",
			       " Participante.piso as pisoparticipantenter, ",
			       " Participante.CP as cpparticipantenter, ",
			       " Participante.municipio as municipioparticipantenter",
			      " FROM EMPRESA,PARTICIPANTE,TIENDA WHERE ",
			      "(EMPRESA.CODIGOPARTY=TIENDA.CODIGOEMPRESA) AND ",
			      "(PARTICIPANTE.CODIGOPARTY=EMPRESA.CODIGOPARTY) ",
			      "AND (TIENDA.CODIGOTIENDA=",aux,")",NULL);


    g_free(aux);

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    g_free(gchar_query);


    
    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if ((*tuples)==1) {
   	    gda_recordset_move_first(record_set);
	    enterprise = m_enterprise_create (record_set);
	} else {
	    enterprise = NULL;
	}
        gda_recordset_close(record_set);
    } else {
        enterprise = NULL;
    }


    g_free(tuples);
    g_free(column);

    return enterprise;


}

/**
 * m_enterprise_clear_array:
 * @enterprises: Array de empresas a liberar.
 *
 * Libera la memoria utilizada por el array de empresas.
 *
 */
void m_enterprise_clear_array (Enterprise ** enterprises) {
    if (enterprises!=NULL) {
	gint index = 0;
	while (enterprises[index]!=NULL) {
	    d_enterprise_clear (enterprises[index]);
	    index++;
	}
	g_free(enterprises);
    }
    enterprises  = NULL;
}

