/* exchange_piece_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "exchange_piece_model.h"
#include "stock_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"
#include "time.h"


#define TABLE      "Intercambio"

/**
 * m_exchange_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_exchange_get_table () {
    return TABLE;
}


/**
 * m_exchange_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Exchange con los campos rellenados con los datos de la base de datos.
 */
Exchange  *m_exchange_create (GdaRecordset *rs){
    Exchange * result=d_exchange_create_empty ();
    gchar  *temp;
    GdaField * field = NULL;
    

    field = gda_recordset_field_name(rs,"codigoempresaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoalmacenMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->warehouse_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->movement_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"unidadesMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->quantity = u_round_to_decimal (igalia_field_get_double_value (field),3);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    result->date = (*my_time);
	    g_free(my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }



    field = gda_recordset_field_name(rs,"codigoEmpresaDestinoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> consignee_enterprise_code=    igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"entradaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> in=    gda_field_get_boolean_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoAlmacenDestinoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> consignee_warehouse_code=    igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "nombreAlmacenDestinoMovimiento");
    if (field!=NULL){
	if (!gda_field_is_null(field)) {
	    temp = gda_stringify_value(NULL, 0, field);
	    result->consignee_warehouse_name = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoProductoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> product_code=igalia_field_get_bigint_value (field)  ;
	}
	gda_field_free(field);
    }



    return result;
}


/**
 * m_exchange_insert:
 * @exchange: Intercambio a insertar.
 * @stock_in: Stock de entrada que se modificar al realizar el intercambio.
 * @stock_out: Stock de salida que se modificar al realizar el intercambio.
 * 
 * Inserta un intercambio en la base de datos.
 *
 * Returns: %TRUE si se inserta en la base de datos, %FALSE en otro caso.
 */
gboolean m_exchange_insert (Exchange *exchange, Stock *stock_in,Stock *stock_out) {
    gchar         *gchar_query_exchange;
    gchar         *gchar_query_exchange_aux;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    time_t         fecha;
    gboolean       correct;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,ULONG_MAX,logged_warehouse_code);

    correct = m_stock_update (stock_in, connection, command);
    correct = m_stock_update (stock_out, connection, command);

    gchar_query_exchange = "";
    gchar_query_exchange = g_strconcat("INSERT INTO ",m_exchange_get_table (),
					     " VALUES ",
					     "('", NULL);

    temp = g_strdup_printf("%lu",d_exchange_get_enterprise_code (exchange));
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange, temp,
						 "', '", NULL);
    g_free(temp);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    temp = g_strdup_printf("%lu",d_exchange_get_warehouse_code (exchange));
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange, temp,
						 "', ", NULL);
    g_free(temp);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    temp = m_model_nextval_sequence ("seqMovimientoAlmacen", connection);
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange,
					   temp, ", ", NULL);
    g_free(gchar_query_exchange);
    g_free(temp);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    fecha = d_exchange_get_date (exchange);

    gchar_query_exchange_aux =
	g_strconcat(gchar_query_exchange, " '",ctime(&fecha),
		    "', '", NULL);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    temp = g_strdup_printf("%.3f",d_exchange_get_quantity (exchange));
    u_commas_to_points (temp);
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange, temp,
						 "', '", NULL);
    g_free(temp);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    temp = g_strdup_printf("%lu",d_exchange_get_consignee_enterprise_code (exchange));
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange, temp,
						 "', '", NULL);
    g_free(temp);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    temp = g_strdup_printf("%lu",d_exchange_get_consignee_warehouse_code (exchange));
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange, temp,
						 "',' ", NULL);
    g_free(temp);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    temp = g_strdup_printf("%lu",d_exchange_get_product_code (exchange));
    gchar_query_exchange_aux = g_strconcat(gchar_query_exchange, temp,
						 "')", NULL);
    g_free(temp);
    g_free(gchar_query_exchange);
    gchar_query_exchange = gchar_query_exchange_aux;
    gchar_query_exchange_aux = NULL;

    correct = correct && m_model_command_execute (&record_set, gchar_query_exchange, connection, command, tuples, 0, FALSE);

    correct = m_model_execute_mutex_modification_logged (connection) && correct;

    if (correct != FALSE) {
	d_stock_set_previous_quantity(stock_in,d_stock_get_quantity(stock_in));
	d_stock_set_previous_quantity(stock_out,d_stock_get_quantity(stock_out));
    }


    g_free(gchar_query_exchange);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);
    return correct;
}


