/* family_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "family_model.h"

#include "util.h"

#define TABLE         "Familia"
#define IDENTIFICATOR "codigoFamilia"
#define ATTRIBUTES    "codigoFamilia as codigoFamiliafamily, \
    descripcion as descripcionfamily"

static Family ** families_list=NULL;
static gint length_of_list=0;




/**
 * m_family_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_family_get_table () {
    return TABLE;
}

/**
 * m_family_get_identificator:
 *
 * Obtiene el identificador de la tabla a la que hace referencia el objeto.
 *
 * Returns:
 * Cadena de caracteres con el identificador del objeto.
 */
gchar *m_family_get_identificator () {
    return IDENTIFICATOR;
}

/**
 * m_family_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_family_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_family_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Family con los campos rellenados con los datos de la base de datos.
 */
Family *m_family_create (GdaRecordset *rs) {
    gchar    *temp;
    GdaField *field;
    Family   *family = d_family_create_empty();

    field = gda_recordset_field_name(rs,"codigofamiliafamily");
    if (field!=NULL) {
	if(!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    family->family_code   = g_strdup(temp);
	    g_free(temp);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"descripcionfamily");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    family->description  = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }
    return family;
}



/**
 * m_family_find_by_code:
 * @family_code: Cdigo identificador de la familia
 *
 *  Busca la familia de una pieza por
 * el cdigo de familia de la misma.
 *
 * Returns: Objeto familia identificada por el cdigo @family_code, o %NULL si no
 * existe una con dicho cdigo.
 */
Family * m_family_find_by_code (char * family_code) {
    GdaCommand    *my_command;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Family        *family;
    gchar         *gchar_query;
    gulong        *tuples;
    gboolean       correct = FALSE;
    gchar * table_family;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    table_family = u_table_put_name (m_family_get_attributes (),m_family_get_table ());
    gchar_query = g_strconcat("SELECT ",
			      table_family,
			      " FROM ", m_family_get_table (),
			      " WHERE ", m_family_get_identificator (),
			      " ILIKE '%", family_code, "%' ", NULL);

    g_free(table_family);
    g_print(gchar_query);
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    correct = m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
    g_free(gchar_query);


    family = NULL;
    if (correct) {
	if (*tuples!=0) {
	    gda_recordset_move_first(record_set);
	    family = m_family_create (record_set);
	}
        gda_recordset_close(record_set);
    } 

    igalia_command_free(my_command);
    gda_recordset_free(record_set);

    g_print(_("Freeing database completed\n"));
    g_free(tuples);
    return family;
}

/**
 * m_family_find_all:
 *
 * Obtiene un array con todas las familias existentes.
 * 
 * Returns: Array con todas las familias existentes, %NULL en caso de non existir ninguna o
 * error con la base de datos
 */
Family ** m_family_find_all () {
    GdaCommand    *my_command;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Family        *family;
    gchar         *gchar_query;
    gulong        *tuples;
    gint           index;
    gboolean       correct = FALSE;


    if (families_list==NULL) {
        gchar * table = u_table_put_name (m_family_get_attributes (),m_family_get_table ());
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);
	gchar_query = g_strconcat("SELECT ",
				  table,
				  " FROM ",m_family_get_table (), NULL);
        g_free(table);
	tuples = (gulong *) g_malloc0(sizeof(gulong));
	correct = m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0,TRUE);
        g_print(gchar_query);
	g_free(gchar_query);

        length_of_list = *tuples;

        families_list = NULL;
	if (correct) {
	    if (*tuples!=0) {
		families_list = (Family **) g_malloc0(sizeof(Family*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    family = m_family_create (record_set);
		    families_list[index-1] = family;
		    index = gda_recordset_move_next(record_set);
		}
		families_list[length_of_list]=NULL;

	    }
	    gda_recordset_close(record_set);
	}

	igalia_command_free(my_command);
	gda_recordset_free(record_set);

	g_free(tuples);
    }
    return families_list;
}


/**
 * m_family_clear_array:
 * @families: Array de familias.
 *
 * Libera la memoria utilizada por un array de familias.
 *
 */
void m_family_clear_array (Family ** families) {
    if (families!=NULL) {
	gint index = 0;
	while (families[index]!=NULL) {
	    d_family_clear (families[index]);
            families[index] = NULL;
	    index++;
	}
    }
    g_free(families);
    families = NULL;

}

/**
 * m_family_clear:
 *
 *  Libera la memoria utilizada por la variable global @families_list.
 *
 */
void m_family_clear () {
    m_family_clear_array (families_list);
    families_list = NULL;
}
