/* find_stocked_pieces_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "find_stocked_pieces_model.h"
#include "stock_model.h"
#include "warehouse_model.h"

static Stock * model_find_stocked_pieces_by_codigoProducto_only(gulong code);
static gint recordset_to_stocked_pieces_array(GdaRecordset *rs, Stock **stock_array);

/**
 * m_stock_find_pieces_by_condition:
 * @condition: Filtro de bsqueda. Permite buscar piezas por eurocode, por descripcin, por cdigo de empresa,
 * cdigo de almacn y por stock mnimo.
 * @tuples: Nmero de piezas que cumplen la condicin.
 *
 * Permite buscar piezas en stock con una o varias condiciones
 * de bsqueda.
 *
 * Returns:
 * Array con la lista de piezas que cumplen la condicin de bsqueda.
 */
Stock ** m_stock_find_pieces_by_condition (ConditionFindStock *condition, gulong * tuples) {

    gint           in;
    GdaCommand    *my_command;
    GString       *g_string_query_select;
    GString       *g_string_query_attributes;
    GString       *g_string_query_from;
    GString       *g_string_query_where;
    GString       *g_string_query_extra;
    gchar         *temp;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Stock        **stock_array;
    gboolean       correct = FALSE;

    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);

    g_string_query_select = g_string_new("SELECT ");
    g_string_query_attributes = g_string_new(" ");
    g_string_query_extra  = g_string_new(" ");
    g_string_query_from   = g_string_new(" FROM ");
    g_string_query_where  = g_string_new(" WHERE ");

    temp = u_table_put_name (m_product_get_attributes (), m_product_get_table ());
    g_string_append(g_string_query_attributes, temp);
    g_free(temp);

    g_string_append(g_string_query_attributes, ", ");

    temp = u_table_put_name (m_piece_get_attributes (), "totaljoin");
    g_string_append(g_string_query_attributes, temp);
    g_free(temp);

    g_string_append(g_string_query_from, " ( ");
    g_string_append(g_string_query_from, m_piece_get_table ());
    g_string_append(g_string_query_from, " join ");
    g_string_append(g_string_query_from, m_warehouse_get_table ());
    g_string_append(g_string_query_from, " on TRUE) as totaljoin left outer join ");
    g_string_append(g_string_query_from, m_stock_get_table ());
    g_string_append(g_string_query_from, " on ");
    g_string_append(g_string_query_from,
		    " ((totaljoin.codigoProducto = Stock.codigoProducto) AND \
		       (totaljoin.codigoAlmacen  = Stock.codigoAlmacen) AND \
		       (totaljoin.codigoEmpresa  = Stock.codigoEmpresa)), \
		      ");
    g_string_append(g_string_query_from, m_product_get_table ());

    g_string_append(g_string_query_where,
		    " (Producto.codigoProducto = totaljoin.codigoProducto) ");

    if (condition->minimun_stock==TRUE) {
	g_string_append(g_string_query_where, " AND (stock.unidades <= stock.stockMinimo) ");
    }

    if (((condition->description) != NULL) && (strlen(condition->description)!=0)) {
	g_string_append(g_string_query_where, " AND (producto.descripcion ILIKE '%");
	g_string_append(g_string_query_where, condition->description);
	g_string_append(g_string_query_where, "%')");
    }

    if (((condition->eurocode) != NULL) && (strlen(condition->eurocode)!=0)) {
	g_string_append(g_string_query_where, " AND (totaljoin.eurocode ILIKE '");
	g_string_append(g_string_query_where, condition->eurocode);
	g_string_append(g_string_query_where, "%') ");
    }

    if ((condition->supplier_code != ULONG_MAX) && (condition->supplier_code!=0)) {
	gchar * temp_supplier=g_strdup_printf("%lu",condition->supplier_code);

	g_string_append(g_string_query_where, " AND (totaljoin.codigoProveedor =");
	g_string_append(g_string_query_where, temp_supplier);
	g_string_append(g_string_query_where, ") ");
        g_free(temp_supplier);
    }

    if ((condition->enterprise_code) !=0) {
        temp = g_strdup_printf("%lu",condition->enterprise_code);
	g_string_append(g_string_query_where, " AND (totaljoin.codigoempresa = ");
	g_string_append(g_string_query_where, temp);
	g_string_append(g_string_query_where, ") ");
        g_string_append(g_string_query_attributes, ", totaljoin.codigoEmpresa  as codigoempresastock ");
        g_free(temp);
    }

    if ((condition->warehouse_code) !=0) {
        temp = g_strdup_printf("%lu",condition->warehouse_code);
	g_string_append(g_string_query_where, "AND (totaljoin.codigoalmacen = ");
	g_string_append(g_string_query_where, temp);
	g_string_append(g_string_query_where, ") ");
        g_string_append(g_string_query_attributes, ", totaljoin.codigoAlmacen  as codigoalmacenstock ");
        g_free(temp);
    }

    g_string_append(g_string_query_extra, " GROUP BY ");
    temp = u_remove_aliases (g_string_query_attributes->str, "as", ",");
    g_string_append(g_string_query_extra, temp);
    g_free(temp);

    g_string_append(g_string_query_attributes, ", ");
    g_string_append(g_string_query_attributes, m_stock_get_sumstock_attributes ());

    g_string_append(g_string_query_select, g_string_query_attributes->str);
    g_string_append(g_string_query_select, g_string_query_from->str);
    g_string_append(g_string_query_select, g_string_query_where->str);
    g_string_append(g_string_query_select, g_string_query_extra->str);

    correct = m_model_command_execute (&record_set, g_string_query_select->str, my_connection, my_command,tuples,0, TRUE);
    g_print(_("\nQuery: %s\n"),g_string_query_select->str);

    stock_array=NULL;
    if (correct) {
	if (*tuples!=0) {
	    stock_array = (Stock **) g_malloc ((sizeof(Stock *))*(*tuples +1));
	    in = recordset_to_stocked_pieces_array(record_set, stock_array);
	}
    }
    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_string_free(g_string_query_select, TRUE);
    g_string_free(g_string_query_from, TRUE);
    g_string_free(g_string_query_where, TRUE);
    g_string_free(g_string_query_extra, TRUE);
    return stock_array;

}

/**
 * m_stock_find_pieces_by_product_code:
 * @codes: Lista de cdigos por los que se desea buscar.
 * @enterprise_code: Empresa a la que pertenece el almacn.
 * @warehouse_code: Almacn en el que se buscar.
 * @number: Nmero de piezas que desean obtener, que coincide con el nmero de elementos de la
 * lista de cdigos.
 *
 * Busca las piezas en stock que tengan como cdigo de producto
 * uno de los de la lista de cdigos.
 *
 * Returns:
 * Lista de piezas en stock que cumplen la condicin.
 */
Stock ** m_stock_find_pieces_by_product_code (gulong *codes,
						     gulong enterprise_code,
						     gulong warehouse_code,
						     gulong *number) {
    gint           index = 0;
    gulong         tuples = 0;
    gchar         *temp = NULL;
    GdaCommand    *my_command = NULL;
    GString       *g_string_query = NULL;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Stock        **stocked_pieces_array = NULL;
    gboolean       correct = FALSE;

    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);

    stocked_pieces_array = (Stock **) g_malloc ((sizeof(Stock *))*(*number));
    while (codes[index]!= -1) {
	g_string_query = g_string_new("SELECT ");
	temp = u_table_put_name (m_product_get_attributes (), m_product_get_table ());
	g_string_append(g_string_query, temp);
	g_free(temp);

	g_string_append(g_string_query, ", ");

        temp = u_table_put_name (m_piece_get_attributes (), m_piece_get_table ());
	g_string_append(g_string_query, temp);
	g_free(temp);

	g_string_append(g_string_query, ", ");

	temp = g_strconcat(" Stock.codigoproducto as codigoproductostock, Stock.codigoAlmacen as codigoalmacenstock,",
			   " Stock.codigoEmpresa as codigoempresastock, ",
			   " to_char(Stock.unidades,'9999999999D999') as unidadesstock, ",
			   " to_char(Stock.stockMinimo,'999999999D999') as stockminimostock, "
			   " ubicacion     as ubicacionstock ",NULL);

	//temp = u_table_put_name (m_stock_get_attributes (), m_stock_get_table ());
	g_string_append(g_string_query, temp);
        g_free(temp);

	g_string_append(g_string_query, " FROM ");
	g_string_append(g_string_query, m_piece_get_table ());
	g_string_append(g_string_query, " left outer join ");
	g_string_append(g_string_query, m_stock_get_table ());
	g_string_append(g_string_query, " on ");
	g_string_append(g_string_query,
			" (Stock.codigoProducto = Pieza.codigoProducto), ");
	g_string_append(g_string_query, m_product_get_table ());

	g_string_append(g_string_query, " WHERE ");
	g_string_append(g_string_query, " (Producto.codigoProducto = Pieza.codigoProducto) ");

	g_string_append(g_string_query, "AND (Pieza.codigoProducto = ");
	temp = g_strdup_printf("%lu", codes[index]);
	g_string_append(g_string_query, temp);
	g_string_append(g_string_query, " ) ");
	g_free(temp);

	temp = g_strdup_printf("%lu", enterprise_code);
	g_string_append(g_string_query, "AND (stock.codigoempresa = ");
	g_string_append(g_string_query, temp);
	g_string_append(g_string_query, ") ");
        g_free(temp);

	temp = g_strdup_printf("%lu", warehouse_code);
	g_string_append(g_string_query, "AND (stock.codigoalmacen = ");
	g_string_append(g_string_query, temp);
	g_string_append(g_string_query, ") ");
        g_free(temp);


	correct = m_model_command_execute (&record_set, g_string_query->str, my_connection,my_command, &tuples, 0,TRUE);

	g_print(_("\nQuery: %s\n"),g_string_query->str);

	gda_recordset_move_first(record_set);

	stocked_pieces_array[index] = NULL;
	if (correct) {
	    if (tuples==1) {
		if (m_model_record_set_contains_piece (record_set)) {
		    stocked_pieces_array[index] = m_stock_create (record_set);
		}
	    } else {
		if (tuples==0) {
		    stocked_pieces_array[index] =
			model_find_stocked_pieces_by_codigoProducto_only(codes[index]);
		    d_stock_set_enterprise_code (stocked_pieces_array[index], enterprise_code);
		    d_stock_set_warehouse_code (stocked_pieces_array[index], warehouse_code);
		} 
	    }
	}
	index++;
    }
    stocked_pieces_array[index] = NULL;
    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_string_free(g_string_query, TRUE);
    return stocked_pieces_array;
}

/**
 * model_find_pieces_by_codigoProducto_only:
 * @code: Lista de cdigos por los que se desea buscar.
 *
 * Busca la pieza en stock que tenga como cdigo de producto @code.
 * @code: Lista de cdigos por los que se desea buscar.
 *
 * Returns:
 * Pieza en stock que cumplen la condicin.
 */
static Stock * model_find_stocked_pieces_by_codigoProducto_only(gulong code) {
    gint           index = 0;
    gulong         tuples = 0;
    gchar         *temp = NULL;
    GdaCommand    *my_command = NULL;
    GString       *g_string_query = NULL;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Stock         *stocked_piece = NULL;
    gboolean       correct = FALSE;

    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);

    if (code!= -1) {
	g_string_query = g_string_new("SELECT ");

	temp = u_table_put_name (m_product_get_attributes (), m_product_get_table ());
	g_string_append(g_string_query, temp);
	g_free(temp);

	g_string_append(g_string_query, ", ");

	temp = u_table_put_name (m_piece_get_attributes (), m_piece_get_table ());
	g_string_append(g_string_query, temp);
	g_free(temp);

	g_string_append(g_string_query, ", ");

	temp = g_strconcat(" Stock.codigoproducto as codigoproductostock, Stock.codigoAlmacen as codigoalmacenstock,",
			   " Stock.codigoEmpresa as codigoempresastock, ",
			   " to_char(Stock.unidades,'9999999999D999') as unidadesstock, ",
			   " to_char(Stock.stockMinimo,'999999999D999') as stockminimostock, "
			   " ubicacion     as ubicacionstock ",NULL);

//        temp = u_table_put_name (m_stock_get_attributes (), m_stock_get_table ());
	g_string_append(g_string_query, temp);
        g_free(temp);

	g_string_append(g_string_query, " FROM ");
	g_string_append(g_string_query, m_piece_get_table ());
	g_string_append(g_string_query, " left outer join ");
	g_string_append(g_string_query, m_stock_get_table ());
	g_string_append(g_string_query, " on ");
	g_string_append(g_string_query,
			" (Stock.codigoProducto = Pieza.codigoProducto), ");
	g_string_append(g_string_query, m_product_get_table ());

	g_string_append(g_string_query, " WHERE ");
	g_string_append(g_string_query, " (Producto.codigoProducto = Pieza.codigoProducto) ");

	g_string_append(g_string_query, "AND (Pieza.codigoProducto = ");
	temp = g_strdup_printf("%lu", code);
	g_string_append(g_string_query, temp);
	g_string_append(g_string_query, " ) ");
	g_free(temp);

	correct = m_model_command_execute (&record_set, g_string_query->str, my_connection,my_command, &tuples, 0, TRUE);

	g_print(_("\nQuery: %s\n"),g_string_query->str);
	gda_recordset_move_first(record_set);


	// Only need generic data from this piece in stock,
	// so we only load the first one
	stocked_piece = NULL;
	if(correct) {
	    if ((tuples>0) && m_model_record_set_contains_piece (record_set)) {
		stocked_piece = m_stock_create (record_set);
		d_stock_set_location (stocked_piece, NULL);
		d_stock_set_quantity (stocked_piece, 0.0);
	    }
	}
	index++;
    }
    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_string_free(g_string_query, TRUE);

    return stocked_piece;
}

/**
 * recordset_to_stocked_pieces_array: 
 * @rs: Record set del que se desea obtener la lista de piezas.
 * @stock_array: Lista de piezas en stock obtenida. Es necesario reservar memoria previamente.
 * 
 * Obtiene una lista de piezas en stock a partir de un record set de la base de datos.
 *
 * Returns:
 * Tamao de la lista de piezas en stock.
 */
static gint recordset_to_stocked_pieces_array(GdaRecordset *rs,
				       Stock **stock_array) {
    gint index;
    index = gda_recordset_move_first(rs);

    while (!gda_recordset_eof(rs)) {

	if (index==GDA_RECORDSET_INVALID_POSITION) return -1;

	stock_array[index-1] = m_stock_create (rs);

	index = gda_recordset_move_next(rs);
    }
    stock_array[index] = NULL;

    return index;
}

/**
 * m_stock_clear_array:
 * @stock_array: Array de piezas en stock que se desea liberar.
 *
 * Libera la memoria utilizada por un array de piezas en stock.
 *
 * Returns: 0
 */

gint m_stock_clear_array (Stock ** stock_array) {
    gulong index = 0;

    if (stock_array!=NULL) {
	while(stock_array[index]!=NULL) {
	    d_stock_clear (stock_array[index]);
            stock_array[index] = NULL;
	    index++;
	}
    }
    g_free(stock_array);
    stock_array=NULL;
    return 0;
}

