/* invoice_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "../controller/control_util.h"
#include "invoice_model.h"
#include "operation.h"
#include "participant_model.h"
#include "operation_model.h"
#include "cash_operation_model.h"
#include "direct_debit_model.h"
#include "account_model.h"

#define TABLE         "FacturaAbono"
#define ATTRIBUTES    "codigoEmpresa as codigoEmpresaFactura, \
                       codigoTienda as codigoTiendaFactura, \
                       numeroFactura as numeroFacturaFactura, \
                       codigoPagador as codigoPagadorFactura, \
                       fechaEmision as fechaEmisionFactura, \
                       dtoFactura as dtoFacturaFactura, \
                       estado as estadoFactura, \
                       codigoReparacion as codigoReparacionFactura, \
                       codigoFactura as codigoFacturaFactura, \
                       codigoPago as codigoPagoFactura";

static gboolean insert_garage_invoice(Invoice *invoice,gchar * invoice_number, gint delivery_payment_index,GdaConnection *connection, GdaCommand *command);
static gboolean invoice_validate_shop_code(Invoice *invoice);
static gboolean invoice_validate_enterprise_code(Invoice *invoice);

/**
 * m_invoice_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_invoice_get_table () {
    return TABLE;
}

/**
 * m_invoice_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_invoice_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_invoice_create:
 * @rs: Record set que contiene los campos de la BD.
 * @delivery_payments: Albaranes que se desea aadir a las facturas.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Invoice con los campos rellenados con los datos de la base de datos.
 */
Invoice * m_invoice_create (GdaRecordset *rs,DeliveryPayment ** delivery_payments) {

    GdaField * field;
    Invoice * invoice;

    invoice = d_invoice_create_empty();

    field = gda_recordset_field_name(rs,"codigoEmpresaFactura");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    /* cando xa est empresa debes quitar esta lia de inicializacin*/
	    gda_field_free(field);
    
	    invoice->enterprise=m_enterprise_create (rs);

	    field = gda_recordset_field_name(rs,"codigoTiendaFactura" );
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
	    
		    invoice->shop = m_shop_create (rs);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaEmisionFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    invoice->issue_date = (*my_time);
		    g_free(my_time);

		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "numeroFacturaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->invoice_number = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "codigoFacturaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->invoice_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "baseImponibleNuevaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    invoice->total = atof(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "baseImponibleModificadaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->modified_total = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "estadoFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->state = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    invoice -> participant = m_participant_create (rs,"pagador");
	    invoice -> payment = m_payment_create (rs);
	    invoice -> delivery_payments = delivery_payments;

	    if (delivery_payments==NULL) {
		invoice -> repair = m_repair_create (rs,NULL);
	    }

	    field = gda_recordset_field_name(rs, "refacturaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->split_invoice = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "tipoIvaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->vat_type = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "dtoFacturaFactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->invoice_discount = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "codigoReparacionRefactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->repair_code_split_invoice= igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "codigoTiendaRefactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->shop_code_split_invoice= igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "codigoEmpresaRefactura");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    invoice->enterprise_code_split_invoice= igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	} else {
	    gda_field_free(field);
	}
    }

    return invoice;
}

/**
 * m_invoice_obtain_data: 
 * @delivery_payments_client: Lista de identificadores de albaranes de abono.
 *
 * Esta funcin crea, a partir de los identificadores de los albaranes de abono de un cliente,
 * una factura con los datos de los albaranes y reparaciones asociadas.
 *
 * Returns:
 * Devuelve los datos de los albaranes con sus reparaciones respectivas a partir de los identificadores en forma
 * de factura.
 */
Invoice * m_invoice_obtain_data (DeliveryPaymentId ** delivery_payments_client) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    DeliveryPayment ** delivery_payments;
    DeliveryPayment * delivery_payment;
    Repair * repair;
    Invoice * invoice;
    gint index;
    gint number_of_delivery_payments;
    gulong * tuples;


    tuples = (gulong *) g_malloc0(sizeof(gulong));


    index = 0;
    if (delivery_payments_client!=NULL) {
	while ((delivery_payments_client[index])!=NULL) {
	    index++;
	}
    }
    number_of_delivery_payments=index;


    delivery_payments = (DeliveryPayment **) g_malloc0(sizeof(DeliveryPayment *)*(number_of_delivery_payments+1));
    index = 0;
    while (index<number_of_delivery_payments) {

	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

       
	gchar_query = g_strconcat("select codigoReparacion from albaranabono where albaranabono.numeroAlbaran=",
				   delivery_payments_client[index]->delivery_payment_number,
				   " and albaranabono.codigoEmpresa=",
				   delivery_payments_client[index]->enterprise_code,
				   " and albaranabono.codigoTienda=",
				  delivery_payments_client[index]->shop_code,
				  NULL);

	g_print(_("\nQuery: %s\n"),gchar_query);


	m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

	if (record_set!=NULL) {
	    if (*tuples!=0) {
		GdaField * field;
		Repair_line ** repair_lines;
		RepairId * repair_id;
		gulong repair_code;

		gda_recordset_move_first(record_set);
		field = gda_recordset_field_name(record_set,"codigoReparacion");
		repair_code = igalia_field_get_bigint_value (field);
                gda_field_free(field);
		repair_id = (RepairId *) g_malloc0(sizeof(RepairId));
		repair_id->enterprise_code = g_strdup(delivery_payments_client[index]->enterprise_code);
		repair_id->shop_code = g_strdup(delivery_payments_client[index]->shop_code);
		repair_id->repair_code = g_strdup_printf("%lu",repair_code);

		repair_lines = m_repair_line_find_by_id (repair_id);

		repair = m_repair_find_by_id (repair_id,repair_lines);

                delivery_payment = m_delivery_payment_find_by_repair (repair_id);
		if (repair!=NULL) {
		    d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
		    d_delivery_payment_set_repair (delivery_payment,repair);
		}
                delivery_payments[index]=delivery_payment;
		delivery_payment=NULL;
		repair=NULL;
		repair_lines=NULL;
		gda_recordset_close(record_set);
		gda_recordset_free(record_set);
		g_free(repair_id->enterprise_code);
                g_free(repair_id->shop_code);
                g_free(repair_id->repair_code);
		g_free(repair_id);
                repair_id=NULL;
	    }
	}
	index++;
	igalia_command_free(my_command);

    }
    delivery_payments[index]=NULL;
    g_free(gchar_query);


    invoice = d_invoice_create_empty ();
    d_invoice_set_repair (invoice,NULL,TRUE);
    d_invoice_set_delivery_payments (invoice,delivery_payments);

    g_free(tuples);


return invoice;
}


/**
 * m_invoice_find_by_id: 
 * @invoice_id: Identificador de la factura, que consta de cdigo de la factura, cdigo de empresa y
 * cdigo de tienda.
 *
 * Obtiene una factura por el identificador de la factura.
 *
 * Returns:
 * Factura completa. Es decir, factura con todos los albaranes y reparaciones asociadas.
 */
Invoice * m_invoice_find_by_id (InvoiceId * invoice_id) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar * gchar_from;
    gchar * gchar_where;
    gchar * gchar_where_aux=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    GdaRecordset       *record_set2 = NULL;
    GdaField * field;
    GdaField * field2=NULL;

    Invoice * invoice=NULL;
    gint index;
    gulong repair_code=ULONG_MAX;
    gulong payment_code=ULONG_MAX;
    gulong * tuples;
    gint error=0;
    gboolean split_invoice=FALSE;

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    gchar_select=g_strconcat("select numeroFactura as numeroFacturaFactura,",
			     "facturaabono.codigoFactura as codigoFacturaFactura,",
			     "facturaabono.codigoTienda as codigoTiendaFactura,",
			     "facturaabono.codigoEmpresa as codigoEmpresaFactura,",
			     "codigoPagador as codigoPartyParticipantPago,",
			     "codigoReparacion as codigoReparacionPago,"
			     "codigoPago as codigoPagoFactura,",
			     "to_char(tipoIva,'999D99') as tipoIvaFactura,",
			     "fechaEmision as fechaEmisionFactura,",
			     "to_char(dtoFactura,'999D99') as dtoFacturaFactura,",
			     "refactura as refacturaFactura,",
			     "to_char(baseImponibleNueva,'99999999D99') as baseImponibleNuevaFactura,",
			     "baseImponibleModificada as baseImponibleModificadaFactura,",
			     "estado as estadoFactura,",
			     "participantePagador.nick as nickParticipantPagador,",
			     "participantePagador.nombreCorto as shortNameParticipantPagador,",
			     "participantePagador.nombre as nombreParticipantPagador,",
			     "participantePagador.cif as cifParticipantPagador,",
			     "participantePagador.codigoParty as codigoPartyParticipantPagador,",
			     "participantePagador.email as emailParticipantPagador,",
			     "participantePagador.telefono as telefonoParticipantPagador,",
			     "participantePagador.telefono2 as telefono2ParticipantPagador,",
			     "participantePagador.fax as faxParticipantPagador,",
			     "participantePagador.calle as calleParticipantPagador,",
			     "participantePagador.numero as numeroParticipantPagador,",
			     "participantePagador.piso as pisoParticipantPagador,",
			     "participantePagador.cp as cpParticipantPagador,",
			     "participantePagador.municipio as municipioParticipantPagador,",
			     "participantePagador.provincia as provinciaParticipantPagador,",
			     "participanteEmpresa.nombre as nombreParticipantEnter,",
			     "participanteEmpresa.nick as nickParticipantEnter,",
			     "participanteEmpresa.cif as cifParticipantEnter,",
			     "participanteEmpresa.codigoParty as codigoPartyParticipantEnter,",
			     "participanteEmpresa.email as emailParticipantEnter,",
			     "participanteEmpresa.telefono as telefonoParticipantEnter,",
			     "participanteEmpresa.telefono2 as telefono2ParticipantEnter,",
			     "participanteEmpresa.fax as faxParticipantEnter,",
			     "participanteEmpresa.calle as calleParticipantEnter,",
			     "participanteEmpresa.numero as numeroParticipantEnter,",
			     "participanteEmpresa.piso as pisoParticipantEnter,",
			     "participanteEmpresa.cp as cpParticipantEnter,",
			     "participanteEmpresa.municipio as municipioParticipantEnter,",
			     "participanteEmpresa.provincia as provinciaParticipantEnter,",
			     "participanteTienda.codigoParty as codigoPartyParticipantShop,",
			     "participanteTienda.nombre as nombreParticipantShop,",
			     "participanteTienda.nick as nickParticipantShop,",
			     "participanteTienda.cif as cifParticipantShop,",
			     "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop",
			     NULL);
    gchar_from = g_strconcat(" from facturaabono,participante participantePagador,",
			     " participante participanteEmpresa,"
			     " participante participanteTienda,tienda ",NULL);
    gchar_where = g_strconcat("where",
			      " facturaabono.codigoPagador=participantePagador.codigoParty ",
			      "and facturaabono.codigoEmpresa=participanteEmpresa.codigoParty ",
			      "and facturaabono.codigoTienda=participanteTienda.codigoParty ",
			      "and facturaabono.codigoTienda=tienda.codigoTienda ",
			      NULL);


    if (((invoice_id->invoice_number) != NULL) && (strlen(invoice_id->invoice_number)!=0))  {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoFactura=",
				      invoice_id->invoice_number,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if (((invoice_id->enterprise_code) != NULL) && (strlen(invoice_id->enterprise_code)!=0))  {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoEmpresa=",
				      invoice_id->enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if (((invoice_id->shop_code) != NULL) && (strlen(invoice_id->shop_code)!=0))  {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoTienda=",
				      invoice_id->shop_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);
    g_print(_("\nQuery: %s\n"),gchar_query);
    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
	    index = gda_recordset_move_first(record_set);
	    field2 = gda_recordset_field_name(record_set,"refacturaFactura");
	    if (field2!=NULL) {
		if (!gda_field_is_null(field2)) {
		    split_invoice=gda_field_get_boolean_value(field2);
		}
		gda_field_free(field2);
	    }
	    field = gda_recordset_field_name(record_set,"codigoReparacionPago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair_code=igalia_field_get_bigint_value (field);
		} else {
		    repair_code=ULONG_MAX;
		}
		gda_field_free(field);
	    }
	    field2 = gda_recordset_field_name(record_set,"codigoPagoFactura");
	    if (field2!=NULL) {
        	if (!gda_field_is_null(field2)) {
		    payment_code=igalia_field_get_bigint_value (field2);
		}
		gda_field_free(field2);
	    }
	    invoice = m_invoice_create (record_set,NULL);
	} else {
            error =1;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
        record_set=NULL;
    }
    g_free(gchar_query);
    g_free(gchar_select);
    g_free(gchar_where);
    g_free(gchar_from);

    
    if (error!=1) {
	PaymentId * payment_id = (PaymentId *) g_malloc0(sizeof(PaymentId));
        payment_id->payment_code=g_strdup_printf("%lu",payment_code);
        payment_id->enterprise_code=g_strdup(invoice_id->enterprise_code);
        payment_id->shop_code=g_strdup(invoice_id->shop_code);
	d_invoice_set_payment (invoice,m_payment_find_by_id (payment_id));
	if (payment_id->payment_code!=NULL)
            g_free(payment_id->payment_code);
	if (payment_id->enterprise_code!=NULL)
            g_free(payment_id->enterprise_code);
	if (payment_id->shop_code!=NULL)
	    g_free(payment_id->shop_code);
        g_free(payment_id);
	if (split_invoice==FALSE) {
	    if (repair_code!=ULONG_MAX) {
		Repair_line ** repair_lines;
		RepairId * repair_id;
		repair_id = (RepairId *) g_malloc0(sizeof(RepairId));
		repair_id->enterprise_code = g_strdup(invoice_id->enterprise_code);
		repair_id->shop_code = g_strdup(invoice_id->shop_code);
		repair_id->repair_code = g_strdup_printf("%lu",repair_code);

		repair_lines =  m_repair_line_find_by_id (repair_id);

		d_invoice_set_repair (invoice,m_repair_find_by_id (repair_id,repair_lines),TRUE);
	    } else {
		DeliveryPayment ** delivery_payments;
		DeliveryPayment ** delivery_payments_garage;
		DeliveryPayment ** delivery_payments_total;

		DeliveryPayment * delivery_payment;
		gchar_query = g_strconcat("select numeroAlbaran,codigoEmpresa,codigoTienda,codigoReparacion from albaranabono where albaranabono.numeroFactura=",
					  invoice_id->invoice_number," and albaranabono.codigoEmpresa=",invoice_id->enterprise_code,
					  " and albaranabono.codigoTienda=",invoice_id->shop_code,NULL);
		g_print(_("\nQuery: %s\n"),gchar_query);
		m_model_command_execute (&record_set2, gchar_query, my_connection,my_command,tuples,0,TRUE);
                g_free(gchar_query);
		if (record_set2!=NULL) {
		    if (*tuples!=0) {
			Repair_line** repair_lines;
			Repair * repair;
			delivery_payments = (DeliveryPayment **) g_malloc0(sizeof(DeliveryPayment *)*(*tuples+1));
			gda_recordset_move_first(record_set2);
                        index=0;
			while (!gda_recordset_eof(record_set2)) {
			    RepairId * repair_id=(RepairId *) g_malloc0(sizeof(RepairId));
			    repair_id->repair_code=NULL;
			    repair_id->shop_code=NULL;
			    repair_id->enterprise_code=NULL;
			    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
			    field = gda_recordset_field_name(record_set2,"codigoReparacion");
			    if (field!=NULL) {
				if (!gda_field_is_null(field)) {
				    repair_id->repair_code=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
				}
				gda_field_free(field);
			    }
			    field = gda_recordset_field_name(record_set2,"codigoEmpresa");
			    if (field!=NULL) {
				if (!gda_field_is_null(field)) {
				    repair_id->enterprise_code=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
				}
				gda_field_free(field);
			    }
			    field = gda_recordset_field_name(record_set2,"codigoTienda");
			    if (field!=NULL) {
				if (!gda_field_is_null(field)) {
				    repair_id->shop_code=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
				}
				gda_field_free(field);
			    }
			    repair_lines = m_repair_line_find_by_id (repair_id);

			    repair = m_repair_find_by_id (repair_id,repair_lines);

			    delivery_payment = m_delivery_payment_find_by_repair (repair_id);

			    if (repair!=NULL) {
				d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
				d_delivery_payment_set_repair (delivery_payment,repair);
			    }

			    delivery_payments[index] = delivery_payment;
			    repair_lines =NULL;
			    repair=NULL;
			    delivery_payment=NULL;
			    gda_recordset_move_next(record_set2);
			    if (repair_id!=NULL) {
				if (repair_id->repair_code!=NULL) g_free(repair_id->repair_code);
				if (repair_id->enterprise_code!=NULL) g_free(repair_id->enterprise_code);
				if (repair_id->shop_code!=NULL) g_free(repair_id->shop_code);
				g_free(repair_id);
			    }
			    index++;

			}
			//delivery_payments[index]=NULL;
                        g_print(_("\n%d\n"),index);
			//gda_recordset_close(record_set2);
		    } else {
			delivery_payments=NULL;
		    }
		    if (record_set2!=NULL)
			gda_recordset_free(record_set2);
		    record_set2=NULL;

		    
		} else {
		    delivery_payments=NULL;
		}
		delivery_payments_garage=m_delivery_payment_find_by_garage (invoice_id->invoice_number,invoice_id->shop_code,invoice_id->enterprise_code);
		if (delivery_payments_garage!=NULL) {
                    delivery_payments_total=m_delivery_payment_join_array (delivery_payments,delivery_payments_garage);
		} else {
                    delivery_payments_total=delivery_payments;
		}
		d_invoice_set_repair (invoice,NULL,TRUE);
		if (d_invoice_get_delivery_payments (invoice)!=NULL)
		    m_delivery_payment_clear_array (d_invoice_get_delivery_payments (invoice));
		d_invoice_set_delivery_payments (invoice,delivery_payments_total);
	    }
	} else {
	    Repair_line ** repair_lines;
	    RepairId * repair_id;
	    repair_id = (RepairId *) g_malloc0(sizeof(RepairId));

	    gchar_query = g_strconcat("select codigoTiendaAbono,codigoEmpresaAbono,codigoReparacion,codigoEmpresaRefactura,codigoTiendaRefactura from refacturacion where ",
				      " refacturacion.numeroRefactura=",invoice_id->invoice_number,
				      " and refacturacion.codigoEmpresaRefactura=",invoice_id->enterprise_code,
				      " and refacturacion.codigoTiendaRefactura=",invoice_id->shop_code,
				      NULL);
	    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0,TRUE);
            g_free(gchar_query);

	    if (record_set!=NULL) {
		if (*tuples==1) {
		    gda_recordset_move_first(record_set);
		    field = gda_recordset_field_name(record_set,"codigoEmpresaAbono");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gulong enterprise_code_aux;
			    enterprise_code_aux=igalia_field_get_bigint_value (field);
			    repair_id -> enterprise_code = g_strdup_printf("%lu",enterprise_code_aux);
			}
			gda_field_free(field);
		    }
		    field = gda_recordset_field_name(record_set,"codigoTiendaAbono");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gulong shop_code_aux;
			    shop_code_aux=igalia_field_get_bigint_value (field);
			    repair_id -> shop_code = g_strdup_printf("%lu",shop_code_aux);
			}
			gda_field_free(field);
		    }
		    field = gda_recordset_field_name(record_set,"codigoEmpresaRefactura");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gulong enterprise_split_invoice_code_aux;
			    enterprise_split_invoice_code_aux=igalia_field_get_bigint_value (field);
			    d_invoice_set_enterprise_code_split_invoice (invoice,enterprise_split_invoice_code_aux);
			}
			gda_field_free(field);
		    }
		    field = gda_recordset_field_name(record_set,"codigoTiendaRefactura");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gulong shop_split_invoice_code_aux;
			    shop_split_invoice_code_aux=igalia_field_get_bigint_value (field);
			    d_invoice_set_shop_code_split_invoice (invoice,shop_split_invoice_code_aux);
			}
			gda_field_free(field);
		    }
		    field = gda_recordset_field_name(record_set,"codigoReparacion");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gulong repair_code_aux;
			    repair_code_aux=igalia_field_get_bigint_value (field);
			    repair_id -> repair_code = g_strdup_printf("%lu",repair_code_aux);
			}
			gda_field_free(field);
		    }

		}
		//	d_invoice_set_shop_code_split_invoice (invoice,model_find_refactura_enterprise_code_invoice(codigoReparacion));

	    }
	    {
		DeliveryPayment ** delivery_payments;
		DeliveryPayment * delivery_payment;
		Repair * repair;
		repair_lines =  m_repair_line_find_by_id (repair_id);



		repair = m_repair_find_by_id (repair_id,repair_lines);
		delivery_payments = (DeliveryPayment **) g_malloc0(sizeof(DeliveryPayment *)*(2));

		delivery_payment = m_delivery_payment_find_by_repair (repair_id);

		if (delivery_payment!=NULL) {
		    if (repair!=NULL) {
			d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
			d_delivery_payment_set_repair (delivery_payment,repair);
			delivery_payments[0]=delivery_payment;
			d_invoice_set_repair (invoice,NULL,TRUE);
			if (d_invoice_get_delivery_payments (invoice)!=NULL)
			    m_delivery_payment_clear_array (d_invoice_get_delivery_payments (invoice));
			d_invoice_set_delivery_payments (invoice,delivery_payments);
		    }
		} else {
		    d_invoice_set_repair (invoice,m_repair_find_by_id (repair_id,repair_lines),TRUE);
		}

		if (record_set!=NULL)
		    gda_recordset_free(record_set);
		record_set=NULL;


	    }
	}
    } else {
	invoice =NULL;
    }

    g_free(tuples);
    return invoice;
}




/**
 * m_invoice_insert: 
 * @invoice: Factura a insertar
 * 
 * Esta es una funcin compleja que inserta una factura en la base de datos. En su interior se inserta un
 * pago asociado a la factura, y en caso de que se trate de una refactura, se insertarn otro pago y otra
 * factura con los datos adecuados.
 *
 * Returns:
 * Cdigo de la factura insertada en la base de datos.
 */
gulong m_invoice_insert (Invoice * invoice) {
    gulong result=0;
    gchar * query1=NULL;
    gchar * query_aux=NULL;
    gchar * query2=NULL;
    gchar * query3=NULL;
    gchar * query=NULL;
    gchar * query4=NULL;
    gchar * query5=NULL;
    gchar * query6=NULL;
    gchar * query7=NULL;
    gchar * query_split_invoice=NULL;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * invoice_number=NULL;
    gchar * invoice_code=NULL;
    gchar * invoice_split_invoice_code=NULL;
    gchar * invoice_split_invoice_number=NULL;
    gulong invoice_code_gulong=ULONG_MAX;
    gchar * delivery_payment_number=NULL;
    gchar * enterprise=NULL;
    gchar * enterprise_aux=NULL;
    gchar * shop=NULL;
    Payment * payment;
    gulong shop_code=ULONG_MAX;
    gchar * temp_invoice;
    gulong enterprise_code=ULONG_MAX;
    gulong payer_code=ULONG_MAX;
    gulong payment_code = ULONG_MAX;
    gdouble new_total = d_invoice_get_total (invoice);
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;
    GdaRecordset *record_set3=NULL;
    GdaRecordset *record_set4=NULL;
    GdaRecordset *record_set5=NULL;
    GdaRecordset *record_set6=NULL;
    GdaRecordset *record_set7=NULL;
    GdaRecordset *record_set_split_invoice=NULL;
    GdaConnection * connection = m_model_get_connection ();
    Repair * repair;
    GdaCommand * command = gda_command_new();
    gint state = d_invoice_get_state (invoice);
    time_t issue_date=d_invoice_get_issue_date (invoice);
    gdouble invoice_discount = d_invoice_get_invoice_discount (invoice);
    gboolean with_payment=FALSE;


    if (invoice->delivery_payments!=NULL) {
        repair = d_delivery_payment_get_repair (invoice->delivery_payments[0]);
    } else {
        repair = d_invoice_get_repair (invoice);
    }
    if (state == D_INVOICE_NOT_ISSUED) {
	shop_code=d_participant_get_code (d_shop_get_participant (d_repair_get_shop (
                                                                                repair
										)));
	enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (
                                                                                              repair
											     )));
	
    }

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);

    if (d_invoice_get_participant (invoice)!=NULL) {
	payer_code = d_participant_get_code (d_invoice_get_participant (invoice));
    }

    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
        gchar * temp_nextval=NULL;
	payment = d_invoice_get_payment (invoice);
	if (d_invoice_get_payment (invoice)!=NULL) {
	    /* The payment has data but it can already exists or no  */
	    /* If the payment already exists codigoPago != ULONG_MAX */

	    query1 = g_strconcat(" INSERT INTO PAGO VALUES ",NULL);
	    values = g_strconcat(" (",NULL);
	    payment_code = d_payment_get_payment_code (payment);
	    if (payment_code==ULONG_MAX) {
                time_t time_aux;
		if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
		    gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(values);
		    g_free(temp);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}
		if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
		    gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(values);
		    g_free(temp);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}
                temp_nextval = m_model_nextval_sequence ("seqPago",connection);
		values_aux = g_strconcat(values,temp_nextval,",",NULL);
		g_free(values);
		values = values_aux;
		g_free(temp_nextval);

		if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
		    gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (d_payment_get_achieved_date (payment)!=time(NULL)) {
		    time_t temp = d_payment_get_achieved_date (payment);
		    values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}


		if (d_payment_get_is_hire_purchase (payment)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		time_aux = d_payment_get_init_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&time_aux),"',",NULL);
		g_free(values);
		values = values_aux;


		if (d_payment_get_entry_percentage (payment)!=-1) {
		    gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
                    u_commas_to_points(temp);
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}
		if (d_payment_get_number_expiries (payment)!=-1) {
		    gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_interval_days (payment)!=-1) {
		    gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_is_direct_debit (payment)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (d_payment_get_bank_code (payment)!=-1) {
		    gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_office_code (payment)!=-1) {
		    gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_account_number (payment)!=NULL) {
		    gchar * temp = d_payment_get_account_number (payment);
		    values_aux = g_strconcat(values,temp,",",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}


		if (d_payment_get_is_transfer (payment)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (d_payment_get_is_cheque (payment)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE,",NULL);
		    g_free(values);
		    values = values_aux;
		}


		if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
		    gchar * temp = d_payment_c_util_get_cheque_number (payment);
		    values_aux = g_strconcat(values,"'",temp,"',",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_is_credit_card (payment)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (d_payment_get_credit_card_number (payment)!=NULL) {
		    gchar * temp = d_payment_get_credit_card_number (payment);
		    values_aux = g_strconcat(values,"'",temp,"',",NULL);
		    g_free(temp);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);;
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_expiry_date (payment)!=time(NULL)) {
		    time_t temp = d_payment_get_expiry_date (payment);
		    values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		    g_free(values);
		    values = values_aux;

		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);;
		    g_free(values);
		    values = values_aux;

		}

		if (d_payment_get_is_paid_cash (payment)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE",NULL);
		    g_free(values);
		    values = values_aux;
		}


		query_aux=g_strconcat(query1,values,")",NULL);
		if (query1!=NULL) g_free(query1);
                query1=NULL;
		query1=query_aux;
		query_aux=NULL;
                g_free(values);
	    }

	    if (m_model_command_execute (&record_set1, query1, connection,command,tuples,0, FALSE)==FALSE) {
                result = ULONG_MAX;
	    }

	    query7 = g_strconcat(" select currval('seqPago') ",NULL);



	    if (m_model_command_execute (&record_set7, query7, connection,
				      command,tuples,0,TRUE)==FALSE) {
		result = ULONG_MAX;
	    }

	    if (record_set7!=NULL) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set7);
		    field  = gda_recordset_field_idx(record_set7,0);
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gchar * temp_pago;
			    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
			    temp_pago = gda_stringify_value(NULL, 0, field);
			    payment_code = strtoul(temp_pago,stop,10);
			    if (payment_code!=ULONG_MAX) d_payment_set_payment_code (d_invoice_get_payment (invoice),payment_code);
			    g_free(stop);
			    g_free(temp_pago);
			}
			gda_field_free(field);
		    }

		} else {
		    result = ULONG_MAX;
		}
	    } else {
		result = ULONG_MAX;
	    }

	    with_payment=TRUE;
	} else {
            with_payment=FALSE;
	}


	enterprise = g_strdup_printf("%lu",enterprise_code);
	if (d_invoice_get_vat_type (invoice)!=0)
            enterprise_aux = g_strdup_printf("%lu",enterprise_code);
	else
            /*PARA METER NUMERACIN DIFERENTE EN CASO DE NO IVA*/
	    enterprise_aux = g_strdup_printf("%lu",(gulong) D_ENTERPRISE_NONE);

	shop = g_strdup_printf("%lu",shop_code);

	result = m_invoice_get_number (enterprise_aux, shop,connection, command);
        if (enterprise_aux!=NULL)
	    g_free(enterprise_aux);
        invoice_number = g_strdup_printf("%lu",result);
        invoice_code = m_model_nextval_sequence ("seqFacturaAbono",connection);

	if (result!=ULONG_MAX) {
	    {
		gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
		invoice_code_gulong = strtoul(invoice_code,stop,10);
		if (strcmp(*stop,"")) {
		    invoice_code_gulong=ULONG_MAX;
		}
		g_free(stop);
	    }

	    if (d_invoice_get_shop_code_split_invoice (invoice)==ULONG_MAX) {
		values = g_strconcat(" ",NULL);
		query2 = g_strconcat("INSERT INTO FACTURAABONO VALUES (",NULL);

		temp_invoice = g_strdup_printf("%lu",enterprise_code);
		values_aux = g_strconcat(values,temp_invoice,",",NULL);
		g_free(values);
		g_free(temp_invoice);
		values = values_aux;

		temp_invoice = g_strdup_printf("%lu",shop_code);
		values_aux = g_strconcat(values,temp_invoice,",",NULL);
		g_free(values);
		g_free(temp_invoice);
		values = values_aux;

		values_aux = g_strconcat(values,invoice_number,",",NULL);
		g_free(values);
		values = values_aux;

		if (payer_code!=ULONG_MAX) {
		    temp_invoice = g_strdup_printf("%lu",payer_code);
		    values_aux = g_strconcat(values,temp_invoice,",",NULL);
		    g_free(values);
		    g_free(temp_invoice);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;

		}

		if (with_payment==TRUE) {
		    gchar * temp_currval=m_model_currval_sequence ("seqPago",connection);
		    values_aux = g_strconcat(values,temp_currval,",'",ctime(&issue_date),"',",NULL);
		    g_free(values);
		    values = values_aux;
		    g_free(temp_currval);
		} else {
		    values_aux = g_strconcat(values,"NULL",",'",ctime(&issue_date),"',",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (invoice_discount!=-1) {
		    temp_invoice = g_strdup_printf("to_number('%.2f','999D99')",invoice_discount);
		    values_aux = g_strconcat(values,temp_invoice,",",NULL);
		    g_free(values);
		    g_free(temp_invoice);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (new_total!=G_MINDOUBLE) {
		    temp_invoice = g_strdup_printf("%10.2f",new_total);
		    g_strstrip(temp_invoice);
		    values_aux = g_strconcat(values,"to_number('",temp_invoice,"','99999999D99'),",NULL);
		    g_free(values);
		    g_free(temp_invoice);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		if (d_invoice_get_modified_total (invoice)==TRUE) {
		    values_aux = g_strconcat(values,"TRUE,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"FALSE,",NULL);
		    g_free(values);
		    values = values_aux;
		}


		temp_invoice = g_strdup_printf("%d",state);
		values_aux = g_strconcat(values,temp_invoice,",",NULL);
		g_free(values);
		g_free(temp_invoice);
		values = values_aux;

		if ((invoice->delivery_payments)==NULL) {
		    if ((d_repair_get_repair_code (repair))!=ULONG_MAX) {
			temp_invoice = g_strdup_printf("%lu",d_repair_get_repair_code (repair));
			values_aux = g_strconcat(values,temp_invoice,NULL);
			g_free(values);
			g_free(temp_invoice);
			values = values_aux;
		    } else {
			values_aux = g_strconcat(values,"NULL",NULL);
			g_free(values);
			values = values_aux;
		    }
		} else {
		    values_aux = g_strconcat(values,"NULL",NULL);
		    g_free(values);
		    values = values_aux;
		}

		values_aux = g_strconcat(values,",FALSE,",NULL);
		g_free(values);
		values = values_aux;

		if (d_invoice_get_vat_type (invoice)!=0) {
		    values_aux = g_strconcat(values,"NULL,",NULL);
		    g_free(values);
		    values = values_aux;
		} else {
		    values_aux = g_strconcat(values,"0.00,",NULL);
		    g_free(values);
		    values = values_aux;
		}

		values_aux = g_strconcat(values,invoice_code,")",NULL);
		g_free(values);
		values = values_aux;

		query_aux=g_strconcat(query2,values,NULL);


		if (query2!=NULL) g_free(query2);
		query2=query_aux;
		g_free(values);
		query_aux=NULL;



		if (m_model_command_execute (&record_set2, query2, connection,command,tuples,0, FALSE)==FALSE) {
		    result = ULONG_MAX;
		}

		if (query2!=NULL) g_free(query2);
		query2=NULL;
	    } else {
		gchar * shop_split_invoice_code = g_strdup_printf("%lu",d_invoice_get_shop_code_split_invoice (invoice));
		Enterprise * enterprise_split_invoice =m_enterprise_find_by_shop_in_transaction (command,connection,d_invoice_get_shop_code_split_invoice (invoice));
		gchar * name_enterprise_split_invoice = d_participant_get_name (d_enterprise_get_participant (enterprise_split_invoice));
		gchar * enterprise_original_code_char = g_strdup_printf("%lu",enterprise_code);
		gchar * shop_original_code_char = g_strdup_printf("%lu",shop_code);
		gchar * payment_code_char = g_strdup_printf("%lu",payment_code);
		gchar * issue_date_char = g_strdup(ctime(&issue_date));
		gchar * payer_code_char = g_strdup_printf("%lu",payer_code);
		gulong enterprise_split_invoice_code = d_participant_get_code (d_enterprise_get_participant (enterprise_split_invoice));
		gchar * enterprise_split_invoice_code_char = g_strdup_printf("%lu",enterprise_split_invoice_code);
		gchar * invoice_discount_char=NULL;
		gchar * total_invoice_char = g_strdup_printf("%10.2f",new_total);
		gchar * total_modified_char=NULL;
		gchar * state_char = g_strdup("0");
		gchar * split_invoice_state_char = g_strdup("0");
		gchar * repair_code_char = NULL;
		gchar * repair_split_invoice_code_char = NULL;
		Payment * payment_split_invoice = d_payment_create_empty ();
		Shop * shop_to_payment = m_shop_find_by_shop_code (d_invoice_get_shop_code_split_invoice (invoice));
		gulong payment_code_split_invoice = ULONG_MAX;
		gchar * payment_split_invoice_code_char=NULL;
		gchar * vat_type = NULL;

		d_payment_set_enterprise (payment_split_invoice, enterprise_split_invoice);
		d_payment_set_shop (payment_split_invoice, shop_to_payment);
		d_payment_set_achieved_payments (payment_split_invoice,0);
		d_payment_set_entry_percentage (payment_split_invoice,100);
		d_payment_set_number_expiries (payment_split_invoice,1);
		d_payment_set_interval_days (payment_split_invoice,60);
		d_payment_set_is_transfer (payment_split_invoice,TRUE);

		if (d_invoice_get_modified_total (invoice)!=FALSE) {
		    total_modified_char = g_strdup("TRUE");
		} else {
		    total_modified_char = g_strdup("FALSE");
		}

		if (invoice->delivery_payments==NULL) {
		    repair_code_char = g_strdup_printf("%lu",d_repair_get_repair_code (repair));
		    repair_split_invoice_code_char = g_strdup(repair_code_char);
		} else {
		    repair_code_char = g_strdup("NULL");
		    repair_split_invoice_code_char = g_strdup_printf("%lu",d_repair_get_repair_code (d_delivery_payment_get_repair (invoice->delivery_payments[0])));
		}
		if (invoice_discount!=-1)
		    invoice_discount_char=g_strdup_printf("to_number('%.2f','999D99')",invoice_discount);
		else
		    invoice_discount_char=g_strdup("NULL");

		if (d_invoice_get_shop_code_split_invoice (invoice)!=ULONG_MAX) {
		    if (d_invoice_get_vat_type (invoice)!=0) {
			enterprise_aux = g_strdup(enterprise_split_invoice_code_char);
		    } else {
			enterprise_aux = enterprise_aux = g_strdup_printf("%lu",(gulong) D_ENTERPRISE_NONE);
		    }
		    invoice_split_invoice_number = g_strdup_printf("%lu",m_invoice_get_number (enterprise_aux,shop_split_invoice_code,connection,command));
		    invoice_split_invoice_code = m_model_nextval_sequence ("seqFacturaAbono",connection);
		}

		if (d_invoice_get_vat_type (invoice)!=0) {
		    vat_type = g_strdup("NULL");
		} else {
		    vat_type = g_strdup("0.00");
		}

		query3 = g_strconcat("insert into facturaabono values(",
				     enterprise_original_code_char,",",
				     shop_original_code_char,",",
				     invoice_number,",",
				     enterprise_split_invoice_code_char,",",
				     payment_code_char,",'",
				     issue_date_char,"',",
				     invoice_discount_char,",",
				     "to_number('",total_invoice_char,"','99999999D99'),",
				     total_modified_char,",",
				     state_char,",",
				     repair_code_char,",FALSE,",vat_type,",",invoice_code,")",
				     NULL);
		if (m_model_command_execute (&record_set3, query3, connection, command,tuples,0, FALSE)==FALSE) {
		    result = ULONG_MAX;
		}
		if (record_set3!=NULL)
		    gda_recordset_free(record_set3);
		record_set3=NULL;
		g_print(_("\nInsert: %s\n"),query3);
		g_free(query3);
		payment_code_split_invoice = m_payment_insert_in_transaction (payment_split_invoice,command,connection);
		if (payment_code_split_invoice==ULONG_MAX) {
		    result =ULONG_MAX;
		}
		payment_split_invoice_code_char = g_strdup_printf("%lu",payment_code_split_invoice);
		g_strstrip(total_invoice_char);

		query2 = g_strconcat("insert into facturaabono values(",
				     enterprise_split_invoice_code_char,",",
				     shop_split_invoice_code,",",
				     invoice_split_invoice_number,",",
				     payer_code_char,",",
				     payment_split_invoice_code_char,",'",
				     issue_date_char,"',",
				     invoice_discount_char,",",
				     "to_number('",total_invoice_char,"','99999999D99'),",
				     total_modified_char,",",
				     state_char,",",
				     "NULL,TRUE,",vat_type,",",invoice_split_invoice_code,")",
				     NULL);


		if (m_model_command_execute (&record_set2, query2, connection, command,tuples,0, FALSE)==FALSE) {
		    result = ULONG_MAX;
		}

		g_print(_("\nInsert: %s\n"),query2);

		g_free(query2);
		if (result != ULONG_MAX) {
		    query_split_invoice = g_strconcat("insert into refacturacion values(",
						  enterprise_original_code_char,",",
						  shop_original_code_char,",",
						  invoice_code,",",
						  enterprise_split_invoice_code_char,",",
						  shop_split_invoice_code,",",
						  invoice_split_invoice_code,",",
						  repair_split_invoice_code_char
						  ,")",NULL);
		    if (m_model_command_execute (&record_set_split_invoice, query_split_invoice, connection, command,tuples,0, FALSE)==FALSE) {
			result = ULONG_MAX;
		    }
		}

		if (name_enterprise_split_invoice!=NULL) g_free(name_enterprise_split_invoice);
		if (payment_code_char!=NULL) g_free(payment_code_char);
		if (invoice_discount_char!=NULL) g_free(invoice_discount_char);
		if (total_invoice_char!=NULL) g_free(total_invoice_char);
		if (total_modified_char!=NULL) g_free(total_modified_char);
		if (state_char!=NULL) g_free(state_char);
		if (issue_date_char!=NULL) g_free(issue_date_char);
		if (split_invoice_state_char!=NULL) g_free(split_invoice_state_char);
		if (repair_code_char!=NULL) g_free(repair_code_char);
		if (payer_code_char!=NULL) g_free(payer_code_char);
		if (repair_split_invoice_code_char!=NULL) g_free(repair_split_invoice_code_char);
		if (shop_split_invoice_code!=NULL) g_free(shop_split_invoice_code);
		if (enterprise_original_code_char!=NULL) g_free(enterprise_original_code_char);
		if (shop_original_code_char!=NULL) g_free(shop_original_code_char);
		if (enterprise_split_invoice_code_char!=NULL) g_free(enterprise_split_invoice_code_char);
		if (invoice_split_invoice_number!=NULL) g_free(invoice_split_invoice_number);
		if (invoice_split_invoice_code!=NULL) g_free(invoice_split_invoice_code);


	    }
	    if (enterprise!=NULL)
		g_free(enterprise);
	    if (invoice->delivery_payments!=NULL) {
		gint index = 0;
		while (invoice->delivery_payments[index]!=NULL) {
		    gulong  garage_code=ULONG_MAX;
		    gulong  client_code=ULONG_MAX;
		    gboolean condition1=FALSE;
		    gdouble value_to_condition=FALSE;
		    if (d_repair_get_garage (d_delivery_payment_get_repair (invoice->delivery_payments[index]))!=NULL) {
			client_code = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (invoice->delivery_payments[index]))));
			garage_code = d_participant_get_code (d_invoice_get_participant (invoice));
		    }
		    condition1=m_delivery_payment_is_from_garage (garage_code,client_code,invoice->delivery_payments[index],connection,command);
		    value_to_condition=d_delivery_payment_get_garage_total (invoice->delivery_payments[index]);
		    g_print("%d",condition1);
		    g_print("%.2f",value_to_condition);
		    if (condition1!=FALSE) {
			if (value_to_condition>0) {
			    if (insert_garage_invoice(invoice,g_strdup(invoice_code), index,connection, command) == FALSE)
				result = ULONG_MAX;
			} else {
			    gchar * temp = g_strdup_printf("%lu",d_delivery_payment_get_number (invoice->delivery_payments[index]));
			    shop = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_repair_get_shop (
														       d_delivery_payment_get_repair (invoice->delivery_payments[index])
														      ))));
			    enterprise = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (
																    d_delivery_payment_get_repair (invoice->delivery_payments[index])
																   ))));
			    delivery_payment_number = g_strconcat(temp,NULL);
			    g_free(temp);


			    query4 = g_strconcat(" update albaranabono set numerofactura=",invoice_code,
						 " where codigoEmpresa = ",enterprise,
						 " and codigoTienda= ",shop,
						 " and numeroAlbaran= ",delivery_payment_number,NULL);
			    g_free(enterprise);
			    g_free(shop);
			    g_free(delivery_payment_number);
			    if (m_model_command_execute (&record_set4, query4, connection, command,tuples,0, FALSE)==FALSE) {
				result = ULONG_MAX;
			    }
			    if (record_set4!=NULL)
				gda_recordset_free(record_set4);
			    record_set4=NULL;
			}

		    } else {
			gchar * temp = g_strdup_printf("%lu",d_delivery_payment_get_number (invoice->delivery_payments[index]));
			shop = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_repair_get_shop (
														   d_delivery_payment_get_repair (invoice->delivery_payments[index])
														  ))));
			enterprise = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (
																d_delivery_payment_get_repair (invoice->delivery_payments[index])
															       ))));
			delivery_payment_number = g_strconcat(temp,NULL);
			g_free(temp);



			query4 = g_strconcat(" update albaranabono set numerofactura=",invoice_code,
					     " where codigoEmpresa = ",enterprise,
					     " and codigoTienda= ",shop,
					     " and numeroAlbaran= ",delivery_payment_number,NULL);
			g_free(enterprise);
			g_free(shop);
			g_free(delivery_payment_number);
			if (m_model_command_execute (&record_set4, query4, connection, command,tuples,0, FALSE)==FALSE) {
			    result = ULONG_MAX;
			}
			if (record_set4!=NULL)
			    gda_recordset_free(record_set4);
			record_set4=NULL;
		    }


		    index++;
		}
	    }



	    if (query5!=NULL) g_free(query5);
	    if (query6!=NULL) g_free(query6);
	    if (query7!=NULL) g_free(query7);
	    if (query!=NULL) g_free(query);
	    if (query_split_invoice!=NULL) g_free(query_split_invoice);
	}
	g_free(tuples);
	g_free(invoice_code);
	g_free(invoice_number);

    }

    if (record_set1!=NULL)
	gda_recordset_free(record_set1);
    if (record_set2!=NULL)
	gda_recordset_free(record_set2);
    if (record_set5!=NULL)
	gda_recordset_free(record_set5);
    if (record_set6!=NULL)
	gda_recordset_free(record_set6);
    if (record_set7!=NULL)
	gda_recordset_free(record_set7);
    if (record_set_split_invoice!=NULL)
	gda_recordset_free(record_set_split_invoice);



    if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	result = ULONG_MAX;
    }

    igalia_command_free(command);
    if (result!=ULONG_MAX)
	d_invoice_set_invoice_code (invoice,invoice_code_gulong);

    return result;
}

/**
 * m_invoice_update_state:
 * @invoice: Factura a modificar el estado.
 * @state: Nuevo estado de la factura.
 *
 * Modifica el estado de la factura en la base de datos. Los estados de la
 * factura se pueden examinar en dataobjects/invoice.h
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_invoice_update_state (Invoice * invoice,gint state) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * invoice_number=NULL;
    gchar * shop_code=NULL;
    gchar * enterprise_code=NULL;
    gchar * state_char = NULL;
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    if (invoice !=NULL) {
	invoice_number = g_strdup_printf("%lu",d_invoice_get_invoice_code (invoice));
	enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))));
	shop_code =  g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice))));

	if (invoice_number!=NULL) {
	    gulong enterprise_participant_code = ULONG_MAX;
	    gulong shop_participant_code = ULONG_MAX;

	    if (d_invoice_get_enterprise (invoice)!=NULL) {
		enterprise_participant_code = d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
	    }
	    if (d_invoice_get_shop (invoice)!=NULL) {
		shop_participant_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice)));
	    }

	    gda_command_set_connection(command,connection);

	    state_char = g_strdup_printf("%d",state);
	    query = g_strconcat("update facturaabono set estado=",state_char,
				" where (codigoFactura=",invoice_number,
				") and (codigoEmpresa=",enterprise_code,
				") and (codigoTienda=",shop_code,
				")",NULL);
	    result = m_model_mutex_transactional_command_execute_logged (&record_set,
                                                               query,
							       connection,
							       command, tuples,
							       0, FALSE,
							       enterprise_participant_code,
							       shop_participant_code,
                                                               ULONG_MAX
							      );

	    g_free(invoice_number);

	}
	if (record_set!=NULL)
	    gda_recordset_free(record_set);

	g_free(query);
	g_free(tuples);
	g_free(shop_code);
	g_free(enterprise_code);
        g_free(state_char);

    }

    igalia_command_free(command);

    if (result) {
	g_print(_("\nResult: true\n"));
	d_invoice_set_state (invoice, state);
    }
    else {
	g_print(_("\nResult: false\n"));
    }

    return result;
}


/**
 * m_invoice_find_by_condition:
 * @condition: Condicin de bsqueda. Esta condicin permite buscar por: tipo de factura (factura,refactura o
 * nota), cdigo de la empresa, cdigo de la tienda, nombre del receptor de la factura, cif/nif del receptor
 * de la factura, nmero de factura, estado de la factura, perodo de vencimiento,tipo de pago, cantidad,
 * si es con iva o sin iva, matrcula del coche y cdigo de compaia de seguros de la reparacin.
 * 
 * Esta funcin realiza la bsqueda facturas por condicin. La bsqueda se realizar entre las facturas del taller y
 * facturas normales o refacturas.
 *
 * Returns:
 * Devuelve las facturas que cumplen con la condicin de bsqueda.
 */
Invoice ** m_invoice_find_by_condition (ConditionFindInvoice * condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_select2=NULL;
    gchar              *gchar_select_garage=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_group_garage=NULL;
    gchar              *gchar_where_split_invoice=NULL;
    gchar              *gchar_where_garage=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar              *gchar_where_aux2=NULL;
    gchar              *gchar_where_aux_split_invoice=NULL;
    gchar * gchar_select_general = NULL;
    gchar              *gchar_where2=NULL;
    gchar * gchar_union ;
    gchar              *gchar_from=NULL;
    gchar              *gchar_from_split_invoice=NULL;
    gchar              *gchar_from2_split_invoice=NULL;
    gchar              *gchar_from2_split_invoice_aux=NULL;
    gchar              *gchar_from_garage=NULL;
    gchar              *gchar_from2=NULL;
    gchar              *gchar_order=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set1 = NULL;
    gulong index_of_invoices=0;

    Invoice ** invoices=NULL;
    Invoice * invoice;
    gint index;
    gulong * tuples1;

    my_command = gda_command_new();
    tuples1 = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    if  ((condition->type==D_INVOICE_CODE) || (condition->type==D_INVOICE_SPLIT_INVOICE_CODE)) {
	gchar_union= g_strdup(" union ");
	gchar_select = g_strconcat("(SELECT ","facturaabono.numeroFactura as numeroFacturaFactura,",
				   "facturaabono.codigoFactura as codigoFacturaFactura,",
				   "facturaabono.codigoEmpresa as codigoEmpresaFactura,",
				   "facturaabono.codigoTienda as codigoTiendaFactura,",
				   "facturaabono.estado as estadoFactura,",
				   "facturaabono.tipoIva as tipoIvaFactura,",
				   "facturaAbono.baseImponibleModificada as baseImponibleModificadaFactura,",
				   "to_char(facturaAbono.baseImponibleNueva,'99999999D99') as baseImponibleNuevaFactura,",
				   "refactura as refacturaFactura,",
				   "participanteShop.codigoParty as codigoPartyParticipantShop,",
				   "participanteShop.nombre as nombreParticipantShop,",
				   "participanteCliente.codigoParty as codigoPartyParticipantPagador,",
				   "participanteCliente.nick as nickParticipantPagador,",
				   "participanteCliente.nombreCorto as shortNameParticipantPagador,",
				   "participanteCliente.cif as cifParticipantPagador,",
				   "reparacion.codigoReparacion as codigoReparacionRepair,",
				   "reparacion.codigoTienda as codigoTiendaRepair,",
				   "reparacion.codigoCliente as codigoClienteRepair,",
				   "reparacion.codigoCompania as codigoCompaniaRepair,",
				   "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
				   "facturaAbono.codigoEmpresa as codigoPartyParticipantEnter,",
				   "participanteCompany.nombre as nombreParticipantCompany,",
				   "participanteCompany.nombreCorto as shortNameParticipantCompany,",
				   "participanteCompany.codigoParty as codigoPartyParticipantCompany,",
				   "participanteCompany.cif as cifParticipantCompany, ",
				   "pago.codigoPago as codigoPagoPago, ",
				   "pago.codigobanco as codigoBancoPago, "
				   "pago.codigooficina as codigoOficinaPago, ",
				   "pago.numerocuenta as numeroCuentaPago, ",
				   "pago.esAPlazos as esAPlazosPago, ",
				   "pago.pagosRealizados as pagosRealizadosPago, ",
				   "pago.numeroVencimientos as numeroVencimientosPago, ",
				   "pago.fechaInicio as fechaInicioPago, ",
				   "pago.esdomiciliacion as esdomiciliacionpago ",
				   NULL );
	gchar_select2 = g_strconcat("(SELECT DISTINCT ","facturaabono.numeroFactura as numeroFacturaFactura,",
				   "facturaabono.codigoFactura as codigoFacturaFactura,",
				   "facturaabono.codigoEmpresa as codigoEmpresaFactura,",
				   "facturaabono.codigoTienda as codigoTiendaFactura,",
				   "facturaabono.estado as estadoFactura,",
				   "facturaabono.tipoIva as tipoIvaFactura,",
				   "facturaAbono.baseImponibleModificada as baseImponibleModificadaFactura,",
				   "to_char(facturaAbono.baseImponibleNueva,'99999999D99') as baseImponibleNuevaFactura,",
				   "refactura as refacturaFactura,",
				   "participanteShop.codigoParty as codigoPartyParticipantShop,",
				   "participanteShop.nombre as nombreParticipantShop,",
				   "participanteCliente.codigoParty as codigoPartyParticipantPagador,",
				   "participanteCliente.nick as nickParticipantPagador,",
				   "participanteCliente.nombreCorto as shortNameParticipantPagador,",
				   "participanteCliente.cif as cifParticipantPagador,",
				   "(case WHEN reparacion.codigoCompania is not NULL THEN reparacion.codigoReparacion ELSE NULL END) as codigoReparacionRepair,",
				   "reparacion.codigoTienda as codigoTiendaRepair,",
				   "reparacion.codigoCliente as codigoClienteRepair,",
				   "reparacion.codigoCompania as codigoCompaniaRepair,",
				   "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
				   "facturaAbono.codigoEmpresa as codigoPartyParticipantEnter,",
				   "participanteCompany.nombre as nombreParticipantCompany,",
				   "participanteCompany.nombreCorto as shortNameParticipantCompany,",
				   "participanteCompany.codigoParty as codigoPartyParticipantCompany,",
				   "participanteCompany.cif as cifParticipantCompany, ",
				   "pago.codigoPago as codigoPagoPago, ",
				   "pago.codigobanco as codigoBancoPago, "
				   "pago.codigooficina as codigoOficinaPago, ",
				   "pago.numerocuenta as numeroCuentaPago, ",
				   "pago.esAPlazos as esAPlazosPago, ",
				   "pago.pagosRealizados as pagosRealizadosPago, ",
				   "pago.numeroVencimientos as numeroVecimientosPago, ",
				   "pago.fechaInicio as fechaInicioPago, ",
				   "pago.esdomiciliacion as esdomiciliacionpago ",
				   NULL );
	gchar_select_garage = g_strconcat("(SELECT ","facturaabono.numeroFactura as numeroFacturaFactura,",
				   "facturaabono.codigoFactura as codigoFacturaFactura,",
				   "facturaabono.codigoEmpresa as codigoEmpresaFactura,",
				   "facturaabono.codigoTienda as codigoTiendaFactura,",
				   "facturaabono.estado as estadoFactura,",
				   "facturaabono.tipoIva as tipoIvaFactura,",
				   "facturaAbono.baseImponibleModificada as baseImponibleModificadaFactura,",
				   "to_char(facturaAbono.baseImponibleNueva,'99999999D99') as baseImponibleNuevaFactura,",
				   "refactura as refacturaFactura,",
				   "participanteShop.codigoParty as codigoPartyParticipantShop,",
				   "participanteShop.nombre as nombreParticipantShop,",
				   "participanteCliente.codigoParty as codigoPartyParticipantPagador,",
				   "participanteCliente.nick as nickParticipantPagador,",
				   "participanteCliente.nombreCorto as shortNameParticipantPagador,",
				   "participanteCliente.cif as cifParticipantPagador,",
				   "NULL as codigoReparacionRepair,",
				   "reparacion.codigoTienda as codigoTiendaRepair,",
				   "NULL as codigoClienteRepair,",
				   "NULL as codigoCompaniaRepair,",
				   "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
				   "facturaAbono.codigoEmpresa as codigoPartyParticipantEnter,",
				   "NULL as nombreParticipantCompany,",
				   "NULL as shortNameParticipantCompany,",
				   "NULL as codigoPartyParticipantCompany,",
				   "NULL as cifParticipantCompany, ",
				   "pago.codigoPago as codigoPagoPago, ",
				   "pago.codigobanco as codigoBancoPago, ",
				   "pago.codigooficina as codigoOficinaPago, ",
				   "pago.numerocuenta as numeroCuentaPago, ",
				   "pago.esAPlazos as esAPlazosPago, ",
				   "pago.pagosRealizados as pagosRealizadosPago, ",
				   "pago.numeroVencimientos as numeroVecimientosPago, ",
				   "pago.fechaInicio as fechaInicioPago, ",
				   "pago.esdomiciliacion as esdomiciliacionpago ",
				   NULL );


	gchar_from = g_strconcat(" from reparacion left outer join participante participanteCompany ",
				 " ON (reparacion.codigoCompania=participanteCompany.codigoParty) left outer join siniestro ",
				 " on (siniestro.codigoReparacion=reparacion.codigoReparacion and siniestro.codigoTienda=reparacion.codigoTienda",
				 " and siniestro.codigoEmpresa=reparacion.codigoEmpresa), ",
				 " participante participanteCliente,",
				 " participante participanteShop,",
				 " tienda,",
				 " facturaabono left outer join pago on",
				 " (facturaabono.codigoPago=pago.codigoPago and facturaabono.codigoTienda=pago.codigoTienda and facturaabono.codigoEmpresa=pago.codigoEmpresa) ",
				 NULL);
	gchar_from2 = g_strconcat(" from reparacion left outer join participante participanteCompany ",
				  " ON (reparacion.codigoCompania=participanteCompany.codigoParty) left outer join siniestro ",
				 " on (siniestro.codigoReparacion=reparacion.codigoReparacion and siniestro.codigoTienda=reparacion.codigoTienda",
				 " and siniestro.codigoEmpresa=reparacion.codigoEmpresa), ",
				  " participante participanteCliente,",
				  " participante participanteShop,",
				  " tienda,",
				  " facturaabono left outer join pago on",
				  " (facturaabono.codigoPago=pago.codigoPago and facturaabono.codigoTienda=pago.codigoTienda and facturaabono.codigoEmpresa=pago.codigoEmpresa),",
				  " (select a.codigofactura,albaranabono.codigoReparacion,a.codigoTienda,a.codigoEmpresa from albaranabono,facturaabono a ",
				  " where albaranabono.numeroFactura=a.codigoFactura and a.codigoEmpresa=albaranAbono.codigoEmpresa and a.codigoTienda=albaranabono.codigoTienda) as albaranes ",
				  NULL);
	gchar_from_split_invoice = g_strconcat(" from reparacion left outer join participante participanteCompany ",
				  " ON (reparacion.codigoCompania=participanteCompany.codigoParty) left outer join siniestro ",
				 " on (siniestro.codigoReparacion=reparacion.codigoReparacion and siniestro.codigoTienda=reparacion.codigoTienda",
				 " and siniestro.codigoEmpresa=reparacion.codigoEmpresa), ",
				  " participante participanteCliente,",
				  " participante participanteShop,",
				  " tienda,",
				  " facturaabono left outer join pago on",
				  " (facturaabono.codigoPago=pago.codigoPago and facturaabono.codigoTienda=pago.codigoTienda and facturaabono.codigoEmpresa=pago.codigoEmpresa),",
         			  " (select numeroRefactura as numeroFactura,refacturacion.codigoReparacion as codigoReparacionRefactura, ",
				  " codigoEmpresaRefactura,codigoTiendaRefactura,codigoEmpresaAbono,codigoTiendaAbono from refacturacion,facturaabono ",
				  " where numeroRefactura=facturaabono.codigofactura and ",
				  " codigoEmpresaRefactura=facturaabono.codigoEmpresa and ",
				  " codigoTiendaRefactura=facturaabono.codigoTienda ",
					   NULL);
        gchar_from2_split_invoice = g_strdup(") as refacturas ");
	gchar_from_garage = g_strconcat(" from reparacion left outer join participante participanteCompany ",
					" ON (reparacion.codigoCompania=participanteCompany.codigoParty) left outer join siniestro ",
					" on (siniestro.codigoReparacion=reparacion.codigoReparacion and siniestro.codigoTienda=reparacion.codigoTienda",
					" and siniestro.codigoEmpresa=reparacion.codigoEmpresa), ",
					" participante participanteCliente,",
					" participante participanteShop,",
					" tienda,",
					" facturaabono left outer join pago on",
					" (facturaabono.codigoPago=pago.codigoPago and facturaabono.codigoTienda=pago.codigoTienda and facturaabono.codigoEmpresa=pago.codigoEmpresa), "
					" (select distinct facturaabono.codigofactura as numeroFacturaFactura,facturaabono.codigoPago,albaranabono.numeroFactura, ",
					" codigoreparacionTaller,facturatalleres.codigoEmpresaAlbaran as codigoEMpresaAlbaranTaller,facturatalleres.codigoTiendaAlbaran as codigoTiendaAlbaranTaller from facturaabono ",
					" left outer join albaranabono on (facturaabono.codigoFactura=albaranabono.numeroFactura and (facturaabono.codigoTienda=albaranabono.codigoTienda) and (facturaabono.codigoEMpresa=albaranAbono.codigoEmpresa)),",
					" (select codigoReparacion as codigoReparacionTaller,numeroFacturaTaller as numeroFacturaTaller ",
                                        " ,codigoEmpresaFactura,codigoTiendaFactura,codigoEmpresaAlbaran,codigoTiendaAlbaran "
					" from facturataller,albaranabono where albaranabono.numeroAlbaran=facturaTaller.numeroalbaran",
					" and albaranabono.codigoTienda=facturataller.codigoTiendaAlbaran and albaranabono.codigoEmpresa=facturaTaller.codigoEmpresaAlbaran"," ) ",
					" as facturaTalleres "," where albaranabono.numeroFactura is null and ",
					" facturaabono.codigoFactura=numeroFacturaTaller and facturaabono.codigoTienda=facturaTalleres.codigoTiendaFactura ",
					" and facturaabono.codigoEmpresa=facturaTalleres.codigoEmpresaFactura  "
				        "  ) as facturasTalleres ",
					NULL);

	gchar_where=g_strconcat(" where ",
				"(facturaabono.numeroFactura is not null) ",
				" and facturaabono.codigoTienda=participanteShop.codigoParty ",
				" and facturaabono.codigoTienda=reparacion.codigoTienda ",
                                " and facturaabono.codigoEmpresa=reparacion.codigoEmpresa ",
				" and facturaabono.codigoPagador = participanteCliente.codigoParty ",
				" and tienda.codigoTienda = facturaabono.codigoTienda ",
				NULL);
	gchar_where2=g_strconcat(" where ",
				 "(facturaabono.numeroFactura is not null) ",
				 " and facturaabono.codigoTienda=participanteShop.codigoParty",
				 " and facturaabono.codigoPagador = participanteCliente.codigoParty",
				 " and tienda.codigoTienda = facturaabono.codigoTienda ",
				 " and (albaranes.codigoFactura = facturaAbono.codigoFactura) ",
				 " and (albaranes.codigoEmpresa = facturaAbono.codigoEmpresa) ",
				 " and (albaranes.codigoTienda = facturaAbono.codigoTienda) ",
				 " and (albaranes.codigoReparacion=reparacion.codigoReparacion) ",
				 " and (albaranes.codigoTienda=reparacion.codigoTienda) ",
				 " and (albaranes.codigoEmpresa=reparacion.codigoEmpresa) ",
				 NULL);

	gchar_where_split_invoice=g_strconcat(" where ",
				 " facturaabono.codigoTienda=participanteShop.codigoParty",
				 " and facturaabono.codigoPagador = participanteCliente.codigoParty",
				 " and tienda.codigoTienda = facturaabono.codigoTienda ",
				 " and (refacturas.numeroFactura = facturaAbono.codigoFactura) ",
				 " and (refacturas.codigoReparacionRefactura=reparacion.codigoReparacion) ",
				 " and (refacturas.codigoEmpresaRefactura=facturaabono.codigoEmpresa) ",
				 " and (refacturas.codigoTiendaRefactura=facturaabono.codigoTienda) ",
				 " and (refacturas.codigoEmpresaAbono=reparacion.codigoEmpresa) ",
				 " and (refacturas.codigoTiendaAbono=reparacion.codigoTienda) ",
				 NULL);
	gchar_group_garage=g_strconcat(" group by facturaabono.codigoFactura, facturaabono.numeroFactura,facturaabono.codigoempresa, "
				       "facturaabono.codigotienda, facturaabono.estado, facturaabono.tipoIva,facturaabono.baseimponiblemodificada,",
				       "facturaabono.baseimponiblenueva, facturaabono.refactura,participanteshop.codigoparty, participanteshop.nombre, ",
				       "participantecliente.codigoparty,participantecliente.nick,participanteCliente.nombreCorto,participantecompany.nombre,participantecliente.cif,",
				       "reparacion.codigotienda,tienda.codigoPertenenciaGrupo,pago.codigopago ,pago.codigobanco, pago.codigooficina, pago.numerocuenta, pago.esAPlazos,pago.pagosRealizados,pago.numeroVencimientos,pago.fechainicio, pago.esdomiciliacion",NULL);

                                                        /*??? NON SEI CAL SERA MELLOR PARA ORDENAR*/
	gchar_order=g_strconcat(" order by participantecompany.nombre,facturaabono.numeroFactura) ",NULL);
//	gchar_order=g_strconcat(" order by facturaabono.numeroFactura) ",NULL);
	if (((condition->shop_code) != ULONG_MAX) && ((condition->shop_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->shop_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoTienda=",
					   temp,") and (reparacion.codigoTienda=",temp,")",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.codigoTienda=",
					   temp,") and (reparacion.codigoTienda=",temp
					   ,") and (albaranes.codigoTienda=",temp,")",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (facturaabono.codigoTienda=",
						temp,")",NULL);
            gchar_from2_split_invoice_aux = g_strconcat(" and refacturacion.codigoTiendaRefactura=",temp,gchar_from2_split_invoice,NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    g_free(gchar_from2_split_invoice);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_from2_split_invoice = gchar_from2_split_invoice_aux;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
            gchar_from2_split_invoice_aux=NULL;
	}

	g_print(_("\nEnterprise Code: %lu\n"),condition->enterprise_code);
	if (((condition->enterprise_code) != ULONG_MAX) && ((condition->enterprise_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->enterprise_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoEmpresa=",
					   temp,") and (reparacion.codigoEmpresa=",temp,")",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.codigoEmpresa=",
					   temp,") and (reparacion.codigoEmpresa=",temp
					   ,") and (albaranes.codigoEmpresa=",temp,")",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (facturaabono.codigoEmpresa=",
						temp,")",NULL);
            gchar_from2_split_invoice_aux = g_strconcat(" and refacturacion.codigoEmpresaRefactura=",temp,gchar_from2_split_invoice,NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    g_free(gchar_from2_split_invoice);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_from2_split_invoice = gchar_from2_split_invoice_aux;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
            gchar_from2_split_invoice_aux=NULL;
	}

	if (((condition->client_name) != NULL) && (strlen(condition->client_name)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (participanteCliente.nick ilike '%",
					  condition->client_name,"%')",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (participanteCliente.nick ilike '%",
					   condition->client_name,"%')",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (participanteCliente.nick ilike '%",
					   condition->client_name,"%')",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
	}

	if (((condition->client_legal_id) != NULL) && (strlen(condition->client_legal_id)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (participanteCliente.cif='",
					  condition->client_legal_id,"')",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (participanteCliente.cif='",
					   condition->client_legal_id,"')",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (participanteCliente.cif='",
					   condition->client_legal_id,"')",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice=gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;

	}

	if (((condition->license_number) != NULL) && (strlen(condition->license_number)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.matricula ilike '%",
					  condition->license_number,"%')",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (reparacion.matricula ilike '%",
					   condition->license_number,"%')",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (reparacion.matricula ilike '%",
					   condition->license_number,"%')",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice=gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;

	}

	if (((condition->accident_reference) != NULL) && (strlen(condition->accident_reference)!=0)) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (siniestro.referenciasiniestro ilike '%",
					  condition->accident_reference,"%')",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (siniestro.referenciasiniestro ilike '%",
					   condition->accident_reference,"%')",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (siniestro.referenciasiniestro ilike '%",
					   condition->accident_reference,"%')",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice=gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;

	}

	if (((condition->invoice_number) != 0) && ((condition->invoice_number) != ULONG_MAX))  {
	    gchar * temp = g_strdup_printf("%lu",condition->invoice_number);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.numeroFactura=",
					  temp,")",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.numeroFactura=",
					   temp,")",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (facturaabono.numeroFactura=",
					   temp,")",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;

	}

	if ((condition->invoice_state) >= 0)  {
	    gchar * temp = g_strdup_printf("%d",condition->invoice_state);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.estado=",
					  temp,")",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.estado=",
					   temp,")",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (facturaabono.estado=",
					   temp,")",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
	}

	if ((condition->ending_issue_date)!= NULL)  {
	    gchar * temp = g_strdup(ctime(condition->ending_issue_date));
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.fechaEmision<='",
					  temp,"')",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.fechaEmision<='",
					   temp,"')",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (facturaabono.fechaEmision<='",
					   temp,"')",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
            g_free(temp);
	}

	if ((condition->initial_issue_date)!= NULL)  {
	    gchar * temp = g_strdup(ctime(condition->initial_issue_date));
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.fechaEmision>='",
					  temp,"')",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.fechaEmision>='",
					   temp,"')",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (facturaabono.fechaEmision>='",
					   temp,"')",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
            g_free(temp);
	}

	if (((condition->company_code) != ULONG_MAX) && ((condition->company_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->company_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoCompania=",temp,") and ",
					  "(participanteCOmpany.codigoPArty=",temp,")",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2," and (reparacion.codigoCOmpania=",temp,") and ",
					   "(participanteCOmpany.codigoPArty=",temp,")",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and (reparacion.codigoCompania=",
						temp,") and (participanteCOmpany.codigoPArty=",temp,")",NULL);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
	}

	if ((condition->vat)==FALSE) {
	    gchar_where_aux = g_strconcat(gchar_where, " and facturaabono.tipoIva is not NULL ",
					  NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and facturaabono.tipoIva is not NULL ",
					  NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and facturaabono.tipoIva is not NULL ",
					  NULL);

	} else {
	    gchar_where_aux = g_strconcat(gchar_where, " and facturaabono.tipoIva is NULL ",
					  NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and facturaabono.tipoIva is NULL ",
					   NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, " and facturaabono.tipoIva is NULL ",
						    NULL);
	}
	g_free(gchar_where);
	g_free(gchar_where2);
	g_free(gchar_where_split_invoice);
	gchar_where = gchar_where_aux;
	gchar_where2 = gchar_where_aux2;
	gchar_where_split_invoice = gchar_where_aux_split_invoice;
	gchar_where_aux=NULL;
	gchar_where_aux2=NULL;
	gchar_where_aux_split_invoice=NULL;

	if ((condition->amount) != 0)  {
	    gdouble real_total;
	    gchar * temp;

	    real_total = u_round_to_decimal (condition->amount,2);
	    g_print(_("\nAmount: %.2f\n"),real_total);
	    temp = g_strdup_printf("%10.2f",real_total);
	    g_strstrip(temp);

	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.baseImponibleNueva=to_number('",temp,"','999999D99')",
					  ")",NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2, " and (facturaabono.baseImponibleNueva=to_number('",temp,"','999999D99')",
					   ")",NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice, "  and (facturaabono.baseImponibleNueva=to_number('",temp,"','999999D99')",
						    ")",NULL);

	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
	}

	if ((condition->payment_type)!= ULONG_MAX)  {
	    gchar * payment_condition;
	    if (condition->payment_type==D_PAYMENT_TRANSFER_CODE) {
		payment_condition = g_strconcat(" and (pago.esTransferencia=true) ",NULL);
	    } else if (condition->payment_type==D_PAYMENT_IN_CASH) {
		payment_condition = g_strconcat(" and (pago.esTransferencia=false)",
					       " and (pago.esDomiciliacion=false)",
					       " and (pago.esTarjeta=false)",
					       " and (pago.esTalon=false) ",
					       NULL);
	    } else if (condition->payment_type==D_PAYMENT_DIRECT_DEBIT_CODE) {
		payment_condition = g_strconcat(" and (pago.esDomiciliacion=true)",NULL);
	    } else if (condition->payment_type==D_PAYMENT_CREDIT_CARD) {
		payment_condition = g_strconcat(" and (pago.esTarjeta=true)",NULL);
	    } else if (condition->payment_type==D_PAYMENT_CHEQUE) {
		payment_condition = g_strconcat(" and (pago.esTalon=true)",NULL);
	    } else {
		payment_condition = g_strconcat(" ",NULL);
	    }
	    gchar_where_aux = g_strconcat(gchar_where,payment_condition,NULL);
	    gchar_where_aux2 = g_strconcat(gchar_where2,payment_condition,NULL);
	    gchar_where_aux_split_invoice = g_strconcat(gchar_where_split_invoice,payment_condition,NULL);
	    g_free(payment_condition);
	    g_free(gchar_where);
	    g_free(gchar_where2);
	    g_free(gchar_where_split_invoice);
	    gchar_where = gchar_where_aux;
	    gchar_where2 = gchar_where_aux2;
	    gchar_where_split_invoice = gchar_where_aux_split_invoice;
	    gchar_where_aux=NULL;
	    gchar_where_aux2=NULL;
	    gchar_where_aux_split_invoice=NULL;
	}

	gchar_where_garage = g_strconcat(gchar_where," and facturaabono.codigoFactura=facturasTalleres.numeroFacturaFactura and ",
					 " reparacion.codigoReparacion=facturasTalleres.codigoReparacionTaller and ",
					 " reparacion.codigoTienda = facturasTalleres.codigoTiendaAlbaranTaller and ",
                                         " reparacion.codigoEmpresa = facturasTalleres.codigoEmpresaAlbaranTaller",
					 NULL);

        gchar_where_aux = g_strconcat(gchar_where," and (facturaabono.codigoReparacion = reparacion.codigoReparacion)",
				      NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;

	if (condition->type==D_INVOICE_CODE) {
	    if (((condition->company_code) != ULONG_MAX) && ((condition->company_code) != 0)) {
		gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order,gchar_union,
					  gchar_select2,gchar_from2,gchar_where2,gchar_order,
					  NULL);
	    } else {
		gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order,gchar_union,
					  gchar_select2,gchar_from2,gchar_where2,gchar_order,gchar_union,
					  gchar_select_garage,gchar_from_garage,gchar_where_garage,gchar_group_garage,gchar_order,
					  NULL);
	    }
	} else if (condition->type==D_INVOICE_SPLIT_INVOICE_CODE) {
	    gchar_query = g_strconcat(gchar_select,gchar_from_split_invoice,gchar_from2_split_invoice,gchar_where_split_invoice,gchar_order,
				      NULL);

	}

	g_print(_("\nQuery: %s\n"),gchar_query);

        gchar_select_general=g_strconcat("SELECT * FROM (",gchar_query ,") as fEmi order by fEmi.nombreparticipantCompany,fEmi.numeroFacturaFactura ",NULL);
	m_model_command_execute (&record_set1, gchar_select_general, my_connection, my_command,tuples1,0, TRUE);



	if (record_set1!=NULL) {
	    if (*tuples1!=0) {
		invoices = (Invoice **) g_malloc0(sizeof(Invoice*)*(*tuples1+1));
		index = gda_recordset_move_first(record_set1);
		while (!gda_recordset_eof(record_set1)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    invoice = m_invoice_create (record_set1,NULL);
		    invoices[index_of_invoices] = invoice;
		    index = gda_recordset_move_next(record_set1);
		    index_of_invoices ++;
		}
	    } else {
		invoices=NULL;
	    }
	    gda_recordset_close(record_set1);
	} else {
	    invoices=NULL;
	}

	if (invoices!=NULL) {
	    invoices[index_of_invoices] = NULL;
	}


	g_free(gchar_where);
	g_free(gchar_where2);
	g_free(gchar_where_garage);
	g_free(gchar_select_garage);
	g_free(gchar_group_garage);
	g_free(gchar_select);
	g_free(gchar_select_general);
	g_free(gchar_select2);
	g_free(gchar_from);
	g_free(gchar_from2);
	g_free(gchar_from_garage);
	g_free(gchar_from_split_invoice);
	g_free(gchar_from2_split_invoice);
	g_free(gchar_where_split_invoice);
        g_free(gchar_union);
	g_free(gchar_order);
	g_free(gchar_query);
	g_free(tuples1);

    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return invoices;

}



/**
 * m_invoice_clear_array: 
 * @invoices: Array de facturas de la que se desea liberar la memoria.
 * 
 * Libera la memoria utilizada por el array de facturas.
 *
 */
void m_invoice_clear_array (Invoice ** invoices) {
    if (invoices!=NULL) {
	gint index = 0;
	while (invoices[index]!=NULL) {
	    d_invoice_clear (invoices[index]);
	    index++;
	}
    }
    g_free(invoices);
    invoices=NULL;
}

/**
 * m_invoice_update: 
 * @invoice: Factura que se modificar.
 * 
 * Modifica la factura que se pasa como parmetro en la base de datos.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_invoice_update (Invoice * invoice) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * invoice_number=NULL;
    gchar * shop_code=NULL;
    gchar * enterprise_code=NULL;
    gchar * issue_date_char=NULL;
    time_t  date=d_invoice_get_issue_date (invoice);
    gchar * client_code=NULL;
    gchar * invoice_discount=NULL;
    gchar * total_invoice=NULL;
    gchar * total_modified=NULL;
    gulong vat_type=d_invoice_get_vat_type (invoice);
    gchar * vat_char=NULL;
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    if (invoice !=NULL) {
	invoice_number = g_strdup_printf("%lu",d_invoice_get_invoice_code (invoice));
	enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))));
	shop_code =  g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice))));
	client_code = g_strdup_printf("%lu",d_participant_get_code (d_invoice_get_participant (invoice)));

	if (d_invoice_get_invoice_discount (invoice)>0) {
	    invoice_discount=g_strdup_printf("to_number('%.2f','999D99')",d_invoice_get_invoice_discount (invoice));
	} else {
	    invoice_discount=g_strdup("NULL");
	}
	total_invoice=g_strdup_printf("%.2f",1.0*d_invoice_get_total (invoice));
	if (d_invoice_get_modified_total (invoice)==FALSE) {
            total_modified = g_strdup("FALSE");
	} else {
            total_modified = g_strdup("TRUE");
	}

	if (vat_type!=0) {
            vat_char = g_strdup("NULL");
	} else {
            vat_char = g_strdup("0.00");
	}

	if (invoice_number!=NULL) {
	    gulong enterprise_participant_code = ULONG_MAX;
            gulong shop_participant_code = ULONG_MAX;

	    if (d_invoice_get_enterprise (invoice)!=NULL) {
		enterprise_participant_code = d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
	    }
	    if (d_invoice_get_shop (invoice)!=NULL) {
		shop_participant_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice)));
	    }

	    gda_command_set_connection(command,connection);
	    m_model_connection_begin_mutex_transaction_logged (connection,
						     enterprise_participant_code,
						     shop_participant_code,
                                                     ULONG_MAX
						    );

            issue_date_char = g_strdup(ctime(&date));
	    query = g_strconcat("update facturaabono set ",
                                "fechaEmision='",issue_date_char,
        			"',dtoFactura=",invoice_discount,
                                ",codigoPagador=",client_code,
				",baseImponibleNueva=to_number('",total_invoice,"','99999999D99')"
				",baseImponibleModificada=",total_modified,
                                ",tipoIva=",vat_char,
				" where (codigoFactura=",invoice_number,
				") and (codigoEmpresa=",enterprise_code,
				") and (codigoTienda=",shop_code,
				")",NULL);

	    g_free(issue_date_char);
            g_free(vat_char);
            g_free(client_code);
	    g_print(_("\nQuery: %s\n"),query);
	    result = m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);

            result = result && (m_payment_update_in_transaction (d_invoice_get_payment (invoice),connection,command));

	    if (d_invoice_get_state (invoice)==D_INVOICE_PAID) {
		if ((d_payment_get_is_transfer (d_invoice_get_payment (invoice))==FALSE) &&
		    (d_payment_get_is_cheque (d_invoice_get_payment (invoice))==FALSE) &&
		    (d_payment_get_is_direct_debit (d_invoice_get_payment (invoice))==FALSE) &&
		    (d_payment_get_is_credit_card (d_invoice_get_payment (invoice))==FALSE))
		{
		    result = result && m_cash_operation_update_in_transaction (d_payment_get_payment_code(d_invoice_get_payment (invoice)),
									    d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice))),
									    d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))),
									    d_invoice_get_total (invoice),
									    connection);

		} else {
		    result = result && m_operation_update_in_transaction (d_payment_get_payment_code(d_invoice_get_payment (invoice)),
	       						       d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice))),
								       d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))),
								       d_invoice_get_total (invoice),
								       connection);

		}
	    }

	    result = m_model_execute_mutex_modification_logged (connection) && result;


	}
	if (record_set!=NULL)
	    gda_recordset_free(record_set);

	g_free(query);
	g_free(tuples);
	g_free(invoice_discount);
	g_free(total_invoice);
        g_free(total_modified);
	g_free(enterprise_code);
	g_free(invoice_number);
	g_free(shop_code);

    }
	igalia_command_free(command);

    return result;
}

/**
 * insert_garage_invoice:
 * @invoice: Datos de la factura que se desea insertar.
 * @invoice_number: Nmero de la factura del taller.
 * @delivery_payment_index: ndice del albaran de abono que se procesar en este momento.
 * @connection: Conexin a la base de datos.
 * @command: Comando con el que se ejecutar la sentencia sql.
 *
 * Inserta los datos de una factura y un albarn de abono de la factura de un taller en la tabla
 * de facturas taller
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
static gboolean insert_garage_invoice(Invoice *invoice,gchar * invoice_number, gint delivery_payment_index,GdaConnection *connection, GdaCommand *command) {
    GdaRecordset *record_set1=NULL;
    gboolean result =TRUE;
    gchar * enterprise_code=NULL;
    gchar * shop_code=NULL;
    gchar * query1=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

    enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))));
    shop_code = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice))));

    if (d_invoice_get_delivery_payments (invoice)!=NULL) {
	if (d_delivery_payment_get_garage_total (invoice->delivery_payments[delivery_payment_index])>0) {
	    gulong delivery_payment_number=d_delivery_payment_get_number (invoice->delivery_payments[delivery_payment_index]);
	    gulong shop_delivery_payment_code=d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_delivery_payment_get_repair (invoice->delivery_payments[delivery_payment_index]))));
	    gulong enterprise_delivery_payment_code=d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (invoice->delivery_payments[delivery_payment_index]))));
	    gchar * delivery_payment_number_char=NULL;
	    gchar * shop_delivery_payment_code_char=NULL;
	    gchar * enterprise_delivery_payment_code_char=NULL;

	    delivery_payment_number_char=g_strdup_printf("%lu",delivery_payment_number);
	    shop_delivery_payment_code_char=g_strdup_printf("%lu",shop_delivery_payment_code);
	    enterprise_delivery_payment_code_char=g_strdup_printf("%lu",enterprise_delivery_payment_code);
	    query1 = g_strconcat("insert into facturataller values(",
				 enterprise_code,",",
				 shop_code,",",
				 invoice_number,",",
				 enterprise_delivery_payment_code_char,",",
				 shop_delivery_payment_code_char,",",
				 delivery_payment_number_char,
				 ")",NULL);
	    g_print(_("\nInsert: %s\n"),query1);
	    result = result && m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE);

	    if (enterprise_delivery_payment_code_char!=NULL) g_free(enterprise_delivery_payment_code_char);
	    if (shop_delivery_payment_code_char!=NULL) g_free(shop_delivery_payment_code_char);
	    if (delivery_payment_number_char!=NULL) g_free(delivery_payment_number_char);

	    if (record_set1!=NULL)
		gda_recordset_free(record_set1);
	    record_set1=NULL;
	    delivery_payment_index++;
	}
    }
    if (enterprise_code!=NULL) g_free(enterprise_code);
    if (shop_code!=NULL) g_free(shop_code);
    if (invoice_number!=NULL) g_free(invoice_number);
    if (query1!=NULL) g_free(query1);

    return result;
}

/**
 * m_invoice_get_number: 
 * @enterprise: Cdigo de empresa para la que se obtendr en siguiente nmero de factura.
 * @shop: Cdigo de la tienda para la que se obtendr en siguiente nmero de factura.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecutar la sentencia sql.
 * 
 * Como las facturas necesitan seguir un orden estrictamente correlativo, los nmeros
 * de factura no se implementan con secuencias sino con nmeros de una tabla de la base de datos
 * que se modifica en una transaccin y devuelve el siguiente nmero de factura (cuando se inserta en la
 * tabla de factura).
 *
 * Returns:
 * Siguiente nmero de factura.
 */
gulong  m_invoice_get_number (gchar * enterprise_code,gchar * shop_code, GdaConnection *connection, GdaCommand *command) {
    gchar * query5=NULL;
    gchar * query6=NULL;
    gchar * invoice_number_aux=NULL;
    GdaRecordset *record_set5=NULL;
    GdaRecordset *record_set6=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong result=ULONG_MAX;


    query5 = g_strconcat(" select numeroFactura from numeroscorrelativosFactura where codigoEmpresa=",enterprise_code,
			 " and codigoTienda=",shop_code,NULL);

    if (m_model_command_execute (&record_set5, query5, connection,command,tuples,0,TRUE)==FALSE) {

	result = ULONG_MAX;
    }

    if (record_set5!=NULL) {
	if (*tuples!=0) {
	    GdaField * field;
	    gda_recordset_move_first(record_set5);
	    field  = gda_recordset_field_idx(record_set5,0);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
		    invoice_number_aux = gda_stringify_value(NULL, 0, field);
		    result = strtoul(invoice_number_aux,stop,10);
		    g_free(stop);
		}
		gda_field_free(field);
	    }

	} else {
	    result = ULONG_MAX;
	}
    } else {
	result = ULONG_MAX;
    }

    if (result!=ULONG_MAX)
	result = result +1;

    if (invoice_number_aux!=NULL) {
	g_free(invoice_number_aux);
        invoice_number_aux=g_strdup_printf("%lu",result);
	query6 = g_strconcat(" update numerosCorrelativosFactura set numeroFactura=",invoice_number_aux,"where codigoEmpresa=",enterprise_code,
			     " and codigoTienda=",shop_code,NULL);
	if (m_model_command_execute (&record_set6, query6, connection, command,tuples,0, FALSE)==FALSE) {
	    result = ULONG_MAX;
	}
    }
    if (record_set5!=NULL) {
	gda_recordset_close(record_set5);
	gda_recordset_free(record_set5);
    }
    if (record_set6!=NULL) {
	gda_recordset_close(record_set5);
	gda_recordset_free(record_set6);
    }
    g_free(tuples);
    g_free(invoice_number_aux);
    return result;
}

/**
 * m_invoice_set_state_selected: 
 * @invoices: Array con una serie de facturas.
 * @selected: GArray con los ndices de las facturas que estn seleccionadas del array de facturas.
 * @old_state: Estado antiguo.
 * @new_state: Nuevo estado que se insertar.
 *
 * Esta funcin pasa de un estado a otro una serie de facturas seleccionadas de un array. Adems
 * de cambiar de estado introduce un movimiento en la base de datos, ya que esta funcin se utiliza
 * para dar como pagadas un grupo de facturas.
 *
 */
void m_invoice_set_state_selected  (Invoice ** invoices, GArray * selected, gint old_state, gint new_state) {
  gint i;
  gint index;

  for (i=0; i< selected->len; i++) {
    index  = g_array_index(selected,gint,i); 
    /* Index takes only the selected invoices, given an array of selected indexes */

    if ((d_invoice_get_state (invoices[index]) == old_state) ||
	((d_payment_get_is_hire_purchase (d_invoice_get_payment (invoices[index]))) &&
	 (d_payment_get_is_direct_debit (d_invoice_get_payment (invoices[index]))))){
	if (d_invoice_get_state (invoices[index])!=D_INVOICE_PAID) {
	    if (new_state == D_INVOICE_PAID) { /* We are supposing that the old_state is different than PAGADA, so we need to create a moviment  */
		gint shop_code;
		Enterprise * enterprise;
		Participant * company;
		GList * account_list;
		GList * element_account;
		gchar * description;

		Operation * operation = d_operation_create_empty ();

		shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index])));
		/* We need this because the Enterprise is not in memory (no included in de do_search result set) */
		enterprise = m_enterprise_find_by_shop (shop_code);
		company = d_enterprise_get_participant (enterprise);
		/* Obtaining the CCC for this client (included the calculation of the control code) */

		account_list = m_account_find_by_participant_code (d_participant_get_code (company));

		if (account_list == NULL) g_print(_("\nAccount list is null\n"));

		d_enterprise_clear (enterprise);
		element_account = g_list_first(account_list);

		d_operation_set_account (operation, (Account *) element_account->data);
		d_operation_set_payment (operation, d_invoice_get_payment (invoices[index]));
		if (d_payment_get_is_hire_purchase (d_invoice_get_payment (invoices[index]))==FALSE) {
		    d_operation_set_amount (operation, d_invoice_get_total (invoices[index]));
		} else {
		    PaymentId * payment_id = (PaymentId *)g_malloc0(sizeof(PaymentId));
		    payment_id->payment_code=g_strdup_printf("%lu",d_payment_get_payment_code (d_invoice_get_payment (invoices[index])));
		    payment_id->enterprise_code=g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (d_invoice_get_payment (invoices[index]))))
							    );
		    payment_id->shop_code=g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (d_invoice_get_payment (invoices[index]))))
							   );

		    d_operation_set_amount (operation,(m_payment_calculate_next_total (payment_id,TRUE)));
		    g_free(payment_id->enterprise_code);
		    g_free(payment_id->shop_code);
		    g_free(payment_id->payment_code);
		    g_free(payment_id);
		}

		if (d_payment_get_is_direct_debit (d_invoice_get_payment (invoices[index]))) {
		    description = N_("Direct debit using Cuaderno58");
		} else if ((d_invoice_get_state (invoices[index]))>D_INVOICE_NOT_PAID) {
		    description = N_("Automated payment by CK transfer");
		} else {
		    description = N_("Reinvoice Payment");
		}

		d_operation_set_section (operation, description);

		d_operation_set_date (operation, time(NULL));
		d_operation_set_notification_date (operation, time(NULL));

		g_print(_("(invoice_model.c) Calling to model_insert_operation\n"));
		m_operation_insert (operation);
		if (d_invoice_get_split_invoice (invoices[index])!=FALSE) {
		    d_operation_set_amount (operation, -d_invoice_get_total (invoices[index]));
		    m_operation_insert (operation);
		}

		d_account_free_glist (account_list); /* Was created by model_find_account_by_party_code */

		g_print(_("(invoice_model.c) Calling to update_state_invoice\n"));

		if ((d_payment_get_is_hire_purchase (d_invoice_get_payment (invoices[index]))!=FALSE) &&
		    ((d_payment_get_is_direct_debit (d_invoice_get_payment (invoices[index])))!=FALSE)) {
		    if (d_invoice_get_state (invoices[index])==D_INVOICE_MANAGING) {
			m_invoice_update_state (invoices[index], new_state);
			m_direct_debit_set_state_from_invoice (d_invoice_get_invoice_code (invoices[index]),
							    d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index]))),
							    d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index]))),D_DIRECT_DEBIT_PAID);
		    }
		} else {
		    m_invoice_update_state (invoices[index], new_state);
		    m_direct_debit_set_state_from_invoice (d_invoice_get_invoice_code (invoices[index]),
							d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index]))),
							d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index]))),D_DIRECT_DEBIT_PAID);

		}

	    }
	} else if ((d_invoice_get_state (invoices[index])==D_INVOICE_PAID) && (new_state==D_INVOICE_NOT_PAID)) {
	    Payment * payment;
	    gulong payment_code = d_payment_get_payment_code(d_invoice_get_payment (invoices[index]));
	    gulong enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index])));
	    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index])));
	    gint operation_type;
	    PaymentId * payment_id = (PaymentId *)g_malloc0(sizeof(PaymentId));
	    payment_id->payment_code=g_strdup_printf("%lu",payment_code);
	    payment_id->shop_code=g_strdup_printf("%lu",shop_code);
	    payment_id->enterprise_code=g_strdup_printf("%lu",enterprise_code);
	    payment = m_payment_find_by_id (payment_id);
	    g_free(payment_id->enterprise_code);
	    g_free(payment_id->shop_code);
	    g_free(payment_id->payment_code);
	    g_free(payment_id);
	    if ((d_payment_get_is_transfer (payment)==FALSE) &&
		(d_payment_get_is_cheque (payment)==FALSE) &&
		(d_payment_get_is_direct_debit (payment)==FALSE) &&
		(d_payment_get_is_credit_card (payment)==FALSE))
	    {
		operation_type = 1;
	    } else {
		operation_type = 0;
	    }
	    d_payment_clear (payment);
	    if (old_state==2)
		m_payment_delete (shop_code,enterprise_code,payment_code,operation_type);


	    if ((d_payment_get_is_hire_purchase (d_invoice_get_payment (invoices[index]))!=FALSE) &&
		((d_payment_get_is_direct_debit (d_invoice_get_payment (invoices[index])))!=FALSE)) {
		if (d_invoice_get_state (invoices[index])==D_INVOICE_MANAGING) {
		    m_invoice_update_state (invoices[index], new_state);
		    m_direct_debit_set_state_from_invoice (d_invoice_get_invoice_code (invoices[index]),
							d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index]))),
							d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index]))),D_DIRECT_DEBIT_NO_USED);
		}
	    } else {
		m_invoice_update_state (invoices[index], new_state);
		m_direct_debit_set_state_from_invoice (d_invoice_get_invoice_code (invoices[index]),
						    d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index]))),
						    d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index]))),D_DIRECT_DEBIT_NO_USED);

	    }

	}

    }

  }
}


/**
 * m_invoice_update_number: 
 * @enterprise_code: Cdigo de la empresa para la que se modificar el siguiente nmero de factura.
 * @shop_code: Cdigo de la tienda para la que se modificar el siguiente nmero de factura.
 * @invoice_number_char: Nmero de la factura que se isertar en la base de datos por encima de antiguo.
 * 
 * Esta funcin establece el nmero siguiente de factura para una tienda a un nmero deseado, ya que por ejemplo
 * sera interesante reetablecerlo a 0 a comienzo de cada ao.
 *
 * Returns:
 * %TRUE si se ha realizado la operacin, %FALSE en otro caso.
 */
gboolean m_invoice_update_number (gulong enterprise_code,gulong shop_code,gchar * invoice_number_char) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * shop_code_char=NULL;
    gchar * enterprise_code_char=NULL;
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong logged_enterprise_code=ULONG_MAX;

    enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    shop_code_char =  g_strdup_printf("%lu",shop_code);

    if (enterprise_code==0) {
        Shop * shop = m_shop_find_by_shop_code (shop_code);
	logged_enterprise_code = d_shop_get_enterprise_code (shop);
        d_shop_clear (shop);
    } else {
        logged_enterprise_code = enterprise_code;
    }

    gda_command_set_connection(command,connection);

    query = g_strconcat("update numerosCorrelativosFactura set numeroFactura=",invoice_number_char,
			"where (codigoEmpresa=",enterprise_code_char,
			") and (codigoTienda=",shop_code_char,
			")",NULL);
    result = m_model_mutex_transactional_command_execute_logged (&record_set,
							      query,
							      connection,
							      command, tuples,
							      0, FALSE,
							      logged_enterprise_code,
							      shop_code,
							      ULONG_MAX
							     );


    if (record_set!=NULL)
	gda_recordset_free(record_set);

    g_free(query);
    g_free(tuples);
    g_free(shop_code_char);
    g_free(enterprise_code_char);


    igalia_command_free(command);

    return result;
}

/**
 * m_invoice_find_last_payment_from_client: 
 * @invoice: Factura de la cual nos interesa extraer el cliente y consultar el ltimo pago.
 *
 * Busca el ltimo pago realizado por un cliente.
 * Funcin utilizada para asociar siempre a un cliente un tipo de pago desde el momento que
 * le emitimos una factura. As, si se desea siempre se podr recupar el tipo de pago sin necesidad
 * de tener que consultarlo en las facturas emitidas.
 *
 * Returns:
 * ltimo pago realizado por el cliente a la empresa.
 */
PaymentId * m_invoice_find_last_payment_from_client (Invoice * invoice) {
    gchar * query;
    gchar * payer_code_char=NULL;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    gulong payer_code=d_participant_get_code (d_invoice_get_participant (invoice));
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    PaymentId * result=NULL;


    gda_command_set_connection(command,connection);

    if (payer_code !=ULONG_MAX) {
        payer_code_char = g_strdup_printf("%lu",payer_code);
	query = g_strconcat(" select facturaabono.codigoTienda as tienda,",
			    " facturaabono.codigoEmpresa as empresa, ",
			    " facturaabono.codigoFactura as factura,",
			    " facturaabono.codigoPago as pago,fechaEmision from facturaabono,pago",
			    " where ",
			    " facturaabono.codigoPago = pago.codigoPAgo and ",
			    " facturaabono.codigoTienda=pago.codigoTienda ",
			    " and facturaabono.codigoEmpresa=pago.codigoEmpresa and ",
			    " facturaabono.codigoPAgador=",
                            payer_code_char,
                            " order by fechaEmision desc,codigoFactura desc ",
			    NULL);


	g_free(payer_code_char);

        g_print(_("\nQuery %s\n"),query);
	if (m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE)==FALSE) {
	    result = NULL;
	}

        g_free(query);
	if (record_set!=NULL) {
	    if (*tuples>=2) {
		GdaField * field;
		gda_recordset_move_first(record_set);
		gda_recordset_move_next(record_set);
		result = (PaymentId *) g_malloc0(sizeof(PaymentId));
		field  = gda_recordset_field_name(record_set,"tienda");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> shop_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field  = gda_recordset_field_name(record_set,"empresa");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> enterprise_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field  = gda_recordset_field_name(record_set,"pago");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			result -> payment_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}

	    } else {
		result = NULL;
	    }
	} else {
	    result = NULL;
	}

    }
    if (record_set!=NULL)
	gda_recordset_free(record_set);
    g_free(tuples);

    igalia_command_free(command);

    return result;

}

/**
 * m_invoice_get_state_from_direct_debit: 
 * @direct_debits: Domiciliaciones que contiene los identificadores de la factura y se han devuelto.
 * @connection: Conexin a la base de datos.
 *
 * Devuelve el estado de la factura a %D_INVOICE_NOT_PAID cuando es devuelta
 * en el cuaderno 58.
 *
 * Returns:
 * %TRUE si se ha realizado la operacin, %FALSE en otro caso.
 */
gboolean m_invoice_get_state_from_direct_debit (DirectDebit ** direct_debits,GdaConnection * connection) {
    gboolean result=TRUE;
    gchar * payment_code=NULL;
    GdaCommand *command  = gda_command_new();
    gchar * query = NULL;
    gchar * query_delete = NULL;
    gint index=0;
    GdaRecordset * record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

    gda_command_set_connection(command,connection);

    while (direct_debits[index]!=NULL) {
	gchar * invoice_code=g_strdup_printf("%lu",d_direct_debit_get_invoice_code (direct_debits[index]));
	gchar * enterprise_code=g_strdup_printf("%lu",d_direct_debit_get_enterprise_code (direct_debits[index]));
	gchar * shop_code=g_strdup_printf("%lu",d_direct_debit_get_shop_code (direct_debits[index]));
        Payment * payment;

	query = g_strconcat(" UPDATE FACTURAABONO SET ESTADO = 1 WHERE "
			    " CODIGOFACTURA=",invoice_code," AND CODIGOEMPRESA = ",
			    enterprise_code," AND CODIGOTIENDA= ",shop_code," AND ESTADO=3",NULL);
	result = result && m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE);

	if (result !=FALSE) {
	    payment = m_payment_find_by_invoice_id (d_direct_debit_get_invoice_code (direct_debits[index]),
						       d_direct_debit_get_shop_code (direct_debits[index]),
						       d_direct_debit_get_enterprise_code (direct_debits[index]));

	    if (d_payment_get_is_hire_purchase (payment)) {
		result = result && m_payment_get_instalment (&payment,connection);
	    }
	}
	
	if (record_set!=NULL) {
	    gda_recordset_free(record_set);
	    record_set=NULL;
	}
	if (invoice_code!=NULL)
            g_free(invoice_code);
	if (enterprise_code!=NULL)
            g_free(enterprise_code);
	if (shop_code!=NULL)
            g_free(shop_code);

        index++;
    }
    igalia_command_free(command);
    g_free(tuples);
    if (payment_code!=NULL)
        g_free(payment_code);
    if (query!=NULL)
        g_free(query);
    if (query_delete!=NULL)
	g_free(query_delete);

    return result;
}

/**
 * m_invoice_set_state_of_invoices:
 * @invoice_codes: Identificadores de las facturas.
 * @new_state: Nuevo estado de la factura
 *
 * Establece el estado de las facturas a #new_state
 *
 * Returns:
 * %TRUE si se ha realizado la operacin, %FALSE en otro caso.
 */
gboolean m_invoice_set_state_of_invoices (InvoiceId ** invoice_codes,gint new_state) {
    gboolean result=TRUE;
    GdaCommand *command  = gda_command_new();
    gchar * query = NULL;
    gint index=0;
    gchar * new_state_char = g_strdup_printf("%d",new_state);
    GdaRecordset * record_set=NULL;
    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaConnection *connection = m_model_get_connection ();


    gda_command_set_connection(command,connection);


    while (invoice_codes[index]!=NULL) {
	InvoiceId * invoice_id = invoice_codes[index];
	gulong  enterprise_code=ULONG_MAX;
	gulong  shop_code=ULONG_MAX;

        if (invoice_id->enterprise_code!=NULL)
	    enterprise_code=strtoul(invoice_id->enterprise_code,stop,10);
        if (invoice_id->shop_code!=NULL)
	    shop_code=strtoul(invoice_id->shop_code,stop,10);

        if ((invoice_id->shop_code!=NULL) || (invoice_id->enterprise_code!=NULL))
	    query = g_strconcat(" UPDATE FACTURAABONO SET ESTADO =", new_state_char," WHERE "
				" CODIGOFACTURA=",invoice_id->invoice_number," AND CODIGOEMPRESA = ",
				invoice_id->enterprise_code," AND CODIGOTIENDA= ",invoice_id->shop_code,NULL);
	else
	    query = g_strconcat(" UPDATE FACTURAABONO SET ESTADO =", new_state_char," WHERE "
				" CODIGOFACTURA=",invoice_id->invoice_number,NULL);

	g_print(_("\nUpdate: %s\n"),query);
	result = result && m_model_mutex_transactional_command_execute_logged (&record_set,
							      query,
							      connection,
							      command, tuples,
							      0, FALSE,
							      enterprise_code,
							      shop_code,
							      ULONG_MAX
							     );

	if (record_set!=NULL) {
	    gda_recordset_free(record_set);
	    record_set=NULL;
	}
        index++;
    }
    igalia_command_free(command);
    g_free(tuples);
    g_free(stop);
    if (query!=NULL)
        g_free(query);

    return result;
}

InvoiceId * m_invoice_get_split_invoice (gulong invoice_code,gulong shop_code,gulong enterprise_code) {
    gchar * query;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * invoice_code_char = g_strdup_printf("%lu",invoice_code);
    gchar * shop_code_char = g_strdup_printf("%lu",shop_code);
    gchar * enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    InvoiceId * result = NULL;
    gboolean no_error=TRUE;
    
    gda_command_set_connection(command,connection);

    query = g_strconcat(" SELECT NUMEROREFACTURA,CODIGOTIENDAREFACTURA,CODIGOEMPRESAREFACTURA FROM",
			" REFACTURACION WHERE NUMEROFACTURAABONO=",invoice_code_char,
			" AND CODIGOEMPRESAABONO=",enterprise_code_char,
			" AND CODIGOTIENDAABONO=",shop_code_char,NULL);

    no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE);

	if (record_set!=NULL) {
	    if (GDA_IS_RECORDSET(record_set)) {
		if (*tuples!=0) {
		    GdaField * field;
		    result = (InvoiceId *) g_malloc0(sizeof(InvoiceId));
		    gda_recordset_move_first(record_set);
		    gda_recordset_move_next(record_set);
		    field  = gda_recordset_field_name(record_set,"numeroRefactura");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    result -> invoice_number = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
			}
			gda_field_free(field);
		    }
		    field  = gda_recordset_field_name(record_set,"codigoempresaRefactura");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    result -> enterprise_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
			}
			gda_field_free(field);
		    }
		    field  = gda_recordset_field_name(record_set,"codigoTiendaRefactura");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    result -> shop_code = g_strdup_printf("%lu",igalia_field_get_bigint_value (field));
			}
			gda_field_free(field);
		    }
		}
		gda_recordset_free(record_set);

	    }
	}

	igalia_command_free(command);
	g_free(tuples);
	g_free(invoice_code_char);
	g_free(enterprise_code_char);
	g_free(shop_code_char);
	if (query!=NULL)
	    g_free(query);

	return result;
}

/**
 * m_invoice_delete:
 * @invoice: Factura a borrar.
 *
 * Borra una factura en la base de datos.
 *
 * Returns:
 * %NULL si se ha borrado correctamente, un mensaje de error en otro caso.
 */
gchar *m_invoice_delete (Invoice * invoice) {
    gchar * query;
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand *command  = gda_command_new();
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong invoice_code = d_invoice_get_invoice_code (invoice);
    gulong enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice)));
    gulong payment_code = d_payment_get_payment_code(d_invoice_get_payment (invoice));
    gchar * payment_code_char = NULL; 
    gchar * invoice_code_char = NULL;
    gchar * shop_code_char = NULL;
    gchar * enterprise_code_char = NULL;
    gchar * split_invoice_number = NULL;
    gchar * shop_split_invoice_code = NULL;
    gchar * enterprise_split_invoice_code = NULL;
    gchar * payment_split_invoice_code = NULL;
    gboolean split_invoice=FALSE;
    gchar * result = NULL;
    gint invoice_state=D_INVOICE_PAID;
    gint split_invoice_state=D_INVOICE_PAID;
    gboolean no_error=TRUE;

    gda_command_set_connection(command,connection);

    invoice_code_char = g_strdup_printf("%lu",invoice_code);
    enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    shop_code_char = g_strdup_printf("%lu",shop_code);
    payment_code_char = g_strdup_printf("%lu",payment_code);

    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);

    query = g_strconcat(" select estado from facturaabono ",
			" WHERE CODIGOFACTURA=",invoice_code_char,
			" AND CODIGOEMPRESA=",enterprise_code_char,
			" AND CODIGOTIENDA=",shop_code_char,NULL);
    no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE);
    g_print(_("\nInvoice Deleting. State: %s\n"),query);

    if (no_error!=FALSE) {
	if (record_set!=NULL) {
	    if (GDA_IS_RECORDSET(record_set)) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set);
		    field  = gda_recordset_field_name(record_set,"estado");
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    invoice_state= gda_field_get_smallint_value(field);;
			}
		    }
		}
		gda_recordset_free(record_set);
		record_set=NULL;
	    }
	}
    }
    if (query!=NULL)
	g_free(query);
    query=NULL;

    if (invoice_state>=D_INVOICE_PAID) {
	result = g_strdup(_("Paid or managing invoice.\nPaid and managing invoices can't be deleted."));
    } 

    if (result==NULL) {
	/* Comprueba si la factura forma parte de un proceso de refacturacin, en cuyo caso solo
	 se podr borrar si no hay ninguna pagada, y en consecuencia se borrarn las dos facturas */
	query = g_strconcat(" select refacturacion.numeroRefactura, ",
			    " refacturacion.codigoTiendaRefactura as codigoTienda,",
			    " refacturacion.codigoEmpresaRefactura as codigoEmpresa,",
			    " factura.estado as estadoFactura, ",
			    " refactura.estado as estadoRefactura,pago.codigoPago as codigoPagoRefactura ",
			    " from refacturacion,facturaabono factura,facturaabono refactura,pago ",
			    " where factura.codigoFactura=refacturacion.numeroFacturaabono and ",
			    " factura.codigoTienda=refacturacion.codigoTiendaAbono and ",
			    " refactura.codigofactura=refacturacion.numeroRefactura and ",
			    " refactura.codigoTienda=refacturacion.codigoTiendaRefactura and ",
			    " refactura.codigoPago = pago.codigoPago and",
			    " refactura.codigoTienda = pago.codigoTienda and",
			    " refactura.codigoEmpresa = pago.codigoEmpresa and",
			    " (numeroFacturaAbono = ",invoice_code_char,
			    ") and (codigoEmpresaAbono = ",enterprise_code_char,
			    ") and (codigoTiendaAbono = ",shop_code_char,")",
			    " union ",
			    " select refacturacion.numeroFacturaAbono, ",
			    " refacturacion.codigoTiendaAbono as codigoTienda,",
			    " refacturacion.codigoEmpresaAbono as codigoEmpresa,",
			    " factura.estado as estadoFactura, ",
			    " refactura.estado as estadoRefactura,pago.codigoPago as codigoPagoRefactura ",
			    " from refacturacion,facturaabono factura,facturaabono refactura,pago ",
			    " where factura.codigoFactura=refacturacion.numeroFacturaabono and ",
			    " factura.codigoTienda=refacturacion.codigoTiendaAbono and ",
			    " factura.codigoPago = pago.codigoPago and",
			    " factura.codigoTienda = pago.codigoTienda and",
			    " factura.codigoEmpresa = pago.codigoEmpresa and",
			    " refactura.codigofactura=refacturacion.numeroRefactura and ",
			    " refactura.codigoTienda=refacturacion.codigoTiendaRefactura and ",
			    " (numeroRefactura = ",invoice_code_char,
			    ") and (codigoEmpresaRefactura = ",enterprise_code_char,
			    ") and (codigoTiendaRefactura = ",shop_code_char,")",
			    NULL
			   );
    g_print(_("\nQuery: %s\n"),query);

	no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE);
	if (query!=NULL)
	    g_free(query);
	query=NULL;

	if (no_error!=FALSE) {
	    if (record_set!=NULL) {
		if (GDA_IS_RECORDSET(record_set)) {
		    if (*tuples!=0) {
			GdaField * field;
			gchar * temp;
			gda_recordset_move_first(record_set);
			field  = gda_recordset_field_name(record_set,"numeroFacturaAbono");
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				temp=gda_stringify_value(NULL, 0, field);
				split_invoice_number = g_strdup(temp);
				g_free(temp);
				gda_field_free(field);

			    }
			    gda_field_free(field);
			}
			field  = gda_recordset_field_name(record_set,"numeroRefactura");
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				split_invoice=TRUE;
				temp=gda_stringify_value(NULL, 0, field);
				split_invoice_number = g_strdup(temp);
				g_free(temp);
			    }
			    gda_field_free(field);
			}

			field  = gda_recordset_field_name(record_set,"estadoRefactura");
			if (field!=NULL) {
			    if (!gda_field_is_null(field)) {
				split_invoice_state= gda_field_get_smallint_value(field);;
			    }
			}

			if ((invoice_state<=D_INVOICE_NOT_PAID) && (split_invoice_state<=D_INVOICE_NOT_PAID)) {
			    field  = gda_recordset_field_name(record_set,"codigoTienda");
			    if (field!=NULL) {
				if (!gda_field_is_null(field)) {
				    temp=gda_stringify_value(NULL, 0, field);
				    shop_split_invoice_code = g_strdup(temp);
				    g_free(temp);
			    }
				gda_field_free(field);
			    }
			    field  = gda_recordset_field_name(record_set,"codigoEmpresa");
			    if (field!=NULL) {
				if (!gda_field_is_null(field)) {
				    temp=gda_stringify_value(NULL, 0, field);
				    enterprise_split_invoice_code = g_strdup(temp);
				    g_free(temp);
				}
				gda_field_free(field);
			    }
			    field  = gda_recordset_field_name(record_set,"codigoPagoRefactura");
			    if (field!=NULL) {
				if (!gda_field_is_null(field)) {
				    temp=gda_stringify_value(NULL, 0, field);
				    payment_split_invoice_code = g_strdup(temp);
				    g_free(temp);
				}
				gda_field_free(field);
			    }

			} else {
			    if (result == NULL)
				result = g_strdup(_("Invoice or reinvoice associated is paid or in managing."));
			    else {
				gchar * temp;
				temp = g_strconcat(result,"\n",_("Invoice or reinvoice associated is paid or in managing."),NULL);
				g_free(result);
				result = temp;
			    }
			}


		    }
		    gda_recordset_free(record_set);
                    record_set=NULL;
		}
	    }
	    if (result == NULL) {

	    }
	} else {
	    if (result == NULL)
		result = g_strdup(_("Error checking if it's a reinvoice."));
	    else {
		gchar * temp;
		temp = g_strconcat(result,"\n",_("Error checking if it's a reinvoice."),NULL);
		g_free(result);
		result = temp;
	    }
	}




	if (result == NULL) {
	    query = g_strconcat(" UPDATE ALBARANABONO set numeroFactura=null ",
				" WHERE NUMEROFACTURA=",invoice_code_char,
				" AND CODIGOEMPRESA=",enterprise_code_char,
				" AND CODIGOTIENDA=",shop_code_char,NULL);
	    no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);

	    g_print(_("\nDeleting Invoices. Delivery payment invoice: %s\n"),query);

	    if (query!=NULL)
		g_free(query);
	    query=NULL;

	    gda_recordset_free(record_set);
	    record_set=NULL;

	    if (no_error!=FALSE) {
		if (split_invoice_number!=NULL) {
		    query = g_strconcat(" UPDATE ALBARANABONO set numeroFactura=null ",
					" WHERE NUMEROFACTURA=",split_invoice_number,
					" AND CODIGOEMPRESA=",enterprise_split_invoice_code,
					" AND CODIGOTIENDA=",shop_split_invoice_code,NULL);
		    no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);
		    g_print(_("\nDeleting Invoices. Delivery Payment Reinvoice: %s\n"),query);

		    if (query!=NULL)
			g_free(query);
		    query=NULL;

		    gda_recordset_free(record_set);
		    record_set=NULL;
		}
		if (no_error==FALSE)
		    result = g_strdup(_("Error deleting delivery payments."));
	    } else {
		result = g_strdup(_("Error deleting delivery payments."));
	    }

	    if (result == NULL) {
		query = g_strconcat(" DELETE FROM PAGO ",
				    " WHERE CODIGOPAGO=",payment_code_char,
				    " AND CODIGOEMPRESA=",enterprise_code_char,
				    " AND CODIGOTIENDA=",shop_code_char,NULL);

		no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);
		g_print(_("\nDeleting invoices. Invoice Payment: %s\n"),query);

		if (query!=NULL)
		    g_free(query);
		query=NULL;

		gda_recordset_free(record_set);
		record_set=NULL;

		if (split_invoice_number!=NULL) {
		    query = g_strconcat(" DELETE FROM PAGO ",
					" WHERE CODIGOPAGO=",payment_split_invoice_code,
					" AND CODIGOEMPRESA=",enterprise_split_invoice_code,
					" AND CODIGOTIENDA=",shop_split_invoice_code,NULL);

		    g_print(_("\nDeleting invoices. Reinvoice Payment: %s\n"),query);
		    no_error = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);

		    if (query!=NULL)
			g_free(query);
		    query=NULL;

		    gda_recordset_free(record_set);
		    record_set=NULL;

		}
	    }
	}
    }
    m_model_execute_mutex_modification_logged (connection);

    if (query!=NULL)
	g_free(query);
    query=NULL;
    g_free(tuples);

    if (invoice_code_char!=NULL)
	g_free(invoice_code_char);
    if (enterprise_code_char!=NULL)
	g_free(enterprise_code_char);
    if (shop_code_char!=NULL)
	g_free(shop_code_char);
    if (payment_code_char!=NULL)
	g_free(payment_code_char);
    if (split_invoice_number!=NULL)
	g_free(split_invoice_number);
    if (shop_split_invoice_code!=NULL)
	g_free(shop_split_invoice_code);
    if (enterprise_split_invoice_code!=NULL)
	g_free(enterprise_split_invoice_code);
    if (payment_split_invoice_code!=NULL)
	g_free(payment_split_invoice_code);

    return result;

}



/**
 * m_invoice_get_shop_code_as_gulnog:
 * @invoice: Factura de la que se quiere saber el cdigo.
 *
 * Determina el cdigo de la tienda a la que pertenece la factura pasada como parmetro.
 *
 * Returns: El cdigo de tienda como un tipo 'gulong'.
 * %ULONG_MAX  si no hay cdigo asociado
 */
gulong m_invoice_get_shop_code_as_gulnog (Invoice *invoice) {
			
	if (invoice_validate_shop_code(invoice)) {
		
		Shop *shop = d_invoice_get_shop (invoice);
		Participant *part = d_shop_get_participant (shop);

		return d_participant_get_code (part);
	}
	
	return ULONG_MAX;
}

/**
 * m_invoice_get_enterprise_code_as_gulong:
 * @invoice: Factura de la que se quiere saber el cdigo.
 *
 * Determina el cdigo de la empresa a la que pertenece la factura pasada como parmetro.
 *
 * Returns : El cdigo de empresa como un tipo 'gulong'.
 * %ULONG_MAX  si no hay cdigo asociado
 */
gulong m_invoice_get_enterprise_code_as_gulong (Invoice *invoice) {

	if (invoice_validate_enterprise_code(invoice)) {
		
		Enterprise *enterprise = d_invoice_get_enterprise (invoice);
		Participant *part = d_enterprise_get_participant (enterprise);

		return d_participant_get_code (part);
	}
	
	return ULONG_MAX;	
}


/**
 * m_invoice_get_shop_code_as_gchar:
 * @invoice: Factura de la que se quiere saber el cdigo.
 *
 * Determina el cdigo de la tienda a la que pertenece la factura pasada como parmetro.
 *
 * Returns : El cdigo de tienda como un tipo 'gchar'. 
 * %NULL si no hay cdigo asociado
 */
gchar *m_invoice_get_shop_code_as_gchar (Invoice *invoice) {
		
	if (invoice_validate_shop_code(invoice)) {
			
		Shop *shop = d_invoice_get_shop (invoice);
		Participant *part = d_shop_get_participant (shop);

		return g_strdup_printf("%lu" , d_participant_get_code (part));
	}
	
	return NULL;
}

/**
 * m_invoice_get_enterprise_code_as_gchar:
 * @invoice: Factura de la que se quiere saber el cdigo.
 *
 * Determina el cdigo de la empresa a la que pertenece la factura pasada como parmetro.
 *
 * Returns: El cdigo de empresa como un tipo 'gchar'.
 * %NULL si no hay cdigo asociado
 */
gchar *m_invoice_get_enterprise_code_as_gchar (Invoice *invoice) {

	if (invoice_validate_enterprise_code(invoice)) {
			
		Enterprise *enterprise = d_invoice_get_enterprise (invoice);
		Participant *part = d_enterprise_get_participant (enterprise);
		
		return g_strdup_printf("%lu" , d_participant_get_code (part));
	}
	
	return NULL;
}

/**
 * invoice_validate_shop_code:
 * @invoice: Factura de la que se desea validar
 *
 * Determina si la factura tiene una tienda asociada.
 *
 * Returns: 
 * %FALSE si no hay ninguna tienda asociada.
 */
static gboolean invoice_validate_shop_code(Invoice *invoice) {
	
	Shop * shop = d_invoice_get_shop (invoice);
	if ((shop != NULL) && (d_shop_get_participant (shop) != NULL)) return TRUE;	
	

	return FALSE;
}
	
	
/**
 * invoice_invoice_validate_enterprise_code:
 * @invoice: Factura de la que se desea validar
 *
 * Determina si la factura tiene una empresa asociada.
 *
 * Returns: 
 * %FALSE si no hay ninguna empresa asociada.
 */
static gboolean invoice_validate_enterprise_code(Invoice *invoice) {
	
	Enterprise *enterprise = d_invoice_get_enterprise (invoice);
	if ((enterprise != NULL) && (d_enterprise_get_participant (enterprise) != NULL)) return TRUE;	
	
	return FALSE;
}
	
/**
 * get_invoice_number_as_gchar_from_invoice.
 * @invoice: Factura de la que se quiere saber el cdigo.
 *
 * Determina el codifgo de la factura pasada como parmetro.
 *
 * Returns: El cdigo de factura como un tipo 'gchar'.
 * %NULL si no hay cdigo asociado
 */
gchar *m_invoice_get_number_as_gchar (Invoice *invoice) {

	if (d_invoice_get_invoice_code (invoice) != ULONG_MAX) {
			
		return g_strdup_printf("%lu" , d_invoice_get_invoice_code (invoice));
	}
	
	return NULL;
}

	
/**
 * m_invoice_get_id:
 * @invoice: Factura de la que se quiere saber el identificador.
 *
 * Determina el identificador de la factura en la base de datos.
 *
 * Returns: El identificador de la factura como tipo %InvoiceId.
 * %NULL si no hay cdigo asociado
 */
InvoiceId * m_invoice_get_id (Invoice *invoice) {

	InvoiceId *invoice_id = NULL;
	gchar *enterprise_code 	= m_invoice_get_enterprise_code_as_gchar (invoice);
	gchar *shop_code 	= m_invoice_get_shop_code_as_gchar (invoice);
	gchar *invoice_code 	= m_invoice_get_number_as_gchar (invoice);	
	
	if ((enterprise_code != NULL) && (enterprise_code != NULL) && (invoice_code != NULL)) {
	
		invoice_id = (InvoiceId *) g_malloc(sizeof(InvoiceId));
		invoice_id->invoice_number = g_strdup(invoice_code);
		invoice_id->shop_code = g_strdup(shop_code);
		invoice_id->enterprise_code = g_strdup(enterprise_code);
	}
		
	g_free(enterprise_code);
	g_free(shop_code);
	g_free(invoice_code);

	return invoice_id;
}
	
/**
 * m_invoice_modify_subtotal:
 * @invoice: Factura.
 * @new_subtotal: Nuevo modulo pactado.
 * @with_discount: Si se aplica el descuento
 *
 * Determina el identificador de la factura en la base de datos.
 *
 * Returns: El identificador de la factura como tipo %InvoiceId.
 * %NULL si no hay cdigo asociado
 */
gdouble m_invoice_modify_subtotal (Invoice * invoice,gdouble new_subtotal,gboolean with_discount) {
    gdouble result = G_MINDOUBLE;
    /*DIFERENTES CASOS*/
    if (with_discount!=FALSE) {
	/*HAI COMPAA DE SEGUROS: Establece el valor de la base imponible al
	 *introducida por el usuario, sin tener en cuenta el descuento de la factura*/
	result = new_subtotal*1.16;
    } else {
	gdouble discount_invoice = (d_invoice_get_invoice_discount (invoice) / 100.0);
	/*NO HAY COMPAA DE SEGUROS: Calcula el descuento y calcula el total de la factura*/

	result = (((new_subtotal)-(new_subtotal*discount_invoice))*1.16);

    }
    return result;
}

/**
 * m_invoice_get_data_to_delivery_payment_line:
 * @description: Descripcin para las lneas de comisin por albarn.
 * @vat: Iva para las lneas de comisin por albarn.
 * @quantity: Unidades para las lneas de comisin por albarn.
 *
 * Devuelve los valores por defecto para las lneas de factura para las lneas de albaranes de comisin a
 * talleres.
 *
 */
void m_invoice_get_data_to_delivery_payment_line(gchar ** description,gchar ** vat,gchar ** quantity) {
    gfloat ivat=16.00;
    gfloat iquantity=1.00;
    *description = g_strdup(N_("PAYMENT VEHICLE REF. DELIVERY PAYMENT:"));
    *vat = g_strdup_printf("%.2f",ivat);
    *quantity = g_strdup_printf("%.2f",iquantity);
}


