/* measure_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "measure_model.h"
#include "warehouse_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "parameter_manager.h"

#include "util.h"

#define TABLE         "Medida"
#define IDENTIFICATOR "codigomedida"
#define ATTRIBUTES    "codigomedida as codigomeasure, \
                       nombre as nombremeasure, \
                       abreviatura as abreviaturameasure"

Measure ** measures_list = NULL;
gint length_of_list;


/**
 * m_measure_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_measure_get_table () {
    return TABLE;
}

/**
 * m_measure_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_measure_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_measure_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Measure con los campos rellenados con los datos de la base de datos.
 */
Measure * m_measure_create (GdaRecordset *rs) {
    GdaField * field;
    Measure * measure = d_measure_create_empty();


    field = gda_recordset_field_name(rs,"codigomeasure");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    measure->measure_code = gda_field_get_integer_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"nombremeasure");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp = gda_stringify_value(NULL, 0, field);
	    measure->measure_name = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs,"abreviaturameasure");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp = gda_stringify_value(NULL, 0, field);
	    measure->abbreviation = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }


    return measure;
}


/**
 * m_meassure_find_by_code:
 * @code: Cdigo de la medida.
 *
 * Busca una medida por cdigo de medida.
 *
 * Returns:
 * Medida que cumple la condicin.
 */
Measure * m_meassure_find_by_code (gint code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Measure * measure;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",code);

    gchar_query = g_strconcat("SELECT ",
			       " codigoMedida as codigomeasure, ",
			       " nombre as nombremeasure, ",
			       " abreviatura as abreviaturameasure ",
			      " FROM medida ",
			      " WHERE (codigoMedida=",aux,")",NULL);


    g_free(aux);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    g_free(gchar_query);
    
    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if ((*tuples)==1) {
   	    gda_recordset_move_first(record_set);
	    measure = m_measure_create (record_set);
	} else {
	    measure = NULL;
	}
        gda_recordset_close(record_set);
    } else {
        measure = NULL;
    }

    igalia_command_free(my_command);

    g_free(tuples);
    g_free(column);

    return measure;
}

/**
 * m_meassure_find_all:
 *
 * Busca todos las medidas en la base de datos. Si ya se han buscado alguna vez devuelve
 * la variable global que las almacena, en otro caso, realiza la busqueda.
 *
 * Returns:
 * Array con todas medidas.
 */
Measure ** m_meassure_find_all () {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Measure * measure;
    gchar * gchar_query;
    gulong * tuples;
    gint * column;
    gint index;


    if (measures_list==NULL) {
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

	gchar_query = g_strconcat("SELECT ",
				  " codigoMedida as codigomeasure, ",
				  " nombre as nombremeasure, ",
				  " abreviatura as abreviaturameasure ",
				  " FROM medida ",NULL);

	g_print(_("\nQuery: %s\n"),gchar_query);
	tuples = (gulong *) g_malloc0(sizeof(gulong));
	m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
	g_free(gchar_query);

        length_of_list = *tuples;
	column = (gint *) g_malloc0(sizeof(gint));

	if (record_set!=NULL) {
	    if (*tuples!=0) {
		measures_list = (Measure **) g_malloc0(sizeof(Measure*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    (*column)=0;
		    measure = m_measure_create (record_set);
		    measures_list[index-1] = measure;
		    index = gda_recordset_move_next(record_set);
		}
		measures_list[length_of_list]=NULL;

	    } else {
		measures_list=NULL;
	    }
	} else {
	    measures_list=NULL;
	}

	gda_recordset_free(record_set);
	igalia_command_free(my_command);

	g_free(tuples);
	g_free(column);


    }
    return measures_list;
}

/**
 * m_meassure_get_name_from_code:
 * @code: Cdigo de la medida a buscar.
 *
 * Busca el nombre de la medida con el cdigo %code.
 *
 * Returns:
 * Nombre de la medida si el cdigo es vlido, %NULL en otro caso.
 */
gchar *m_meassure_get_name_from_code (gulong code) {
    gchar *name = NULL;
    gint   index=0;

    if (measures_list==NULL) {
	m_meassure_find_all ();
    }

    if (measures_list!=NULL) {
      while(measures_list[index]!=NULL) {
	if (code == d_measure_get_measure_code (measures_list[index])) {
	  name = d_measure_get_name (measures_list[index]);
	}
        index++;
      }
    }

    return name;
}

/**
 * m_meassure_abbreviation_from_code:
 * @code: Cdigo de la medida a buscar.
 *
 * Busca el nombre de la medida con el cdigo %code.
 *
 * Returns:
 * Nombre de la medida si el cdigo es vlido, %NULL en otro caso.
 */
gchar *m_meassure_abbreviation_from_code (gulong code) {
    gchar *abbreviation = NULL;
    gint   index=0;

    if (measures_list==NULL) {
	m_meassure_find_all ();
    }
    
    if (measures_list!=NULL) {
      while(measures_list[index]!=NULL) {
	if (code == d_measure_get_measure_code (measures_list[index])) {
	  abbreviation = d_measure_get_abbreviation (measures_list[index]);
	}
        index++;
      }
    }

    return abbreviation;
}



/**
 * m_meassure_get_code_from_name:
 * @name: Nombre de la medida a buscar.
 *
 * Busca el cdigo de la medida con el nombre %name.
 *
 * Returns:
 * Cdigo de la medida si el nombre es vlido, %NULL en otro caso.
 */
gulong m_meassure_get_code_from_name (gchar *name) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (measures_list==NULL) {
	m_meassure_find_all ();
    }

    if (measures_list!=NULL) {
      while(measures_list[index]!=NULL) {
	temp = d_measure_get_name (measures_list[index]);
	if (!strcmp(name, temp)) {
	  code = d_measure_get_measure_code (measures_list[index]);
	  g_free(temp);
	  break;
	}
        g_free(temp);
        index++;
      }
    }

    return code;
}



/**
 * m_meassure_clear_array:
 * @measures: Array de medidas a liberar.
 *
 * Libera la memoria utilizada por una array de medidas.
 *
 */
void m_meassure_clear_array (Measure ** measures) {
    if (measures!=NULL) {
	gint index = 0;
	while (measures[index]!=NULL) {
	    d_measure_clear (measures[index]);
	    index++;
	}
    }
    g_free(measures);
    measures  = NULL;
}

/**
 * m_meassure_reload:
 *
 * Recarga en memoria las medidas existentes en la base de datos.
 *
 * Returns:
 * La lista con todas las medidas.
 */
Measure ** m_meassure_reload (void) {
    m_meassure_clear_array (measures_list);
    measures_list=NULL;
    m_meassure_find_all ();
    return measures_list;
}

/**
 * m_meassure_insert:
 * @measure: Medida (metros,kilogramos,...) a insertar
 *
 * Inserta una medida.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_meassure_insert (Measure * measure) {

    gboolean result = TRUE;
    GdaCommand *command = gda_command_new();
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset *record_set = NULL;
    gchar * name=NULL;
    gchar * temp_name=NULL;
    gchar * abbreviation=NULL;
    gchar * code=NULL;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gulong logged_shop_code;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * query=NULL;
    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command, connection);


    name = d_measure_get_name (measure);

    abbreviation = d_measure_get_abbreviation (measure);

    if ((name!=NULL) && (abbreviation!=NULL)) {
	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);
	if (d_measure_get_measure_code (measure)==ULONG_MAX) {
	    code = m_model_nextval_sequence ("seqMedida",connection);
	    if (code!=NULL) {
		query = g_strconcat("INSERT INTO Medida VALUES (",
				    code,",'",
				    name,"','",
				    abbreviation,
				    "')",
				    NULL);
		g_free(code);
	    } else {
		result =FALSE;
	    }
	} else {

	    query = g_strconcat("UPDATE Medida SET ",
				" nombre = '",name,
				"' ,abreviatura = '",abbreviation,
				"' WHERE",
				" codigoMEdida = ", code,
				NULL);

	}
	g_free(name);
	g_free(abbreviation);

	result = result && m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);

	g_print(_("\nInsert: %s\n"),query);

	result = m_model_execute_mutex_modification_logged (connection) && result;

    } else {
        result = FALSE;
    }




    g_free(tuples);



    if (record_set!=NULL)
	gda_recordset_free(record_set);
    if (query!=NULL)
	g_free(query);

    igalia_command_free(command);

    return result;

}
