
/*  model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

//#include <gda-client.h>

#include "config.h"
#include "model.h"
#include "gda_igalia_util.h"
#include "util.h"
#include "parameter_manager.h"
#include "time.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>


//GdaConnectionPool * connection_pool;

GdaConnection *connection=NULL;

static void print_log(const gchar *query);
static gchar *get_identificator(gchar *enterprise, gchar *shop, gchar *warehouse);
static gboolean execute_modification(GdaConnection * connection);
static void model_connection_begin_transaction(GdaConnection *cnc);
static gchar *get_path(void);
static gchar *get_log_filename(void);


/**
 * m_model_list_tables:
 * @rs: Record set que se desea listar.
 * 
 * Lista las tuplas de un record set.
 *
 */
void m_model_list_tables (GdaRecordset * rs) {
    GdaField *field;
    gint i;

    for (gda_recordset_move_first(rs); !gda_recordset_eof(rs);
	 gda_recordset_move_next(rs)) {
	for (i = 0; i < gda_recordset_rowsize(rs); i++) {
	    field = gda_recordset_field_idx(rs, i);
	    g_print("%s=%s\t", gda_field_get_name(field),gda_stringify_value(NULL, 0, field));
	}
	g_print("\n");
    }
    gda_recordset_free(rs);
}


/**
 * m_model_list_providers:
 * 
 * Lista los proveedores de GDA. Son necesarios para poder conectarse a la
 * base de datos que deseamos.
 *
 * Returns:
 * Nombre del provider. En nuestro caso nos interesa el de Postgres
 */
gchar* m_model_list_providers () {
    GList* list;
    GList* node;
    GdaProvider* provider;
    int i = 0;
    gchar* selected = NULL;

    list = gda_provider_list ();

    if (!list)  {
	g_print (_("\n*** Error ***\n"));
	exit (1);
    }

    g_print (_("\nThe next %d GDA provider is avaliable : \n"),
	     g_list_length (list));

    for (node = g_list_first (list); node; node = g_list_next (node)) {
	provider = (GdaProvider*) node->data;
	g_print ("%d: %s\n", ++i, GDA_PROVIDER_NAME (provider));

	//		if (i == 2)
	if (!g_strcasecmp(GDA_PROVIDER_NAME(provider),"OAFIID:GNOME_GDA_Provider_Postgres_Connection"))
	    selected = g_strdup (GDA_PROVIDER_NAME (provider));
    }
    gda_provider_free_list (list);
    return (selected);
}


/**
 * model_init 
 * @argc: Nmero de parmetros de entrada.
 * @argv: Parmetros de entrada.
 * 
 * Reinicia el servidor gda.
 *
 * Returns: 0 si sale correctamente.
 */
gint m_model_init (int argc, char *argv[])
{

    u_kill_gda ();
    gda_init (_("Fisterra: Open Source ERP"), NULL, argc, argv);

    //  gnome_init ("Fisterra", "0.0.1", argc, argv);
    return 0;

}

/**
 * m_model_get_connection:
 *
 * Obtiene una conexin a la base de datos, con los datos obtenidos
 * de los parmetros de configuracin. En esta apliaccin los datos son obtenidos de un fichero xml
 * con los parmetros adecuados. Esta funcin comprueba si la variable global @connection es una
 * conexin a la BD, si lo es la devuelve, sino la abre.
 *
 * Returns:
 * Conexin a la base de datos.
 */
GdaConnection *  m_model_get_connection () {

    gchar * provider = NULL;
    gchar * user     = u_parameter_manager_get_parameter ("db_user");
    gchar * password = u_parameter_manager_get_parameter ("db_password");
    gchar * db_name  = u_parameter_manager_get_parameter ("db_name");
    gchar * db_hostname = u_parameter_manager_get_parameter ("db_hostname");
    gchar * dsn      = g_strconcat("DATABASE=", db_name,
				   ";HOST=", db_hostname,
				   NULL);


    /*   GdaConnection* my_connection; */

    if (GDA_IS_CONNECTION(connection)) {
	if (!(gda_connection_is_open(connection))) {
	    g_free(connection);
	    connection=NULL;
	}

	if (connection==NULL) {
	    g_print(_("Connection \n"));
	    connection = gda_connection_new(gda_corba_get_orb());
	    provider = m_model_list_providers ();
	    g_print(_("Name : %s\n"),provider);
	    gda_connection_set_provider (connection, provider);
	    gda_connection_open(connection,dsn,user,password);
	    g_free(provider);
	}

    } else {
	g_print(_("Connection was not freed correctly at any time\n"));
	connection = gda_connection_new(gda_corba_get_orb());
	provider = m_model_list_providers ();
	g_print(_("Name: %s\n"),provider);
	gda_connection_set_provider (connection, provider);
	gda_connection_open(connection,dsn,user,password);
	g_free(provider);
    }

    g_free(db_name);
    g_free(db_hostname);
    g_free(dsn);
    g_free(user);       
    g_free(password);   

    return connection;
}


/**
 * m_model_destroy_connection:
 *
 * Destruye la conexin a la base de datos, liberando la memoria reservada por
 * @connection.
 *
 */
void  m_model_destroy_connection () {


    if (gda_connection_is_open(connection))
	gda_connection_close(connection);

    g_print(_("Closing connection\n"));
    if (connection!=NULL)
	gda_connection_free(connection);

}

/**
 * m_model_command_execute:
 * @record_set: Record set en el que se devolvern los resultados de la query. Si se trata de una
 * select se devuelven las tuplas.
 * @query: Cadena con la sentencia sql a ejecutar.
 * @cnc: Conexin a la base de datos.
 * @cmd: Comando con el que se ejecutar la sentencia sql.
 * @reccount: Tuplas que se devuelven en el record set.
 * @flags: Flags utilizados por gda_command_execute.
 * @readonly: Si es de solo lectura no imprime la sentencia en el log.
 *
 * Ejecuta una sentencia sql en la base de datos.
 *
 * Returns:
 * %TRUE si se ha realizado la operacin correctamente, %FALSE en otro caso.
 */
gboolean m_model_command_execute (GdaRecordset **record_set,
			       gchar *query,
			       GdaConnection *cnc,
			       GdaCommand *cmd,
			       gulong *reccount,
			       gulong flags, gboolean readonly) {

    gboolean correct_execute = FALSE;
    gchar *text = NULL;
    GList * errors=NULL;

    if (!readonly) {
	u_delete_intros (query);
	print_log(query);
    }

    if(*record_set!=NULL){
	gda_recordset_close(*record_set);
	gda_recordset_free(*record_set);
	*record_set = NULL;
    }

    gda_command_set_text(cmd, query);

    *record_set = gda_command_execute(cmd, reccount, flags);

    errors = gda_connection_get_errors(cnc);
    correct_execute = ((*record_set)!=NULL) && (errors==NULL);

    if (errors!=NULL)
	g_list_free(errors);

    if (!readonly) {
	text = g_strdup_printf("%d", correct_execute);
	print_log(text);
	g_free(text);
	text = NULL;
    }

    //    g_assert(correct_execute);
    return correct_execute;
}

/**
 * m_model_transactional_command_execute:
 * @record_set: Record set en el que se devolvern los resultados de la query. Si se trata de una
 * select se devuelven las tuplas.
 * @query: Cadena con la sentencia sql a ejecutar.
 * @cnc: Conexin a la base de datos.
 * @cmd: Comando con el que se ejecutar la sentencia sql.
 * @reccount: Tuplas que se devuelven en el record set.
 * @flags: Flags utilizados por gda_command_execute.
 * @readonly: Si es de solo lectura no imprime la sentencia en el log.
 * 
 * Ejecuta una sentencia sql en la base de datos, dentro de una transaccin.
 *
 * Returns:
 * %TRUE si se ha realizado la operacin correctamente, %FALSE en otro caso.
 */
gboolean m_model_transactional_command_execute (GdaRecordset **record_set,
					     gchar *query,
					     GdaConnection *cnc,
					     GdaCommand *cmd,
					     gulong *reccount,
					     gulong flags, gboolean readonly) {
    gboolean correct_execute = FALSE;
    model_connection_begin_transaction(cnc);

    correct_execute =
	m_model_command_execute (record_set, query, cnc, cmd, reccount, flags, readonly);
    correct_execute = execute_modification(cnc) && correct_execute;

    //    g_assert(correct_execute);
    return correct_execute;
}

/**
 * m_model_mutex_transactional_command_execute_logged:
 * @record_set: Record set en el que se devolvern los resultados de la query. Si se trata de una
 * select se devuelven las tuplas.
 * @query: Cadena con la sentencia sql a ejecutar.
 * @cnc: Conexin a la base de datos.
 * @cmd: Comando con el que se ejecutar la sentencia sql.
 * @reccount: Tuplas que se devuelven en el record set.
 * @flags: Flags utilizados por gda_command_execute.
 * @readonly: Si es de solo lectura no imprime la sentencia en el log.
 * @enterprise_code: Cdigo de la empresa que se escribir en el log.
 * @shop_code: Cdigo de la tienda que se escribir en el log.
 * @warehouse_code: Cdigo del almacn que se escribir en el log.
 *
 * Funcin que comienza una transaccin, ejecuta el comando en la
 * transaccin y cierra la transaccin. A empezar una transaccin utiliza model_connection_begin_mutex_transaction_logged
 * para poner un lock al fichero de log, empieza la transaccin en el log, ejecuta el comando
 * y utiliza execute_mutex_modification_logged para cerrar la transaccin y quitar el lock del fichero.
 *
 * Returns:
 * %TRUE si se ha realizado la operacin correctamente, %FALSE en otro caso.
 */
gboolean m_model_mutex_transactional_command_execute_logged (GdaRecordset **record_set,
							  gchar *query,
							  GdaConnection *cnc,
							  GdaCommand *cmd,
							  gulong *reccount,
							  gulong flags, gboolean readonly,
							  gulong enterprise_code,
							  gulong shop_code,
							  gulong warehouse_code) {
    gboolean correct_execute = FALSE;
    m_model_connection_begin_mutex_transaction_logged (cnc, enterprise_code,
						    shop_code, warehouse_code);

    correct_execute =
	m_model_command_execute (record_set, query, cnc, cmd, reccount, flags, readonly);
    correct_execute = m_model_execute_mutex_modification_logged (cnc) && correct_execute;

    //    g_assert(correct_execute);
    return correct_execute;
}


/**
 * model_connection_begin_transaction:
 * @cnc: Conexin al base de datos para la que se comenzar la transacin.
 * 
 * Comienza una transacccin en la base de datos.
 *
 */
void model_connection_begin_transaction(GdaConnection *cnc) {
    //    gchar * temp;
    //    time_t dia_actual = time(NULL);
    //    u_mutex_on ();
    gda_connection_begin_transaction(cnc);
    //    temp = g_strdup_printf("Begin Transaction %s", ctime(&dia_actual));
    //    print_log(temp);
    //    g_free(temp);

}

/**
 * m_model_connection_begin_mutex_transaction_logged:
 * @cnc: Conexin a la base de datos.
 * @enterprise_code: Cdigo de la empresa que se escribir en el log.
 * @shop_code: Cdigo de la tienda que se escribir en el log.
 * @warehouse_code: Cdigo del almacn que se escribir en el log.
 *
 * Comienza una transaccin en la base de datos escribiendo
 * en el fichero de log la cabecera de transaccin.
 *
 */
void m_model_connection_begin_mutex_transaction_logged (GdaConnection *cnc,
						     gulong enterprise_code,
						     gulong shop_code,
						     gulong warehouse_code) {
    gchar * temp;
    time_t current_day = time(NULL);

    if ((enterprise_code == ULONG_MAX) && (shop_code == ULONG_MAX) && (warehouse_code==ULONG_MAX)) {
        exit(-1);
    }
    if (enterprise_code==ULONG_MAX)
        exit(-1);

    u_mutex_on ();

    temp = get_path();
    u_set_lock (temp);
    g_free(temp);
    temp=NULL;

    //    gda_connection_begin_transaction(cnc);
    model_connection_begin_transaction(cnc);
    temp = g_strdup_printf("Begin Transaction (%lu,%lu,%lu) %s",
			   enterprise_code,shop_code,warehouse_code,
			   ctime(&current_day));
    print_log(temp);
    g_free(temp);

}

/**
 * execute_modification: 
 * @connection: Conexin a la base de datos sobre la que se ejecuta el commit.
 *
 * Ejecuta un commit sobre la base de datos.
 *
 * Returns:
 * %TRUE si se ha realizado correctamente, %FALSE en otro caso.
 */
gboolean execute_modification(GdaConnection * connection){


    gboolean correct_execute;

    correct_execute =  (gda_connection_get_errors (connection)==NULL) &&
	(gda_connection_commit_transaction(connection)==0);


    return correct_execute;
}


/**
 * m_model_execute_mutex_modification_logged: 
 * @connection: Conexin a la base de datos.
 *
 * Ejecuta un commit sobre la base de datos y cierra la transacin
 * en el log, quitanto el lock del fichero de log.
 *
 * Returns:
 * %TRUE si se ha producido correctamente, %FALSE en otro caso.
 */
gboolean m_model_execute_mutex_modification_logged (GdaConnection * connection){

    gboolean correct_execute;
    gchar *text;
    gchar *temp;

    correct_execute = execute_modification(connection);

    text = g_strdup_printf("End Transaction %d", correct_execute);
    print_log(text);
    g_free(text);
    text = NULL;
    u_mutex_off ();

    temp = get_path();
    u_remove_lock (temp);
    g_free(temp);

    return correct_execute;
}

/**
 * m_model_record_set_contains_piece:
 * @rs: Record set en el que se comprobar.
 *
 * Combrueba si el actual campo del record set es una pieza.
 *
 * Returns:
 * %TRUE si el campo apuntado por el record set se trata de una pieza, %FALSE en otro caso.
 */
gboolean m_model_record_set_contains_piece (GdaRecordset *rs) {
    GdaField   *field;
    field = gda_recordset_field_name(rs,"eurocodepiece");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gda_field_free(field);
	    return TRUE;
	}
    }
    return FALSE;

}

/**
 * m_model_record_set_contains_man_power:
 * @rs: Record set en el que se comprobar.
 * 
 * Combrueba si el actual campo del record set es una mano de obra.
 *
 * Returns:
 * %TRUE si el campo apuntado por el record set se trata de una mano de obra, %FALSE en otro caso.
 */
gboolean m_model_record_set_contains_man_power (GdaRecordset *rs) {
    GdaField   *field;
    field = gda_recordset_field_name(rs,"codigomanodeobramanodeobra");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gda_field_free(field);
	    return TRUE;
	}
    }
    return FALSE;

}

/**
 * m_model_nextval_sequence:
 * @sequence: Nombre de la secuencia para la que se quiere obtener el siguiente valor.
 * @connection: Conexin a la base de datos.
 *
 * Obtiene el siguiente valor de la secuencia solicitada.
 *
 * Returns:
 * Siguiente valor de la secuencia si no se ha producido ningn error, %NULL en otro caso.
 *
 */
gchar *m_model_nextval_sequence (const gchar *sequence, GdaConnection *connection) {
    gboolean result = FALSE;
    gchar *query    = g_strdup_printf(" SELECT nextval('%s') as value ", sequence);
    gchar *value    = NULL;
    gchar *temp     = NULL;
    GdaRecordset  *record_set = NULL;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaCommand    *command = gda_command_new();
    GdaField      *field = NULL;
    gint           index;

    gda_command_set_connection(command,connection);
    result = m_model_command_execute (&record_set, query, connection,
				   command, tuples, 0, TRUE);

    if (result) {
	index = gda_recordset_move_first(record_set);
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

	field = gda_recordset_field_name(record_set,"value");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp = gda_stringify_value(NULL, 0, field);
		value = g_strdup(temp);
		g_free(temp);
	    }
	    gda_field_free(field);
	}
    }

    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
    g_free(query);
    igalia_command_free(command);
    g_free(tuples);

    return value;
}

/**
 * m_model_currval_sequence: 
 * @sequence: Nombre de la secuencia para la que se desea obtener el siguiente valor.
 * @connection: Conexin a la base de datos.
 *
 * Devuelve el valor actual de la secuencia solicitada.
 *
 * Returns:
 * Valor actual de la secuencia si no se ha producido ningn error, %NULL en otro caso.
 */
gchar *m_model_currval_sequence (const gchar *sequence, GdaConnection *connection) {
    gboolean result = FALSE;
    gchar *query    = g_strdup_printf(" SELECT currval('%s') as value ", sequence);
    gchar *value    = NULL;
    gchar *temp     = NULL;
    GdaRecordset  *record_set = NULL;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaCommand    *command = gda_command_new();
    GdaField      *field = NULL;
    gint           index;

    gda_command_set_connection(command,connection);
    result = m_model_command_execute (&record_set, query, connection,
				   command, tuples, 0, TRUE);

    if (result) {
	index = gda_recordset_move_first(record_set);
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

	field = gda_recordset_field_name(record_set,"value");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp = gda_stringify_value(NULL, 0, field);
		value = g_strdup(temp);
		g_free(temp);
	    }
	    gda_field_free(field);
	}
    }

    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
    g_free(query);
    igalia_command_free(command);
    g_free(tuples);

    return value;
}

/**
 * get_log_filename:
 *
 * Obtiene el nombre del log en el que debe escribir la aplicacin actualmente corriendo,
 * dependiendo de los parmetros del fichero parameters.xml
 *
 * Returns:
 * Nombre del fichero de log.
 */
static gchar *get_log_filename() {
    gchar *temp = NULL;
    gchar *path = NULL;
    gchar *path_aux = NULL;
    gchar *enterprise_name = u_parameter_manager_get_parameter ("empresa");
    gchar *shop_name = u_parameter_manager_get_parameter ("tienda");
    gchar *warehouse_name = u_parameter_manager_get_parameter ("almacen");


    path_aux = g_strconcat("bd_modifications.",
			   NULL);
    g_free(temp);

    path = path_aux;
    path_aux = NULL;

    temp =  get_identificator(enterprise_name, shop_name, warehouse_name);

    path_aux = g_strconcat(path, temp, NULL);
    g_free(temp);
    g_free(path);
    g_free(enterprise_name);
    g_free(shop_name);
    g_free(warehouse_name);
    path = path_aux;
    path_aux = NULL;

    path_aux = g_strconcat(path, "log", NULL);
    g_free(path);
    path = path_aux;
    path_aux = NULL;

    return path;

}

/**
 * get_path: 
 * 
 * Obtiene el path en el que se encuentra el fichero de log.
 *
 * Returns:
 * Path completo del directorio de logs.
 */
static gchar *get_path() {
    gchar *temp = NULL;
    gchar *path = NULL;
    gchar *path_aux = NULL;

    path_aux = g_strconcat(DIR_SPOOL, "/logs/",
			   NULL);
    g_free(temp);
    path = path_aux;
    path_aux = NULL;

    temp =  get_log_filename();
    path_aux = g_strconcat(path, temp, NULL);
    g_free(temp);
    g_free(path);
    path = path_aux;
    path_aux = NULL;

    return path;

}

/**
 * print_log: 
 * @query: Cadena con la query que se desea imprimir en el fichero de log.
 * 
 * Imprime la query actual en el fichero de log.
 *
 */
static void print_log(const gchar *query) {
    gint   file = -1;
    gchar *eol  = "\n";
    gchar *path = get_path();
    mode_t mask;

    mask = umask(0);
    file = open(path, O_SYNC|O_CREAT|O_APPEND|O_RDWR,
		S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
    umask(mask);

    if(file==-1) {
	perror("model/model.c/print_log:open");
	printf(_("\tTrying to open: %s\n"), path);
        exit(-1);
    }

    if(write(file, query, strlen(query))==-1) {
	perror("model/model.c/print_log:write(query)");
	printf(_("\tTrying to write: %s\n"), path);
        exit(-1);
    }

    if(write(file, eol, strlen(eol))==-1) {
	perror("model/model.c/print_log:write(eol)");
	printf(_("\tTrying to write: %s\n"), path);
        exit(-1);
    }

    if(close(file)==-1) {
	perror("model/model.c/print_log:close");
	printf(_("\tTrying to close: %s\n"), path);
        exit(-1);
    }

    g_free(path);
}

/**
 * get_identificator:
 * @enterprise: Nombre de la empresa.
 * @shop: Nombre de la tienda
 * @warehouse: Nombre del almacn. Este parmetro actualmente no se utiliza.
 *
 * Obtiene el identificador para el nombre del log que se corresponde con
 * la empresa y tienda que se pasa como parmetro.
 *
 * Returns:
 * El nombre del fichero de log obtenido a partir de combinar el nombre de la empresa y de la tienda.
 */
static gchar *get_identificator(gchar *enterprise, gchar *shop, gchar *warehouse) {
    gchar *identificator = " ";
    gchar *identificator_aux = NULL;

    if (enterprise!=NULL) {
	identificator_aux = g_strconcat(identificator, enterprise, ".", NULL);
    } else {
	identificator_aux = g_strconcat(identificator, "null.", NULL);
    }
    
    identificator = identificator_aux;
    identificator_aux = NULL;

    if (shop!=NULL) {
	identificator_aux = g_strconcat(identificator, shop, ".", NULL);
    } else {
	identificator_aux = g_strconcat(identificator, "null.", NULL);
    }
    g_free(identificator);
    identificator = identificator_aux;
    identificator_aux = NULL;

    /*YA NO ES NECESARIO EL NOMBRE DEL ALMACN*/
/*XAVI    if (warehouse!=NULL) {
	identificator_aux = g_strconcat(identificator, warehouse, ".", NULL);
    } else {
	identificator_aux = g_strconcat(identificator, "null.", NULL);
    }
    g_free(identificator);
    identificator = identificator_aux;
    identificator_aux = NULL;
*/
    return identificator;
}

