/* office_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "office_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"

#define TABLE         " Banco,Oficina "
#define ATTRIBUTES    "Banco.codigoBanco as codigoBancoOffice , \
                       Banco.nombreBanco as nombreBancoOffice , \
                       Oficina.codigoOficina as codigoOficinaOffice , \
                       Oficina.nombre as nombreOffice , \
                       Oficina.direccion as direccionOffice "


/**
 * m_office_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_office_get_table () {
    return TABLE;
}

/**
 * m_office_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_office_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_office_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Office con los campos rellenados con los datos de la base de datos.
 */
Office *m_office_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    Office *office = d_office_create_empty();


    field = gda_recordset_field_name(rs,"codigoOficinaoffice");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    office->office_code = gda_field_get_smallint_value(field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"codigoBancooffice");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    office->bank_code = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"nombreBancooffice");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    office->bank_name = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"nombreoffice");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    office->name = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"direccionoffice");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    office->address = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }
    return office;
}



/**
 * m_office_find_ine_code: 
 * @bank_code: El cdigo del banco, es el primer nmero de un nmero de cuenta.
 * @office_code: El cdigo de la oficina, es el segundo nmero de un nmero de cuenta.
 *
 * Para el cuaderno 58 se necesita el cdigo INE del banco.
 *
 * Returns: El cdigo Ine de la oficina. %ULONG_MAX si no lo encuentra.
 */
gulong m_office_find_ine_code (gulong bank_code,gulong office_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar * gchar_bank = g_strdup_printf("%lu",bank_code);
    gchar * gchar_office = g_strdup_printf("%lu",office_code);
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gint index;
    gulong * tuples;
    gulong result = ULONG_MAX;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat(" SELECT codigoIne from codigoIneOficina where codigoBanco=",gchar_bank," and ",
                              " codigoOficina=",gchar_office,
			       NULL );

    g_print(_("\nQuery: %s\n"),gchar_query);
    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
	    gchar * result_gchar = NULL;
	    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
            GdaField * field;
	    index = gda_recordset_move_first(record_set);
	    if (index==GDA_RECORDSET_INVALID_POSITION) return ULONG_MAX;
	    field = gda_recordset_field_idx(record_set,0);
	    result_gchar  = gda_stringify_value(NULL, 0, field);
	    result = strtoul(result_gchar,stop,10);
	    g_free(stop);
            g_free(result_gchar);
	}
	gda_recordset_close(record_set);
    }

    g_free(gchar_query);
    g_free(gchar_bank);
    g_free(gchar_office);
    g_free(tuples);


    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;
}
