/* operation_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "gda_igalia_util.h"
#include "operation_model.h"
#include "parameter_manager.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "payment_model.h"
#include "warehouse_model.h"
#include "account_model.h"

#define TABLE         "Movimiento "
#define ATTRIBUTES    "Movimiento.codigoBanco as codigoBancoOperation , \
                       Movimiento.codigoOficina as codigoOficinaOperation , \
                       Movimiento.codigoMovimiento as codigoMovimientoOperation , \
                       Movimiento.codigoPago as codigoPagoOperation , \
                       Movimiento.numeroCuenta as numerocuentaOperation , \
                       Movimiento.descripcion as descripcionOperation , \
                       Movimiento.fecha as fechaOperation , \
                       Movimiento.referenciaRecibo as referenciaReciboOperation , \
                       Movimiento.cantidad as cantidadOperation , \
                       Movimiento.fechaNotificacion as fechaNotificacionOperation "



/**
 * m_operation_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_operation_get_table () {
    return TABLE;
}

/**
 * m_operation_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_operation_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_operation_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Operation con los campos rellenados con los datos de la base de datos.
 */
Operation *m_operation_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    Operation *operation = d_operation_create_empty();

    field = gda_recordset_field_name(rs,"codigoMovimientoOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    operation->operation_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);


	    operation->account = m_account_create (rs);
	    operation->payment = m_payment_create (rs);

	    field = gda_recordset_field_name(rs,"descripcionoperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    operation->section = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"referenciaRecibooperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    operation->receipt_reference = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaOperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    operation->date = (*my_time);
		    g_free(my_time);

		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaNotificacionOperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    operation->notification_date = (*my_time);
		    g_free(my_time);

		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"cantidadoperation");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    operation->amount = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }
    return operation;
}


/**
 * m_operation_insert:
 * @operation: Movimiento bancario a insertar en la BD.
 *
 * Inserta un movimiento producido en una cuenta corriente de alguna de las empresas de la empresa.
 *
 * Returns:
 * %TRUE si se insert correctamente, %FALSE en otro caso.
 */
gboolean m_operation_insert (Operation * operation) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gchar * query_payment=NULL;
    gchar * query_aux=NULL;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * payment_code_aux=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaRecordset *record_set_payment=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    time_t date;
    gint bank_code = d_office_get_bank_code (d_account_get_office (d_operation_get_account (operation)));
    gint office_code = d_office_get_office_code (d_account_get_office (d_operation_get_account (operation)));
    gulong payment_code = d_payment_get_payment_code (d_operation_get_payment (operation));
    gchar * account_number = d_account_get_account_number (d_operation_get_account (operation));
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);


    if ((bank_code!=-1) && (office_code!=-1) && (account_number!=NULL)) {
	gchar * temp=NULL;
        gchar * temp_nextval;

	m_model_connection_begin_mutex_transaction_logged (connection,
						       logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	query = g_strconcat(" INSERT INTO MOVIMIENTO VALUES ",NULL);
	values = g_strconcat(" (",NULL);

        temp_nextval=m_model_nextval_sequence ("seqMovimiento",connection);
	values_aux = g_strconcat(values,temp_nextval,",",NULL);
	g_free(values);
	values = values_aux;
	g_free(temp_nextval);

	temp = g_strdup_printf("%d",bank_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;

	temp = g_strdup_printf("%d",office_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;

	temp = g_strdup_printf("%lu",payment_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;

	values_aux = g_strconcat(values,account_number,",",NULL);
	g_free(values);
	values = values_aux;

	if (d_operation_get_section (operation)!=NULL) {
	    temp = d_operation_get_section (operation);
	    values_aux = g_strconcat(values,"'",temp,"',",NULL);
	    g_free(values);
	    g_free(temp);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}
	date = d_operation_get_date (operation);
	values_aux = g_strconcat(values,"'",ctime(&date),"',",NULL);
	g_free(values);
	values = values_aux;


	if (d_operation_get_amount (operation)!=-1) {
	    temp = g_strdup_printf("%f",d_operation_get_amount (operation));
	    values_aux = g_strconcat(values,"to_number('",temp,"','99999999D99'),",NULL);
	    g_free(values);
	    g_free(temp);                         
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}
	if (d_operation_get_receipt_reference (operation)!=NULL) {
	    temp = d_operation_get_receipt_reference (operation);
	    values_aux = g_strconcat(values,"'",temp,"',",NULL);
	    g_free(values);
	    g_free(temp);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}

	date = d_operation_get_notification_date (operation);
	values_aux = g_strconcat(values,"'",ctime(&date),"'",NULL);
	g_free(values);
	values = values_aux;

	query_aux=g_strconcat(query,values,")",NULL);
	g_free(query);
	query=query_aux;
	query_aux=NULL;

	if (payment_code!=ULONG_MAX) {
            time_t done_date=time(NULL);
	    payment_code_aux = g_strdup_printf("%lu",payment_code);
	    query_payment = g_strconcat("update pago set fecharealizado='",ctime(&done_date),"' where codigoPago=",payment_code_aux,NULL);
	    g_print(_("\nUpdate: %s\n"),query_payment);
            g_free(payment_code_aux);
	}


	result = m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);

	if (query_payment!=NULL) {
	    result = m_model_command_execute (&record_set_payment, query_payment, connection, command,tuples,0, FALSE) && result;
	}

	result = m_model_execute_mutex_modification_logged (connection) && result;


    }
    if (query!=NULL) g_free(query);
    if (query_payment!=NULL) g_free(query_payment);
    if (tuples!=NULL) g_free(tuples);
    if (account_number!=NULL) g_free(account_number);

    gda_recordset_free(record_set);
    if (record_set_payment!=NULL) gda_recordset_free(record_set_payment);
    igalia_command_free(command);

    return result;
}

/**
 * m_operation_update_in_transaction:
 * @payment_code: Pago al que se refiere este movimiento.
 * @shop_code: Cdigo de la tienda en la que se realiz en movimiento.
 * @enterprise_code: Cdigo de la empresa a la que pertenece la tienda.
 * @amount: Cantidad nueva que se meter.
 * @connection: Conexin a la base de datos.
 *
 * Modifica la cantidad de  un movimiento.
 *
 * Returns:
 * %TRUE si se modific correctamente, %FALSE en otro caso.
 */
gboolean m_operation_update_in_transaction (gulong payment_code,gulong shop_code,gulong enterprise_code,gdouble amount,GdaConnection * connection) {
    gboolean result=TRUE;
    GdaCommand *command  = gda_command_new();
    gchar * query = NULL;
    GdaRecordset * record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * payment_code_char = g_strdup_printf("%lu",payment_code);
    gchar * amount_char = g_strdup_printf("%.2f",u_round_to_decimal (amount,2));

    u_commas_to_points (amount_char);
    gda_command_set_connection(command,connection);

    query = g_strconcat(" UPDATE MOVIMIENTO SET cantidad=",amount_char," WHERE ",
			" codigoPAgo=",payment_code_char,
			NULL);

    result = m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);

    g_print(_("\nUpdate: %s\n"),query);

    if (record_set!=NULL) {
	if (GDA_IS_RECORDSET(record_set)) {
	    gda_recordset_close(record_set);
	}
    }
    g_free(payment_code_char);
    g_free(tuples);
    g_free(query);
    igalia_command_free(command);
    return result;
}



