/* product_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "product_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"
#include "util.h"

#define TABLE         "Producto"
#define IDENTIFICATOR "codigoProducto"
#define ATTRIBUTES    "codigoProducto as codigoproductoproduct, \
    codigoFamilia as codigofamiliaproduct, \
    descripcion as descripcionproduct, tipoIVA as tipoivaproduct, \
    precio as precioproduct, \
    dtomax as dtomaxproduct"

/**
 * m_product_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_product_get_table () {
    return TABLE;
}

/**
 * m_product_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_product_get_attributes () {
    return ATTRIBUTES;
}



/**
 * m_product_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Product con los campos rellenados con los datos de la base de datos.
 */
Product m_product_create (GdaRecordset *rs) {
    gchar    *temp;
    GdaField *field;
    Product   product;

    d_product_init (&product);

    field = gda_recordset_field_name(rs,"codigoproductoproduct");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    product.product_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"codigofamiliaproduct");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    product.family_code = g_strdup(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"descripcionproduct");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    product.description = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"tipoivaproduct");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    product.vat_type = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"precioproduct");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    u_points_to_commas (temp);
		    product.price = atof(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"dtomaxproduct");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    product.max_discount = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	} else {
	    gda_field_free(field);
	}
    }

    return product;
}


/**
 * m_product_insert:
 * @product: Producto que se inserta en la base de datos.
 *
 * Inserta un producto en la base de datos.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso. Una vez se ha insertado se inserta en
 * el producto el cdigo devuelto de la base de datos.
 */
gboolean m_product_insert (Product *product) {
    gboolean       result = FALSE;

    if (product != NULL) {
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();
	gulong         product_code = ULONG_MAX;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	gchar         *temp= NULL;
	gulong logged_shop_code;
	gulong logged_enterprise_code;
	gulong logged_warehouse_code;
	gchar * temp_name;

	temp_name = u_parameter_manager_get_parameter ("empresa");
	logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("tienda");
	logged_shop_code = m_shop_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("almacen");
	logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

	g_free(temp_name);

	gda_command_set_connection(command,connection);            

	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	result = m_product_insert_in_transaction (product, connection, command);

        temp = m_model_currval_sequence ("seqproducto", connection);

	result = result && (temp!=NULL);

	if (temp != NULL) {
	    gchar **stop = (gchar **) g_malloc0(sizeof(gchar *));

	    product_code= strtoul(temp,stop,10);

	    g_free(temp);
	    g_free(stop);
	}

	d_product_set_product_code (product, product_code);

	result = m_model_execute_mutex_modification_logged (connection) && result;
	
	igalia_command_free(command);

	g_free(tuples);
    }
    return result;
}

/**
 * m_product_insert_in_transaction:
 * @product: Producto que se inserta.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la sentencia sql.
 *
 * Inserta un producto en la base de datos, dentro de una transaccin.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_product_insert_in_transaction (Product *product, GdaConnection *connection, GdaCommand *command) {
    gchar         *gchar_query_product = NULL;
    gchar         *gchar_query_product_aux = NULL;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    gboolean       result = FALSE;

    temp = m_model_nextval_sequence ("seqProducto", connection);
    gchar_query_product_aux = g_strconcat("INSERT INTO ", m_product_get_table (),
					  " VALUES ",
					  "(",temp,", '", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp = d_product_get_family_code (product);
    gchar_query_product_aux = g_strconcat(gchar_query_product, temp,
					  "', '",NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp =d_product_get_description (product);
    if (temp == NULL)
      gchar_query_product_aux = g_strconcat(gchar_query_product,
					    "', ",NULL);      
    else
      gchar_query_product_aux = g_strconcat(gchar_query_product, temp,
					    "', ",NULL);
    
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp = g_strdup_printf("%.2f",d_product_get_vat_type (product));
    u_commas_to_points (temp);
    gchar_query_product_aux = g_strconcat(gchar_query_product, "'",temp,
					  "', ", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    if (d_product_get_price (product)!=G_MINDOUBLE) {
	temp = g_strdup_printf("%.2f",d_product_get_price (product)*1.0);
	u_commas_to_points (temp);
	gchar_query_product_aux = g_strconcat(gchar_query_product,"'", temp,
					      "',", NULL);
    } else {
	temp = g_strdup("NULL");
	gchar_query_product_aux = g_strconcat(gchar_query_product, temp,
					      ",", NULL);

    }
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp = g_strdup_printf("%.2f",d_product_get_max_discount (product));
    u_commas_to_points (temp);
    gchar_query_product_aux = g_strconcat(gchar_query_product,
					  "'",temp,"')", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    g_print(_("\nInsert: %s\n"),gchar_query_product);
    result = m_model_command_execute (&record_set, gchar_query_product, connection,command,tuples,0,FALSE);

    g_free(gchar_query_product);
    gda_recordset_free(record_set);
    g_free(tuples);
    return result;
}

/**
 * m_product_update:
 * @product: Producto a modificar.
 * @company_man_power: Boolean que indica si la modificacin se produce desde una mano
 *                   de obra con compaa.
 *
 * Modifica un producto en la base de datos. Aber una transaccin, utiliza la funcin m_product_update_in_transaction () y
 * cierra la transaccin.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_product_update (Product *product,gboolean company_man_power) {
    gboolean       result = FALSE;

    if (product != NULL) {
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();
	gulong logged_shop_code;
	gulong logged_enterprise_code;
	gulong logged_warehouse_code;
	gchar * temp_name;

	temp_name = u_parameter_manager_get_parameter ("empresa");
	logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("tienda");
	logged_shop_code = m_shop_get_code_from_name (temp_name);
	g_free(temp_name);
	temp_name = u_parameter_manager_get_parameter ("almacen");
	logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

	g_free(temp_name);

	gda_command_set_connection(command,connection);            

	m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

	result = m_product_update_in_transaction (product, connection, command,company_man_power);
	result = m_model_execute_mutex_modification_logged (connection) && result;
	
	igalia_command_free(command);
    }

    return result;
}

/**
 * m_product_update_in_transaction:
 * @product: Producto a insertar.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la sentencia sql.
 * @company_man_power: Boolean que indica si la modificacin se produce desde una mano
 *                   de obra con compaa.
 *
 * Inserta un producto en la BD, dentro de una transaccin.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_product_update_in_transaction (Product *product, GdaConnection *connection, GdaCommand *command,gboolean company_man_power) {
    GdaRecordset *record_set = NULL;
    gchar        *gchar_query_product;
    gchar        *gchar_query_product_aux;
    gchar        *temp;
    gulong       *tuples = (gulong *) g_malloc0(sizeof(gulong));
    gboolean      result = FALSE;

    gchar_query_product = "";
    gchar_query_product = g_strconcat("UPDATE ", m_product_get_table (),
				      " SET ", NULL);

    temp = d_product_get_family_code (product);
    gchar_query_product_aux = g_strconcat(gchar_query_product,
					  " codigoFamilia = '",
					  temp,
					  "', ", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp = d_product_get_description (product);
    gchar_query_product_aux = g_strconcat(gchar_query_product,
					  " descripcion = '",
					  temp,
					  "', ", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp = g_strdup_printf("%.2f",d_product_get_vat_type (product));
    u_commas_to_points (temp);
    gchar_query_product_aux = g_strconcat(gchar_query_product,
					  "tipoIVA = '",temp,
					  "', ", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    if (company_man_power!=TRUE) {
	if (d_product_get_price (product)!=G_MINDOUBLE) {
	    temp = g_strdup_printf("%.2f",d_product_get_price (product)*1.0);
	    u_commas_to_points (temp);
	    gchar_query_product_aux = g_strconcat(gchar_query_product,
						  "precio = '",temp,
						  "',",
						  NULL);
	} else {
	    temp = g_strdup("NULL");
	    gchar_query_product_aux = g_strconcat(gchar_query_product,
						  "precio = ",temp,
						  ",",
						  NULL);

	}

	g_free(temp);
	g_free(gchar_query_product);
	gchar_query_product = gchar_query_product_aux;
	gchar_query_product_aux = NULL;
    }
    temp = g_strdup_printf("%.2f",d_product_get_max_discount (product));
    u_commas_to_points (temp);
    gchar_query_product_aux = g_strconcat(gchar_query_product,
					  "dtomax = '",temp,
					  "' ",
					  NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    temp = g_strdup_printf("%lu",d_product_get_product_code (product));
    gchar_query_product_aux = g_strconcat(gchar_query_product,
					  " WHERE codigoProducto = '",temp,
					  "' ", NULL);
    g_free(temp);
    g_free(gchar_query_product);
    gchar_query_product = gchar_query_product_aux;
    gchar_query_product_aux = NULL;

    g_print (gchar_query_product);

    result = m_model_command_execute (&record_set, gchar_query_product, connection, command,tuples,0, FALSE);

    g_free(gchar_query_product);
    gda_recordset_free(record_set);
    g_free(tuples);

    return result;
}

/**
 * m_product_delete_in_transaction:
 * @product: Producto a borrar.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la sentencia sql.
 *
 * Borra un producto de la base de datos.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_product_delete_in_transaction (Product *product, GdaConnection *connection, GdaCommand *command) {

    gulong product_code = d_product_get_product_code (product);
    gboolean      result = FALSE;

    if (product_code!=-1){
	gchar        * query;
	gchar        * temp;
	gulong       * tuples = (gulong *) g_malloc0(sizeof(gulong));
	GdaRecordset *record_set = NULL;

	temp = g_strdup_printf("%lu", product_code);
	query = "";
	query = g_strconcat("DELETE FROM ", m_product_get_table ()," WHERE ",
			    "codigoProducto ILIKE '",temp,"'",
			    NULL);
	g_free(temp);

	result = m_model_command_execute (&record_set, query, connection, command,tuples,0,FALSE);
	gda_recordset_free(record_set);

	g_free(query);
	g_free(tuples);
    }
    return result;
}
