/* province_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "province_model.h"

#include "util.h"

#define TABLE         "Provincia"
#define IDENTIFICATOR "codigoprovincia"
#define ATTRIBUTES    "codigoprovincia as codigoprovince, \
                       nombreprovincia as nombreprovince"

Province ** provinces_list = NULL;
gint length_of_list;

/**
 * m_province_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_province_get_table () {
    return TABLE;
}

/**
 * m_province_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_province_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_province_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Province con los campos rellenados con los datos de la base de datos.
 */
Province * m_province_create (GdaRecordset *rs) {
    GdaField * field;
    Province * province = d_province_create_empty();


    field = gda_recordset_field_name(rs,"codigoprovince");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    province->province_code = gda_field_get_integer_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"nombreprovince");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp = gda_stringify_value(NULL, 0, field);
	    province->name = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }


    return province;
}


/**
 * m_province_find_by_code:
 * @code: Cdigo de la provincia.
 *
 * Busca una provincia por cdigo de provincia.
 *
 * Returns:
 * Provincia que cumple la condicin.
 */
Province * m_province_find_by_code (gint code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Province * province;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",code);

    gchar_query = g_strconcat("SELECT ",
			       " codigoProvincia as codigoprovince, ",
			       " nombreProvincia as nombreprovince ",
			      " FROM provincia ",
			      " WHERE (codigoProvincia=",aux,")",NULL);


    g_free(aux);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);

    g_free(gchar_query);
    
    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if ((*tuples)==1) {
   	    gda_recordset_move_first(record_set);
	    province = m_province_create (record_set);
	} else {
	    province = NULL;
	}
        gda_recordset_close(record_set);
    } else {
        province = NULL;
    }

    igalia_command_free(my_command);

    g_free(tuples);
    g_free(column);

    return province;
}

/**
 * m_province_find_all:
 *
 * Busca todos las provincias en la base de datos. Si ya se han buscado alguna vez devuelve
 * la variable global que las almacena, en otro caso, realiza la busqueda.
 *
 * Returns:
 * Array con todas provincias.
 */
Province ** m_province_find_all () {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Province * province;
    gchar * gchar_query;
    gulong * tuples;
    gint * column;
    gint index;


    if (provinces_list==NULL) {
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

	gchar_query = g_strconcat("SELECT ",
				  " codigoProvincia as codigoprovince, ",
				  " nombreProvincia as nombreprovince ",
				  " FROM provincia ",NULL);

	g_print(_("\nQuery: %s\n"),gchar_query);
	tuples = (gulong *) g_malloc0(sizeof(gulong));
	m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
	g_free(gchar_query);

        length_of_list = *tuples;
	column = (gint *) g_malloc0(sizeof(gint));

	if (record_set!=NULL) {
	    if (*tuples!=0) {
		provinces_list = (Province **) g_malloc0(sizeof(Province*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    (*column)=0;
		    province = m_province_create (record_set);
		    provinces_list[index-1] = province;
		    index = gda_recordset_move_next(record_set);
		}
		provinces_list[length_of_list]=NULL;

	    } else {
		provinces_list=NULL;
	    }
	} else {
	    provinces_list=NULL;
	}

	gda_recordset_free(record_set);
	igalia_command_free(my_command);

	g_free(tuples);
	g_free(column);


    }
    return provinces_list;
}

/**
 * m_province_get_name_from_code:
 * @code: Cdigo de la provincia a buscar.
 *
 * Busca el nombre de la provincia con el cdigo %code.
 *
 * Returns:
 * Nombre de la provincia si el cdigo es vlido, %NULL en otro caso.
 */
gchar *m_province_get_name_from_code (gulong code) {
    gchar *name = NULL;
    gint   index=0;

    if (provinces_list==NULL) {
	m_province_find_all ();
    }

    while(provinces_list[index]!=NULL) {
	if (code == d_province_get_province_code (provinces_list[index])) {
            name = d_province_get_name (provinces_list[index]);
	}
        index++;
    }

    return name;
}

/**
 * m_province_get_code_from_name:
 * @name: Nombre de la provincia a buscar.
 *
 * Busca el cdigo de la provincia con el nombre %name.
 *
 * Returns:
 * Cdigo de la provincia si el nombre es vlido, %NULL en otro caso.
 */
gulong m_province_get_code_from_name (gchar *name) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (provinces_list==NULL) {
	m_province_find_all ();
    }

    while(provinces_list[index]!=NULL) {
	temp = d_province_get_name (provinces_list[index]);
	if (!strcmp(name, temp)) {
	    code = d_province_get_province_code (provinces_list[index]);
	    g_free(temp);
           break;
	}
        g_free(temp);
        index++;
    }

    return code;
}



/**
 * m_province_clear_array:
 * @provinces: Array de provincias a liberar.
 *
 * Libera la memoria utilizada por una array de provincias.
 *
 */
void m_province_clear_array (Province ** provinces) {
    if (provinces!=NULL) {
	gint index = 0;
	while (provinces[index]!=NULL) {
	    d_province_clear (provinces[index]);
	    index++;
	}
    }
    g_free(provinces);
    provinces  = NULL;
}


