/* regularization_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "regularization_model.h"
#include "stock_model.h"
#include "util.h"

#define TABLE      "Regularizacion"

/**
 * m_regularization_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_regularization_get_table () {
    return TABLE;
}

/**
 * m_regularization_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Regularization con los campos rellenados con los datos de la base de datos.
 */
Regularization  *m_regularization_create (GdaRecordset *rs){
    Regularization * result=d_regularization_create_empty ();
    gchar  *temp;
    GdaField * field = NULL;
    

    field = gda_recordset_field_name(rs,"codigoempresaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoalmacenMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->warehouse_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->movement_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"unidadesMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->quantity = u_round_to_decimal (igalia_field_get_double_value (field),3);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    result->date = (*my_time);
	    g_free(my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoProductoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> product_code=igalia_field_get_bigint_value (field)  ;
	}
	gda_field_free(field);
    }


    return result;

}

/**
 * m_regularization_insert:
 * @regularization: Regularizacin que se desea insertar en la base de datos.
 * @stock: Stock que se modificar como resultado de la regularizacin.
 *
 * Inserta una regularizacin de un almacn con otro.
 *
 * Returns:
 * %TRUE si se ha realizado correctamente, %FALSE en otro caso.
 */
gboolean m_regularization_insert (Regularization *regularization, Stock *stock) {
    gboolean result = FALSE;

    gchar         *gchar_query_regularization;
    gchar         *gchar_query_regularization_aux;
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));

    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    time_t         date;
    gulong enterprise_code_regularization = ULONG_MAX;
    gulong warehouse_code_regularization = ULONG_MAX;

    gda_command_set_connection(command,connection);

    if (regularization!=NULL) {
	enterprise_code_regularization=d_regularization_get_enterprise_code (regularization);
    }
    if (regularization!=NULL) {
	warehouse_code_regularization=d_regularization_get_warehouse_code (regularization);
    }

    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_regularization,
								ULONG_MAX,
								warehouse_code_regularization
							       );

    result = m_stock_update (stock, connection, command);

    gchar_query_regularization = "";
    gchar_query_regularization = g_strconcat("INSERT INTO ",m_regularization_get_table (),
					     " VALUES ",
					     "('", NULL);

    temp = g_strdup_printf("%lu",d_regularization_get_enterprise_code (regularization));
    gchar_query_regularization_aux = g_strconcat(gchar_query_regularization, temp,
						 "', '", NULL);
    g_free(temp);
    g_free(gchar_query_regularization);
    gchar_query_regularization = gchar_query_regularization_aux;
    gchar_query_regularization_aux = NULL;

    temp = g_strdup_printf("%lu",d_regularization_get_warehouse_code (regularization));
    gchar_query_regularization_aux = g_strconcat(gchar_query_regularization, temp,
						 "', ", NULL);
    g_free(temp);
    g_free(gchar_query_regularization);
    gchar_query_regularization = gchar_query_regularization_aux;
    gchar_query_regularization_aux = NULL;

    temp = m_model_nextval_sequence ("seqMovimientoAlmacen", connection);
    gchar_query_regularization_aux = g_strconcat(gchar_query_regularization, temp,
						 ", ", NULL);
    g_free(gchar_query_regularization);
    g_free(temp);
    gchar_query_regularization = gchar_query_regularization_aux;
    gchar_query_regularization_aux = NULL;

    date = d_regularization_get_date (regularization);

    gchar_query_regularization_aux =
	g_strconcat(gchar_query_regularization, " '", ctime(&date),
		    "', '", NULL);
    g_free(gchar_query_regularization);
    gchar_query_regularization = gchar_query_regularization_aux;
    gchar_query_regularization_aux = NULL;

    temp = g_strdup_printf("%.3f",d_regularization_get_quantity (regularization));
    u_commas_to_points (temp);
    gchar_query_regularization_aux = g_strconcat(gchar_query_regularization, temp,
						 "',' ", NULL);
    g_free(temp);
    g_free(gchar_query_regularization);
    gchar_query_regularization = gchar_query_regularization_aux;
    gchar_query_regularization_aux = NULL;

    temp = g_strdup_printf("%lu",d_regularization_get_product_code (regularization));
    gchar_query_regularization_aux = g_strconcat(gchar_query_regularization, temp,
						 "') ", NULL);
    g_free(temp);
    g_free(gchar_query_regularization);
    gchar_query_regularization = gchar_query_regularization_aux;
    gchar_query_regularization_aux = NULL;

    result = result && m_model_command_execute (&record_set, gchar_query_regularization, connection, command,tuples,0, FALSE);

    g_print(_("\nInsert: %s\n"),gchar_query_regularization);

    result = m_model_execute_mutex_modification_logged (connection) && result ;

    if (result !=FALSE) {
        d_stock_set_previous_quantity(stock,d_stock_get_quantity(stock));
    }
    g_free(gchar_query_regularization);
    g_free(tuples);

    gda_recordset_free(record_set);
    igalia_command_free(command);
    return result;
}


