
/* repair_line_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"

#include "repair_line_model.h"
#include "find_pieces_model.h"
#include "repair_movement_model.h"
#include "stock_model.h"
#include "manodeobra_model.h"
#include "piece.h"
#include "piece_model.h"
#include "product_model.h"

#define M_FIRST_ERROR_MESSAGE N_("Error in Repair Lines data")

#define TABLE         "LineaReparacion"
#define ATTRIBUTES    "numeroLinea as numeroLinearepairline, \
                       precio as preciorepairline, \
                       dto as dtorepairline, \
                       preciocoste as preciocosterepairline, \
                       preciohistorico as preciohistoricorepairline, \
                       tipoiva as tipoivarepairline, \
                       tipoivaHistorico as tipoivaHistoricorepairline, \
                       unidades as unidadesrepairline"


/**
 * m_repair_line_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_repair_line_get_table () {
    return TABLE;
}

/**
 * m_repair_line_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_repair_line_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_repair_line_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Repair_line con los campos rellenados con los datos de la base de datos.
 */
Repair_line  *m_repair_line_create (GdaRecordset *rs){					    
    GdaField    *field;
    Repair_line *repair_line = d_repair_line_create_empty();
    gchar       *temp;
    Product * product;    

    if (m_model_record_set_contains_piece (rs)==TRUE) {
	product = (Product *) m_piece_create (rs);
	repair_line->stock = m_stock_create (rs);
    } else if (m_model_record_set_contains_man_power (rs)==TRUE) {
        product = (Product *) m_man_power_create (rs);
    } else {
      product = (Product *) g_malloc (sizeof(Product));
      
      d_product_init (product);

      *product = m_product_create (rs);
    }

    field = gda_recordset_field_name(rs,"numeroLinearepairline");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    repair_line->line_number = igalia_field_get_bigint_value (field);

	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"unidadesrepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair_line->quantity  = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"unidadesantesrepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair_line->earlier_quantity  = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"preciorepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    repair_line->price = atof(temp);
		}
		gda_field_free(field);
	    }


	    field = gda_recordset_field_name(rs,"dtorepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair_line->discount = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"preciocosterepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    repair_line->cost_price = atof(temp);
		}
		gda_field_free(field);
	    }


	    field = gda_recordset_field_name(rs,"preciohistoricorepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    repair_line->historical_price = atof(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"tipoIvahistoricorepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    repair_line->historical_vat_type = atof(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"tipoivarepairline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair_line->vat_type = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    if(product!=NULL) {
		repair_line->product = product;
	    }
	} else {
	    gda_field_free(field);
	}

    }

    return repair_line;

}


/**
 * m_repair_line_find_by_id:
 * @condition: Identificador de la reparacin
 *
 * Busca las lneas de una reparacin.
 *
 * Returns:
 * Array con las lneas que pertenecen a la reparacin.
 */
Repair_line **  m_repair_line_find_by_id (RepairId *condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query;
    gchar              *gchar_select;
    gchar              *gchar_where;
    gchar              *gchar_from;
    gchar              *gchar_order;
    gchar              *gchar_where_aux;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    Repair_line ** repair_lines;
    gulong * tuples;
    Repair_line * repair_line;
    gint index;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));

    gda_command_set_connection(my_command,my_connection);




    gchar_select = g_strconcat("SELECT ",
			       "LineaReparacion.numeroLinea as numeroLinearepairline,",
			       "LineaReparacion.codigoProducto as codigoproductorepairline,",
			       "to_char(LineaReparacion.unidades,'9999999999D999') as unidadesrepairline,",
			       "to_char(LineaReparacion.unidades,'9999999999D999') as unidadesantesrepairline,",
			       "to_char(LineaReparacion.preciohistorico,'99999999D99') as precioproduct,",
			       "to_char(LineaReparacion.precio,'99999999D99') as preciorepairline, ",
			       "to_char(LineaReparacion.dto,'999D99') as dtorepairline,",
			       "to_char(LineaReparacion.precioCoste,'99999999D99') as preciocosterepairline, ",
			       "Producto.codigoProducto as codigoproductoproduct,",
			       "Producto.codigoFamilia as codigofamiliaproduct,",
			       "Producto.descripcion as descripcionproduct,",
			       "to_char(LineaReparacion.tipoIVAHistorico,'999D99') as tipoivaHistoricorepairline,",
			       "to_char(Producto.tipoIVA,'999D99') as tipoivaProduct,",
/* 			       "to_char(Producto.precio,'99999999D99') as precioproduct,", */
			       "to_char(Producto.dtomax,'999D99') as dtomaxproduct,",
			       "Pieza.eurocode as eurocodepiece,",
			       "Pieza.codigoProducto as codigoproductopiece,",
			       "Pieza.largo as largopiece,",
			       "Pieza.ancho as anchopiece,",
			       "to_char(Pieza.precioCoste,'99999999D99') as precioCostepiece,",
			       "stock.codigoempresa as codigoempresastock,",
			       "stock.codigoalmacen as codigoalmacenstock,",
			       "to_char(stock.unidades,'9999999999D999') as unidadesstock,",
			       "to_char(stock.stockminimo,'9999999999D999') as stockminimostock,",
			       "to_char(Pieza.precioCoste,'99999999D99') as precioCosteproduct, ",
			       "ManoDeObra.codigoManoDeObra as codigomanodeobramanodeobra,",
			       "ManoDeObra.codigoProducto as codigoproductomanodeobra ",
			       NULL);
    gchar_from = g_strconcat(" from Producto, ",
			     " lineareparacion left outer join pieza ON (lineareparacion.codigoProducto = Pieza.codigoProducto)",
			     " left outer join manodeobra ON (lineareparacion.codigoProducto = manodeobra.codigoProducto)",
			     " left outer join stock ON (lineareparacion.codigoProducto = stock.codigoProducto AND "
                                                         "lineareparacion.codigoalmacen = stock.codigoalmacen)",
			     NULL);


    gchar_where=g_strconcat(" where lineaReparacion.codigoProducto = Producto.codigoProducto",
			    NULL);

    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (lineaReparacion.codigoEmpresa=",
				      condition->enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (lineaReparacion.codigoTienda=",
				      condition->shop_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if (((condition->repair_code) != NULL) && (strlen(condition->repair_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (lineaReparacion.codigoReparacion=",
				      condition->repair_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    gchar_order = g_strconcat(" ORDER BY LINEAREPARACION.NUMEROLINEA ",NULL);

    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where, gchar_order,NULL);

    g_print(_("Query: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {

	    repair_lines = (Repair_line **) g_malloc0(sizeof(Repair_line*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		repair_line = m_repair_line_create (record_set);
		repair_lines[index-1] = repair_line;
		    index = gda_recordset_move_next(record_set);
	    }
	    repair_lines[*tuples]=NULL;
	} else {
		repair_lines=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	repair_lines=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(tuples);
    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_from);
    g_free(gchar_order);

    return repair_lines;
    
}

/**
 * m_repair_line_clear_array:
 * @repair_lines: Array de lineas de reparacin que se desea liberar.
 *
 * Libera la memoria utilizada por el array de lneas de reparacin.
 *
 */
void m_repair_line_clear_array (Repair_line ** repair_lines) {
    if (repair_lines!=NULL) {
	gint index = 0;
	while (repair_lines[index]!=NULL) {
	    d_repair_line_clear (repair_lines[index]);
	    index++;
	}
    }
    g_free(repair_lines);

}

/**
 * m_repair_line_convert_from_pieces:
 * @list: Puntero a la lista enlazada que se desea rellenar con las lneas de reparacin.
 * @pieces_ids: Lista con los identificadores de las piezas.
 * @number: Nmero de piezas que se pasan en el array de piezas.
 *
 * Construye una lista de lneas de reparacin a partir de un array de piezas.
 *
 */
void m_repair_line_convert_from_pieces (GList **list, gulong *pieces_ids, gulong *number) {
  gint index = 0;
  Piece ** pieces = m_piece_find_by_product_code (pieces_ids, number);
  
  if (pieces!=NULL){
    GList *last = g_list_last(*list);
    gint last_pos = 0;

    if (last != NULL ) last_pos = (g_list_position(*list, last))+1;

    while (pieces[index]!=NULL) {
      Repair_line *repair_line = d_repair_line_create_empty ();
      d_repair_line_set_product (repair_line, (Product *) pieces[index]);
      d_repair_line_set_quantity (repair_line, 1.0);
      d_repair_line_set_line_position_number (repair_line, last_pos+1);
      *list = g_list_insert(*list, repair_line, last_pos++);

      index++;
    }
  }

}

/**
 * m_repair_line_convert_from_stocks:
 * @list: Puntero a la lista enlazada que se desea rellenar con las lneas de reparacin.
 * @stocks: Lista con los stocks a utilizar.
 *
 * Construye una lista de lneas de reparacin a partir de un array de stocks.
 *
 */
void m_repair_line_convert_from_stocks (GList **list, Stock **stocks) {
  gint index = 0;
  
  if (stocks != NULL){
    GList *last = g_list_last(*list);
    gint last_pos = 0;

    if (last != NULL ) last_pos = (g_list_position(*list, last))+1;

    while (stocks[index]!=NULL) {
        Product product = d_piece_get_product(d_stock_get_piece(stocks[index]));
	gulong new_product_code = d_product_get_product_code(&product);
	gulong product_code=ULONG_MAX;
        Repair_line * existent_repair_line=NULL;
	GList * first = g_list_first(*list);
	gboolean exists = FALSE;

	while (first!=NULL) {
	    existent_repair_line = (Repair_line *) first->data;
	    product_code = d_product_get_product_code(d_repair_line_get_product(existent_repair_line));
	    if (product_code == new_product_code) {
                exists =TRUE;
		break;
	    }
	    first = g_list_next(first);
	}


	if (exists == FALSE) {
	    Stock *stock = d_stock_duplicate (stocks[index]);
	    Repair_line *repair_line = d_repair_line_create_empty ();

	    d_repair_line_set_product (repair_line, (Product *) d_stock_get_piece (stock));
	    d_repair_line_set_stock (repair_line,stock);

	    d_stock_set_piece (stock, NULL);
	    d_repair_line_set_quantity (repair_line, 1.0);
	    d_repair_line_set_earlier_quantity (repair_line, 0.0);
	    d_repair_line_set_line_position_number (repair_line, last_pos+1);
	    d_repair_line_set_historical_vat_type(repair_line, d_product_get_vat_type(d_repair_line_get_product(repair_line)));
	    g_print(_("Historic VAT: %.2f\n"),d_repair_line_get_historical_vat_type(repair_line));
	    g_print(_("Product VAT: &2f\n"),d_product_get_vat_type(d_repair_line_get_product(repair_line)));
	    *list = g_list_insert(*list, repair_line, last_pos++);
	} else {
	    if (d_repair_line_get_line_number(existent_repair_line)!=ULONG_MAX) {
	    //    d_repair_line_set_earlier_quantity (existent_repair_line,d_repair_line_get_quantity(existent_repair_line));
	    }
	    d_repair_line_set_quantity (existent_repair_line,d_repair_line_get_quantity(existent_repair_line)+1);
	}
	index++;
    }
  }

}

/**
 * m_repair_line_apply_discount:
 * @list: Lista enlazada a la que se desea aplicar el descuento.
 * @discount: Descuento que se desea aplicar a la lista de lneas de reparacin.
 *
 * Aplica un descuento a todas las lneas de reparacin de una lista enlazada.
 *
 */
void m_repair_line_apply_discount (GList *list, gdouble discount){

  GList *element = g_list_first (list);
    
  while (element != NULL){    
    d_repair_line_set_discount (element->data,discount);
    element = g_list_next(element);
  }

}

/**
 * m_repair_line_reallocate:
 * @list: Lista con las lneas de reparacin a recolocar.
 *
 * Resigna los nmeros de lnea a una lista de lneas de reparacin por si quedasen alteradas
 * por alguna modificacin, por ejemplo, el borrado de alguna.
 *
 */
void m_repair_line_reallocate (GList *list){

  GList *element = g_list_first (list);
  gint line = 1;
 
  while (element != NULL){
    d_repair_line_set_line_position_number (element->data, line);
    element = g_list_next(element);
    line ++;
  }

}

/**
 * m_repair_line_array_to_glist:
 * @repair_lines: Array con las lneas de reparacin a transformar.
 *
 * Transforma un array con lneas de reparacin en una lista enlazada con
 * las mismas.
 *
 * Returns:
 * La lista enlazada creada a partir del array, %NULL en otro caso.
 */
GList * m_repair_line_array_to_glist (Repair_line ** repair_lines) {
  
  GList *result = NULL;
      
  if (repair_lines!=NULL) {
    gint index = 0;
    while (repair_lines[index]!=NULL) {
      result = g_list_append(result, d_repair_line_duplicate (repair_lines[index]));
      index++;
    }
  }

  m_repair_line_reallocate (result);

  return result;
}


/**
 * m_repair_line_glist_to_array:
 * @list:
 * @subtotal_sum:
 *
 * Construye un array de lneas de reparacin a partir de un lista enlazada con lneas de reparacin. Adems,
 * calcula el importe total de las lneas de la #GList.
 *
 * Returns:
 * Array con las lneas de reparcin. En el @subtotal_sum devuelve el clculo realizado.
 */
Repair_line **m_repair_line_glist_to_array (GList *list, gdouble *subtotal_sum) {

  Repair_line **repair_lines = NULL;

  if (list != NULL){
    GList *element = g_list_last(list);
    gint number_elements = g_list_position(list, element);

    guint line;
    
    repair_lines = (Repair_line **) g_malloc0(sizeof(Repair_line*)*(number_elements+2));
    
    repair_lines[number_elements+1] = NULL;
    
    *subtotal_sum = 0.0;
    
    for (line = 0; line <= number_elements; line ++) {
      gdouble subtotal = G_MINDOUBLE;

      repair_lines[line] = d_repair_line_duplicate (g_list_nth_data(list,line));
      
      subtotal = d_repair_line_get_price (repair_lines[line]);

      if (subtotal != G_MINDOUBLE)
	*subtotal_sum = *subtotal_sum + subtotal*u_round_to_decimal (d_repair_line_get_quantity (repair_lines[line]),3);
      else {

	Product *product = d_repair_line_get_product (repair_lines[line]);
	gdouble price = d_product_get_price (product);
	gdouble quantity = u_round_to_decimal (d_repair_line_get_quantity (repair_lines[line]),3);
	gdouble discount = d_repair_line_get_discount (repair_lines[line]);

	if (price!=G_MINDOUBLE) {
	    *subtotal_sum = *subtotal_sum + (price - (price*discount/100))*quantity;
	} else {
	    *subtotal_sum = *subtotal_sum + 0;
	}

      }
    }
  }

  return repair_lines;
}


/**
 * m_repair_line_insert:
 * @repair_line: Linea de reparacin a insertar en la base de datos.
 * @repair_code: Cdigo de la reparacin a la que pertenece la lnea.
 * @enterprise_code: Cdigo de la empresa en la que se realiz la reparacin
 * @shop_code: Cdigo de la tienda en la que se realiz la reparacin
 * @update: Indica si se inserta o se modifica la reparacin.
 * @command: Comando en el que se ejecutar la sentencia sql.
 * @connection: Conexin a la base de datos.
 * @exception: Objeto que se utiliza para indicar si se ha producido un error y el tipo de error.
 *
 * Inserta una lnea reparacin en la base de datos, asociada a su reparacin, dentro de una transaccin. Si
 * update es %TRUE se modifica la lnea de reparacin, dando por supuesto que ya existe.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_repair_line_insert (Repair_line *repair_line, gchar *repair_code, gchar *enterprise_code, gchar *shop_code, gboolean update, GdaCommand * command, GdaConnection *connection, Exception *exception) {
 
  gboolean result = FALSE;

  if(repair_line != NULL) {
    gchar * query = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    GdaRecordset * recordset_linea = NULL;
    gulong line_number_gulong = d_repair_line_get_line_number (repair_line);
    gchar *line_number = NULL;
    gdouble quantity_gulong = u_round_to_decimal (d_repair_line_get_quantity (repair_line),3);
    gchar *quantity = NULL;
    gdouble subtotal_gdouble = d_repair_line_get_price (repair_line);
    gchar *subtotal = NULL;
    gdouble historic_price_gdouble = d_product_get_price (d_repair_line_get_product(repair_line));
    gdouble historic_vat_type_gdouble = d_repair_line_get_historical_vat_type(repair_line);

    gchar *historic_price = NULL;
    gchar *historic_vat_type = NULL;
    gdouble discount_gint = d_repair_line_get_discount (repair_line);
    gchar *discount = NULL;

    Product *product = d_repair_line_get_product (repair_line);
    Stock *stock= d_repair_line_get_stock (repair_line);
    gulong product_code_gulong = d_product_get_product_code(product);
    gchar *product_code = NULL;   
    gulong warehouse_code_gulong = ULONG_MAX;
    gchar *warehouse_code = NULL;
    gchar ** stop=(gchar **)g_malloc0(sizeof(gchar *));
    gulong enter_code_gulong=ULONG_MAX;
    gulong shop_code_gulong=ULONG_MAX;
    if (stock != NULL) warehouse_code_gulong = d_stock_get_warehouse_code (stock);

    if (warehouse_code_gulong!=ULONG_MAX)
      warehouse_code = g_strdup_printf("%lu",warehouse_code_gulong);
    else warehouse_code = g_strdup("NULL");    
       
    if (line_number_gulong!=ULONG_MAX)
      line_number = g_strdup_printf("%lu",line_number_gulong);
    else line_number = g_strdup("NULL");

    if (quantity_gulong!=G_MINDOUBLE)
      quantity = g_strdup_printf("%.3f",quantity_gulong);
    else quantity = g_strdup("NULL");
    u_commas_to_points (quantity);

    if (subtotal_gdouble != G_MINDOUBLE)
      subtotal = g_strdup_printf("to_number('%.2f','99999999D99')",subtotal_gdouble);
    else subtotal = g_strdup("NULL");

    if (historic_price_gdouble != G_MINDOUBLE)
      historic_price = g_strdup_printf("to_number('%.2f','99999999D99')",historic_price_gdouble);
    else historic_price = g_strdup("NULL");

    if (historic_vat_type_gdouble != G_MINDOUBLE)
      historic_vat_type = g_strdup_printf("to_number('%.2f','99999999D99')",historic_vat_type_gdouble);
    else historic_vat_type = g_strdup("NULL");

    if (discount_gint!=-1)
      discount = g_strdup_printf("to_number('%.2f','999D99')",discount_gint);
    else discount = g_strdup("NULL");

    if (product_code_gulong != ULONG_MAX)
      product_code = g_strdup_printf("%lu",product_code_gulong);
    else product_code = g_strdup("NULL");

    if (update) {
      query = g_strconcat("UPDATE ",m_repair_line_get_table ()," SET ",
			  " codigoproducto = ",product_code,",",
			  " unidades = ",quantity,",",
			  " precio = ",subtotal,",",
			  " dto = ",discount,",",
			  " preciohistorico = ",historic_price,","
			  " tipoIvaHistorico = ",historic_vat_type,
			  " WHERE codigoreparacion = ",repair_code,
			  " AND codigoempresa = ",enterprise_code,
			  " AND codigotienda = ",shop_code,
			  " AND numerolinea = ",line_number,
			  NULL);
      
    } else {
      gchar * temp =  m_model_nextval_sequence ("seqlineareparacion",connection);
      
      query = g_strconcat("INSERT INTO ",m_repair_line_get_table ()," VALUES (",
			  repair_code,",",
			  enterprise_code,",",
			  shop_code,",",
			  warehouse_code,",",
			  temp,",",
			  product_code,",",
			  quantity,",",
			  subtotal,",",
			  discount,",",
			  "NULL,",
			  historic_price,",",
			  historic_vat_type,
			  ")",
			  NULL);          
    
      g_free(temp);
    }

    g_print(_("\nRepair Line Insert: %s\n"),query);

    if (command == NULL) {
      GdaConnection * connection = m_model_get_connection ();
      GdaCommand * local_command = gda_command_new();

      gda_command_set_connection(local_command,connection);
      enter_code_gulong=strtoul(enterprise_code,stop,10);
      shop_code_gulong=strtoul(shop_code,stop,10);
      g_free(stop);

      m_model_connection_begin_mutex_transaction_logged (connection,enter_code_gulong,shop_code_gulong,ULONG_MAX);

      result = m_model_command_execute (&record_set, query, connection,
				     local_command,tuples,0,FALSE);
      
      if (!result) d_exception_set_message (exception, _(M_FIRST_ERROR_MESSAGE));

      if (!update) {
	
	if ((stock != NULL)&&(d_product_get_is_piece (product)))
	  result = result && m_repair_movement_insert (repair_line, repair_code, enterprise_code, shop_code, FALSE, command, connection, exception);

	g_free(query);

	query =g_strdup("select currval('seqlineareparacion')");
	
	m_model_command_execute (&recordset_linea, query,
			      connection,local_command,tuples,0,TRUE);
      } else {
	if ((stock != NULL)&&(d_product_get_is_piece (product)))
	  result = result && m_repair_movement_insert (repair_line, repair_code, enterprise_code, shop_code, TRUE, command, connection, exception);
      }
      
      
      result = m_model_execute_mutex_modification_logged (connection);
      
      g_print(query);
      
      gda_recordset_free(record_set);
      
      if (!update) {
	if (recordset_linea != NULL) {
	  GdaField * field = NULL;

	  gda_recordset_move_first(recordset_linea);
      
	  field = gda_recordset_field_name(recordset_linea,"currval");
      
	  if (field!=NULL) {
	    if (!gda_field_is_null(field)){
	      gchar **stop = (gchar **) g_malloc0(sizeof(gchar *));
	      gchar *temp = gda_stringify_value(NULL, 0, field);
	      line_number_gulong = strtoul(temp,stop,10);
	      
	      d_repair_line_set_line_number (repair_line, line_number_gulong);

	      g_free(temp);
	      g_free(stop);
	    }
	    
	    gda_field_free(field);
	  }

	  gda_recordset_free(recordset_linea);
	}
      }
     
      igalia_command_free(command);
    }
    else {
      result = m_model_command_execute (&record_set, query, connection,
				     command,tuples,0,FALSE);
      
      if (!result) d_exception_set_message (exception, _(M_FIRST_ERROR_MESSAGE));

      if (!update) {

	if ((stock != NULL)&&(d_product_get_is_piece (product)))
	  result = result && m_repair_movement_insert (repair_line, repair_code, enterprise_code, shop_code, FALSE, command, connection, exception);
	
	g_free(query);
	
	query =g_strdup("select currval('seqlineareparacion')");
	
	m_model_command_execute (&recordset_linea, query,
			      connection,command,tuples,0,TRUE);     
      } else {
	if ((stock != NULL)&&(d_product_get_is_piece (product)))
	  result = result && m_repair_movement_insert (repair_line, repair_code, enterprise_code, shop_code, TRUE, command, connection, exception);
      }
      
      g_print(query);

      if (!update) {
	if (recordset_linea != NULL) {
	  GdaField * field = NULL;
	  
	  gda_recordset_move_first(recordset_linea);
	  
	  field = gda_recordset_field_name(recordset_linea,"currval");
	  
	  if (field!=NULL) {
	    if (!gda_field_is_null(field)){
	      gchar **stop = (gchar **) g_malloc0(sizeof(gchar *));
	      gchar *temp = gda_stringify_value(NULL, 0, field);
	      line_number_gulong = strtoul(temp,stop,10);
	      
	      d_repair_line_set_line_number (repair_line, line_number_gulong);

	      g_free(temp);
	      g_free(stop);
	    }
	    
	    gda_field_free(field);
	  }
	}
	
	gda_recordset_free(recordset_linea);
      }
      
      if (record_set != NULL) gda_recordset_free(record_set);

/*       result = TRUE; */
    }
    
    g_free(query);
    g_free(line_number);
    g_free(quantity);
    g_free(subtotal);
    g_free(historic_price);
    g_free(discount);
    g_free(product_code);
    g_free(tuples);
  }
  
  return result;
}

/**
 * m_repair_line_is_new:
 * @repair_code: Cdigo de la reparacin para la que se comprobar la existencia de la lnea.
 * @shop_code: Cdigo de la tienda en la que se realiz la reparacin.
 * @enterprise_code: Cdigo de la empresa en la que se realiz la reparacin.
 * @line_number: Nmero de lnea de la reparacin que se desea comprobar.
 *
 * Comprueba si una linea de reparacin existe en la base de datos para una repararacin.
 *
 * Returns:
 * %TRUE si ya existe la lnea de la reparacin, %FALSE en otro caso.
 */
gboolean m_repair_line_is_new (gchar *repair_code, gchar *shop_code, gchar *enterprise_code, gchar *line_number){

  gboolean result = TRUE;
  GdaCommand *command = gda_command_new();
  gchar *gchar_query = NULL;
  GdaConnection *my_connection = m_model_get_connection ();
  GdaRecordset *record_set = NULL;
  gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

  gda_command_set_connection(command,my_connection);

  gchar_query = g_strconcat("SELECT numerolinea FROM lineareparacion",
			    " WHERE codigoreparacion = ", repair_code,			      
			    " AND codigoempresa = ", enterprise_code,
			    " AND codigotienda = ", shop_code,
			    " AND numerolinea = ", line_number,
			    NULL);

  g_print(_("\nQuery: %s\n"),gchar_query);

  m_model_command_execute (&record_set, gchar_query, my_connection,
			command,tuples,0,TRUE);

  igalia_command_free(command);

  result = !(*tuples);

  if (record_set != NULL) 
    gda_recordset_free(record_set);

  g_free(gchar_query);
  g_free(tuples);
    
  return result;

}

/**
 * m_repair_line_delete:
 * @repair_code: Cdigo de la reparacin a la que pertenece la lnea.
 * @shop_code: Cdigo de la tienda en la que se realiz la reparacin.
 * @enterprise_code: Cdigo de la empresa en la que se realiz la reparacin.
 * @line_number: Nmero de lnea de reparacin para la que se desea borrar la lnea.
 * @quantity: Unidades que contena la lnea de reparacin para reestablecer el stock.
 * @stock: Stock que se debe modificar cuando se borre la lnea de reparacin.
 *
 * Borra una lnea de reparacin de la base de datos.
 *
 * Returns:
 * %TRUE si se borra la lnea correctamente, %FALSE en otro caso.
 */
gboolean m_repair_line_delete (gchar *repair_code, gchar *shop_code, gchar *enterprise_code, gchar *line_number, gdouble quantity, Stock *stock) {

  gboolean result = FALSE;
  
  if (repair_code!=NULL) {
    gchar * query;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gchar ** stop=(gchar **)g_malloc0(sizeof(gchar *));
    gulong enter_code_gulong=ULONG_MAX;
    gulong shop_code_gulong=ULONG_MAX;
    
    gda_command_set_connection(command,connection);

    enter_code_gulong=strtoul(enterprise_code,stop,10);
    shop_code_gulong=strtoul(shop_code,stop,10);
    g_free(stop);

/*
 if (stock!=NULL) {
	warehouse_code_gulong = d_stock_get_warehouse_code (stock);
    }
 */
          
    m_model_connection_begin_mutex_transaction_logged (connection,enter_code_gulong,shop_code_gulong,ULONG_MAX);
    
    query = g_strconcat("DELETE FROM ",m_repair_line_get_table ()," WHERE",
			      " codigoreparacion = ", repair_code,
			      " AND codigoempresa = ", enterprise_code,
			      " AND codigotienda = ", shop_code,
			      " AND numerolinea = ", line_number,
			      NULL);

    result = m_model_command_execute (&record_set, query, connection,
				   command,tuples,0,FALSE);

    gda_recordset_free(record_set);    
    
    g_free(query);
    g_free(tuples);

    if ((stock!=NULL)&&(result)) {
      gdouble quantity_stock_gulong = u_round_to_decimal (d_stock_get_quantity (stock),3);
      
      d_stock_set_quantity (stock,quantity_stock_gulong + quantity);
      
      m_stock_update (stock, connection, command);
    }
      
    result = m_model_execute_mutex_modification_logged (connection) && result;

    igalia_command_free(command);
    
  }
    
  return result;
}

/**
 * m_repair_line_get_changed:
 * @repair_lines: Lneas de reparacin que se modificaron.
 *
 * Establece las unidades anteriores, una vez se han modificado las lneas.
 *
 */
void m_repair_line_get_changed (Repair_line **repair_lines) {

  if (repair_lines!=NULL) {
    gint index = 0;
    while (repair_lines[index]!=NULL) {
      d_repair_line_set_earlier_quantity (repair_lines[index], d_repair_line_get_quantity (repair_lines[index]));

      index++;
    }
  }
  
}
