/* stock_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "piece_model.h"
#include "product_model.h"
#include "warehouse_model.h"
#include "stock_model.h"
#include "find_stocked_pieces_model.h"

#define TABLE      "Stock"
#define ATTRIBUTES "codigoproducto as codigoproductostock, \
    codigoAlmacen as codigoalmacenstock, \
    codigoEmpresa as codigoempresastock, \
    unidades as unidadesstock, \
    stockMinimo as stockminimostock, \
    ubicacion     as ubicacionstock "

#define D_ACCUMULATE_ATTRIBUTES "codigoproducto as codigoproductostock "

#define D_SUM_ATTRIBUTES "sum(stock.unidades) as sumstock"

#define D_NOT_ACCUMULATE_ATTRIBUTES "codigoAlmacen as codigoalmacenstock, \
    codigoEmpresa as codigoempresastock, \
    unidades as unidadesstock, \
    stockMinimo  as stockminimostock, \
    ubicacion     as ubicacionstock "

static gboolean exists_stock(Stock *stock, GdaConnection *connection, GdaCommand *command);



/**
 * m_stock_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_stock_get_table () {
    return TABLE;
}


/**
 * m_stock_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_stock_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_stock_get_sumstock_attributes:
 *
 * Funcin que obtiene los atributos de agrupacin por suma.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_stock_get_sumstock_attributes () {
    return D_SUM_ATTRIBUTES;
}


/**
 * m_stock_get_accumulative_attributes:
 *
 * Funcin que obtiene los atributos acumulativos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_stock_get_accumulative_attributes () {
    return D_ACCUMULATE_ATTRIBUTES;
}


/**
 * m_stock_get_not_accumulative_attributes:
 *
 * Funcin que obtiene los atributos no acumulativos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_stock_get_not_accumulative_attributes () {
  return D_NOT_ACCUMULATE_ATTRIBUTES;
}


/**
 * m_stock_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Stock con los campos rellenados con los datos de la base de datos.
 */
Stock * m_stock_create (GdaRecordset *rs) {
    gchar  *temp;
    GdaField * field = NULL;
    Stock * stock = d_stock_create_empty();

    d_stock_set_piece (stock, m_piece_create (rs));
                                         
    field = gda_recordset_field_name(rs,"codigoempresastock");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    stock->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoalmacenstock");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    stock->warehouse_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"unidadesstock");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    stock->quantity = u_round_to_decimal (igalia_field_get_double_value (field),3);
	    stock->previous_quantity = stock->quantity;
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"sumstock");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
            stock->quantity = u_round_to_decimal (igalia_field_get_double_value (field),3);
	    stock->previous_quantity = stock->quantity;
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"stockminimostock");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    stock->minimum_stock = u_round_to_decimal (igalia_field_get_double_value (field),3);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "ubicacionstock");
    if (field!=NULL){
	if (!gda_field_is_null(field)) {
	    temp = gda_stringify_value(NULL, 0, field);
	    stock->location = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    return stock;
}


/**
 * exists_stock:
 * @stock: Stock que se desea comprobar.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la sentencia sql.
 *
 * Comprueba si existe stock para un producto en la base de datos dentro de una transaccin.
 *
 * Returns:
 * %TRUE si existe, %FALSE en otro caso.
 */
static gboolean exists_stock(Stock *stock, GdaConnection *connection, GdaCommand *command) {

    gboolean result = FALSE;
    gchar         *gchar_query_stock = NULL;
    gchar         *gchar_query_stock_aux = NULL;
    GdaRecordset  *record_set = NULL;
    gchar         *temp = NULL;
    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong result_gulong  = 0;

    gchar_query_stock = g_strconcat("SELECT codigoProducto FROM ",
				    m_stock_get_table (),
				    " WHERE (codigoProducto = ",
				    NULL);

    temp = g_strdup_printf("%lu",d_product_get_product_code ((Product *)d_stock_get_piece (stock)));
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
				        NULL);
    /*
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
					") AND (codigoEmpresa = ",
					NULL);
    g_free(temp);
    g_free(gchar_query_stock);
    gchar_query_stock = gchar_query_stock_aux;
    gchar_query_stock_aux = NULL;

    temp = g_strdup_printf("%lu",d_stock_get_enterprise_code (stock));
					*/
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
					") AND (codigoAlmacen = ",
					NULL);

    g_free(temp);
    g_free(gchar_query_stock);
    gchar_query_stock = gchar_query_stock_aux;
    gchar_query_stock_aux = NULL;

    temp = g_strdup_printf("%lu",d_stock_get_warehouse_code (stock));
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
					") ",
					NULL);
    g_free(temp);
    g_free(gchar_query_stock);
    gchar_query_stock = gchar_query_stock_aux;
    gchar_query_stock_aux = NULL;

    result = m_model_command_execute (&record_set, gchar_query_stock, connection, command, tuples, 0, TRUE);

    g_free(gchar_query_stock);
    gda_recordset_free(record_set);

    result_gulong = *tuples;
    return result_gulong;
}

/**
 * exists_stock_in_other_warehouse:
 * @stock: Stock de otra tienda del que  se comprobar la existencia.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la sentencia sql.
 *
 * Comprueba si existe stock en otro almacn para un producto en la base de datos dentro de una transaccin.
 *
 * Returns:
 * %TRUE si existe, %FALSE en otro caso.
 */
/*gboolean exists_stock(Stock *stock,gulong codigoAlmacen,gulong codigoEmpresa) {

    gboolean result = FALSE;
    gchar         *gchar_query_stock = NULL;
    gchar         *gchar_query_stock_aux = NULL;
    GdaRecordset  *record_set = NULL;
    gchar         *temp = NULL;
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();

    gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong result_gulong  = 0;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    gchar_query_stock = g_strconcat("SELECT codigoProducto FROM ",
				    m_stock_get_table (),
				    " WHERE (codigoProducto = ",
				    NULL);

    temp = g_strdup_printf("%lu",d_product_get_product_code ((Product *)d_stock_get_piece (stock)));
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
				        NULL);
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
					") AND (codigoAlmacen = ",
					NULL);

    g_free(temp);
    g_free(gchar_query_stock);
    gchar_query_stock = gchar_query_stock_aux;
    gchar_query_stock_aux = NULL;

    temp = g_strdup_printf("%lu",d_stock_get_warehouse_code (stock));
    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
					") ",
					NULL);
    g_free(temp);
    g_free(gchar_query_stock);
    gchar_query_stock = gchar_query_stock_aux;
    gchar_query_stock_aux = NULL;

    result = m_model_command_execute (&record_set, gchar_query_stock, connection, command, tuples, 0, TRUE);

    g_free(gchar_query_stock);
    gda_recordset_free(record_set);

    result_gulong = *tuples;
    return result_gulong;
}
*/

/**
 * m_stock_insert:
 * @stock: Stock a introducir en la Bd.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecutarn las sentencias sql.
 *
 * Inserta un stock en la base de datos para un producto.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_stock_insert (Stock *stock, GdaConnection *connection, GdaCommand *command) {
    gboolean result = FALSE;

    if (d_product_get_is_piece ((Product *)d_stock_get_piece (stock))) {
	gchar         *gchar_query_stock = NULL;
	gchar         *gchar_query_stock_aux = NULL;
	GdaRecordset  *record_set = NULL;
	gchar         *temp = NULL;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));

	if (!exists_stock(stock, connection, command)) {

	    gchar_query_stock = "";
	    gchar_query_stock = g_strconcat("INSERT INTO ",m_stock_get_table (),
					    " VALUES ",
					    "('", NULL);

	    temp = g_strdup_printf("%lu",d_product_get_product_code ((Product *)d_stock_get_piece (stock)));
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
						"', '",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

	    temp = g_strdup_printf("%lu",m_warehouse_get_enterprise_code_from_code (d_stock_get_warehouse_code (stock)));
//	    temp = g_strdup_printf("%lu",d_stock_get_enterprise_code (stock));
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
						"', '",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

	    temp = g_strdup_printf("%lu",d_stock_get_warehouse_code (stock));
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
						"', ",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

	    if (d_stock_get_quantity (stock)!=G_MINDOUBLE) {
		temp = g_strdup_printf("'%.3f'",d_stock_get_quantity (stock));
	    } else {
                temp = g_strdup("NULL");
	    }
	    u_commas_to_points (temp);
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
						", ",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

	    if (d_stock_get_minimum_stock (stock)!=G_MINDOUBLE)  {
		temp = g_strdup_printf("'%.3f'",d_stock_get_minimum_stock (stock));
	    } else {
                temp = g_strdup("NULL");

	    }
	    u_commas_to_points (temp);
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock, temp,
						", ",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

	    if (d_stock_get_location (stock)!=NULL) {
		temp = d_stock_get_location (stock);
		gchar_query_stock_aux = g_strconcat(gchar_query_stock," '", temp,
						    "') ", NULL);
		g_free(temp);
		g_free(gchar_query_stock);
		gchar_query_stock = gchar_query_stock_aux;
		gchar_query_stock_aux = NULL;
	    } else {
		gchar_query_stock_aux = g_strconcat(gchar_query_stock,
						    "null) ", NULL);
		g_free(gchar_query_stock);
		gchar_query_stock = gchar_query_stock_aux;
		gchar_query_stock_aux = NULL;
	    }

	    g_print(_("\nStock:%s\n"),gchar_query_stock);
	    result = m_model_command_execute (&record_set, gchar_query_stock, connection, command, tuples, 0, FALSE);

	    g_free(gchar_query_stock);
	    gda_recordset_free(record_set);
	} else {
	    result = m_stock_update (stock, connection, command);
	}
	g_free(tuples);
    }
    return result;
}

/**
 * m_stock_update:
 * @stock: Stock a modificar en la Bd.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecutarn las sentencias sql.
 *
 * Modifica las unidades de stock de un producto en la base de datos.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_stock_update (Stock *stock, GdaConnection *connection, GdaCommand *command) {
    gboolean result = FALSE;

    if (d_product_get_is_piece ((Product *)d_stock_get_piece (stock))) {
	gchar         *gchar_query_stock = NULL;
	gchar         *gchar_query_stock_aux = NULL;
	GdaRecordset  *record_set = NULL;
	gchar         *temp = NULL;
        gboolean      first=TRUE;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	if (exists_stock(stock, connection, command)) {

	    gchar_query_stock = "";
	    gchar_query_stock = g_strconcat("UPDATE ",m_stock_get_table (),
					    " SET ",
					    NULL);

	    if (d_stock_get_quantity (stock)!=G_MINDOUBLE) {
		gdouble quantity_dif =d_stock_get_quantity(stock)-d_stock_get_previous_quantity(stock);

		g_print(_("\nBefore:  %.3f"),d_stock_get_quantity(stock));
		g_print(_("\nAfter: %.3f"),d_stock_get_previous_quantity(stock));
		temp = g_strdup_printf("%.3f",quantity_dif);

		u_commas_to_points (temp);
		gchar_query_stock_aux = g_strconcat(gchar_query_stock,
						    " unidades = unidades + '",temp,
						    "' ",
						    NULL);


		g_free(temp);
		g_free(gchar_query_stock);
		gchar_query_stock = gchar_query_stock_aux;
		gchar_query_stock_aux = NULL;
                first=FALSE;
	    }
	    if (d_stock_get_minimum_stock (stock)!=G_MINDOUBLE)  {
		temp = g_strdup_printf("%.3f",d_stock_get_minimum_stock (stock));
		u_commas_to_points (temp);
                if (first!=FALSE)
		    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
							" stockMinimo ='",temp,
							"' ",
							NULL);
		else
		    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
							" ,stockMinimo ='",temp,
							"' ",
							NULL);

		g_free(temp);
		g_free(gchar_query_stock);
		gchar_query_stock = gchar_query_stock_aux;
		gchar_query_stock_aux = NULL;
                first=FALSE;
	    }
	    if (d_stock_get_location (stock)!=NULL) {
		temp = d_stock_get_location (stock);
                if (first !=FALSE)
		    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
							" ubicacion ='",temp,
							"' ",
							NULL);
		else
		    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
							", ubicacion ='",temp,
							"' ",
							NULL);

		g_free(temp);
		g_free(gchar_query_stock);
		gchar_query_stock = gchar_query_stock_aux;
		gchar_query_stock_aux = NULL;
                first=FALSE;
	    }

	    temp = g_strdup_printf("%lu",d_product_get_product_code ((Product *)d_stock_get_piece (stock)));
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
						" WHERE (codigoProducto = ",temp,
						") AND ",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

 /*           temp = g_strdup_printf("%lu",d_stock_get_enterprise_code (stock));
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
						" (codigoEmpresa = ",temp,
						") AND ",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;
 */
	    temp = g_strdup_printf("%lu",d_stock_get_warehouse_code (stock));
	    gchar_query_stock_aux = g_strconcat(gchar_query_stock,
						" (codigoAlmacen = ",temp,
						") ",
						NULL);
	    g_free(temp);
	    g_free(gchar_query_stock);
	    gchar_query_stock = gchar_query_stock_aux;
	    gchar_query_stock_aux = NULL;

	    g_print(_("\nStock:%s\n"),gchar_query_stock);
	    result = m_model_command_execute (&record_set, gchar_query_stock, connection, command, tuples, 0, FALSE);

	    //if (result != FALSE)
	    //    d_stock_set_previous_quantity(stock,d_stock_get_quantity(stock));

	    g_free(gchar_query_stock);
	    gda_recordset_free(record_set);
	} else {
	    result = m_stock_insert (stock, connection, command);
	}
	g_free(tuples);
    }
    return result;
}



