/* supplier_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "supplier_model.h"
#include "participant_model.h"
#include "util.h"

#define TABLE         "Proveedor"
#define IDENTIFICATOR "codigoParty"  

/**
 * m_supplier_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_supplier_get_table ()
{  
       return TABLE;
}




/**
 * m_supplier_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Supplier con los campos rellenados con los datos de la base de datos.
 */
Supplier * m_supplier_create (GdaRecordset *rs) {

    Supplier * supplier;

    supplier = d_supplier_create_empty();

    supplier->participant = m_participant_create (rs,"supplier");

    return supplier;
}


/**
 * m_supplier_find_all:
 *
 * Busca todos los proveedores de cualquier empresa del grupo.
 *
 * Returns:
 * Array con los proveedores, %NULL en caso de error.
 */
Supplier ** m_supplier_find_all () {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Supplier * supplier;
    Supplier ** suppliers_list;
    gchar * gchar_query;
    gulong * tuples;
    gint * column;
    gint index;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat("SELECT ",
			      " Participante.codigoParty as codigopartyparticipantsupplier, ",
			      " Participante.nombre as nombreparticipantsupplier, ",
			      " Participante.cif as cifparticipantsupplier, ",
			      " Participante.nick as nickparticipantsupplier, ",
			      " Participante.telefono as telefonoparticipantsupplier, ",
			      " Participante.telefono2 as telefono2participantsupplier, ",
			      " Participante.fax as faxparticipantsupplier, ",
			      " Participante.email as emailparticipantsupplier, ",
			      " Participante.calle as calleparticipantsupplier, ",
			      " Participante.numero as numeroparticipantsupplier, ",
			      " Participante.piso as pisoparticipantsupplier, ",
			      " Participante.CP as cpparticipantsupplier, ",
			      " Participante.municipio as municipioparticipantsupplier, ",
			      " Participante.nombrecorto as shortNAmeparticipantsupplier ",
			      " FROM ",m_supplier_get_table (),",",m_participant_get_table (),
			      " WHERE PROVEEDOR.CODIGOPARTY=PARTICIPANTE.CODIGOPARTY order by participante.nick "
			      ,NULL
			     );

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0,TRUE);
    g_print(gchar_query);
    g_free(gchar_query);

    column = (gint *) g_malloc0(sizeof(gint));

    if (record_set!=NULL) {
	if (*tuples!=0) {
	    suppliers_list = (Supplier **) g_malloc0(sizeof(Supplier*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		(*column)=0;
		supplier = m_supplier_create (record_set);
		suppliers_list[index-1] = supplier;
		index = gda_recordset_move_next(record_set);
	    }
	    suppliers_list[*tuples]=NULL;

	} else {
	    suppliers_list=NULL;
	}
    } else {
	suppliers_list=NULL;
    }

    igalia_command_free(my_command);
    gda_recordset_free(record_set);

    g_print(_("Freeing database completed\n"));
    g_free(tuples);
    g_free(column);


    return suppliers_list;
}

/**
 * m_supplier_find_by_code:
 * @supplier_code: Cdigo de proveedor que se desea buscar.
 *
 * Busca un proveedor por cdigo de proveedor.
 *
 * Returns:
 * Proveedor de cualquier empresa del grupo que cumple la condicin.
 */
Supplier * m_supplier_find_by_code (gint supplier_code) {
    GdaCommand         * my_command;
    GdaConnection      * my_connection = m_model_get_connection ();
    GdaRecordset       * record_set = NULL;
    Supplier * supplier=NULL;
    gchar * gchar_query;
    gchar * aux;
    gulong * tuples;
    gint * column;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux = g_strdup_printf("%d",supplier_code);

    gchar_query = g_strconcat("SELECT ",
			      " Participante.codigoParty as codigopartyparticipantsupplier, ",
			      " Participante.nombre as nombreparticipantsupplier, ",
			      " Participante.cif as cifparticipantsupplier, ",
			      " Participante.nick as nickparticipantsupplier, ",
			      " Participante.telefono as telefonoparticipantsupplier, ",
			      " Participante.telefono2 as telefono2participantsupplier, ",
			      " Participante.fax as faxparticipantsupplier, ",
			      " Participante.email as emailparticipantsupplier, ",
			      " Participante.calle as calleparticipantsupplier, ",
			      " Participante.numero as numeroparticipantsupplier, ",
			      " Participante.piso as pisoparticipantsupplier, ",
			      " Participante.CP as cpparticipantsupplier, ",
			      " Participante.municipio as municipioparticipantsupplier, ",
			      " Participante.nombreCorto as shortNameparticipantsupplier ",
			      " FROM PROVEEDOR,PARTICIPANTE WHERE ",
			      "(PARTICIPANTE.CODIGOPARTY=PROVEEDOR.CODIGOPARTY) ",
			      "AND (PROVEEDOR.CODIGOPARTY=",aux,")",NULL);



    g_free(aux);

    g_print(gchar_query);
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0, TRUE);
    g_free(gchar_query);


    column = (gint *) g_malloc0(sizeof(gint));
    (*column)=0;
    if (record_set!=NULL) {
	if (*tuples!=0) {
	    gda_recordset_move_first(record_set);
	    supplier = m_supplier_create (record_set);

	} else {
	    supplier=NULL;
	}
    } else {
	supplier=NULL;
    }

    igalia_command_free(my_command);
    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
    g_print(_("Freeing database completed\n"));
    g_free(tuples);
    g_free(column);
    return supplier;
}


/**
 * m_supplier_clear_array:
 * @suppliers: Array de proveedores a liberar.
 *
 * Libera la memoria utilizada por el array de proveedores.
 *
 */
void m_supplier_clear_array (Supplier ** suppliers) {
    if (suppliers!=NULL) {
	gint index = 0;
	while (suppliers[index]!=NULL) {
	    d_supplier_clear (suppliers[index]);
	    index++;
	}
    }
    g_free(suppliers);

}

/**
 * m_supplier_get_name_from_code:
 * @code: Cdigo de proveedor del que se quiere buscar el nombre
 *
 * Busca el nombre de un proveedor a partir de cdigo de proveedor.
 *
 * Returns:
 * Nombre del proveedor que cumple la condicin.
 * 
 */
gchar *m_supplier_get_name_from_code (gulong code) {
    gchar *name = NULL;
    Supplier * supplier = NULL;

    supplier =  m_supplier_find_by_code (code);
    name = d_participant_get_nick (d_supplier_get_participant (supplier));
    d_supplier_clear (supplier);
//    g_free(supplier);

    return name;
}


