/* shoplogs2todos.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "config.h"
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include "log2todo.h"
#include "shoplogs2todos.h"

#define R_QUERY_EXTENSION ".query"
#define R_TODO_EXTENSION  ".todo"
#define R_BLOCK_SIZE   4*1024

static void copy_valid(const gchar* in_file_name, const gchar *out_file_name);
static gint migration(gchar *in_directory_name, gchar *out_file_name);
static gint is_valid(const gchar *query);
static gchar *help(gchar *executable);


static const gchar *valids[] = {
    "PRODUCTO", "MANODEOBRA", "PIEZA", "PARTICIPANTE", "DIRECCIONENVIO",
    "CLIENTE", "EMPRESA", "TIENDA", "BANCO", "OFICINA",
    "CUENTACORRIENTE", "MOVIMIENTO", "CCPARTY", "PROVEEDOR", "TALLER",
    "PARTICULAR", "CONSORCIO", "COMPANIA", "POLIZA", "RECIBO",
    "TASACION", "TASACION", "VEHICULO", "ALMACEN", "STOCK",
    "DOCUMENTO", "NECESITADOCUMENTO","MOVIMIENTOPORREPARACION",
    NULL};

static gint is_valid(const gchar *query) {
    if (query!=NULL) {
	char *temp = g_strdup(query);
	gint  index = 0;

	g_strup(temp);
	while(valids[index]!=NULL) {
	    if(strstr(temp, valids[index])!=NULL) {
                break;
	    }
	    index++;
	}

	g_free(temp);

	if(valids[index]==NULL) {
	    return 0;
	}
	return 1;
    }
    return 0;
}

static void copy_valid(const gchar* in_file_name, const gchar *out_file_name) {
    FILE  *in_file  = fopen(in_file_name, "r");
    FILE  *out_file = fopen(out_file_name, "a");
    gchar *buffer = NULL;
    gint   old_position = 0;
    gint   new_position = 0;

    buffer = (gchar *) g_malloc0(sizeof(gchar *)*R_BLOCK_SIZE);
    new_position = r_read_line (&buffer, old_position, in_file);

    while((new_position!=old_position) && (buffer!=NULL)) {

	if (buffer!=NULL) {
	    if (is_valid(buffer)) {
		fwrite(buffer, strlen(buffer), 1, out_file);
	    }
	}
	new_position = r_read_line (&buffer, old_position, in_file);
    }

    if(buffer!=NULL) {
	g_free(buffer);
    }

    fclose(in_file);
    fclose(out_file);
}

static gint migration(gchar *in_directory_name, gchar *out_file_name) {
    DIR           *in_directory = NULL;
    struct dirent *dir_entry     = NULL;
    struct stat   *statbuf;
    gchar         *in_file_name  = NULL;
    gchar        **file_name_query_array = (gchar **) g_malloc(sizeof(gchar *)*2);
    gchar        **aux  = NULL;
    gchar         *temp = NULL;
    gint           files = 0;

    if ((in_directory = opendir(in_directory_name))==NULL) {
	perror("replication/generate_migration.c/migration:opendir");
        m_model_destroy_connection ();
	exit(-1);
    }

    dir_entry = readdir(in_directory);
    while (dir_entry!=NULL) {

	statbuf      = (struct stat *) g_malloc(sizeof(struct stat));
        temp = g_strconcat(in_directory_name, dir_entry->d_name, NULL);
	stat(temp, statbuf);
        g_free(temp);

	if (S_ISDIR(statbuf->st_mode)) {
	    dir_entry = readdir(in_directory);
            g_free(statbuf);
            continue;
	}

	g_free(statbuf);
	files++;
	aux = file_name_query_array;
	file_name_query_array = (gchar **)realloc(aux, sizeof(gchar *)*(files+1));

	in_file_name = g_strdup_printf("%s%s", in_directory_name, dir_entry->d_name);
	file_name_query_array[files-1] = g_strconcat(in_directory_name, dir_entry->d_name, R_QUERY_EXTENSION, NULL);

	r_log_to_todo (in_file_name, file_name_query_array[files-1]);

	dir_entry = readdir(in_directory);
	g_free(in_file_name);
    }
    file_name_query_array[files] = NULL;

    files = 0;
    while(file_name_query_array[files]!=NULL) {
	copy_valid(file_name_query_array[files], out_file_name);
	files++;
    }

    // Liberations
    files = 0;
    while(file_name_query_array[files]!=NULL) {
	g_free(file_name_query_array[files]);
	file_name_query_array[files] = NULL;
	files++;
    }
    g_free(file_name_query_array);

    if (closedir(in_directory)==-1) {
	perror("replication/generate_migration.c/migration:closedir");
        m_model_destroy_connection ();
	exit(-1);
    }

    return 1;

}

static gchar *help(gchar *executable) {
    gchar *help_text = NULL;

    help_text = g_strconcat(_("Use: "), executable, _(" (log central directory) (migration file_name)\n"), NULL);
    return help_text;
}

int main(int argc, char *argv[]) {

    gint index = 1;
    if (argc!=3) {
	g_print("\n%s\n", help(argv[0]));
        m_model_destroy_connection ();
	exit(-1);
    }

    while (index--) {
	migration(argv[1], argv[2]);
    }

    return 0;

}
