/* parse_file.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "log2todo.h"
#include "shop_model.h"
#include "enterprise_model.h"
#include "enterprise.h"
#include "model.h"
#include "parse_file.h"

// This function parse a log_file
// A log file is well build if there are only transactions
// which are not merge, in other words, a "Begin Transaction"
// is always followed by a "End Transaction", and a
// "End Transaction" is always followed by a "Begin Transaction".

/**
 * r_parse_file:
 * @file_name: Fichero a parsear
 *
 * Parsea un fichero de log para comprobar si el fichero est correctamente construdo.
 * Comprueba si a cada "Begin Transaction" le sigue un "End Transaction" y si las ternas (codigoEmpresa,codigoTienda,codigoAlmacen)
 * son vlidas.
 *
 * Returns:
 * 0 si se ha parseado correctamente el fichero de log, en otro caso la lnea en la que fall el parseo.
 */
gint r_parse_file (const gchar *file_name) {
    gboolean  well_build = TRUE;
    gint      old_position = 0;
    gint      new_position = 0;
    gchar    *buffer = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    FILE     *in_file = NULL;
    gint      line = 0;
    Shop ** shops =m_shop_find_all ();
    PairShopEnterprise ** pairs = NULL;
    gint index_of_shop=0;
    gint      return_value = 0;

    while (shops[index_of_shop]!=NULL) {
        index_of_shop++;
    }
    pairs = g_malloc0(sizeof(PairShopEnterprise*)*(index_of_shop+1));
    index_of_shop=0;
    while (shops[index_of_shop]!=NULL) {
	Enterprise * enterprise = m_enterprise_find_by_shop (d_participant_get_code (d_shop_get_participant (shops[index_of_shop])));
	gulong enterprise_code = d_participant_get_code (d_enterprise_get_participant (enterprise));
        pairs[index_of_shop] = (PairShopEnterprise *) g_malloc0(sizeof(PairShopEnterprise));
	pairs[index_of_shop]->shop_code=g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (shops[index_of_shop])));
	pairs[index_of_shop]->enterprise_code=g_strdup_printf("%lu",enterprise_code);
        d_enterprise_clear (enterprise);
	index_of_shop++;
    }

    in_file  = fopen(file_name, "r");

    if(in_file==NULL) {
	perror("replication/parse_file.c/valid:fopen");
	printf(_("\tTrying open: %s\n"), file_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    new_position = r_read_line (&buffer, old_position, in_file);
    line++;

    while ((new_position!=old_position) && well_build
	   && (buffer!=NULL)) {

	while((new_position!=old_position)
	      && !strstr(buffer, R_BEGIN_TRANSACTION)
	      && !strstr(buffer, R_END_TRANSACTION)
	      && !r_is_query (buffer)
              && !r_is_value (buffer)
	      && (buffer!=NULL)) {

	    new_position = r_read_line (&buffer, old_position, in_file);
	    line++;
	}

	if((new_position!=old_position)
	   && strstr(buffer, R_BEGIN_TRANSACTION)
	   && (buffer!=NULL)) {

	    line++;
	    well_build = FALSE;
	    if (strstr(buffer, R_BEGIN_TRANSACTION)) {
		if ((new_position!=old_position)
		    && (buffer!=NULL)) {
		    gint index_of_pair=0;
		    gboolean coincide=FALSE;

		    while (pairs[index_of_pair]!=NULL) {
			gchar * ulong_max_char = g_strdup_printf("%lu",ULONG_MAX);
			gchar * char_to_compare1 = g_strconcat("(",pairs[index_of_pair]->enterprise_code,",",pairs[index_of_pair]->shop_code,NULL);
			gchar * char_to_compare2 = g_strconcat("(",pairs[index_of_pair]->enterprise_code,",",ulong_max_char,NULL);
			if ((strstr(buffer, char_to_compare1)) || (strstr(buffer,char_to_compare2))) {
			    coincide=TRUE;
			}
			index_of_pair++;
		    }
		    if (coincide == FALSE) {
			well_build = FALSE;
                        break;
		    }

		}
	    }
	}



	if((new_position!=old_position)
	   && strstr(buffer, R_END_TRANSACTION)
	   && (buffer!=NULL)) {
	    line++;
	    well_build = FALSE;
	    continue;
	}

	new_position = r_read_line (&buffer, old_position, in_file);

	while((new_position!=old_position)
	      && !strstr(buffer, R_BEGIN_TRANSACTION)
	      && !strstr(buffer, R_END_TRANSACTION)
	      && (buffer!=NULL)) {

	    new_position = r_read_line (&buffer, old_position, in_file);
	    line++;
	}

	if ((new_position!=old_position)
	    && (buffer!=NULL)) {
	    if (strstr(buffer, R_END_TRANSACTION)) {
		well_build = TRUE;
	    } else if (strstr(buffer, R_BEGIN_TRANSACTION)) {
		well_build = FALSE;
	    }
	}
	new_position = r_read_line (&buffer, old_position, in_file);
	line++;
    }

    g_free(buffer);


    if (well_build) {
        return_value =  0;
    } else {
        return_value = line;
    }

    if (pairs!=NULL) {
        gint index_of_pair = 0;
	while (pairs[index_of_pair]!=NULL) {
	    if (pairs[index_of_pair]->enterprise_code !=NULL) {
                g_free(pairs[index_of_pair]->enterprise_code);
	    }
	    if (pairs[index_of_pair]->shop_code !=NULL) {
                g_free(pairs[index_of_pair]->shop_code);
	    }
            g_free(pairs[index_of_pair]);
            index_of_pair++;
	}
        g_free(pairs);
    }
    g_print(_("If file is well build returns 0 otherwise returns the last line parsed: %d\n"), return_value);

    return return_value;

}

