/* util.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "util.h"
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/mman.h>
#include <ctype.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>

#define U_LOCK_EXTENSION ".lock"
#define U_HALF_SECOND  500000000

static int             sem_id = -1;
static struct timespec lock_time = {0,U_HALF_SECOND};


/**
 * u_commas_to_points: 
 * @text: Texto a transformar.
 *
 * Transforma un texto con comas a puntos. Utilizado para la insercin en la
 * BD, ya que los decimales en C son con comas y para la BD son con puntos.
 *
 */
void u_commas_to_points (char *text) {
    while (*text!=0) {
	if (*text==',') {
            *text='.';
	}
	text++;
    }
}

/**
 * u_points_to_commas:
 * @text: Texto a transformar.
 *
 * Transforma un texto con puntos a comas. Utilizado para le recuperacin
 * de valores con decimales de la base de datos.
 *
 */
void u_points_to_commas (char *text) {
    while (*text!=0) {
	if (*text=='.') {
            *text=',';
	}
	text++;
    }
}

/**
 * u_blanks_to_percentage: 
 * @text: Texto a transformar.
 *
 * Transforma una cadena con blancos en una cadena con porcentajes para poder separar en tokens
 * de bsqueda en la BD.
 *
 */
void u_blanks_to_percentage (char *text) {
    g_strstrip(text);
    while (*text!=0) {
	if ((*text==' ')||(*text=='\n')||(*text=='\t')) {
	    *text='%';
	}
	text++;
    }
}

/**
 * u_delete_intros:
 * @text: Texto a transformar.
 *
 * Borra los "\n" de un texto.
 *
 */
void u_delete_intros (char *text) {
    while (*text!=0) {
	if (*text=='\n') {
            *text=' ';
	}
	text++;
    }
}


/* Function to get the index to the value "val" in the array "rows"*/
/**
 * u_get_index: 
 * @rows: Array en el que se quiere encontrar un valor.
 * @val: Valor a buscar en el array.
 * 
 * Busca el ndice que ocupa un valor en un GArray de enteros.
 *
 * Returns:
 * El ndice del valor solicitado.
 *
 */
gint u_get_index (GArray * rows,gint val) {
  gint i=0;
    gint * arrayVal=NULL;

    arrayVal = &g_array_index(rows,gint,i);
    while (val!=(*arrayVal)) {
	i++;
	arrayVal = &g_array_index(rows,gint,i);
    }

    return i;
}


/**
 * u_reverse:
 * @string: Cadena que se desea invertir.
 *
 * Invierte una cadena de caracteres.
 *
 * Returns:
 * Cadena invertida.
 *
 */
gchar *
u_reverse (gchar *string) {
    gint j = 0;
    gint i;
    gchar *temp;
    gint length = strlen(string);
    temp = (gchar *) g_malloc0(length+1);

    for(i=length-1; i>=0; i--, j++) {
	temp[j] = string[i];
    }
    temp[j]='\0';
    return temp;
}

/**
 * u_weight_legal_id:
 * @string: Cadena de la que se calcularn los pesos de cada carcter.
 *
 * Se calculan los pesos de cada carcter de la cadena para despus calcular el digito
 * de control a partir de los cdigos de banco y oficina y del nmero de cuenta.
 *
 * Returns:
 * Los pesos en una cadena.
 * 
 */
gchar *
u_weight_legal_id (gchar *string) {
    guint64 weight[] = {0, 6, 3, 7, 9, 10, 5, 8, 4, 2, 1};
    guint64 total = 0;
    gint j;
    gint i;
    gint length = strlen(string);
    gchar *temp;

    j = length ;

    temp = (gchar *) g_malloc0(2);

    for(i = 0; i<length; i++) {
	strncpy(temp, &string[i], 1);
	temp[1] = '\0';

	total = total + (strtoul(temp, (gchar **) NULL, 10) * weight[j]);
        j--;
    }

    i = 11 - (total % 11);

    if (i == 10) {
        i = 1;
    } else if (i == 11) {
        i = 0;
    }

    sprintf(temp, "%d", i);

    return temp;
}

/**
 * u_calculate_control_digit:
 * @entity: Entidad financiera a la que pertenece la cuenta corriente.
 * @office: Oficina bancaria que pertenece a la entidad.
 * @account: Nmero de cuenta a la que pertenece la cuenta corriente.
 *
 * calcula el dgito de control a partir de los cdigos de banco,oficina y nmero  de
 * cuenta.
 *
 * Returns:
 * Dgito de control.
 * 
 */
gchar *
u_calculate_control_digit (gchar *entity, gchar *office, gchar *account) {
    gchar *temp;
    gchar *temp2;
    gchar *dig1;
    gchar *dig2;

    if(!strcmp(entity, "")) {
        entity = g_strdup("0");
    }
    if(!strcmp(account, "")) {
        account = g_strdup("0");
    }
    temp = g_strconcat(entity,office,NULL);
    dig1 = (gchar *) u_weight_legal_id (temp);
    dig2 = (gchar *) u_weight_legal_id (account);
    g_free(temp);
    temp2 = g_strconcat(dig1,dig2);
    g_free(dig1);
    g_free(dig2);

    return temp2;

}


/**
 * u_legal_id_calculate_letter:
 * @legal_id: Nmero de dni al que se le quiere calcular la letra.
 *
 * Calcula la letra del cif a partir del nmero de DNI.
 *
 * Returns:
 * Letra del cif obtenida a partir del dni.
 *
 */
gchar *
u_legal_id_calculate_letter (gulong legal_id) {
    gchar letter_table[] = {'T', 'R', 'W', 'A', 'G', 'M', 'Y', 'F', 'P', 'D',
    'X', 'B', 'N', 'J', 'Z', 'S', 'Q', 'V', 'H', 'L', 'C', 'K', 'E', '\0'};
    gchar *temp;
    gulong number;

    if (strlen(letter_table)!= 23) {
	perror(_("Invalid letters table:"));
        printf("%d",strlen(letter_table));
        exit(-1);
    }

    if (legal_id<23){
	perror(_("Invalid Legal ID"));
        exit(-1);
    }

    number = legal_id%23;
    temp = (gchar *) g_malloc0(2);
    strncpy(temp,&letter_table[number], 1);
    temp[1]='\0';

    return temp;
}

/**
 * u_enterprise_legal_id_is_correct: 
 * @legal_id: Nmero de cif que se desea comprobar.
 *
 * Comprueba si el nmero del cif de una empresa es correcto.
 *
 * Returns:
 * Digito de control del CIF.
 */
gint
u_enterprise_legal_id_is_correct (gulong legal_id) {

    gchar *letters_legal_id;
    gchar * temp;
    gulong legal_id_aux;
    gint i = 0;
    gint a = 0;
    gint b = 0;
    gint operacion = 0;
    gint c;
    gint d;

    letters_legal_id = (gchar *) g_malloc0(7+1);
    temp = (gchar *) g_malloc0(1+1);
    legal_id_aux = legal_id;
    legal_id = legal_id/10;

    sprintf(letters_legal_id,"%lu", legal_id);
    for (i=1; i<strlen(letters_legal_id);) {
	strncpy(temp, &letters_legal_id[i], 1);
        temp[1] = '\0';
        a += atoi(temp);
        i = i+2;
    }

    for(i=0; i<strlen(letters_legal_id);) {
	strncpy(temp, &letters_legal_id[i], 1);
	temp[1] = '\0';
	operacion = atoi(temp)*2;
	b += ((operacion/10) + (operacion%10));
        i = i + 2;
    }

    c = a + b;

    d = 10 -(c % 10);

    if (d == 10) {
        d = 0;
    }

    g_free(temp);
    g_free(letters_legal_id);
    return ((legal_id_aux%10) == d);
}


/**
 * u_person_legal_id_is_correct: 
 * @legal_id: Nmero de carnet de identidad con la letra del nif al final.
 *
 * Comprueba si el dni es correcto.
 *
 * Returns:
 * 0 si el dni es incorrecto.
 */
gint u_person_legal_id_is_correct (gchar * legal_id) {
    gchar * DNI_char_number;
    gint DNIlength;
    gint DNIcorrect=0;
    gchar * DNI_char;
    gchar * ptr_char;
    gulong DNI_number;

    DNIlength = strlen(legal_id);
    if (DNIlength==9) {
	DNI_char_number = (gchar *) g_malloc (sizeof(gchar)*(DNIlength-1));
	DNI_char_number = g_strndup(legal_id,DNIlength-1);

	DNI_char = (gchar *) g_malloc (sizeof(gchar));
        DNI_char = &legal_id[DNIlength-1];
        ptr_char = &legal_id[DNIlength-1];


	DNI_number = strtoul(DNI_char_number,&ptr_char,10);

	if ((DNI_number>0) && (DNI_number<ULONG_MAX)) {
	    if (strcmp(DNI_char,u_legal_id_calculate_letter (DNI_number))==0) {
		DNIcorrect = 1;
	    }
	}
    }
    return DNIcorrect;
}

/**
 * u_table_put_name: 
 * @query: Lista de atributos a los que anteponer el nombre de la tabla.
 * @name: Nombre de la tabla.
 *
 * Antepone el nomre de una tabla de la base de datos a una lista de atributos separados
 * por coma.
 *
 * Returns:
 * Lista de los atributos con el nombre de la tabla delante de los atributos.
 */
gchar *u_table_put_name (gchar *query, gchar *name) {
    gchar *final_query;
    gint words = 1;
    gint size = strlen(query);
    gint long_name = strlen(name);
    const gchar *p = query;
    gchar *p2;
    gchar *d;
    gchar * blank=" ";
    
    gchar * delimiter = ".";

    if (size == 0) return NULL;
    p = strchr(p,',');
    while (p != NULL) {
	words++;
	p = strchr(p+1,',');
    }
    size += (long_name +1)* words + 1;
    final_query = (char *) g_malloc (size * sizeof(char));
    d = final_query;
    p = query;
    do {
	p2 = strchr(p,',');
	memcpy(d,name,long_name);
        memcpy(d+long_name,delimiter,1);
        d +=(long_name+1);
	if (p2 != NULL) {
	    memcpy(d,p,p2-p+1);
            d+=p2-p+1;
	    p = p2 + 1;
	    while (*p==*blank) {
                p++;
	    }
	} else {
	    strcpy(d,p);
	    d+=strlen(p);
	}
    } while (p2 != NULL);

    return final_query;
}

/**
 * u_remove_aliases: 
 * @query: Cadena con la lista de atributos separados por #delim2 y con el formato "nombre delim1 alias".
 * @delim1: Separador del nombre con el alias.
 * @delim2: Separador de cada elemento de la lista.
 *
 * Borra los alias de una lista de atributos separados por #delim2 y que siempre aparecen despus de
 * #delim1
 *
 * Returns:
 * Cadena con la lista de atributos sin los alias.
 *
 */
gchar *u_remove_aliases (gchar *query, const gchar *delim1, const gchar *delim2) {
    gchar *final_query;
    gchar *position_comma;
    gchar *position_alias;
    gint   index=0;
    gboolean finish = FALSE;

    final_query = (gchar *)g_malloc0(sizeof(char)*strlen(query));

    position_alias = query;
    position_comma = position_alias;

    while ((position_alias!=NULL) && (position_comma!=NULL) && (!finish)) {
        if ((*position_alias!='0') && (*position_comma!='0')) {
	    position_alias = strstr(query, delim1);
	    memcpy(final_query+index, query, (position_alias-query)-1);
	    index += (position_alias-query)-1;
	    if ((position_comma=strstr(position_alias, delim2))!=NULL) {
		query = position_comma;
	    }
	} else {
            finish = TRUE;
	}
    }

    return final_query;
}

/**
 * u_get_hour): 
 * @time: Fecha en tipo #time_t. Es un entero con la fecha tomada deste el 1 de Enero 1970.
 *
 * Devuelve la hora a partir de una fecha con tipo #time_t.
 *
 * Returns:
 * La hora solicitada.
 */
gint u_get_hour (time_t time) {
    struct tm * aux_time;

    aux_time = (struct tm *) malloc(sizeof(struct tm *));
    aux_time = localtime(&time);
    return aux_time->tm_hour;

}

/**
 * u_get_minute:
 * @time: Fecha en tipo #time_t. Es un entero con la fecha tomada deste el 1 de Enero 1970.
 *
 * Devuelve el minuto a partir de una fecha con tipo #time_t.
 *
 * Returns:
 * Los minutos solicitados.
 */
gint u_get_minute (time_t time) {
    struct tm * aux_time;

    aux_time = (struct tm * ) malloc(sizeof(struct tm *));
    aux_time = localtime(&time);
    return aux_time->tm_min;

}

/**
 * u_parse_date: 
 * @date: FEcha en formato texto obtenido de la base de datos.
 *
 * Parsea la fecha obtenida de la base de datos y lo convierte en una fecha de tipo #time_t
 *
 * Returns:
 * Puntero a la fecha parseada y transformada.
 */
time_t * u_parse_date (gchar * date) {
    time_t * time_of_gchar= (time_t *) g_malloc0(sizeof(time_t));
    gchar * delimiters ="/ :";
    gchar new_delimiter = '-';
    gchar * date_to_convert;
    gchar * time_change=NULL;
    struct  tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
    gchar ** tokens;
/*    gchar ** tokens = (gchar**) g_malloc0(sizeof(gchar *));*/

    date_to_convert = g_strdup(date);
    g_strdelimit(date_to_convert,delimiters,new_delimiter);

    tokens = g_strsplit(date_to_convert,"-",7);
    time_tm ->  tm_sec = atoi(tokens[5]);
    time_tm ->  tm_min = atoi(tokens[4]);
    time_tm ->  tm_hour = atoi(tokens[3]);
    time_tm ->  tm_mon = atoi(tokens[0])-1;
    time_tm ->  tm_year = atoi(tokens[2])-1900;
    time_tm ->  tm_mday = atoi(tokens[1]);
    time_tm ->  tm_isdst = 1;
    time_change = tokens[6];

    if (!strcmp(time_change,"CET"))
	*time_of_gchar = mktime(time_tm)+3600;
    else
	*time_of_gchar = mktime(time_tm);

    g_free(date_to_convert);
    g_strfreev(tokens);
    g_free(time_tm);

    return time_of_gchar;
}

/**
 * u_get_current_date: 
 *
 * Devuelve la fecha actual con respecto al 1 de Enero de 1970.
 *
 * Returns:
 * Un puntero a la fecha actual.
 */
time_t * u_get_current_date (void) {
    time_t * time_of_gchar= (time_t *) g_malloc0(sizeof(time_t));
    time_t  time_with_hours;
    struct  tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
    struct  tm * time_tm_with_hours ;

    time_with_hours=time(NULL);
    time_tm_with_hours = localtime(&time_with_hours);
    time_tm ->  tm_sec = 0;
    time_tm ->  tm_min = 0;
    time_tm ->  tm_hour = 0;
    time_tm ->  tm_mon = time_tm_with_hours ->  tm_mon;
    time_tm ->  tm_year = time_tm_with_hours ->  tm_year;
    time_tm ->  tm_mday = time_tm_with_hours -> tm_mday;
    time_tm ->  tm_isdst = 1;

    *time_of_gchar = mktime(time_tm);
    g_free(time_tm);

    return time_of_gchar;
}


/*Returns a shoter date format than ctime*/

/**
 * u_get_parsed_date:
 * @date: Fecha a formatear en una cadena de caracteres.
 *
 * Transforma una fecha en una cadena de caracteres con un formato adecuado.
 *
 * Returns:
 * Cadena con la fecha en formato dd/mm/aaaa.
 *
 */
gchar  *u_get_parsed_date (time_t *date) {
    //    struct tm *fecha = gmtime(date);
    struct tm *fecha = localtime(date);
    gchar     *shorten = g_strdup_printf("%02d/%02d/%4d", fecha->tm_mday,
					 fecha->tm_mon + 1 ,
					 fecha->tm_year + 1900);
    return shorten;
}


/* if digits of number_to_fill are minor than number_total_digits */
/* then fill with zeros at left until number_total_digits */
/* else return the same number */
/**
 * u_fill_with_zeros_until: 
 * @number_to_fill: Nmero a rellenar.
 * @number_total_digits: Nmero de dgitos totales a rellenar. 
 * 
 * Rellena el #number_to_fill con ceros (a la izquierda) hasta un mximo de #number_total_digits, es decir,
 * si el nmero a rellenar
 * tiene 4 cifras y el nmero de cifras es 10 mete 6 ceros a la izquierda del nmero a rellenar.
 *
 * Returns:
 * El nmero con los ceros a la izquierda.
 */
gchar * u_fill_with_zeros_until (gchar * number_to_fill,gint number_total_digits) {
    gchar * number_filled = g_strdup(number_to_fill);
    gint length;
    g_strstrip(number_filled);
    length = strlen(number_filled);

    if (length<number_total_digits) {
	gchar * zero = "0";
	gchar * zeros = g_strconcat("",NULL);
        gchar * number_filled_aux;
	gint index=length;
	while (index<number_total_digits) {
	    gchar * zeros_aux;
	    zeros_aux = g_strconcat(zeros,zero,NULL);
	    g_free(zeros);
	    zeros = zeros_aux;
	    zeros_aux=NULL;
	    index++;
	}
        number_filled_aux = g_strconcat(zeros,number_filled,NULL);
	g_free(number_filled);
	number_filled = number_filled_aux;
	number_filled_aux =NULL;
    }
    return number_filled;
}

/**
 * u_mutex_init: 
 *
 * Inicializa el semforo mutex.
 *
 */
void u_mutex_init (void)
{
    union semun {
	int val;
	struct semid_ds *buf;
	ushort *array;
    };
    union semun ick;

    sem_id = semget(999, 1, IPC_CREAT | 0666);
    if (sem_id < 0) {
       perror ("util/util.c/u_mutex_init:semget");
       exit (1);
    }
    ick.val = 1;
    if (semctl(sem_id, 0, SETVAL, ick) < 0) {
	perror ("util/util.c/u_mutex_init:semctl");
        exit(1);
    }
}

/**
 * u_mutex_on: 
 *
 * Suma uno al semforo y pone el proceso en espera en caso de estar cerrado.
 *
 */
void u_mutex_on (void)
{
    struct sembuf op;

    op.sem_flg = SEM_UNDO;
    op.sem_num = 0;
    op.sem_op  = -1;
    if (semop(sem_id, &op, 1) < 0) {
	perror ("util/util.c/u_mutex_on:semop");
        exit (1);
    }
}

/**
 * u_mutex_off:
 *
 * Libera el semforo y si hay algn  proceso esperando lo despierta.
 *
 */
void u_mutex_off (void)
{
    struct sembuf op;

    op.sem_num = 0;
    op.sem_op  = 1;
    op.sem_flg = SEM_UNDO;
    if (semop(sem_id, &op, 1) < 0) {
        perror ("util/util.c/u_mutex_off:semop");
        exit (1);
    }
}

/**
 * filename2lockname: 
 * @path: Nombre (con path completo) de fichero al que se aadir el lock.
 *
 * Devuelve el nombre del fichero de lock a partir de un nombre de fichero.
 *
 * Returns:
 * El nombre de fichero con la extensin de lock aadida al final.
 */
static gchar *filename2lockname(const gchar *path) {
    gchar *temp = NULL;
    temp = g_strconcat(path, U_LOCK_EXTENSION, NULL);
    return temp;
}

/**
 * u_set_lock: 
 * @path: Nombre de fichero de log que se desea bloquear.
 *
 * Crea un fichero de lock para que nadie pueda escribir en el fichero de log.
 *
 */
void u_set_lock (const gchar *path) {
    gchar *temp =  filename2lockname(path);
    mode_t mask;

    mask = umask(0);
    while(open(temp, O_SYNC|O_EXCL|O_CREAT|O_RDONLY, S_IRUSR) ==-1) {
	nanosleep(&lock_time, NULL);
    }
    umask(mask);
    g_free(temp);
}

/**
 * u_remove_lock: 
 * @path: Nombre de fichero de log que se desea desbloquear.
 *
 * Borra el fichero de lock que est bloqueando la escritura en el fichero de log.
 *
 */
void u_remove_lock (const gchar *path) {
    gchar *temp =  filename2lockname(path);
    gint file = -1;

    file = unlink(temp);
    if(file==-1) {
	printf(_("\tTrying delete: %s\n"), temp);
	perror("util/util.c/u_remove_lock:unlink");
    }
    g_free(temp);
}

/**
 * u_fill_n_spaces: 
 * @char_to_fill: Cadena al que se le desean aadir los espacios.
 * @n: Nmero de espacios a aadir.
 * 
 * Aade n espacios a una cadena de caracteres.
 *
 * Returns:
 * La cadena con los espacios aadidos al final.
 */
gchar * u_fill_n_spaces (gchar * char_to_fill,gint n) {
    gchar * char_filled;
    gint length;
    gint length_aux;
    if (char_to_fill==NULL) {
        char_filled = g_strdup("");
    } else {
	char_filled = g_strdup(char_to_fill);
    }
    length = strlen(char_filled);
    length_aux = length;


    if (length<n+length_aux) {
	gchar * space = " ";
	gchar * spaces = g_strconcat("",NULL);
        gchar * char_filled_aux;
	gint index=0;
	while (index<n) {
	    gchar * spaces_aux;
	    spaces_aux = g_strconcat(spaces,space,NULL);
	    g_free(spaces);
	    spaces = spaces_aux;
	    spaces_aux=NULL;
	    index++;
	}
        char_filled_aux = g_strconcat(char_filled,spaces,NULL);
	g_free(char_filled);
	char_filled = char_filled_aux;
	char_filled_aux =NULL;
    }
    return char_filled;

}

/**
 * u_get_name_of_month:
 * @number: Nmero del mes.
 *
 * Obtiene el nombre del mes con nmero #numnber.
 *
 * Returns:
 * El nombre del mes solicitado, %NULL si el nmero es mayor que 12 o menor que 1.
 */
gchar * u_get_name_of_month (gint number) {
    gchar * temp=NULL;

    switch (number) {
    case 1:{
        temp = g_strdup(_("January"));
        break;
    }
    case 2:{
        temp = g_strdup(_("February"));
        break;
    }
    case 3:{
        temp = g_strdup(_("March"));
        break;
    }
    case 4:{
        temp = g_strdup(_("April"));
        break;
    }
    case 5:{
        temp = g_strdup(_("May"));
        break;
    }
    case 6:{
        temp = g_strdup(_("June"));
        break;
    }
    case 7:{
        temp = g_strdup(_("July"));
        break;
    }
    case 8:{
        temp = g_strdup(_("August"));
        break;
    }
    case 9:{
        temp = g_strdup(_("September"));
        break;
    }
    case 10:{
        temp = g_strdup(_("October"));
        break;
    }
    case 11:{
        temp = g_strdup(_("November"));
        break;
    }
    case 12:{
        temp = g_strdup(_("December"));
        break;
    }
    }
    return temp;
}

/**
 * u_extract_element: 
 * @number: Cadena caracteres con palabras separadas por el carcter #delimit.
 * @index: Posicin de la cadena que se desea obtener dentro de la cadena #number.
 * @delimit: Carcter por el que se desea separar las palabras.
 * @number_of_tokens: Nmero de palabras mximo en las que se desea dividir #number.
 *
 * Extrae el elemento que ocupa la posicin #index dentro de una cadena de caracteres
 * con tokens (palabras/nmeros) separados por #delimit. Utilizada para extraer los cdigos de banco,
 * oficina, dgito de control y nmero de cuenta dentro de un CCC.
 *
 * Returns:
 * La cadena con el elemento solicitado.
 */
gchar * u_extract_element (gchar * number,gint index,const gchar * delimit,gint number_of_tokens) {
    if (number_of_tokens<index) {
	g_print(_("\nError at the access to an element beyond the list scope.\n"));
        return NULL;
    } else {
	gchar * result=NULL;
	gchar ** result_aux = NULL;
	result_aux = g_strsplit(number,delimit,number_of_tokens);
	result = g_strdup(result_aux[index]);
	g_strfreev(result_aux);
        return result;
    }
}

/**
 * u_kill_gda: 
 * 
 * Mata el servidor gda.
 *
 */
void u_kill_gda () {
    system("ps augwx | grep gda-postgres-srv | grep $USER | awk '{ print $2 }' | xargs kill -9");
}


/**
 * u_get_substring: 
 * @original: Cadena de caracteres a partir de la que se obtendr la subcadena.
 * @pos_ini: Posicin inicial de la subcadena.
 * @pos_fin: Posicin final de la subcadena.
 *
 * Obtiene el substring de la cadena #original que empieza en la posicin inicial #pos_ini y la
 * posicin final #pos_fin.
 *
 * Returns:
 * El substring solicitado.
 * 
 */
gchar * u_get_substring (gchar * original,gint pos_ini,gint pos_fin) {
    gchar * inicio;
    gchar * result=NULL;

    if (original!=NULL) {
	if (pos_fin>pos_ini) {
            gint length=strlen(original);
	    if (length<pos_fin) {
		pos_fin=length;
	    }
	    inicio=original+sizeof(gchar)*(pos_ini);


	    result = g_strndup(inicio,(pos_fin-pos_ini+1));

	}
    }


    return result;
}

/**
 * u_get_substring_complete_word: 
 * @original: Cadena a partir de la cual se desea obtener la subcadena.
 * @pos_ini: Posicin inicial a partir de la que se obtendr la subcadena.
 * @pos_fin: Posicin final de la subcadena. En caso de que se posicin final no sea
 * un espacio se va retrasando la posicin hasta encontrar el primer espacio hacia atrs y
 * queda modificado el valor apuntado por #pos_fin a la posicin final.
 *
 * Obtiene una subcadena de una cadena original, descartando las letras que no forman una palabra completa al
 * final, para ello vuelve hacia atrs hasta que encuentra el ltimo espacio en la cadena.
 *
 * Returns:
 * Subcadena requerida, donde el texto est formado por palabras completas.
 */
gchar * u_get_substring_complete_words (gchar * original,gint pos_ini,gint * pos_fin) {
    gchar * inicio;
    gchar * final;
    gchar * result=NULL;

    if (original!=NULL) {
	if ((*pos_fin)>pos_ini) {
	    gint length=strlen(original);

	    inicio=original+sizeof(gchar)*(pos_ini);
	    if (length>(*pos_fin)) {

		final=original+sizeof(gchar)*((*pos_fin)-pos_ini+1);

		while ((*final!=' ') && (*final!='.')) {
		    final = final - sizeof(gchar);
		    (*pos_fin) = (*pos_fin) -1;
		}

		result = g_strndup(inicio,((*pos_fin)-pos_ini+1));
	    } else {
		result = g_strndup(inicio,((length)-pos_ini+1));
	    }
            /*saltar el ultimo caracter y el espacio*/
            *pos_fin=*pos_fin+2;
	}
    }


    return result;
}

/**
 * u_exists: 
 * @path: Path completo del fichero de configuracin.
 *
 * Comburba si existe el fichero de configuracin en el lugar indicado.
 *
 * Returns:
 * 0 si existe, -1 en otro caso.
 */
gint u_exists (const gchar *path) {
    FILE      *parameter_file;
    parameter_file  = fopen(path, "r");

    if(parameter_file==NULL) {
	perror("util/parameter_manager.c/u_parameter_manager_init:fopen");
	printf(_("\tTrying open: %s\n"), path);
	exit(-1);
    }

    if(fclose(parameter_file)) {
	perror("util/parameter_manager.c/u_parameter_manager_init:fclose");
	printf(_("\tTrying close: %s\n"), path);
	exit(-1);
    }

    return 0;
}

/**
 * u_replace: 
 * @original: Cadena originarl
 * @delimiter: Delimitadores que se desean substituir.
 * @new_delimiter: Nuevo delimitador para substituir los delimitadores solicitados.
 *
 * Transforma #original buscando los #delimiter y substituyendolos por #new_delimiter.
 *
 *
 * Returns:
 * Devuelve una zona diferente de memoria con los delimitadores substuidos.
 */
gchar *u_replace (gchar *original, gchar *delimiter, gchar *new_delimiter) {
    gchar *copy     = (gchar *) g_malloc(sizeof(gchar));
    gchar *copy_aux = NULL;

    gint  size_new_delimiter = strlen(new_delimiter);
    gint  size_delimiter = strlen(delimiter);
    gint  size = 0;
    gint  i    = 0;

    gchar *position = original;
    gchar *position_delim = NULL;

    while((position_delim=strstr(position, delimiter))) {

	copy_aux = copy;
	copy = (gchar *)
	    realloc(copy_aux,sizeof(gchar)*
		    (size+(position_delim-position)+size_new_delimiter));

	memcpy(copy+size, position, (position_delim-position));
	memcpy(copy+size+(position_delim-position), new_delimiter,
	       size_new_delimiter);

	size += (position_delim-position)+size_new_delimiter;

	position = position_delim + size_delimiter;
    }

    if (position_delim == NULL) {
	position_delim=strchr(position, '\0');
    }

    copy_aux = copy;
    copy = (gchar *) realloc(copy_aux,sizeof(gchar)*
			     (size+(position_delim-position)+1));

    // Clean the memory realloc
    for (i=size+1; i<size+(position_delim-position)+1; i++) {
	copy[i] = '\0';
    }

    memcpy(copy+size, position, (position_delim-position));

    return copy;
}

/**
 * u_send_mail: 
 * @email: Direccin de correo.
 * @file_path: Path del fichero adjunto que se desea enviar.
 * @subject: Subject del correo.
 *
 * Abre el evolution con la opcin enviar nuevo email.
 *
 * Returns:
 * 0 si se ha enviado correctamente, -1 en otro caso.
 */
gint   u_send_mail (gchar *email, gchar *file_path, gchar *subject) {
    if (email != NULL) {
	gchar * command = g_strconcat("evolution --no-splash mailto:",
				     email, NULL);
	gint result = system(command);

	if ((result==127)||(result==-1)){
	    perror("util/send_mail.system");
	    g_print("%s\n",command);
	    exit(-1);
	}
	g_print("e:%s\tFP:%s\tS:%s\t\n", email, file_path, subject);
        return 0;
    }
    return -1;
}

/**
 * u_round_to_decimal: 
 * @value: Valor a redondear
 * @number_of_decimals: Nmero de decimales que se desean redondear.
 *
 * Redondea un valor en #gdouble(valor real) en #number_of_decimals decimales.
 *
 * Returns:
 * El valor redondeado.
 * 
 */
gdouble u_round_to_decimal (gdouble value,gint number_of_decimals) {
    glong auxiliar = pow (10,number_of_decimals);
    gdouble result=0;

    result = value * auxiliar ;
    if ((ceil(result)  - result)!=0) {
	if ((ceil(result)  - result) <= (result - floor(result))) {
	    result = floor(result);
	    result = result + 1;
	} else {
	    result = floor(result);
	    result = result ;
	}
    } else {
    }
//    result = ceil(result);
    result = result / auxiliar;

    return result;
}

/**
 * u_string_fill_left: 
 * @character: Carcter con el que se desea rellenar por la izquierda.
 * @total_size: Tamao total que se desea rellenar.
 * @string: String que se rellenar.
 *
 * Rellena con el carcter #character hasta un total de #total_size por la izquierda la cadena
 * #string. La memoria utilizada por #string es liberada.
 *
 * Returns:
 * La cadena rellenada por la izquierda.
 * 
 */
gchar * u_string_fill_left (gchar character, gint total_size, gchar * string) {
  gchar * aux_string;
  gint number_of_chars;
  gchar * result;

  number_of_chars = total_size - strlen(string); /* Chars needed to fill the string */

  if (number_of_chars > 0) { /* If we need to fill the string */
    aux_string = g_strnfill(number_of_chars, character);
    result = g_strconcat(aux_string,string,NULL);
    g_free(aux_string);
    g_free(string);
    return result;
  }
  else return string;
}

/* THIS FUNCTION FREES THE MEMORY USED BY THE PARAMETER 'string' */
/**
 * u_string_fill_right: 
 * @character: Carcter con el que se desea rellenar por la derecha.
 * @total_size: Tamao total que se desea rellenar.
 * @string: String que se rellenar.
 *
 * Rellena con el carcter #character hasta un total de #total_size por la derecha la cadena
 * #string. La memoria utilizada por #string es liberada.
 *
 * Returns:
 * La cadena rellenada por la derecha.
 * 
 */
gchar * u_string_fill_right (gchar character, gint total_size, gchar * string) {
  gchar * aux_string;
  gint number_of_chars;
  gchar * result;

  number_of_chars = total_size-strlen(string);

  if (number_of_chars > 0) {
    aux_string = g_strnfill(number_of_chars, character);
    result = g_strconcat(string,aux_string,NULL);
    g_free(aux_string);
    g_free(string);
    return result;
  }
  else return string;
}

/**
 * u_cif_strcmp: Compara dos cifs de empresas.
 * @legal_id1: Primer cif a comparar.
 * @legal_id2: Segundo cif a comparar.
 *
 * Compara la igualdad de dos cifs.
 *
 * Returns:
 * 0 si son iguales. Otro valor en caso de ser diferentes.
 */
gint u_cif_strcmp (gchar * legal_id1,gchar * legal_id2) {
    gchar ** aux1=NULL;
    gchar ** aux2=NULL;
    gchar * aux_legal_id1=NULL;
    gchar * aux_legal_id2=NULL;
    gchar * auxlegal_id1=NULL;
    gchar * auxlegal_id2=NULL;
    gchar * legal_id1_dup = g_strdup(legal_id1);
    gchar * legal_id2_dup = g_strdup(legal_id2);
    gboolean result;

    auxlegal_id1 =  g_strdelimit(legal_id1_dup,G_STR_DELIMITERS,' ');
    auxlegal_id2 =  g_strdelimit(legal_id2_dup,G_STR_DELIMITERS,' ');
    aux1 = g_strsplit(auxlegal_id1," ",4);
    aux2 = g_strsplit(auxlegal_id2," ",4);

    aux_legal_id1 = g_strjoinv(NULL,aux1);
    aux_legal_id2 = g_strjoinv(NULL,aux2);

    g_strfreev(aux1);
    g_strfreev(aux2);

    result = strcmp(aux_legal_id1,aux_legal_id2);
    g_free(aux_legal_id1);
    g_free(aux_legal_id2);
    g_free(auxlegal_id1);
    g_free(auxlegal_id2);

    return result;
}

/**
 * legal_id_tokens
 * @legal_id: Cif del que se desea obtener los tokens.
 *
 * Obtiene los tokens de un cif.
 *
 * Returns:
 * Lista de tokens del cif.
 *
 */

gchar ** u_cif_get_tokens (gchar * legal_id) {
    gchar caracter;
    gint index = 0;
    gint number_of_tokens=1;
    gboolean actual_isdigit=FALSE;
    gchar ** result;
    gchar * result_temp;

    if (legal_id!=NULL) {
	gint equal_number = 0;
	gint indice_tokens=0;
	gchar * temp;
        g_strstrip(legal_id);
	actual_isdigit = isdigit((gchar) legal_id[index]);
	while (legal_id[index]!='\0') {
	    caracter = (gchar) legal_id[index];
	    if (isdigit(caracter)!=FALSE) {
		if (actual_isdigit == FALSE) {
		    number_of_tokens++;
                    actual_isdigit=TRUE;
		}
	    } else {
		if (actual_isdigit != FALSE) {
		    number_of_tokens++;
                    actual_isdigit=FALSE;
		}
	    }
            index++;
	}
	result = (gchar **) g_malloc0(sizeof(gchar *)*(number_of_tokens+1));
        index=0;
	actual_isdigit = isdigit((gchar) legal_id[index]);
	temp=&legal_id[index];
	while (legal_id[index]!='\0') {
	    caracter = (gchar) legal_id[index];
	    if (isdigit(caracter)!=FALSE) {
		if (actual_isdigit == FALSE) {
                    result_temp = g_strndup(temp,equal_number);
		    result[indice_tokens]= g_strdelimit(result_temp,G_STR_DELIMITERS,' ');
		    g_strstrip(result[indice_tokens]);
		    actual_isdigit=TRUE;
		    indice_tokens++;
		    equal_number=1;
		    temp=&legal_id[index];

		} else {
                    equal_number++;
		}
	    } else {
		if (actual_isdigit != FALSE) {
                    result_temp = g_strndup(temp,equal_number);
		    result[indice_tokens]= g_strdelimit(result_temp,G_STR_DELIMITERS,' ');
                    g_strstrip(result[indice_tokens]);
		    indice_tokens++;
                    equal_number=1;
                    actual_isdigit=FALSE;
		    temp=&legal_id[index];
		} else {
                    equal_number++;
		}
	    }
            index++;
	}
	result_temp = g_strndup(temp,equal_number);
	result[indice_tokens]= g_strdelimit(result_temp,G_STR_DELIMITERS,' ');
	g_strstrip(result[indice_tokens]);
        


    } else {
	result = NULL;
    }
return result;
}

